///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   BisectSolver.h
//! \brief  Defines the mcBisectSolver class.
//!
//! See BisectSolver.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef BISECTSOLVER_H
#define BISECTSOLVER_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "BisectSolver.h"
#endif

// required includes
#include "mc/Solver.h"


//! The amplitude of the approximation range: result returned by mcBisectSolver
//! will be +/- mcBISECTSOLVER_DEFAULT_DELTA.
#define mcBISECTSOLVER_DEFAULT_DELTA			mcRealValue(wxString(wxT("1e-10")))

// this will be defined later
class mcMathLine;


//! Implements the bisection algorithm to solve nonlinear
//! equations.
class mcBisectSolver : public mcSolver
{
protected:
	
	//! The precision to which the algorithm searches
	//! the solution: is m_fDelta is 0.01 and the exact
	//! solution of #m_pCurrLine is 1.567, then
	//! mcBisectSolver will return a value comprised
	//! between 1.567-0.01 and 1.567+0.01.
	mcRealValue m_fDelta;

	//! The range where the bisection algorithm is applied at the beginning.
	mcExtRange m_rStart;

protected:

	//! The recursive bisect function.
	//! \param a,b The range where one solution of the current
	//!            equation is surely located.
	//!            Must be f(a)*f(b) < 0.
	//! \param fa, fb The value of the function (equation)
	//!               in the points a, b.
	mcRealValue math_Bisect(mcMathLine &tosolve, const mcSymbolProperties *unk,
					const mcRealValue &a, const mcRealValue &b,
					const mcRealValue &fa, const mcRealValue &fb);
	
	//! Starts the bisection algorithm in the given range, calculating
	//! the values of the function in the two given points and then
	//! calls the #Bisect function.
	mcRealValue math_Bisect(mcMathLine &tosolve, const mcSymbolProperties *unk,
							const mcRealValue &a, const mcRealValue &b);

public:
	mcBisectSolver() {
		m_strName = wxT("Bisection equation solver");
		m_strDesc = wxT("Finds an approximated solution of all types of equations.");
		m_nType = mcST_BISECTION;
		m_fDelta = mcBISECTSOLVER_DEFAULT_DELTA;
	//	m_rStart.DeepCopy(*mcRange::pR);
	}

	virtual ~mcBisectSolver() {}


	//! Solves the given mcMathLine for the given unknown with
	//! the bisection algorithm.
	//! You can find good docs about this algorithm on the web:
	//! it is based over the theorem of zero existence.
	bool math_SolveLine(mcMathLine &, const mcSymbolProperties *unk, 
						mcSystemStepArray &);

	//! Returns TRUE if the given system does not contain more
	//! than one type of unknown and if it does not contain parameters.
	bool math_WorksOn(const mcMathOrSystem &m) const;
	
	
	bool math_isReady() const;
	wxPanel *gui_GetOptionsPanel() const				{ return NULL; }


	//! Changes the delta which defines the approximation error which
	//! is accepted by the program.
	void math_SetDelta(const mcRealValue &d)			{ m_fDelta=d; }

	//! Sets the start range for the algorithm.
	//! If the given range (for the given system) is wrong, then FALSE
	//! is returned and mcSolver::GetLastErr() can be used to retrieve
	//! the error description; returns TRUE on success.
	bool math_SetStartRange(const mcRange &r, const mcMathLine &p,
							const mcSymbolProperties *);
};


#endif		// BISECTSOLVER_H
