///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Bracket.h
//! \brief  Defines the mcBracketData, mcBracketGUI, mcBracketMath,
//!         mcBracketIO, mcBracket classes.
//!
//! See Bracket.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef BRACKET_H
#define BRACKET_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "Bracket.h"
#endif

// required includes
#include "mc/ExpContainer.h"


mcDEFINE_HELPER_CLASSES(mcBracket)



#define mcBRACKET_SIMPLE_BRACKETS			1
#define mcBRACKET_SQUARE_BRACKETS			2



//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
class mcBracketHelpers : public mcExpContainerHelpers
{
	mcDEFINE_REFERENCE_DATA(mcBracket, mcET_BRACKET);

#ifdef mcENABLE_GUI
public:				// customizable variables

	//! If TRUE, then, when the user types wxT('('), wxT('[') or '{', also
	//! the other symbol wxT(')'), wxT(']') or '}' is automatically created
	//! and the content is automatically set as empty.
	//! If FALSE, then the contents of the bracket is automatically
	//! set as everything which follows the '(' symbol.
	static bool sgui_bCreateBothSymbols;

	static int sgui_nSpaceAboveBelow;
	static int sgui_nSpaceLeftRight;

	virtual int gui_GetSpaceAboveBelow() const		{ return sgui_nSpaceAboveBelow; }
	virtual int gui_GetSpaceLeftRight() const		{ return sgui_nSpaceLeftRight; }

	static wxString sgui_strLeftParentheses, sgui_strRightParentheses;

	//! Returns TRUE if the given character is a left parenthesis.
	static bool gui_isLeftParenthesis(const wxChar c) {
		return (c != 0 && sgui_strLeftParentheses.Contains(c));
	}

	//! Returns TRUE if the given character is a right parenthesis.
	static bool gui_isRightParenthesis(const wxChar c) {
		return (c != 0 && sgui_strRightParentheses.Contains(c));
	}

protected:

	//! Font used to draw the parentheses (it's updated each time
	//! the content changes its height).
	wxFont mgui_fLastFont;
	
	//! Size of the '(' bracket symbol. It is supposed to be equal to
	//! the size of the ')' character.
	wxSize mgui_szBracket;

public:

	//! Is the LEFT or RIGHT bracket being removed ?
	bool mgui_bRemoveLeftBracket;

#endif



public:

	mcBracketHelpers() { data_Init(); }
	virtual ~mcBracketHelpers() {}



#ifdef mcENABLE_DATA
public:

#ifdef __MCDEBUG__
	
	wxString data_BaseDebug(long flags) const {
		return wxT("mcBracket [\n") + 
			data_GetContent().data_GetDebug(
			mcMathCore::Get()->m_nIndentationStep, flags) + 
			wxT("mcBracket ]\n");
	}

#endif

	//! Returns a code which is used internally by mcBracket to auto-determine
	//! the 'bracket depth' of this instance, which is used to choose among
	//! normal, square and curly brackets.
	int data_GetBracketDepth() const {
 		//return data_m_nBracketType;
		return data_ChooseBracketType();
	}


	wxString data_GetBracketLeftSymbol(int n = -1) const;
	wxString data_GetBracketRightSymbol(int n = -1) const;

	int data_ChooseBracketType() const;

#endif		// mcENABLE_DATA




#ifdef mcENABLE_GUI
public:

	///////////////////////////////////////////////////////////////////
	//! \name mcExpContainer and mcExpElement abstract functions.
	//! Functions required by compiler to be present. @{

	// brackets begin when the user presses the '(' symbol
	bool gui_isBeginKey(const mcKey &ev) const {
		wxChar c = ev.GetKeyCode();
		if (!sgui_bCreateBothSymbols)	// don't build mcBrackets...
			return FALSE;
		if (c == wxT('(') || c == wxT('[') || c == wxT('{'))
			return TRUE;
		return FALSE;
	}


	//! Works as mcExpContainer::gui_BaseInput but it have two other
	//! tasks:
	//! 1) to change, eventually, the type of brackets drawn by
	//!    this function to reflect the input.
	//! 2) if bCreateBothSymbols == FALSE, to intercept the
	//!    ')' keypresses.
	mcInputRes gui_BaseInput(const mcKey &key, mcElement *pnew);
	mcInputRes gui_HandleContentInput(mcInputRes r, mcElement *);

	int gui_GetContentOffsetX() const;
	int gui_GetContentOffsetY() const;

	bool gui_isLeftPosEnabled() const		{ return TRUE; }
	bool gui_isRightPosEnabled() const		{ return TRUE; }

	void gui_DrawContainer(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const;

	//@}


	///////////////////////////////////////////////////////////////////
	//! \name mcBracketGUI specific functions.
	//! Some utilities + some overrides. @{

	void gui_DoRecalcBaseSize();

	mcInputRes gui_OnRemoveLeftBracket(mcElement *);
	mcInputRes gui_OnRemoveRightBracket(mcElement *);

	static wxFont gui_RebuildFont(const mcElementHelpers *elem, int h);
	
	//@}

#endif		// mcENABLE_GUI




#ifdef mcENABLE_MATH
public:

	//mcExpSimRes math_Distribute(mcPolynomial &, mcMonomial &);
	mcExpSimRes math_RemoveExp();

	mcExpSimRes math_ExpandContents(long flags, mcElement *newelem);
	mcExpSimRes math_SimplifyContents(long flags, mcElement *newelem);
	mcBasicOpRes math_MakeReciprocal(mcElement *);
	
	mcRealValue math_GetBaseLenght() const	{ return mcExpContainerHelpers::math_GetBaseLenght() + 2; }
	int math_GetOrderPos() const			{ return 2; }

#endif		// mcENABLE_MATH




#ifdef mcENABLE_IO
protected:

	wxString io_GetMathMLPresentationTag() const	 { return wxT("mfenced"); }

public:

	bool io_isBeginChar(const wxString &str) const {
		return io_isBracketBeginChar(str);
	}

	static bool io_isBracketBeginChar(const wxString &str) {
		wxChar c = str.GetChar(0);
		if (c == wxT('(') || c == wxT('[') || c == wxT('{'))
			return TRUE;
		return FALSE;
	}

	// inlined expr IO
	wxString io_GetBaseInlinedExpr() const;
	bool io_ImportBaseInlinedExpr(const wxString &str, int *, wxString &);

#endif		// mcENABLE_IO
};


//! Implements the brackets in the program using a mcPolynomial to 
//! hold as content any polynomial.
//! Bracket derives from mcExpContainer because it can have an
//! exponent. Because of its nature (a mcExpContainer and a container
//! for a mcPolynomial class), this class is very simple because
//! recalls methods of the underlying class.
//! The only problem to solve for this class is the bracket's height:
//! brackets must be adaptable to the height of the contained
//! expression.
class mcBracket : public mcExpContainer
{
	mcDEFINE_MAIN_CLASS(Bracket, mcExpContainer);

public:

	mcBracket(const mcPolynomial &content)
	{ data_SetRefData(new mcBracketHelpers()); data_SetContent(content); }
	
	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_BRACKET; }

	int data_GetBracketDepth() const
		{ return hlp()->data_GetBracketDepth(); }
};


#endif // BRACKET_H



