///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Decoration.h
//! \brief  Defines the mcDecorationData, mcDecorationGUI,
//!         mcDecorationIO, mcDecorationMath classes.
//!
//! See Decoration.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef mcDECORATION_H
#define mcDECORATION_H


// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "Decoration.h"
#endif


// required includes
#include "mc/Element.h"


mcDEFINE_HELPER_CLASSES(mcDecoration)


//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
class mcDecorationHelpers : public mcElementHelpers
{
#ifdef mcENABLE_DATA
public:		// DATA variables

	//! mcDecorationHelpers have no children (by default).
	const mcElement &data_GetConstChild(int n) const
		{ return mcEmptyElement; }
#endif



#ifdef mcENABLE_GUI
protected:	// GUI variables

	int mgui_nCursorPos;

#define mcDECORATION_LEFT			0
#define mcDECORATION_RIGHT			1

#endif




public:
	mcDecorationHelpers() { /* data_Init() must be called by derived class */ }
	virtual ~mcDecorationHelpers() {}


protected:

	void gui_Init() {
		mcElementHelpers::gui_Init();
		mgui_nCursorPos = mcDECORATION_RIGHT;
	}




#ifdef mcENABLE_DATA
public:

	void data_DeepCopy(const mcElementHelpers *p) {
		const mcDecorationHelpers *d = (const mcDecorationHelpers *)p;
		mgui_nCursorPos = d->mgui_nCursorPos;
		mcElementHelpers::data_DeepCopy(p);
	}

#ifdef __MCDEBUG__

	wxString data_Debug(long flags) const
		{ return wxT("mcDecoration\n"); }

#endif
#endif		// mcENABLE_DATA



#ifdef mcENABLE_GUI
public:

	/////////////////////////////////////////////////////////////////////////////////
	//! \name mcElementGUI overloaded functions
	//! Implements the behaviour of a general decoration element. @{
	
	//! Always returns FALSE: decorations generally are created 
	//! by the program itself...
	bool gui_isBeginKey(const mcKey &key) const {
		return FALSE;
	}
	
	//! Always returns FALSE: mcEmptyBox::gui_Input function handles all the keypresses
	//! returning mcIR_DELETE_this-> Empty boxes infact, should always be replaced
	//! by a new class.
	bool gui_isEndKey(const mcKey &key) const;
		
	mcInputRes gui_Input(const mcKey &key, mcElement *pnew);
	mcMoveCursorRes gui_MoveCursor(mcMoveCursorFlag flag, long modifiers);
	mcInsertRes gui_Insert(const mcElement &, mcElement *)		{ return mcINSR_OKAY; }

	int gui_MoveCursorUsingPoint(wxDC &dc, const wxPoint &p);
	int gui_GetRelCursorPos(wxDC &dc, wxPoint *pt) const;
	void gui_SetCursorPos(const mcCursorPos &code);
	void gui_GetCursorPos(mcCursorPos &) const;
	mcCursorPos gui_GetCursorPos() const
		{ return mcElementHelpers::gui_GetCursorPos(); }

	//@}
#endif		// mcENABLE_GUI



#ifdef mcENABLE_MATH
public:

	mcExpSimRes math_Simplify(long flags, mcElement *)			{ return mcESR_DONE; }
	mcExpSimRes math_Expand(long flags, mcElement *)			{ return mcESR_DONE; }

	// these do not apply on this object
	bool math_CanBeAddedWith(const mcElement &p) const	{ return FALSE; }
	bool math_CanBeMultWith(const mcElement &p) const	{ return FALSE; }
	bool math_CanBeDivBy(const mcElement &p) const		{ return FALSE; }

	virtual mcBasicOpRes math_Add(const mcElement &, mcElement *p, bool add)	{ return mcBOR_REMOVE_OPERAND; }
	virtual mcBasicOpRes math_Subtract(const mcElement &, mcElement *p) { return mcBOR_REMOVE_OPERAND; }
	virtual mcBasicOpRes math_MultiplyBy(const mcElement &, mcElement *p) { return mcBOR_REMOVE_OPERAND; }
	virtual mcBasicOpRes math_DivideBy(const mcElement &, mcElement *p) { return mcBOR_REMOVE_OPERAND; }
	
	void math_SetExp(const mcPolynomial &p)				{ /* do nothing */ }
	mcBasicOpRes math_MakeReciprocal(mcElement *e)		{ return mcBOR_REMOVE_OPERAND; }
	mcBasicOpRes math_RaiseTo(const mcPolynomial &)		{ return mcBOR_REMOVE_OPERAND; }

	mcMathType math_GetMathType() const 
 		{ return mcMathType(mcMTL1_POLYNOMIAL, mcMTL2_ALGEBRAIC, mcMTL3_CONSTANT); }

	bool math_Compare(const mcElement &p, long flags) const	
		{ return math_CompareThisOnly(p, flags); }

	mcRealValue math_Evaluate() const							{ return *mcRealValue::pNAN; }
	mcRealValue math_GetLenght() const							{ return *mcRealValue::pNAN; }


	int math_GetOrderPos() const							{ return -1; }

	mcMonomial math_GetLCM(const mcElement &) const;//			{ return mcEmptyElement; }
	mcMonomial math_GetGCD(const mcElement &) const;//			{ return mcEmptyElement; }
	//mcPolynomial &math_GetFactors() const					{ return mcEmptyElement; }
#endif		// mcENABLE_MATH




#ifdef mcENABLE_IO
public:	

	bool io_isBeginTag(const wxXml2Node &tag) const {
		if (tag.GetName() == wxT("mdecoration"))
			return TRUE;
		return FALSE;
	}

	bool io_isBeginChar(const wxString &str) const {
		return FALSE;
	}

	wxXml2Node io_GetMathML(bool bGetPresentation) const;
	wxString io_GetInlinedExpr() const;

	bool io_ImportPresentationMathML(wxXml2Node tag, wxString &pErr);	
	bool io_ImportInlinedExpr(const wxString &str, int *count, wxString &pErr);

#endif		// mcENABLE_IO
};




//! A decoration element is an element without any math meaning,
//! which is used only for GUI input.
class mcDecoration : public mcElement 
{
	mcDEFINE_ABSTRACT_CLASS(Decoration, mcElement);

public:

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_DIVOP || t == mcET_MULTOP ||
				t == mcET_ADDOP || t == mcET_SUBOP ||
				t == mcET_PARENTHESIS || t == mcET_TEXT || 
				t == mcET_EMPTYBOX; }
};

#endif	// mcDECORATION_H

