///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Element.h
//! \brief  Defines the mcElementData, mcElementGUI, mcElementMath,
//!         mcElementIO and mcElement classes.
//!
//! See Element.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef ELEMENT_H
#define ELEMENT_H


// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "Element.h"
#endif

// required includes
#include <wx/memory.h>
#include "mc/Setup.h"
#include "mc/Styles.h"
#include "mc/MathUtils.h"
#include "mc/MathHelpers.h"
#include "wx/xml2.h"

#include "mc/Object.h"


//! The following value defines a limit to the maximum number of exponent
//! recursions; it also defines the number of mcElementGUI styles.
//! This #define cannot be placed in mcElementMath because it's required
//! by mcElementGUI, whose declaration comes first.
#define mcELEMENTMATH_MAX_EXPDEPTH							4

//! The maximum number of cycles executed in simplification/expansion 
//! routines to try to perform complete simplification/expansion.
#define mcELEMENTMATH_MAX_PROCESS_CYCLES					100


#define mcELEMENT_DBG_EXPDEPTH		1		//!< Adds expdepth info in mcElement::data_GetDebug().
#define mcELEMENT_DBG_SELECTION		2		//!< Adds selection status in mcElement::data_GetDebug().
#define mcELEMENT_DBG_ID			4		//!< Adds ID info in mcElement::data_GetDebug().


//! The mcElementHelpers::math_Find() function will use the
//! mcElementHelpers::math_Compare() function to perform
//! non-strict comparisons. See #math_Compare() for more info.
#define mcFIND_NOFLAGS				0

//! The mcElementHelpers::math_Find() function will use the
//! mcElementHelpers::math_Compare() function to perform
//! a strict comparison. See #math_Compare() for more info.
#define mcFIND_STRICT				1

//! The mcElementHelpers::math_Find() function will return
//! not the element itself but its parent.
#define mcFIND_PARENT				2


//! The mcElementHelpers::gui_MoveCursor function will just move
//! the cursor without any special behaviour.
#define mcMCF_NOMODIFIERS			0

//! The mcElementHelpers::gui_MoveCursor function will move the
//! cursor, extending the selection in the direction specified
//! by the mcMoveCursorFlag given to the function.
#define mcMCF_EXTEND_SELECTION		1


// compiler needs to know that mcElementGUI, mcElementMath, mcElementIO
// and mcElement classes exist, before we define the mcElementData class.
mcDEFINE_HELPER_CLASSES(mcElement);


// defined later
class mcPolynomial;
class mcMonomial;
extern mcElement mcEmptyElement;
extern mcPolynomial mcEmptyPolynomial;




//! A little class which contains all the data referenced by a mcElement.
//! This is the base class for the specialized classes containing the
//! instances of their helper mcElementData-, mcElementGUI-, mcElementMath-
//! mcElementIO-derived classes.
//! mcElement-derived do not contain pointers to allocated classes:
//! in this way we would have an enormous number of allocations:
//! 1 due to mcElement + 4 due to helper classes (data, gui, math, io).
class mcElementHelpers : public mcObjectRefData
{
	friend class mcElement;


#ifdef mcENABLE_DATA
public:

	//! An array containing one instance of each class type; used by
	//! isCharBeginChar(char) function; it uses this array to call
	//! the io_isBeginChar() function of each element type to do its work
	//! (the pElem array is initialized at the moment of the allocation
	//! of the namespace; see MathCore.cpp).
	static mcElement *sdata_pElem[mcNUM_ELEMENT_TYPES];



	///////////////////////////////////////////////////////////////////////////
	//! \name BASIC CLASS SECTION
	//! The class identifiers of MathCore classes (the possible values
	//! of mcElement::m_nType), and the  functions which handle them. @{

	//! Returns the identifier code (mcET_****) of the class that uses
	//! the given char as begin char; mcET_INVALID if the function can't
	//! find any result. If you specify a pointer for the last argument,
	//! and the class found has derived classes, subtype will be set
	//! to the subtype identifier code for that class.
	static mcElementType gui_isKeyBeginKey(const mcKey &ev);
	static mcElementType io_isCharBeginChar(const wxString &str);
	static mcElementType io_isTagBeginTag(const wxXml2Node &tag);

	//! Allocates a new element of given type and set his parent to
	//! the given pointer (which must be NULL only for the root of
	//! the tree structure); then, eventually initializes it with
	//! the given inlined expression (if IO section is enabled !).
	//! This function can allocate any type of mcElement-derived class.
	//! \sa mcElement::Newhlp()
	static mcElement data_NewElem(mcElementType type, 
								const mcElementHelpers *towrap = NULL);

	//! Returns a pointer to an instance of the given element type.
	//! The given pointer must not be modified and it is completely
	//! unlinked from any tree (no parents, no children).
	static mcElement *data_GetInstanceOf(mcElementType type)
		{ return sdata_pElem[type]; }

	//! Returns a string containing the sizes of the various mcElement
	//! classes.
	static wxString data_GetGlobalMemoryInfo();

	//! data_Updates the internal array of elements used by #GetInstanceOf
	//! and the isChar/Key/Tag/BeginXXXX() functions.
	//! This function is typically used by mcMathCore functions only;
	//! however it is public to allow special uses.
	static void data_UpdateElemArray();

	//! Generates a unique number in the entire MathCore session,
	//! applying a fixed formula based on the current time and the
	//! random generator provided by standard libraries.
	//! Exists a litte possibility that this function produces two
	//! identical values in the same MathCore session, but it's
	//! almost impossible.
	//! \param bFirstMember If TRUE ID generation starts from zero,
	//! 	if FALSE from 0xFFFF
	//! \return a 32-bit unique code.
	static int data_GenID(bool bFirstMember = TRUE);

	//! Inits all the static data of mcElement.
	static void Init();

	//! Destroys all the static data of mcElement.
	static void Cleanup();

	//@} END OF BASIC CLASS SECTION


protected:			// DATA variables

	//! The ID of this element. This should be a unique value in the
	//! entire MathCore session: see MathCore::GenID() for more info
	//! about its generation at run-time. This code is used to identify
	//! one specific element inside the input data.
	int mdata_nID;

	//! The properties of this element, stored as bit array (use values
	//! defined in Element.h to check and set flags).
	int mdata_nProperties;

#endif



#ifdef mcENABLE_GUI
protected:			// GUI variables

	//! Array of MathCore styles; access this array indirectly
	//! using the gui_GetStyle() function.
	//!
	//! example:
	//! \code
	//! [...]
	//! gui_SelectObject(hDC, gui_GetStyle(this).hFont);
	//! SetTextColor(hDC, gui_GetStyle(this).clrText);
	//! [...]
	//! \endcode
	const mcStyleArray *mgui_pStyleArray;

	//! TRUE if the element is selected (totally or partially).
	//! This variable cannot be accessed by derived classes: forcing
	//! use of the functions #gui_OnSelect() and #gui_DeSelect() avoids
	//! errors and nasty bugs because when the selection states
	//! of member classes' must change...
	bool mgui_bSelected;

	//! Size of the element (in pixels). It should be always up-to-date;
	//! so, if you modify size in some way, call the #gui_RecalcSize()
	//! function.
	wxSize mgui_sz;

	//! The exponent level of this element. A null value means that
	//! this element is not an exponent; 1 means this is an exponent;
	//! 2 means this is an exponent of an exponent and so on...
	//! This variable can't be accessed by derived classes: forcing
	//! use of the functions #SetExpDepth() and #GetExpDepth() avoid
	//! errors and nasty bugs because when this value changes also
	//! the values of derived classes' exponent depth must change...
	//!
	//! \note Why do we have to sore the exponent depth besides the
	//!       parent which owns this element ? The parent always has
	//!       the exponent level of this element minus 1, isn't it ?
	//!       Well, the problem is that it's not always true: consider
	//!       a bracket and its contained polynomial. The polynomial
	//!       has the bracket as parent, but its same exponent depth.
	//!       Instead, the exponent of a mcNumber has the mcNumber as
	//!       parent, but an higher exponent level.
	//!       In conclusion, we do really need to keep two values
	//!       (exponent depth and parent) in each element.
	int mgui_nExpDepth;

public:			// customizable variables

	//! The brush used to draw the selected portions of the math data.
	//! When user selects a piece of math data, its background is filled
	//! with this colour.
	static wxBrush *sgui_pSelectionBrush;

	//! The brush used to draw active elements (elements which are located
	//! under the cursor).
	//! All the elements are highlighted when the mouse is moved over them. This
	//! is the background colour of the highlight.
	static wxBrush *sgui_pActivationBrush;

	//! The array of styles used by default by all mcElements.
	//! To change the style of a particular element (and the style of
	//! all its children), use the mcElement::SetStyle() function.
	//! To change the default style used by all mcElements, change
	//! this pointer.
	static mcStyleArray *sgui_pDefaultStyles;

	//! The additional number of pixels on the left & right of the active element
	//! to fill with #sgui_pActivationBrush.
	//! This should be a small value which should be use to adjust the activation
	//! rectangle which is drawn too small on some platforms.
	static int sgui_nAdditionalActivationSpaceLeftRight;

#endif


public:
		
	///////////////////////////////////////////////////////////////////////////
	//! \name mcELEMENT PROPERTIES SECTION
	//! mcElementHelpers::m_nProperties variable stores flags as bit array;
	//! see #data_AddProperty() and #data_RemoveProperty() functions
	//! to get more info about setting and removing a mcElement property. These
	//! properties must be properties regarding the current element only; for
	//! changes that applies also to the children of the current element you
	//! must change the mcElement definition.
	//@{

	// properties which are set by the element itself and are handled by the parent

	//! The element has no special properties. This is the default initial property
	//! of every mcElement. If you want to add a property to the element, add in the
	//! mcElement constructor a call to mcElement::data_AddProperty.
	#define mcEP_NOPROPERTIES 						0

	//! The parent calls the mcElement::gui_OnParentSizeChange when the parent size
	//! changes...
	#define mcEP_NOTIFY_PARENTSIZE_CHANGE			1

	//! The parent calls the mcElement::data_OnNeighborChange() function when the one of the
	//! adjacent (on left/right, tipically) elements of this object, changes its size.
	#define mcEP_NOTIFY_NEIGHBOR_CHANGE				2

	//! The element is a container and it has the ability to "take out" elements
	//! to simplify itself.
	#define mcEP_FACTOREOUT_ALLOWED					4

	//! If set, the element has been correctly initialized. If not, the element is
	//! not ready yet or it does not use this flag (which is generally used for
	//! GUI input only).
	#define mcEP_INITIALIZED						8


	// properties which can be set by mcElement-external functions but which must be
	// handled by the element.

	//! The exponent of the element is allocated. 
	//! This flag is necessary; an alternative would be the use of mcEP_HIDDEN 
	//! on the exponent itself; however you cannot check the mcEP_HIDDEN flag 
	//! on the exponent when it doesn't exist (this happens often: to
	//! optimize performances, exponents are created on-the-fly, when required...).
	#define mcEP_HASEXPONENT							16

	//! The subscript of the element is allocated. 
	//! This flag is necessary; an alternative would be the use of mcEP_HIDDEN 
	//! on the subscript itself; however you cannot check the mcEP_HIDDEN flag 
	//! on the subscript when it doesn't exist (this happens often: to
	//! optimize performances, subscripts are created on-the-fly, when required...).
	#define mcEP_HASSUBSCRIPT							32

	//! This is the maximum value used by the standard mcElement property functions;
	//! If a mcElement-derived class needs to define a new property, it can use the
	//! multiples of this value (2*mcEP_NONSTANDARD_BASE, 3*mcEP_NONSTANDARD_BASE) to
	//! define it...
	#define mcEP_NONSTANDARD_BASE						64

	//@}
	// END OF mcELEMENT PROPERTIES SECTION



public:

	//! Default constructor.
	mcElementHelpers();

	virtual ~mcElementHelpers() {}



	
#ifdef mcENABLE_DATA
public:

	//////////////////////////////////////////////////////////////
	//! \name Properties control functions
	//! See the previous section for more info about properties. @{

	//! Adds the given property to this element.
	virtual void data_AddProperty(int p, void *val1 = NULL, void *val2 = NULL) {

		// m_nProperties stores info as bit array; use OR op
		// to add properties, AND op to check if a property is set
		mdata_nProperties |= p;
	}

	//! Removes the given property to this element.
	virtual void data_RemoveProperty(int p) {
		mdata_nProperties &= ~p;
	}

	//! Returns TRUE if this element has the given property set.
	virtual bool data_hasProperty(int p) const {
		return ((mdata_nProperties & p) != 0);
	}

	//@}



	//////////////////////////////////////////////////////////////
	//! \name Recursive properties control functions
	//! See the previous section for more info about properties. @{

	//! math_Adds the given property to this element.
	virtual void data_AddRecursiveProperty(mcElementType t, int p, void *val1 = NULL, void *val2 = NULL);

	//! Removes the given property to this element.
	virtual void data_RemoveRecursiveProperty(mcElementType t, int p);

	//! Returns TRUE if this element has the given property set.
	virtual bool data_hasRecursiveProperty(mcElementType t, int p) const;

	//@}


	
	//////////////////////////////////////////////////////////////
	//! \name Abstract general functions
	//! Functions which must be implemented in derived classes. @{

	//! Returns a pointer to a new object identical to the current element.
	//! Derived classes must implement this function creating a new identical
	//! mcElement and returning it.
	//! See also mcIMPLEMENT_CLONE_FNC macro for a quick way to create this function.
	//! \return A pointer to a new element which must be deleted by the caller
	virtual mcElementHelpers *data_Clone() const = 0;	

	//! Returns the size of this class and all its helper classes
	//! calculated using the sizeof() operator.
	//! See the mcDEFINE_GETMEMORYINFO_FNC macro for a quick way to implement
	//! this function.
	virtual int data_GetMemoryInfo() const = 0;

	//! Returns the identifier code for this class.
	//! The identifier code is one of the mcElementType enumeration values
	//! and it should uniquely identify this mcElmentHelpers-derived class.
	virtual mcElementType data_GetType() const = 0;

	//@}



	//////////////////////////////////////////////////////////////
	//! \name Standard methods (overridables)
	//! Override only if necessary. @{

	//! Deep copies all the info contained in this class. The mcElement implementation
	//! should be enough because all the element's data should be contained inside
	//! the associated mcElementData derived class. In fact, mcElementData does not
	//! provide a standard implementation of the DeepCopy() function. mcElement-derived
	//! classes, instead, should not contain any other variable, so this function
	//! should be enough....
	virtual void data_DeepCopy(const mcElementHelpers *p);

	//! Performs a deep comparison of the data hold by this class and
	//! the data hold by the given mcElementHelpers object.	
	virtual bool data_isSameAs(const mcElementHelpers *) const;
	
	//! Returns TRUE if this element is allowed to be directly on the left
	//! of the given element type. If this function returns FALSE
	//! (default implementation returns TRUE), the container element
	//! will interpose a mcMultOp between this element and the following.
	//! \note This is not a GUI-specific function because it is used
	//!       also by non-GUI functions, like mcMonomial::data_Repair().
	virtual bool data_isAllowedBefore(mcElementType type) const		 { return TRUE; }
	
	//! Returns TRUE if this element is allows to be placed directly on the
	//! right of the given element type.
	//! See #isAllowedBefore for more info.
	virtual bool data_isAllowedAfter(mcElementType type) const		 { return TRUE; }

	//! Returns the element with the given ID.
	//! Non-container classes should just return their ID (which is the
	//! default implementation behaviour).
	//! Container classes, instead, should search recursively among their
	//! children and then return the one which matches with the given ID.
	//! If the element with the given ID cannot be found, this function
	//! returns NULL.
	virtual mcElement &data_GetElemFromID(int id) const;

	//! Called by parent when neighbors change.
	//! Override this function if you need to do something when
	//! neighbor elements change (remember to set up the flag
	//! mcEP_NOTIFY_NEIGHBOR_CHANGE in the constructor).
	virtual void data_OnNeighborChange() {
		// default implementation does nothing
	}

	//! Updates this element and all its children (recursive function).
	//! Default implementation of #data_Update() does nothing: override it
	//! if you need to do something else...
	virtual void data_Update();

	//! Returns simple info about this element. It should return something
	//! in this form:
	//!
	//! [class name: 14 char, pad with spaces the end of the name if less than 14
	//! characters][info specific for this element][with final '\\n']
	//!
	//! For example, for the mcSymbol class defining 'a', default debug output is:
	//!
	//! "mcSymbol      a\n"
	//!
#ifdef __MCDEBUG__
	virtual wxString data_Debug(long flags) const
		{ return wxT("mcElement\n"); }
	
	//! Returns the name of this class.
	//! This function is different from mcElementHelpers::data_Debug()
	//! function because it returns always only the name of the class without any
	//! additional info about its status & its children (while data_GetDebug() does).
	virtual wxString data_GetDebugName() const
		{ return wxT("mcElement"); }

#else
	// this function should be removed by the linker...
	inline wxString data_Debug(long) const 
		{ return wxEmptyString; }
	inline wxString data_GetDebugName() const 
		{ return wxEmptyString; }
#endif	

	//@}




	/////////////////////////////////////////////////////////////////////
	//! \name Functions which MUST be overridden by container classes
	//! Functions which _MUST_ be overridden by container classes to ensure
	//! a correct behaviour of the element. @{

	//! Does some standard checks on the element integrity. If something is
	//! wrong an mcASSERT fails. Except for some short moments (the element
	//! creation/deallocation...) none of the checks done by the function
	//! should never fail.
	//! This is a recursive function.
#ifdef __MCDEBUG__
	virtual void data_Check() const;
#else
	// this function should be removed by the linker...
	inline wxString data_Check() const { return wxEmptyString; }
#endif

	//! Returns the number of children attached to this tree node.
	//! This function can be used on *any* mcElement node because this
	//! is a virtual function which is overridden by container classes
	//! which thus return the correct number of children.
	//!
	//! This function is used by various mcElement functions like
	//! #data_Check() const #data_GetElemFromID(), ... which must call some
	//! standard mcElement functions on all nodes attached
	//! to this element.
	virtual int data_GetChildrenCount() const		{	return 0; }

	//! Returns the n-th tree node attached to this element.
	//! You can use this function to traverse the entire tree:
	//!
	//! \code
	//! mcElement rootnode = [...];
	//! MyWalkerFnc(rootnode);
	//!
	//! void MyWalkerFnc(mcElement p) {
	//!
	//!     // do what you want on the "p" node...
	//!
	//!     for (int i=0; i < p.data_GetChildrenCount(); i++)
	//!         MyWalkerFnc(p.data_GetChild(i));
	//! }
	//! \endcode
	//!
	//! \param n The n-th child of this object.
	virtual const mcElement &data_GetConstChild(int n) const;
	
	//! Like #data_GetConstChild, but this function returns a non-const pointer.
	//! \note This function *must* have a different name from #data_GetConstChild
	//!       otherwise, stack overflows would occur since this function
	//!       calls #data_GetConstChild.
	virtual mcElement &data_GetChild(int n);
	
	//! math_Replaces the idx-th child of this element with a copy of the given one.	
	virtual void data_SetChild(int idx, const mcElement &newchild) {
		// non-container elements should just do nothing...
 	}

	//! Returns the index of the children which is allocated in the given
	//! memory address.
	//! If the given pointer does not identify a children of this element,
	//! the returned value is -1.
	virtual int data_GetChildIdx(const mcElement &p) const;
	
 	//! Tries to attach the children of *this to the given element,
 	//! respecting the math meaning of each one.
 	//! If the function does not manage to perform its task, it returns
 	//! FALSE (for example, when the given element's type and this.data_GetType()
 	//! do not match).
	virtual bool data_AddChildrenTo(mcElement p);

	// Called by mcElement's standard functions to update the parent-child
	// relationships of member classes.
	// Override this function if you need to change parent-relationships.
	// virtual void data_UpdateParent()
	//
	// WARNING: THIS FUNCTION IS USELESS: WHEN AN OBJECT HAS MEMBER CLASSES
	//          WHICH ARE TIED TO THE ELEMENT ITSELF AS SIBLINGS OR CHILDREN,
	//          IT'S NOT REQUIRED TO UPDATE THEM... see also #m_nExpDepth

	//@}




	//////////////////////////////////////////////////////////////
	//! \name Standard methods (not overridables)
	//! You just cannot override these functions... @{

	//! Returns the ID for this element.
	int data_GetID() const			{ return mdata_nID;	}

	//! Inits this class calling the #gui_Init, #math_Init and #io_Init
	//! functions, if required. This function should called by the
	//! last overload of the constructor of the mcElementHelpers-derived
	//! class, that is, when the build process is completed and the
	//! xxx_Init() functions called are the functions of the derived class.
	void data_Init();

	//! Returns more complete debug information about this element
	//! (using the #data_Debug function).
	//! \param indent The indentation to leave at the beginning of the string.
	//! \param flags Allows to specify additional info to get
	//!              through the mcELEMENT_DBG_ID, mcELEMENT_DBG_EXPDEPTH flags...
#ifdef __MCDEBUG__
	wxString data_GetDebug(int indent, long flags) const;
#else
	// this function should be removed by the linker
	inline wxString data_GetDebug(int, long) const 
		{ return wxEmptyString; }
#endif

	//@}

#endif		// mcENABLE_DATA





#ifdef mcENABLE_GUI
protected:

	//! Inits this section of the element. 
	//! Acts like a constructor (it's called by #mcElementHelpers).
	virtual void gui_Init();

public:

	//////////////////////////////////////////////////////////////
	//! \name mcElement univocal begin and end characters
	//! The characters that univocally defines the beginning and the
	//! ending of this element. @{

	//! Returns a positive value if the given character marks the begin
	//! of this element; FALSE otherwise.
	//! The begin character cannot be dependent from any variable; it
	//! must be always the same.
	//! The character is given both as ASCII code and as virtual key code.
	//! \param ev The wxWidgets key event for the keypress. It contains the
	//!	           untraslated code for the input key; see wxWidgets
	//!	           KeyCodes for more info.
	//! \return A value identifying the subclass of this element that
	//!	        matched the given character; if the element has no subclasses
	//!	        it should return a null value when the given character is
	//!	        the begin character of the element. See MathCore::NewElem
	//!	        for more info.
	virtual bool gui_isBeginKey(const mcKey &ev) const = 0;

	//! Returns TRUE if the given character marks the end of this element;
	//! this function is always called before calling ::gui_Input function.
	//! isEndChar can return values dependent from the cursor position.
	//! The character is given both as ASCII code and as virtual key code
	//! \param ev The wxWidgets key event for the keypress. It contains the
	//!		untraslated code for the input key; see wxWidgets
	//!		KeyCodes for more info.
	virtual bool gui_isEndKey(const mcKey &ev) const = 0;

	//@}




	/////////////////////////////////////////////////////////////////////
	//! \name Pure virtual functions
	//! Functions which must be implemented in the derived classes. @{

	// functions grouped under this section belong to the
	// USER INPUT SYSTEM group

	//! Called by the parent node of the mcElement to draw the element.
	//! The function must draw the element on the given device context
	//! (DC) at the given position (x;y). The function can change every
	//! text-related setting on the DC and select every font or brush
	//! it needs, but it cant affect other DC properties like the clipping
	//! region, the mapping mode or the current logical function.
	//!
	//! Before begin drawing, the element should check for the mcEP_HIDDEN
	//! property: if it's set the function should just return mcDRW_NOACTIVEELEM.
	//! Just place the two following lines at the beginning of the function:
	//! \code
	//! if (this->data_hasProperty(mcEP_HIDDEN))
	//!	 return mcDRW_NOACTIVEELEM;
	//! \endcode
	//!
	//! The element must draw itself using correct fonts and colors; these
	//! properties cannot be set by the element, they depend from the elements
	//! exponent level depth. The element can retrieve its style using the
	//! mcElementGUI function #gui_GetStyle() (for more info, please refer to the
	//! STYLES SECTION in Element.h). The function must always, except
	//! when otherwise specified by activation state, draws the element with
	//! a transparent background.
	//!
	//! The style structure contains the font and the color to use for each
	//! main type of graphical output: it contains different colors for symbols,
	//! functions, numbers... you must use the one which matches with your elements
	//! type. To select in a quick way text colors and fonts, you should use
	//! the mcElementGUI function #gui_SelectStyle().
	//!
	//! Last, if you need to draw lines or boxes or other primitives, you
	//! must draw them with a specific thickness: use mcElementGUI function
	//! #gui_GetThickness() to get it.
	//!
	//! This function is called a lot of times; thus it should be as fast and
	//! accurate as possible.
	//!
	//! \param dc The device context where the element must be drawn
	//! \param x, y The coordinates where the element must be placed
	//! \param pt If this pointer is equal to mcDRW_ALLACTIVE, the entire
    //!        element must be drawn with mcElementHelpers::sgui_m_pActivationBrush background
    //!        brush; if this pointer is equal to mcDRW_NONACTIVE, the function must
    //!        simply draw everything with default transparent mode; if the pointer
    //!        is not one of the previous values, the function must use the pointer
    //!        to find the subelement where it lies. Then, only the subelement's gui_Draw()
    //!        function must be called with the initial pointer. The subelements
    //!        must be called with mcDRW_ALLACTIVE flag only for special reasons.
    //! \return The ID of the element marked as active (the element which is drawn
    //!         with mcElementHelpers::sgui_m_pActivationBrush as background), or mcDRW_NOACTIVEELEM
    //!         flag if this element does not contain the mouse cursor.
    virtual int gui_Draw(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const = 0;

	//! This function is called by mcElement and non-mcElement functions when the 
	//! internal size variable must be updated due to a change in the structure of 
	//! the element.
	//! This function should be as fast and accurate as possible and it must store
	//! the new size in the mgui_sz variable. 
	//! \note It's very important, to maintain the size member updated, to call, 
	//!       in the other mcElement functions, #gui_RecalcSize() every time size could 
	//!       have changed (for example, in #gui_Input(), mcElementData setters...).
	virtual void gui_DoRecalcSize() = 0;

	//! Returns the height of the cursor and the (x;y) position
	//! of the cursor (in the given pointer) relative to the top-left point
	//! of this element. The function must fill the
	//! wxPoint pointer using coordinates relative to the top-left point
	//! of its bounding rectangle. When this function is called, the cursor
	//! is always inside the element.
	//! \param dc The device context where the element was drawn last time.
	//! \param pt The coordinates, relative to the element (assume the
	//!	    element placed at 0;0), indicating the top-left point of the
	//!	    cursor.
	virtual int gui_GetRelCursorPos(wxDC &dc, wxPoint *pt) const = 0;

	//! This function is called to move the cursor which is inside the
	//! element, one 'step' left, right, up or down (when
	//! flag=mcMCF_LEFT/mcMCF_RIGHT/mcMCF_UP/mcMCF_DOWN). 
	//! Elements which cannot move the cursor
	//! in the given direction, should call mcMathCore::Get()->SyntaxError()
	//! function. If the cursor must 'exit' the element (because its at
	//! the end, for example) the function must return mcMCR_SETFOCUS_PREVIOUS
	//! or mcMCR_SETFOCUS_NEXT.
	//! This function must always
	//! return the code mcMCR_SETFOCUS_* or mcMCR_OKAY. When this function is called,
	//! cursor is always inside the element.
	//! \param flag One of the Move Cursor flags (see MathTypes.h)
	virtual mcMoveCursorRes gui_MoveCursor(mcMoveCursorFlag flag, long modifiers) = 0;

	//! Moves the cursor inside this element in the closest available
	//! position to the given point. The function must use the given
	//! point to set its internal cursor-position variables (when
	//! this function is called, the given point is always inside
	//! the bounding rectangle of the element, but maybe the cursor
	//! is not).
	//! \param dc The device context where the element was drawn last time
	//! \param p a valid pointer to a wxPoint
	//!	       (containing coord. relative to the top-left point of the element)
	//!	       which must be used to find the selected subelement; otherwise this
	//!	       is a NULL value
	virtual int gui_MoveCursorUsingPoint(wxDC &dc, const wxPoint &p) = 0;

	//! Returns the cursor position as Cursor Position flag.
	//! It must return mcCP_BEGIN if cursor is at the leftmost
	//! point of the element, mcCP_END if the cursor is at the rightmost
	//! point of the element, mcCP_INSIDE otherwise. When this function is
	//! called the cursor is always inside the element.
	virtual void gui_GetCursorPos(mcCursorPos &) const = 0;

	//! Returns the cursor position after creating mcCursorPos.
	//! This function uses the #gui_GetCursorPos but it is a little slower
	//! since it allocates a mcCursorPos oject while the other overload
	//! uses an already existing object given by the caller.
	virtual mcCursorPos gui_GetCursorPos() const
		{ mcCursorPos ret; gui_GetCursorPos(ret); return ret; }

	//! Sets the cursor position inside the element using Cursor Position flags.
	//! When it's called to set the cursor at the end of the base, code==mcCP_END and
	//! data_hasProperty(mcEP_HASEXPONENT) == FALSE, when it's called to set the cursor at the
	//! beginning of the base, code==mcCP_BEGIN. The function doesnt need to care
	//! for the third Cursor Position flag: mcCP_INSIDE, actually this function will
	//! never be called with this code.
	virtual void gui_SetCursorPos(const mcCursorPos &code) = 0;

	//! This function is called when the cursor is inside the element and
	//! the user presses a key (which is not the element's end tag).
	//! The way the element should process the
	//! given character is dependent to the element type: a mcNumber will insert
	//! vk next to the cursor position if vk is a digit. If vk is not acceptable,
	//! the function should call the MathCore error-handler function,
	//! mcMathCore::Get()->SyntaxError.
	//! This function should return one of the values defined for mcInputRes:
	//! mcIR_OKAY, mcIR_DELETE_THIS, mcIR_DIRECT_DELETE, mcIR_DELETE_NEXT.
	//! \param ev The wxWidgets key event for the keypress. It contains the
	//!		untraslated code for the input key; see wxWidgets
	//!		KeyCodes for more info.
	//! \param newelem If the function returns mcIR_REPLACE_THIS, the caller will
	//!		           use this pointer as a substitute for this element (which
	//!		           will be deleted).
	virtual mcInputRes gui_Input(const mcKey &ev, mcElement *newelem) = 0;

	//! Inserts the given element at the right of the cursor position.
	virtual mcInsertRes gui_Insert(const mcElement &toinsert, mcElement *newelem) = 0;

	//@}




	/////////////////////////////////////////////////////////////////////
	//! \name Functions which MUST be overridden by container classes
	//! Functions which _MUST_ be overridden by container classes to ensure
	//! a correct behaviour of the element. Non-container classes should
	//! avoid to overload these functions. @{

	//! Returns TRUE if this element is a container class; FALSE if this element
	//! is just a simple element without any subelement. This function is used
	//! by some functions to decide what must be done. For example, when a container
	//! element is selected, maybe there are non-selected subelements inside it.
	//! In this case it shouldn't be enclosed in

	//! This function is called when the bounding box of this element intersects
	//! with the selection rectangle created by the user with the mouse cursor.
	//! The standard implementation just selects this element. If you are
	//! implementing a container element, you must override this function and
	//! check contained elements' bounding boxes and then call the gui_OnSelect()
	//! function of the contained elements whose bounding boxes intersect the
	//! given selection rectangle.
	virtual void gui_OnSelect(wxDC &, wxRect &rc) {
		gui_Select();
	}

	//! Returns the selected portion of this element. If this element is entirely
	//! selected (this is the default for non-container elements), the entire
	//! element should be returned (this is what the default implementation does).
	//! Container elements should override this function returning only the
	//! selected elements (in mcMonomials, mcBrackets, mcPolynomials...) or the
	//! entire element itself if it is entirely selected.
	//! This functions returns an object and not a reference to an object
	//! (as for example does #gui_GetActiveElem) because the selection cannot
	//! be modified through this function.
	virtual mcElement gui_GetSelection() const;

	//! Deletes the selected children of this element, without breaking the
	//! integrity of this element: a data_Check() constcall after this function should
	//! throw no errors.
	//! The caller should first check if the element is entirely selected;
	//! if it is, the element should be deleted (by the caller) and this
	//! function should not be called.
	//! This is a recursive function.	
	virtual void gui_DeleteSelection();

	//! Returns the currently active element; that is, the element currently
	//! under the mouse cursor (whose position is given) or NULL if there is
	//! no active element inside this one (for container elements).
	//! The default implementation of this function returns the element itself
	//! but container classes should return the active subelement.
	virtual mcElement &gui_GetActiveElem(int x, int y, const wxPoint &pt) const;

	//@}




	/////////////////////////////////////////////////////////////////////
	//! \name Overrideables functions
	//! Functions which can be overridden but that should work without changes. @{

	//! Changes recursively the styles of this element and the styles
	//! of all its children.
	virtual void gui_SetStyleArray(const mcStyleArray *p);

	//! Called by the mcElementHelpers::sgui_RecalcSize
	virtual void gui_OnRecalcSize(wxSize old);

	//! Called by parent when parent's size changes.
	//! Override this function if you need to do something when
	//! parent's size changes (remember to set up the flag
	//! mcEP_NOTIFY_PARENTSIZE_CHANGE in the constructor).
	virtual void gui_OnParentSizeChange() {
		// default implementation does nothing
	}

	//! Recalculates the size of the elements and all the subelements contained
	//! into this one (this is not done normally by gui_RecalcSize). This function
	//! must be obviously overridden by container classes. The standard
	//! implementation recalculates the size of this element and the sizes
	//! of the children.
	virtual void gui_DeepRecalcSize();

	//! Returns TRUE only if the element and all its children are completely selected.
	//! This is a recursive function.
	virtual bool gui_isAllSelected() const;

	//! gui_Selects the entire element and all its children.
	//! This is a recursive function.
	virtual void gui_SelectAll();

	//! If this element is selected, this function deselects it.
	//! Use this function to turn off selection; you can't
	//! directly access mcElement::mgui_bSelected variable because when
	//! selection state change, also derived classes' selection states
	//! must change. This function ensures this is done, calling the
	//! #data_Updategui_SelectionState() function.
	//! To select the element, see #gui_OnSelect function().
	virtual void gui_DeSelect();

	//! gui_Splits this element in two parts. The first is stored in this element,
	//! the second is returned as a pointer (the caller must then delete it when
	//! it's no longer necessary). The split point must be determined using the
	//! current cursor position. This function has a sense only for few elements,
	//! see as an example mcNumber::gui_Split: when the user tries to insert a new
	//! operator like '/' and the cursor is inside a number, the number is split
	//! in two parts and this function is used to get the second half.
	//! If the cursor is placed in at the leftmost/rightmost point of the element
	//! and, thus, the first/second half of the element would be empty, then
	//! the function should return NULL.
	//! The classes that cannot be splitted should return NULL. This is what
	//! the standard implementation does.
	//! \return TRUE if *this must be deleted; FALSE if it must be kept.
	virtual bool gui_Split(mcElement *p)		{ return FALSE;	}

	//! Merges the given element with this object.
	//! Like splitting, very few elements support merging, but those elements
	//! which handles splitting, should also handle merging... see #gui_Split().
	//! The function should merge the given element in *this and return TRUE
	//! if everything was okay; FALSE if the merging could not be done
	//! (because, for example, the given element is not of the right type...).
	virtual bool gui_MergeWith(const mcElement &) {
		return FALSE; // default implementation does nothing
	}

	//! Returns TRUE if this element supports splitting and, thus, also
	//! merging. The default implementation returns FALSE because this
	//! is a quite unused feature. Elements which override this function
	//! and return TRUE, must override also the #gui_Split() and #gui_MergeWith()
	//! functions to actually do the work...
	virtual bool gui_isSplittable() const {
		return FALSE;
	}

	//! Sets this element as selected. This function doesn't call
	//! the #data_Updategui_SelectionState() function because the gui_OnSelect()
	//! function should care about setting up the selection states
	//! for the member classes. Use this function with attention.
	virtual void gui_Select() {

		// we don't need to do anything else: size hasn't changed
		mgui_bSelected = TRUE;
	}

	//! Returns TRUE if this element is selected (totally or partially).
	//! This is a very quick function because all the elements owns a
	//! specific flag for selection state.
	virtual bool gui_isSelected() const {
		return mgui_bSelected;
	}

	//! Returns TRUE if this element can accept cursor (and then calls
	//! the #gui_SetCursorPos() function with the given code) or FALSE
	//! if the element, for some reason, cannot accept cursor (in this
	//! case this element should be skipped).
	//! The standard implementation returns TRUE and calls #gui_SetCursorPos().
	//! \param code The position where the cursor must be placed in this
	//!        element: if mcCP_END, then the cursor is assumed to be
	//!        entering from right; if mcCP_BEGIN, the cursor is assumed
	//!        to be entering from left.
	virtual bool gui_LetInCursor(mcCursorPos code) {
		gui_SetCursorPos(code);
		return TRUE;
	}

	//! Recalculates the size of the element, calling in any case the
	//! mcElement::gui_OnRecalcSize function. 
	//! See #gui_DoRecalcSize() for more info.
	virtual void gui_RecalcSize();

	//! Returns the exponent depth of this element.
	//! This is a positive value; for more info see #mgui_nExpDepth.
	int gui_GetExpDepth() const				{ return mgui_nExpDepth; }

	//! Returns the size; the returned size can be not up-to-date
	//! (this function cannot update size if it is invalid, because
	//! this is a CONST function; use gui_RecalcSize() before calling
	//! this function to be sure to get updated results).
	virtual wxSize gui_GetSize() const		{ return mgui_sz; }

	//! Returns the width of the element. See #GetConstSize()
	//! for some important info about the usage of this function.
	virtual int gui_GetWidth() const		{ return mgui_sz.GetWidth(); }

	//! Returns the width of the element. See #GetConstSize()
	//! for some important info about the usage of this function.
	virtual int gui_GetHeight() const		{ return mgui_sz.GetHeight(); }

	//! Returns the updated size (in pixels) of the element.
	//! \note This function cannot be CONST because it could be
	//!       forced to call the #gui_RecalcSize() function; use the
	//!       #GetConstSize() function instead.
	virtual wxSize gui_GetSize() {
		if (mgui_sz.GetWidth() == mgui_sz.GetHeight() && 
			mgui_sz.GetWidth() == -1)
			gui_RecalcSize();
		return mgui_sz;
	}

	//! Returns the updated width, in pixels, of this element.
	//! \note This function cannot be CONST because it could be
	//!       forced to call the #gui_RecalcSize() function; use the
	//!       #GetConstSize() function instead.
	virtual int gui_GetWidth() {
		if (mgui_sz.GetWidth() == -1)
			gui_RecalcSize();
		return mgui_sz.GetWidth();
	}

	//! Returns the updated height, in pixels, of this element.
	//! \note This function cannot be CONST because it could be
	//!       forced to call the #gui_RecalcSize() function.
	virtual int gui_GetHeight() {
		if (mgui_sz.GetHeight() == -1)
			gui_RecalcSize();
		return mgui_sz.GetHeight();
	}

	//! Returns the y amount of this mcElement to subtract from
	//! the center line to draw this element.
	virtual int gui_GetYAnchor() const {
		return gui_GetSize().GetHeight()/2;
	}

	//@}



	///////////////////////////////////////////////////////////////////////////
	//! \name Styles STATIC functions
	//! Defines a lot of attributes which are used to create
	//! the graphical output for all MathCore classes. @{

	//! Returns the exponent-depth encoded in the given flags which should
	//! be of the type of those given to a mcElementHelpers::gui_Draw-like
	//! function (that is, a composition of mcDRW_XXXX flags).
	//static int gui_GetExpDepthFromDrawFlags()		
	//	{ return flags & mcDRW_FLAGS_STARTBIT; }

	//! Resets the style array to the defaults, eventually deleting
	//! previous styles. This function should be called at the
	//! beginning of the program, before styles are used.
	//! \sa mcMathCore::Setup()
	static void gui_InitDefaultStyles();

	//! Removes the allocated styles from the default style array of mcElementGUI.
	//! This function must be called only when styles will no longer be
	//! used (like in program shutdown).
	//! \sa mcMathCore::Cleanup()
	static void gui_DeleteDefaultStyles();

	//! math_Replaces all the styles used by MathCore classes to create
	//! the GUI output. The given array is *NOT* copied; instead, it
	//! is linked to mcElementGUI. In this way, the user of MathCore
	//! classes (and some special MathCore classes, like mcMathMng)
	//! can set globally different
	//static void gui_SetGlobalStyleArray(const mcStyleArray *);

	//! Sets the style of this element to the first valid style that
	//! the function finds climbing up the tree (that is, searching
	//! among its parents).
	//bool SyncStyleWithParent();

	//! Returns the style for the n-th exponent level.
	//! This function is static, so it can be used in this way:
	//!
	//! \code
	//| [...]   mcElementHelpers::sgui_GetStyle(0).[...]
	//! \endcode
	static const mcStyle *gui_GetDefaultStyle(int n);

	//! Returns the number of pixels per point size using the current screen
	//! settings to compute them.
	//! This is an utility function used by #gui_PointSize2Pixels & #gui_Pixels2PointSize.
	static double gui_GetPointSize();

	//! Utility function: use it to convert point size units to pixels units.
	//! Point size units is the type of unit used by wxWidgets font functions.
	static double gui_PointSize2Pixels(double pointsize);

	//! Utility function: use it to convert pixels to point size units.
	//! Point size units is the type of unit used by wxWidgets font functions.
	//! See also the #gui_PointSize2Pixels function.
	static double gui_Pixels2PointSize(double pointsize);

	//! Returns the string height and width (using the selected font to determine them).
	//! \param dc A valid pointer to the DC whose selected font will be used to
	//! 	calculate string size, or a NULL value to use a temporary DC.
	//! \param str The string whose size will be calculated.
	static wxSize gui_GetSizeOf(wxDC *dc, const wxString &str);

	static int gui_GetHeightOf(wxDC *, const wxString &);	//!< Returns only the height of ::gui_GetSizeOf.
	static int gui_GetWidthOf(wxDC *, const wxString &);   //!< Returns only the width of ::gui_GetSizeOf.

	//@}



	///////////////////////////////////////////////////////////////////////////
	//! \name Styles non-STATIC functions.
	//! Functions which makes style-access easier but requires a THIS pointer. @{

	//! Called by mcElement's standard functions to update the exponent
	//! depth of member classes.
	//! Override this function if you need to update member variables
	//! when the exponent level of this element changes (this happens 
	//! each time #gui_SetExpDepth() function is called; thus this 
	//! happens each time #gui_SetAsExpOf or #gui_SetAtSameLevelOf
	//! functions are called).
	//! \note This function DOES NOT call the gui_RecalcSize() function,
	//!       even if probably it is required after such function.
	virtual void gui_UpdateExpDepth();

	//! Sets this element as exponent of the given pointer.
	//! This function automatically calls the gui_UpdateExpDepth() function.
	//! \param p The element which will be the base of the current element.
	void gui_SetAsExpOf(const mcElement &p);

	//! Sets this element at the same level of the given element.
	//! For 'level', I mean exponent depth. This function is called at
	//! least once just after the creation of the element.
	//! This function automatically calls the gui_UpdateExpDepth() function.
	//! \param p The element from which exponent depth will be copied.
	void gui_SetAtSameLevelOf(const mcElement &p);

	//! Sets the exponent depth of this element.
	//! This function automatically calls the gui_UpdateExpDepth() function.
	void gui_SetExpDepth(int n)
		{ mgui_nExpDepth = n; gui_UpdateExpDepth(); }

	//! Returns the style associated with the n-th exponent depth using
	//! the style array owned by this element.
	//! If you want to get the default style for the n-th exponent depth,
	//! you can use the #gui_GetDefaultStyle() function.
	const mcStyle *gui_GetStyle(int n) const;

	//! Returns the style array associated with this element.
	const mcStyleArray *gui_GetStyleArray() const
		{ return mgui_pStyleArray; }

	//! Returns the size of a character (assume that fonts used are
	//! fixed pitch fonts and thus all characters have the same size)
	//! in the style used by the given pointer;
	//! \param dc A valid pointer to the DC which will be used to select
	//! 	   the style of the element and to calculate the character
	//! 	   size, or a NULL value to use a temporary DC.
	//! \param p The element whose style will be used to calculate the char size;
	//! 	   if you use a NULL pointer, currently selected style will be used
	wxSize gui_GetSizeOfChar(wxDC *dc = NULL, const mcElement &p = mcEmptyElement) const;

	int gui_GetHeightOfChar(wxDC *, const mcElement &) const;	//!< Returns only the height of #gui_GetSizeOfChar
	int gui_GetWidthOfChar(wxDC *, const mcElement &) const;    //!< Returns only the width of #gui_GetSizeOfChar.

	//! The style used to draw each element is chosen using the
	//! element's exponent level; use this function to retrieve
	//! the style which should be used for the currentelement.
	//!
	//! VERY IMPORTANT: the style shouldn't be stored in any
	//! variable; it should be chosen on the fly during draw functions
	//! (style could change between two 'steps').
	//! \return A pointer to the style for the given element
	const mcStyle *gui_GetStyleForThis() const;

	//! Automatically selects the style for the given element in the given DC,
	//! recognizing its mcElementType.
	//! \param dc The device context where the style will be selected
	void gui_SelectStyle(wxDC &dc) const;

	//! Returns the thickness (in pixels) the caller should use to draw
	//! decoration elements (for example, the fraction line).
	//! \param dc A valid pointer to the DC which will be used to select
	//! 	the style of the element and to calculate the default
	//! 	thickness, or a NULL value to use a temporary DC.
	int gui_GetThickness(wxDC *dc) const;

	//! Use the following function to get the num of pixels between
	//! the exponent miny and the base miny; that is, the empty space
	//! between the exponent and the base on y axis.
	//! This function should be used every time an exp must be drawn;
	//! in this way, we ensure that all the exponents appear always
	//! at the same level over the base.
	//! \param h The height (in pixels) of the exponent.
	int gui_GetBaseOffsety(int h) const;

	//! Returns the num of pixels between the base's bottom border and
	//! the subscript's (of the base) bottom border.
	//! \param h The height (in pixels) of the subscript.
	int gui_GetSubscriptOffsety(int h) const;

	//@}

#endif		// mcENABLE_GUI







#ifdef mcENABLE_MATH
protected:		// utils

	//! Initializes the MATH section of this element.
	virtual void math_Init() {}

	//! Recursively search the n-th occurrence of the given element
	//! both in *this and both in *this' children using
	//! #math_RecursiveFindInChildrenOnly().
	//! If this function cannot find in *this or *this' children
	//! the n-th occurrence of the given element, then it returns
	//! NULL and to \c n the number of elements found is subtracted.
	//!
	//! \note This function is used by #math_Find() to perform its work.
	//!       You should use #math_Find() wherever it's possible since
	//!       it hides the reference need.
	virtual mcElement math_RecursiveFind(int &n, 
				const mcElement &p, long flags) const;	

	//! Works exactly like #math_RecursiveFind() except that this function
	//! does not perform comparison between \c p and *this.
	virtual mcElement math_RecursiveFindInChildrenOnly(int &n, 
				const mcElement &p, long flags) const;


public:


	/////////////////////////////////////////////////////////////////////////
	//! \name Search & replace
	//! Various versions of math_Find() and math_Replace() functions. @{

	//! Returns the n-th occurrence of an element with the same contents
	//! (this function uses #math_CompareWith, not #hasSameContentOf to perform
	//! the comparison !) or NULL if there is no an n-th occurrence of such 
	//! an element.
	//! The element to search is not required to be of the same type of *this,
	//! since this function recursively calls itself on the children of *this.
	mcElement math_Find(int n, const mcElement &p, long flags = mcFIND_NOFLAGS) const;

	//! Works like #math_Find() but its action is limited to the children 
	//! (recursively) of *this only and not also *this.
	mcElement math_FindInChildrenOnly(int n, const mcElement &p, long flags = mcFIND_NOFLAGS) const;

	//! Works like #math_FindInChildrenOnly() but this function does not search
	//! recursively; that is, this function won't search in the children of
	//! the children of *this.
	//! \return The index of the n-th children which match the given 
	//!         element (comparison is performed through #math_Compare()).
	int math_NonRecursiveFindInChildren(int n, const mcElement &p, long flags = mcFIND_NOFLAGS) const;

	//! Returns the number of elements with the same contents of the given
	//! one contained into this one.
	virtual int math_GetCountOf(const mcElement &p, long flags = mcFIND_NOFLAGS) const;

	//! Works like #math_GetCountOf() but does not compare the given element
	//! with *this, just with the children of *this.
	virtual int math_GetCountOfChildrenOnly(const mcElement &p, long flags = mcFIND_NOFLAGS) const;

	//! Returns the number of symbols of the given type contained in *this
	//! element and its children (recursive search).
	virtual int math_GetCountOf(const mcSymbolProperties *sym, long flags = mcFIND_NOFLAGS) const;

	//! Returns the number of symbols belonging to the given mcSymbolArray are
	//! contained in *this element and its children (recursive search). 
	virtual int math_GetCountOf(const mcSymbolArray *arr, long flags = mcFIND_NOFLAGS) const;

	//! Unlike #math_GetCountOf(const mcSymbolArray *, long) this function does
	//! not return the total number of symbols contained: it returns 
	//! how many *types* of symbols of the given array are contained into *this
	//! and its children (recursive search).
	//! Eg:
	//!         4ax^2+bx+x   -> math_GetCountOf(mcSymbol::arrUnknowns) = 3
	//!                         (because "x" is repeated three times)
	//!                      -> math_GetCountOfSymbolsType(mcSymbol::arrUnknowns) = 1
	//!                         (because only one type of unknown is used, "x")	
	//!         where wxT("x") is registered as unknown and wxT("a") and "b" are not.
	virtual int math_GetCountOfSymbolsType(const mcSymbolArray *arr, long flags = mcFIND_NOFLAGS) const;

	//! Replaces the n-th occurrence of the element 'tofind' with a copy
	//! of the element 'replacement'. If n == -1, all the occurrences of
	//! the element to find are replaced.
	//! If bAttachChildren == TRUE, then this function will try to attach
	//! the children of the just replaced element to the copy of
	//! 'replacement' using the mcElement::AttachChildrenTo.
	//! In any case, this function won't replace *this if *this matches
	//! the 'tofind' element.
	//! \return The number of occurrences replaced.
	//!
	//! VERY IMPORTANT: this function should never be used to replace
	//!                 operators or to replace elements with operators,
	//!                 since operators follow special rules in
	//!                 mcElementArrays and could broke data integrity.
	virtual int math_Replace(const mcElement &tofind, int n, long flags,
						const mcElement &replacement, bool bAttachChildren);

	//! Returns TRUE if the given element is contained inside this one.
	//! While #hasSameContentOf returns TRUE if the two elements involved have
	//! exactly the same contents (nothing more, nothing less), this function
	//! returns TRUE even if the given element is only a subset of those
	//! contained.
	//! This function does *not* work with math_MaxSimplify()ed copies of the
	//! two elements involved.
	//!
	//! Some examples:
	//! - the polynomial wxT("2x+5^2") contains "x"
	//! - the monomial wxT("(8^3)*(1/y)") contains wxT("3") but it doesn't contain "y^(-1)"
	//! - the polynomial wxT("x^7+x^x") contains 3 times "x"
	virtual bool math_Contains(const mcElement &p, long flags = mcFIND_NOFLAGS) const;		

	//! See #math_Contains.
	//bool math_Contains(const mcElement &p, long flags = mcFIND_NOFLAGS) const;		

	//! Returns TRUE if one of the symbols registered in the 
	//! given array is contained inside this element.
	bool math_ContainsOneOf(const mcSymbolArray *) const;

	//! Returns TRUE if this element contains the given type of
	virtual bool math_ContainsSymbol(const mcSymbolProperties *s, 
						const mcPolynomial &exp = mcEmptyPolynomial) const;
	virtual bool math_ContainsNumber(const mcRealValue &n, 
						const mcPolynomial &exp = mcEmptyPolynomial) const;

	//! Returns TRUE if this element contains one or more unknowns.
	//! The unknowns which are checked are those registered in
	//! mcSymbol::arrUnknowns.
	//! This function performs a recursive search in this element.
	virtual bool math_ContainsUnknowns() const;

	//! Like #math_ContainsUnknowns() but the array checked is mcSymbol::arrConstants.
	virtual bool math_ContainsConstants() const;

	//! Like #math_ContainsUnknowns() but the array checked is mcSymbol::arrParameters.
	virtual bool math_ContainsParameters() const;

	//! Returns TRUE if this element contains at least one unknown/constant/parameter.
	//! This function does not check for unregistered symbols...
	virtual bool math_ContainsSymbols() const;

	//! Returns TRUE if this element contains unregistered symbols.
	virtual bool math_ContainsInvalidSymbols() const;

	//@}


	
	/////////////////////////////////////////////////////////////////////////
	//! \name Math engine handy functions.
	//! Virtual functions implemented using the abstract mcElementMath
	//! functions. @{

	//! Use the following function to check if the given element can
	//! allocate an exponent.
	//! \param p The element to be tested.
	static bool math_isExpAllowed(mcElement p);
	static bool math_isSubAllowed(mcElement p);			//! See #math_isExpAllowed.

	//! Completely simplifies this element.
	//! This function typically works calling #math_Simplify until it does not
	//! return mcESR_DONE. Anyway, it doesn't modify the operation counter.
	virtual mcExpSimRes math_MaxSimplify(long flags);

	//! Returns TRUE if the element is completely simplified;
	//! FALSE if calls to  math_Simplify(long flags) will make changes to the element.
	virtual bool math_isMaxSimplified(long flags) const;

	//! Returns the number of symbols identic to the one given
	//! (uses #math_CompareWith to perform comparison) and place a pointer
	//! to each symbol in the given array of pointers of size \c 'size'.
	//!
	//! If the array is too small, then the function returns the number
	//! of symbols of the given type which have been found but with a
	//! negative sign.
	//! If no symbols of the given type are found, then the function returns 0.
	virtual int math_GetSymbolList(mcSymbol **, int size, const mcSymbol &tofind) const;
	virtual int math_GetSymbolList(mcSymbol **, int size, mcSymbolProperties &tofind) const;

	//! Returns TRUE if this element contains mcSymbol registered as constants
	//! and/or mcNumbers only.
	virtual bool math_isConstant() const;

	//! Returns TRUE if this element can be evaluated (see #math_Evaluate())
	//! and its value matches the given one.
	virtual bool math_EvaluatesTo(const mcRealValue &val) const
		{ return math_Evaluate() == val; }

	//! Returns the result of the #math_Evaluate function called on a copy
	//! of this element with the given symbol substituted by the given value.
	virtual mcRealValue math_EvaluateAt(const mcSymbolProperties *sym, 
										const mcRealValue &symval) const;

	//! Returns TRUE if this element evaluates to a finite value;
	//! returns FALSE if this element could not be evaluated (result is NAN)
	//! or this element evaluates to a infinite value.
	virtual bool math_isFinite() const;
	
	//! Returns the lenght of this element, taking in count also the
	//! children. Uses the #math_GetLenght() function to do the job.
	virtual mcRealValue math_GetTotalLenght() const;

	//! Adds the given element to this element using the #math_Add() function
	//! and then checking that the return value is mcBOR_REMOVE_OPERAND:
	//! that is, the operation was completed successfully and *this now
	//! contains the result of the operation. If the #math_Add() function does
	//! not return mcBOR_REMOVE_OPERAND then an ASSERT fails: use math_SimpleXXXX
	//! functions only when you are sure that the operation can be performed
	//! without special operations.
	virtual void math_SimpleAdd(const mcElement &p, bool add = TRUE);
	virtual void math_SimpleSubtract(const mcElement &p)	{ math_SimpleAdd(p, FALSE); }
	virtual void math_SimpleMultiplyBy(const mcElement &p);
	virtual void math_SimpleDivideBy(const mcElement &p);

	//! Raises this element to the given mcPolynomial, checking that
	//! the return value is mcBOR_REMOVE_OPERAND.
	//! If it is not, then a mcASSERT will fail.
	void math_SimpleRaiseTo(const mcPolynomial &);

	//! Returns TRUE if *this element is math consistent,
	//! that is, it is not a mcParenthesis or a mcEmptyBox and it does not
	//! contain invalid math elements (this function is recursive).
	bool math_isValidMath() const;

	//! Returns TRUE if *this should be put before the given element
	//! in an ordered math list: this function is used by mcElementArray-derived
	//! classes to reorder their elements.
	//! The default implementation of this function just compares the
	//! order position (retrieved through the #math_GetOrderPos function) of
	//! *this with the order position of the given element.
	virtual bool math_isListedBeforeOf(const mcElement &p) const;

	//@}




	/////////////////////////////////////////////////////////////////////////
	//! \name Math engine ABSTRACT functions.
	//! Abstract functions to implement in mcElementMath-derived classes. @{

	//! A powerful recognizer function which automatically classifies
	//! the type of the math data contained in this class and returns
	//! the respective identifier.
	virtual mcMathType math_GetMathType() const = 0;
		
	//! Returns the 'lenght' of this element without taking in count
	//! its children.
	//! The wxT('lenght') of this element is the 'complexity' represented
	//! by the data placed in the mcElementData of this element.
	//! This is a key function for the simplify and expand routines of
	//! MathCore because through the use of this value, the system
	//! is able to decide if an operation determines a simplification
	//! or a complication. The simple example below shows how this
	//! can be done:
	//!
	//! \code
	//!      mcPolynomial mypol(NULL);
	//!      mypol.Init("2^16");
	//!      mcRealValue n1 = mypol.math_GetTotalLenght();
 	//!
	//!      // after this op, mypol == 65536
	//!      mypol.math_Expand(long flags);
	//!      mcRealValue n2 = mypol.math_GetTotalLenght();
	//!
	//!      // the complexity of mypol should now be higher because
	//!      // 65536 must be considered longer than 2^8
	//!      bool shouldbetrue = n2 > n1;
	//! \endcode
	//!
	//! (The #math_GetTotalLenght() function works using this function !)
	//! The default implementation returns zero. This is a valid behaviour
	//! only for container elements, that is, for those elements where only
	//! the children carry information. For simple elements and for those
	//! containers which stand for an operation (like mcFraction which
	//! implies a division), the default implementation must be overridden.
	//!
	//! A lenght/complexity of 1 means that this element would require 
	//! only a single byte of memory if stored in an ASCII string. An 
	//! element long 2 units (like mcBracket) would require 2 bytes (one 
	//! for the left parenthesis and one for the right one)...
	virtual mcRealValue math_GetLenght() const = 0;
	
	//! Simplifies this element one "step"; that is, after one call to
	//! this function the element must be slightly changed. For example,
	//! this monomial requires three calls to math_Simplify to be completely
	//! reduced in a basic form:
	//!
	//! 12ax*3 + 4a/(x^-1)  .	first call to math_Simplify
	//! 36ax + 4a/(1/x)	   .	second call
	//! 36ax + 4ax	   .	third call
	//! 40ax
	//!
	//! \return Returns TRUE if the element is completely simplied and
	//!	        thus, the following calls to math_Simplify won't have effect.
	virtual mcExpSimRes math_Simplify(long flags, mcElement *newelem) = 0;

	//! Expands this element one "step". expanding means that the aim of
	//! this function is exactly the opposite of #math_Simplify: instead
	//! making the function body shorter, it tries to make it as long as
	//! possible:
	//!
	//! (x - 2)^3              . first call to math_Expand
	//! x^3 - 6x^2 + 12x -8
	//!
	virtual mcExpSimRes math_Expand(long flags, mcElement *newelem) = 0;

	//! Compares *this element with the given one and then compare the
	//! children of *this and the children of the given element, too.
	//! If they are the same, then the function returns TRUE; FALSE otherwise.
	//! This function is different from 
	//! #math_hasSameContentOf() because it doesn't work on MaxSimply()ed copies.
	//! This function tipically uses the #math_CompareThisOnly() function.
	virtual bool math_Compare(const mcElement &p, long flags) const = 0;

	//! Compares this element and the given one and then stops, unlike
	//! #math_Compare() that compares children too.
	virtual bool math_CompareThisOnly(const mcElement &p, long flags) const;

	//! Evaluates the numeric value of this element and then returns it.
	//! The evaluation can proceed *only* if this element does _not_ contain
	//! any parameter or unknown; in case it contains one of these, then
	//! the returned value is *mcRealValue::pNAN.
	virtual mcRealValue math_Evaluate() const = 0;

	//! Returns the order position for this element.
	virtual int math_GetOrderPos() const = 0;

	//! Factores-out everything which can be factored out from this element.
	//! Returns a mcPolynomial containing a copy of those elements factored-out.
	//! This function is CONST since many times not all the factores which can
	//! be taken out will be used and thus is caller's duty the modification
	//! of this element, eventually calling the #math_DivideBy function.
	virtual mcMonomial math_GetFactors() const;

	//@}


	///////////////////////////////////////////////////////////////////////////
	//! \name BASIC operations
	//! Abstract functions defining basic add/sub/mult/div operations. @{

	//! Returns TRUE if this element can be added with the given element.
	//! If the function returns TRUE, then the #math_Add() function should
	//! return one of mcBOR_REMOVE_OPERAND, mcBOR_REPLACE_BOTH or
	//! mcBOR_REPLACE_OPERAND.
	virtual bool math_CanBeAddedWith(const mcElement &p) const = 0;

	//! Works like #math_CanBemath_AddeWith(), just for multiplication.
	virtual bool math_CanBeMultWith(const mcElement &p) const = 0;

	//! Works like #math_CanBemath_AddeWith(), just for divisions.
	virtual bool math_CanBeDivBy(const mcElement &p) const = 0;

	//! Sums or subtracts this element with the given \c e element.
	//! A unique function for add/subtract operations is used because
	//! the algorithms used to handle this two operations are usually
	//! almost identic and thus this avoids a lot of repetitions.
	//!
	//! If the two elements cannot be summed, then an ASSERT should fail
	//! since before using this function, the caller should always use
	//! the #math_CanBeAddedWith() function.
	//! 
	//! The function returns one of mcBasicOpRes values, eventually
	//! using the \c pp pointer to store the replacement element which
	//! will be used by the caller in case this function returns
	//! mcBOR_REPLACE_OPERAND or mcBOR_REPLACE_BOTH.
	virtual mcBasicOpRes math_Add(const mcElement &e, mcElement *pp, bool add = TRUE) = 0;

	//! math_s two mcElements of the same type and stores the result in
	//! *this object.
	mcBasicOpRes math_Subtract(const mcElement &e, mcElement *pp)
		{ return math_Add(e, pp, FALSE); }

	//! Multiplies two mcElements of the same type and stores the result in
	//! this object.
	//! \param p A pointer to a mcElement of the same type of this mcElement,
	//!	      which will be multiplied with this mcElement.
	virtual mcBasicOpRes math_MultiplyBy(const mcElement &e, mcElement *pp) = 0;

	//! Divides two mcElements of the same type and stores the result in
	//! this object.
	//! \param p A pointer to a mcElement of the same type of this mcElement,
	//!	      which will be used to divide this mcElement.
	virtual mcBasicOpRes math_DivideBy(const mcElement &e, mcElement *pp) = 0;

	//! Returns the LCM between *this and the given element.
	virtual mcMonomial math_GetLCM(const mcElement &) const = 0;
	virtual mcMonomial math_GetGCD(const mcElement &) const = 0;

	//@}



	///////////////////////////////////////////////////////////////////////////
	//! \name BASIC operations regarding EXPONENTS
	//! Abstract functions defining basic raise/set op on the exp. @{

	//! Raises *this element to the given polynomial.	
	//! Returns mcBOR_INVALID if the operation could not be performed
	//! (this is typically because the base is negative), or
	//! mcBOR_REMOVE_OPERAND if the operation was okay.
	//!
	//! \note All MathCore elements must implement this function,
	//!       not only mcExpElement-derived classes because this
	//!       operation *must* be feasible on every element: those
	//!       elements which do not hold an exponent, must raise their
	//!       children directly.
	//!       mcFraction, for example, must implement this operation
	//!       directly raising the numerator & denominator.
	virtual mcBasicOpRes math_RaiseTo(const mcPolynomial &) = 0;

	//! Calculates the reciprocal of this element; tipically, this
	//! function creates a mcFraction whose numerator is 1 and the 
	//! denominator contains *this.
	//! This should be the default behaviour. A simple multiplication
	//! of the current exponent by -1 should be avoided when possible.
	//! \note The #math_RaiseTo function could be used instead of this
	//!       one but this function should be preferred over
	//!                   math_RaiseTo(mcPolynomial(-1));
	//!       because this function ?????????? FIXME
	virtual mcBasicOpRes math_MakeReciprocal(mcElement *replacement);

	//! Sets the exponent of this element to match the given polynomial.
	//! As for #math_RaiseTo, for elements which do not have an exponent
	//! by their own, this function should just call the children's
	//! math_SetExp functions.
	virtual void math_SetExp(const mcPolynomial &);	

	//@}


#endif		// mcENABLE_MATH







#ifdef mcENABLE_IO
protected:

	//! Initializes the IO section of this element.
	virtual void io_Init() {}

	//! Post-processes this element after the import-process.
	//! This function will be called after the entire process performed
	//! by #io_ImportInlinedExpr and #io_ImportPresentationMathML and thus
	//! when all the data tree has already been built (from this function
	//! you can access neighbor elements, for example).
	virtual void io_PostProcess() {}

	//! Calls the #io_PostProcess functions of all the children of this
	//! element; call this one just before exiting ##io_ImportInlinedExpr
	//! and ##io_ImportPresentationMathML.
	virtual void io_PostProcessChildren();

public:

	//! Inits this element with the given INLINED expression; for more info
	//! about inlined expressions, see the mcElement::io_ImportInlinedExpr
	//! function. If the given string is not valid for this element as
	//! init string, then this function returns FALSE.
	virtual bool io_Init(const wxString &str);

	//////////////////////////////////////////////////////////////
	//! \name Input & output functions (abstract functions)
	//! These functions import or export the element's data. @{

	//! Returns a positive value if the given MathML content tag
	//! marks the begin of this element; FALSE otherwise.
	//! The begin tag cannot be dependent from any variable; it
	//! must be always the same.
	//! \param tag The string containing the TAG to test.
	//! \return A value identifying the subclass of this element that
	//!	        matched the given character; if the element has no subclasses
	//!	        it should return a null value when the given character is
	//!	        the begin character of the element. See MathCore::NewElem
	//!	        for more info
	virtual bool io_isBeginTag(const wxXml2Node &tag) const = 0;

	//! Returns a positive value if the given inlined expression
	//! marks the begin of this element;  FALSE otherwise.
	//! The begin tag cannot be dependent from any variable; it
	//! must be always the same.
	virtual bool io_isBeginChar(const wxString &str) const = 0;

	//! Returns the MathML code for this element as presentation markup
	//! (http://www.w3.org/TR/MathML2/chapter3.html) or as content markup
	//! (http://www.w3.org/TR/MathML2/chapter4.html).
	//! \param bGetPresentation TRUE if you want to get the presentation
	//!                         markup, FALSE if you want the content MathML.
	//! \return The XML tree containing the MathML code for this element.
	virtual wxXml2Node io_GetMathML(bool bGetPresentation) const = 0;

	//! Returns the inlined expression for this element as a wxString.
	//! "Inlined expression" means a math representation of the data
	//! in a single line: to allow such a restriction (everything must
	//! be on one text line), some standard conventions (and many
	//! brackets !!!) are used:
	//! - "^" introduces an exponent
	//! - "_" introduces a subscript
	//! Some examples of inlined expressions are: "4x^2+5a*sin(x/4)",
	//! "cos^2(PI/2)+ (123ax+b)/c >= 0", ....
	virtual wxString io_GetInlinedExpr() const = 0;


	//! Imports presentation MathML reading the contents of the given
	//! XML node (without modifying it). This function is called only
	//! if #io_isBeginTag function returns a positive value for the name
	//! of parent tag of \c pTag.
	//! \param pTag The wxXML_***_NODE class to parse.
	//! \param pErr The string where the error description is placed, if
	//!	       the function returns FALSE.
	//! \return TRUE on success (no mismatched or invalid tags).
	virtual bool io_ImportPresentationMathML(wxXml2Node tag, wxString &pErr) = 0;

	//! Imports an inlined expression.
	//! "Inlined expression" means a string containing an encoded
	//! representation of math data: 234^2*ax+6y/2=0 is an inlined
	//! expression. This is the form which is used to input and output
	//! math data by a lot of math programs.
	//! Compability with these programs is crucial.
	//! However, because of the nature of this representation, the
	//! algorithm which recognizes (through #io_isBeginChar functions)
	//! the elements to create, needs to know the lenght of the token
	//! which encodes this element; this is accomplished with the
	//! \c count parameter which must *absolutely* be set by this function.
	//! \param str The string containing the inlined expression to import.
	//! \param count A pointer where is stored the number of characters
	//!        of \c str imported by this function.
	//! \param pErr The string where the error description is placed, if
	//!	       the function returns FALSE.
	//! \return TRUE on success: the first (and eventually the following)
	//!         characters of the given string represent an encoded inline
	//!         expression of this element and they were successfully
	//!         recognized and imported.
	virtual bool io_ImportInlinedExpr(const wxString &str, int *count, wxString &pErr) = 0;	

	//@}

#endif		// mcENABLE_IO
};



//! Base abstract class for nearly all MathCore classes.
//! This is the base class of an element typed by the user
//! (a mcNumber, mcFraction, mcSymbol...); this is an abstract
//! class and cannot be allocated directly; you can just
//! use pointers and down-casts with mcElement classes.
//! You can be sure that each mcElement derived class has the
//! methods declared here, so you can always cast an unknown
//! derived class to mcElement and use the mcElement functions:
//!
//! \code
//! mcElement mynewelem;
//! ((mcElement )pt).gui_Input(wxT('2'), '2');	// if you want to feed the class
//!										// with a char, just use gui_Input function
//! if (((mcElement )pt).data_GetType() == mcET_SYMBOL)	// use data_GetType() if you want to run
//!	     char c = ((mcElement )pt).GetSymbol();	// specific functions defined only
//!													// in a specific derived class
//! \endcode
class mcElement : public mcObject
{
protected:

	//! Use this function to access m_nExpDepth private variable.
	//! This function provides a safe and easy way to access the
	//! exponent level of the element. Tipically this function will be
	//! called by SetAsExpOf and by SetAtSameLevelOf functions.
	//! \param n The new exponent depth for this element
	//void SetExpDepth(int n);
	
public:
			
	///////////////////////////////////////////////////////////////////////////
	//! \name COW functions
	//! The mcElement Copy-On-Write (COW) system. @{

	//! Returns a private copy of the currently shared data.
	mcObjectRefData *data_MakePrivateCopy();

	//! Changes the reference data but first checks that this object can
	//! safely contain the given mcElementHelpers-derived class, using the
	//! #data_isValidContainerFor function.
	void data_Ref(const mcObject &toclone);

	//! Returns a private copy of the associated mcElement.
	//! If this class is sharing it with another mcElement, then this function makes
	//! a personal copy for our private uses.
	mcElementHelpers *phlp()				{ return (mcElementHelpers *)data_MakePrivateCopy(); }

	//! Returns the (shared) helpers class container.
	//! This function has different name from #GetHelpers() since its work-mode is different.
	//! \note Never cast the returned pointer to something non-const since the given
	//!       pointer could be a shared pointer.
	const mcElementHelpers *chlp() const	{ return (mcElementHelpers *)data_GetRefData(); }

	//! Returns TRUE if the referenced data is allocated.
	//! If the reference data is not allocated, then almost all the mcElement
	//! function won't work and will raise an exception.
    bool data_isOk() const					{ return (mdata_refData != NULL); }

	//@}



public:

	//! Main constructor.
	//! Requires a valid pointer to the parent of the element.
	//! \param p A pointer to the parent element; if NULL, this element
	//!          is set as the root of the mcElement objects tree.
	//! \note Be sure that derived classes' constructors do not
	//!       call any function which uses mcMathCore::isKeyBeginKey.
	mcElement() {}

	//! Constructs a mcElement wrapping the given helpers.
	mcElement(const mcElementHelpers *helpers);

	//! Destructor of the element.
	virtual ~mcElement();


	// as for all mcElement-derived classes, we define through
	// macros the following functions:
	// - hlp() const      which uses chlp()
	// - hlp() non-const  which uses phlp()
	// - operator==
	// - operator!=
	mcDEFINE_HLP_GETTER(mcElement)
	mcDEFINE_COMPARISON_OPERATORS(mcElement)	


	
#ifdef mcENABLE_DATA
public:

	//! Returns a pointer to a new object identical to the current element.
	//! Derived classes must implement this function creating a new identical
	//! mcElement and returning it.
	//! See also mcIMPLEMENT_CLONE_FNC macro for a quick way to create this function.
	//! \return A pointer to a new element which must be deleted by the caller
	virtual mcElement data_Clone() const
		{ return mcElement(chlp()); }
	
	//! Deep copies all the info contained in this class. The mcElement implementation
	//! should be enough because all the element's data should be contained inside
	//! the associated mcElementData derived class. In fact, mcElementData does not
	//! provide a standard implementation of the DeepCopy() function. mcElement-derived
	//! classes, instead, should not contain any other variable, so this function
	//! should be enough....
	virtual void data_DeepCopy(const mcElement &p) 
		{ hlp()->data_DeepCopy(p.hlp()); }

	//! Creates a new element of the same type of *this and returns it empty.
	//! This function is very useful in the base classes (mcExpElement, 
	//! mcElementArray...) where the programmer (and the compiler !) cannot
	//! know what type of derived class is the one executing that code.
	virtual mcElement data_GetNewElemLikeThis() const 
		{ return mcElementHelpers::data_NewElem(data_GetType());	}

	//! Returns TRUE if this element can safely contain the given element type.
	//! Without this function a lot of errors could be undetectable:
	//! 
	//! \code
	//!    mcPolynomial pol;	// creates a new polynomial
	//!    mcElement e(pol);	// references it
	//!    [...]				// the programmers forgots what "e" references
	//!    mcFraction f(e);		// mcFraction wraps a mcPolynomialHelpers class !
	//!    f.data_GetNum();		// CRASH
	//! \endcode
	//!
	//! This function must be overridden by mcElement-derived classes in order
	//! to provide a list of the mcElementType it can safely contain.
	virtual bool data_isValidContainerFor(mcElementType t) const
		{ return TRUE; }

#ifdef __MCDEBUG__

	//! Checks that this is a valid container for the current helpers referenced
	//! using the #data_isValidContainerFor function.
	//! This check is quite fast since it's not recursive.
	void data_CheckContainer() const;

	//! Just calls the #data_CheckContainer() function and, if this
	//! class is referencing a valid mcElementHelpers-derived class,
	//! the helper's data_Check() function.
	void data_Check() const;

	//! Returns the name of this class like mcElementHelpers::data_GetDebugName()
	//! does. However, this function cannot just call the data_GetDebugName()
	//! of the mcElementHelpers-derived class attached because we could need
	//! the name of this class, even if it is an empty element (like mcEmptyElement) !
	virtual wxString data_GetDebugName() const
		{ return wxT("mcElement"); }

	// the wrapper for the data_GetDebug function (which 
	// is implemented by the helpers section)
	mcWRAPPER wxString data_GetDebug(int n, long f) const
		{ return hlp()->data_GetDebug(n, f); }	

#else

	// these functions should be removed by the linker
	inline void data_Check() const				{}
	inline void data_CheckContainer() const		{}
	inline wxString data_GetDebug(int, long) const 
		{ return wxEmptyString; }
	inline wxString data_GetDebugName(int, long) const 
		{ return wxEmptyString; }

#endif		// __MCDEBUG__


	mcWRAPPER void data_AddProperty(int p, void *val1 = NULL, void *val2 = NULL) 
		{ hlp()->data_AddProperty(p, val1, val2); }
	mcWRAPPER void data_RemoveProperty(int p) 
		{ hlp()->data_RemoveProperty(p); }
	mcWRAPPER bool data_hasProperty(int p) const
		{ return hlp()->data_hasProperty(p); }

	mcWRAPPER void data_AddRecursiveProperty(mcElementType t, int p, 
		void *val1 = NULL, void *val2 = NULL)
		{ hlp()->data_AddRecursiveProperty(t, p, val1, val2); }
	mcWRAPPER void data_RemoveRecursiveProperty(mcElementType t, int p)
		{ hlp()->data_RemoveRecursiveProperty(t, p); }
	mcWRAPPER bool data_hasRecursiveProperty(mcElementType t, int p) const
		{ return hlp()->data_hasRecursiveProperty(t, p); }	

	mcWRAPPER int data_GetChildrenCount() const	
		{ return hlp()->data_GetChildrenCount(); }
	mcWRAPPER const mcElement &data_GetConstChild(int n) const 
		{ return hlp()->data_GetConstChild(n); }
	mcWRAPPER mcElement &data_GetChild(int n)
		{ return hlp()->data_GetChild(n); }
	mcWRAPPER void data_SetChild(int idx, const mcElement &newchild)
		{ hlp()->data_SetChild(idx, newchild); }
	mcWRAPPER int data_GetChildIdx(const mcElement &p) const
		{ return hlp()->data_GetChildIdx(p); }
 	mcWRAPPER bool data_AddChildrenTo(mcElement p)
		{ return hlp()->data_AddChildrenTo(p); }

	mcWRAPPER bool data_isSameAs(const mcElement &p) const
		{ return hlp()->data_isSameAs(p.hlp()); }
	mcWRAPPER void gui_UpdateExpDepth() 
		{ hlp()->gui_UpdateExpDepth(); }

	mcWRAPPER mcElementType data_GetType() const
		{ return hlp()->data_GetType(); }
	mcWRAPPER int data_GetID() const
		{ return hlp()->data_GetID(); }
	mcWRAPPER int data_GetMemoryInfo() const
		{ return hlp()->data_GetMemoryInfo(); }

	mcWRAPPER mcElement &data_GetElemFromID(int id) const
		{ return hlp()->data_GetElemFromID(id); }

	mcWRAPPER void data_Update()
		{ hlp()->data_Update(); }
	mcWRAPPER bool data_isAllowedBefore(mcElementType type) const	
			{ return hlp()->data_isAllowedBefore(type); }
	mcWRAPPER bool data_isAllowedAfter(mcElementType type) const
			{ return hlp()->data_isAllowedAfter(type); }	

#endif		// mcENABLE_DATA




#ifdef mcENABLE_GUI

	mcWRAPPER void gui_SetAsExpOf(const mcElement &p)
		{ hlp()->gui_SetAsExpOf(p); }
	mcWRAPPER void gui_SetAtSameLevelOf(const mcElement &p)
		{ hlp()->gui_SetAtSameLevelOf(p); }
	mcWRAPPER void gui_SetExpDepth(int n)
		{ hlp()->gui_SetExpDepth(n); }

	mcWRAPPER bool gui_isBeginKey(const mcKey &ev) const
		{ return hlp()->gui_isBeginKey(ev); }	
	mcWRAPPER bool gui_isEndKey(const mcKey &ev) const
		{ return hlp()->gui_isEndKey(ev); }	
    mcWRAPPER int gui_Draw(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const
		{ return hlp()->gui_Draw(dc, x, y, flags, pt); }	
	mcWRAPPER int gui_GetRelCursorPos(wxDC &dc, wxPoint *pt) const
		{ return hlp()->gui_GetRelCursorPos(dc, pt); }	
	mcWRAPPER void gui_GetCursorPos(mcCursorPos &cp) const
		{ hlp()->gui_GetCursorPos(cp); }
	mcWRAPPER mcCursorPos gui_GetCursorPos() const
		{ return hlp()->gui_GetCursorPos(); }
	mcWRAPPER mcElement gui_GetSelection() const
		{ return hlp()->gui_GetSelection(); }
	mcWRAPPER bool gui_isAllSelected() const
		{ return hlp()->gui_isAllSelected(); }
	mcWRAPPER bool gui_isSplittable() const
		{ return hlp()->gui_isSplittable(); }
	mcWRAPPER bool gui_isSelected() const
		{ return hlp()->gui_isSelected(); }


	mcWRAPPER void gui_SetStyleArray(const mcStyleArray *p)
		{ hlp()->gui_SetStyleArray(p); }
	mcWRAPPER void gui_OnRecalcSize(wxSize old)
		{ hlp()->gui_OnRecalcSize(old); }
	mcWRAPPER void gui_DoRecalcSize()
		{ hlp()->gui_DoRecalcSize(); }
	mcWRAPPER mcInputRes gui_Input(const mcKey &ev, mcElement *newelem)
		{ return hlp()->gui_Input(ev, newelem); }
	mcWRAPPER mcInsertRes gui_Insert(const mcElement &toinsert, mcElement *newelem)
		{ return hlp()->gui_Insert(toinsert, newelem); }
	mcWRAPPER int gui_MoveCursorUsingPoint(wxDC &dc, const wxPoint &p)
		{ return hlp()->gui_MoveCursorUsingPoint(dc, p); }	
	mcWRAPPER mcMoveCursorRes gui_MoveCursor(mcMoveCursorFlag flag, long modifiers)
		{ return hlp()->gui_MoveCursor(flag, modifiers); }	
	mcWRAPPER void gui_SetCursorPos(const mcCursorPos &code)
		{ hlp()->gui_SetCursorPos(code); }	


	mcWRAPPER void gui_DeleteSelection()
		{ hlp()->gui_DeleteSelection(); }

	mcWRAPPER mcElement &gui_GetActiveElem(int x, int y, const wxPoint &pt) const
		{ return hlp()->gui_GetActiveElem(x, y, pt); }

	mcWRAPPER void gui_DeepRecalcSize()
		{ hlp()->gui_DeepRecalcSize(); }

	mcWRAPPER void gui_SelectAll()
		{ hlp()->gui_SelectAll(); }
	mcWRAPPER void gui_DeSelect()
		{ hlp()->gui_DeSelect(); }
	mcWRAPPER void gui_Select()
		{ hlp()->gui_Select(); }
	mcWRAPPER void gui_OnSelect(wxDC &dc, wxRect &rc)
		{ hlp()->gui_OnSelect(dc, rc); }

	mcWRAPPER bool gui_Split(mcElement *p)		
		{ return hlp()->gui_Split(p); }
	mcWRAPPER bool gui_MergeWith(const mcElement &p)
		{ return hlp()->gui_MergeWith(p); }

	mcWRAPPER bool gui_LetInCursor(mcCursorPos code)
		{ return hlp()->gui_LetInCursor(code); }

	mcWRAPPER void gui_RecalcSize()
		{ hlp()->gui_RecalcSize(); }
	mcWRAPPER wxSize gui_GetSize() const		
		{ return hlp()->gui_GetSize(); }
	mcWRAPPER int gui_GetWidth() const		
		{ return hlp()->gui_GetWidth(); }
	mcWRAPPER int gui_GetHeight() const		
		{ return hlp()->gui_GetHeight(); }

	/*wxSize gui_GetSize()
	mcWRAPPER int gui_GetWidth()
	mcWRAPPER int gui_GetHeight()*/
	mcWRAPPER int gui_GetExpDepth() const
		{ return hlp()->gui_GetExpDepth(); }

	mcWRAPPER int gui_GetYAnchor() const
		{ return hlp()->gui_GetYAnchor(); }

	mcWRAPPER const mcStyle *gui_GetStyle(int n) const
		{ return hlp()->gui_GetStyle(n); }

	mcWRAPPER wxSize gui_GetSizeOfChar(wxDC *dc = NULL, 
			const mcElement &p = mcEmptyElement) const
		{ return hlp()->gui_GetSizeOfChar(dc, p); }

	mcWRAPPER int gui_GetHeightOfChar(wxDC *dc, const mcElement &p) const	
		{ return hlp()->gui_GetHeightOfChar(dc, p); }
	mcWRAPPER int gui_GetWidthOfChar(wxDC *dc, const mcElement &p) const    
		{ return hlp()->gui_GetWidthOfChar(dc, p); }

	mcWRAPPER const mcStyle *gui_GetStyleForThis() const
		{ return hlp()->gui_GetStyleForThis(); }

	mcWRAPPER void gui_SelectStyle(wxDC &dc) const
		{ hlp()->gui_SelectStyle(dc); }
	mcWRAPPER int gui_GetThickness(wxDC *dc) const
		{ return hlp()->gui_GetThickness(dc); }
	mcWRAPPER int gui_GetBaseOffsety(int h) const
		{ return hlp()->gui_GetBaseOffsety(h); }
	mcWRAPPER int gui_GetSubscriptOffsety(int h) const
		{ return hlp()->gui_GetSubscriptOffsety(h); }

#endif		// mcENABLE_GUI




#ifdef mcENABLE_MATH
public:

	mcWRAPPER mcElement math_Find(int n, const mcElement &p, long flags = mcFIND_NOFLAGS) const
		{ return hlp()->math_Find(n, p, flags); }
	mcWRAPPER mcElement math_FindInChildrenOnly(int n, const mcElement &p, long flags = mcFIND_NOFLAGS) const
		{ return hlp()->math_FindInChildrenOnly(n, p, flags); }
	mcWRAPPER int math_NonRecursiveFindInChildren(int n, const mcElement &p, long flags = mcFIND_NOFLAGS) const
		{ return hlp()->math_NonRecursiveFindInChildren(n, p, flags); }
	mcWRAPPER int math_GetCountOf(const mcElement &p, long flags = mcFIND_NOFLAGS) const
		{ return hlp()->math_GetCountOf(p, flags); }
	mcWRAPPER int math_GetCountOfChildrenOnly(const mcElement &p, long flags = mcFIND_NOFLAGS) const
		{ return hlp()->math_GetCountOfChildrenOnly(p, flags); }
	mcWRAPPER int math_Replace(const mcElement &tofind, int n, long flags,
						const mcElement &replacement, bool bAttachChildren)
		{ return hlp()->math_Replace(tofind, n, flags, replacement, bAttachChildren); }

	mcWRAPPER int math_GetCountOf(const mcSymbolProperties *sym, long flags = mcFIND_NOFLAGS) const
		{ return hlp()->math_GetCountOf(sym, flags); }
	mcWRAPPER int math_GetCountOf(const mcSymbolArray *arr, long flags = mcFIND_NOFLAGS) const
		{ return hlp()->math_GetCountOf(arr, flags); }
	mcWRAPPER int math_GetCountOfSymbolsType(const mcSymbolArray *arr, long flags = mcFIND_NOFLAGS) const
		{ return hlp()->math_GetCountOfSymbolsType(arr, flags); }


	mcWRAPPER bool math_Contains(const mcElement &p, long flags = mcFIND_NOFLAGS) const		
		{ return hlp()->math_Contains(p, flags); }
	mcWRAPPER bool math_ContainsOneOf(const mcSymbolArray *arr) const
		{ return hlp()->math_ContainsOneOf(arr); }

	mcWRAPPER bool math_ContainsSymbol(const mcSymbolProperties *p, 
							const mcPolynomial &exp = mcEmptyPolynomial) const
		{ return hlp()->math_ContainsSymbol(p, exp); }
	mcWRAPPER bool math_ContainsNumber(const mcRealValue &n, 
							const mcPolynomial &exp = mcEmptyPolynomial) const
		{ return hlp()->math_ContainsNumber(n, exp); }

	mcWRAPPER bool math_ContainsUnknowns() const
		{ return hlp()->math_ContainsUnknowns(); }
	mcWRAPPER bool math_ContainsConstants() const
		{ return hlp()->math_ContainsConstants(); }
	mcWRAPPER bool math_ContainsParameters() const
		{ return hlp()->math_ContainsParameters(); }
	mcWRAPPER bool math_ContainsSymbols() const
		{ return hlp()->math_ContainsSymbols(); }
	mcWRAPPER bool math_ContainsInvalidSymbols() const
		{ return hlp()->math_ContainsInvalidSymbols(); }

	mcWRAPPER mcExpSimRes math_MaxSimplify(long flags)
		{ return hlp()->math_MaxSimplify(flags); }

	mcWRAPPER bool math_isMaxSimplified(long flags) const
		{ return hlp()->math_isMaxSimplified(flags); }

	mcWRAPPER int math_GetSymbolList(mcSymbol **arr, int size, const mcSymbol &tofind) const
		{ return hlp()->math_GetSymbolList(arr, size, tofind); }
	mcWRAPPER int math_GetSymbolList(mcSymbol **arr, int size, mcSymbolProperties &tofind) const
		{ return hlp()->math_GetSymbolList(arr, size, tofind); }

	mcWRAPPER bool math_isConstant() const
		{ return hlp()->math_isConstant(); }

	mcWRAPPER bool math_EvaluatesTo(const mcRealValue &val) const		
		{ return hlp()->math_EvaluatesTo(val); }

	mcWRAPPER bool math_isFinite() const
		{ return hlp()->math_isFinite(); }

	mcWRAPPER mcBasicOpRes math_RaiseTo(const mcPolynomial &p)
		{ return hlp()->math_RaiseTo(p); }
	mcWRAPPER mcBasicOpRes math_MakeReciprocal(mcElement *p)
		{ return hlp()->math_MakeReciprocal(p); }
	mcWRAPPER void math_SetExp(const mcPolynomial &p)
		{ hlp()->math_SetExp(p); }


	mcWRAPPER mcRealValue math_GetTotalLenght() const
		{ return hlp()->math_GetTotalLenght(); }

	mcWRAPPER void math_SimpleAdd(const mcElement &p, bool add = TRUE)
		{ hlp()->math_SimpleAdd(p, add); }
	mcWRAPPER void math_SimpleSubtract(const mcElement &p)		
		{ hlp()->math_SimpleSubtract(p); }
	mcWRAPPER void math_SimpleMultiplyBy(const mcElement &p)
		{ hlp()->math_SimpleMultiplyBy(p); }
	mcWRAPPER void math_SimpleDivideBy(const mcElement &p)
		{ hlp()->math_SimpleDivideBy(p); }
	mcWRAPPER void math_SimpleRaiseTo(const mcPolynomial &pol)
		{ hlp()->math_SimpleRaiseTo(pol); }

	mcWRAPPER bool math_isValidMath() const
		{ return hlp()->math_isValidMath(); }

	mcWRAPPER bool math_isListedBeforeOf(const mcElement &p) const
		{ return hlp()->math_isListedBeforeOf(p); }

	mcWRAPPER mcMathType math_GetMathType() const
		{ return hlp()->math_GetMathType(); }
	mcWRAPPER mcRealValue math_GetLenght() const
		{ return hlp()->math_GetLenght(); }

	mcWRAPPER mcExpSimRes math_Simplify(long flags, mcElement *newelem)
		{ return hlp()->math_Simplify(flags, newelem); }
	mcWRAPPER mcExpSimRes math_Expand(long flags, mcElement *newelem)
		{ return hlp()->math_Expand(flags, newelem); }

	mcWRAPPER bool math_Compare(const mcElement &p, long flags) const
		{ return hlp()->math_Compare(p, flags); }
	mcWRAPPER bool math_CompareThisOnly(const mcElement &p, long flags) const
		{ return hlp()->math_CompareThisOnly(p, flags); }

	mcWRAPPER mcRealValue math_Evaluate() const
		{ return hlp()->math_Evaluate(); }
	mcWRAPPER mcRealValue math_EvaluateAt(const mcSymbolProperties *sym, 
											const mcRealValue &symvalue) const
		{ return hlp()->math_EvaluateAt(sym, symvalue); }

	mcWRAPPER int math_GetOrderPos() const
		{ return hlp()->math_GetOrderPos(); }

//	mcPolynomial math_GetFactors() const
//		{ return hlp()->math_GetFactors(); }

	mcWRAPPER bool math_CanBeAddedWith(const mcElement &p) const
		{ return hlp()->math_CanBeAddedWith(p); }
	mcWRAPPER bool math_CanBeMultWith(const mcElement &p) const
		{ return hlp()->math_CanBeMultWith(p); }
	mcWRAPPER bool math_CanBeDivBy(const mcElement &p) const
		{ return hlp()->math_CanBeDivBy(p); }

	mcWRAPPER mcBasicOpRes math_Add(const mcElement &e, mcElement *pp, bool add = TRUE)
		{ return hlp()->math_Add(e, pp, add); }
	mcWRAPPER mcBasicOpRes math_Subtract(const mcElement &e, mcElement *pp) 
		{ return hlp()->math_Subtract(e, pp); }
	mcWRAPPER mcBasicOpRes math_MultiplyBy(const mcElement &e, mcElement *pp)
		{ return hlp()->math_MultiplyBy(e, pp); }
	mcWRAPPER mcBasicOpRes math_DivideBy(const mcElement &e, mcElement *pp)
		{ return hlp()->math_DivideBy(e, pp); }

	// the implementation of these two must be in the source file
	// and thus they cannot be inlined...
	mcMonomial math_GetLCM(const mcElement &p) const;
	mcMonomial math_GetGCD(const mcElement &p) const;
	mcMonomial math_GetFactors() const;

	mcElement operator*(const mcElement &m) const
	{ mcElement ret(*this); ret.math_SimpleMultiplyBy(m); return ret; }
	mcElement operator/(const mcElement &m) const
	{ mcElement ret(*this); ret.math_SimpleDivideBy(m); return ret; }
	mcElement operator-(const mcElement &m) const
	{ mcElement ret(*this); ret.math_SimpleSubtract(m); return ret; }
	mcElement operator+(const mcElement &m) const
	{ mcElement ret(*this); ret.math_SimpleAdd(m); return ret; }

	mcElement operator^(const mcPolynomial &m) const
	{ mcElement ret(*this); ret.math_RaiseTo(m); return ret; }

	// this operator cannot be inlined...
	mcElement operator^(const mcRealValue &m) const;


	// these operators return references

	mcElement &operator^=(const mcPolynomial &m)
	{ *this = (*this)^m;return *this; }
	mcElement &operator^=(const mcRealValue &m)
	{ *this = (*this)^m;return *this; }

	mcElement &operator+=(const mcElement &m)		{ *this = *this+m;return *this; }
	mcElement &operator-=(const mcElement &m)		{ *this = *this-m;return *this; }
	mcElement &operator*=(const mcElement &m)		{ *this = *this*m;return *this; }
	mcElement &operator/=(const mcElement &m)		{ *this = *this/m;return *this; }

#endif		// mcENABLE_MATH


#ifdef mcENABLE_IO
public:

	mcWRAPPER bool io_isBeginTag(const wxXml2Node &tag) const
		{ return hlp()->io_isBeginTag(tag); }
	mcWRAPPER bool io_isBeginChar(const wxString &str) const
		{ return hlp()->io_isBeginChar(str); }

	mcWRAPPER wxXml2Node io_GetMathML(bool bGetPresentation) const
		{ return hlp()->io_GetMathML(bGetPresentation); }

	mcWRAPPER wxString io_GetInlinedExpr() const
		{ return hlp()->io_GetInlinedExpr(); }

	mcWRAPPER bool io_ImportPresentationMathML(wxXml2Node tag, wxString &pErr)
		{ return hlp()->io_ImportPresentationMathML(tag, pErr); }
	mcWRAPPER bool io_ImportInlinedExpr(const wxString &str, int *count, wxString &pErr)	
		{ return hlp()->io_ImportInlinedExpr(str, count, pErr); }

#endif		// mcENABLE_IO
};


#endif	// ELEMENT_H

