///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   ElementArray.h
//! \brief  Defines the mcFilter, mcElementArrayData, mcElementArrayGUI,
//!         mcElementArrayMath, mcElementArrayIO, mcElementArray classes.
//!
//! See ElementArray.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef ELEMENTARRAY_H
#define ELEMENTARRAY_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "ElementArray.h"
#endif

// required includes
#include "mc/Element.h"
#include "mc/MathCore.h"
#include "mc/Operator.h"


// defined later
class mcElementArrayHelpers;
class mcMonomial;
class mcFraction;
class mcBracket;
class mcNumber;


//! The number of new entries added to the main array each time
//! it needs to be enlarged.
#define mcELEMENTARRAY_DEFAULT_ROOM 		4

//! Empty entry identifier in the array; use this define to write
//! readable code.
#define mcELEMENTARRAY_EMPTYENTRY			((mcElement )0xFFFFFFFF)

//! A property specific for mcElementArray: it sets the filter to
//! be used from GUI input. Use mcElement::data_AddRecursiveProperty()
//! to add/remove this property.
#define mcEP_ELEMENTARRAY_FILTER			mcEP_NONSTANDARD_BASE*2

//! Use this value in #SetMaxInstancesFor to allow infinite
//! instances for a specific element type.
#define mcFILTER_NO_INSTANCES_LIMIT			-1




//! An univocally determined entry in a mcElementArrayHelpers-derived class.
class mcArrayEntry : public mcDataHelper 
{
public:

	//! The data index of the monomial containing the element.
	int mdata_idx;

	//! A reference to the element.	
	const mcElementArrayHelpers *mdata_arr;

public:

	mcArrayEntry(const mcElementArrayHelpers *owner = NULL, int idx = -1) 
		: mdata_arr(owner) { mdata_idx=idx; }

	virtual ~mcArrayEntry() {}



	//! \name GET functions
	//! CONST functions which makes member access safe. @{
	
	mcElement &data_GetRef() const;

	int data_GetIdx() const;
	int math_GetIdx() const;

	mcBracket &data_GetBracket() const		{ return (mcBracket &)data_GetRef(); }
	mcFraction &data_GetFraction() const	{ return (mcFraction &)data_GetRef(); }	

	bool data_isValid() const
		{ return (mdata_arr != NULL) && (mdata_idx != -1); }

	//@}


	//! \name SET functions
	//! Safe set functions. @{

	void data_SetOwner(mcElementArrayHelpers *ref)	{ mdata_arr = ref; }
	void data_SetIdx(int idx)						{ mdata_idx = idx; }

	//@}
};


// a global instance of the mcArrayEntry class
extern mcArrayEntry mcEmptyArrayEntry;


//! A simple filter which can be used by mcElementArray to limit
//! the user input.
//! This filter can be attached to a mcElementArray using the
//! mcElementArray::gui_SetFilter function.
//! This class can be used as base class to derive new different
//! filters, if this one is not flexible enough.
class mcFilter : public mcDataHelper
{
protected:

	//! The type of this filter.
	mcFilterType m_nType;

	//! The n-th entry of this array contains the maximum number of 
	//! instances of the n-th element type which are allowed.
	//! If the n-th entry of this array is set to -1, then there's
	//! no limit to the number of instances for the n-th element type.
	int m_nMaxInstances[mcNUM_ELEMENT_TYPES];

public:

	//! Constructs this filter, allowing the user to specify in the
	//! constructor itself, the first type of element type (and its
	//! maximum number of instances) to allow.
	//! \param type The type of filter.
	//! \param t1, count1 The parameters passed to #SetMaxInstancesFor
	mcFilter(mcFilterType type = mcFILTER_INCLUDE_ELEMENT_LIST, 
		mcElementType t1 = mcET_INVALID, int count1 = -1);
	virtual ~mcFilter() {}

	//! Deep copies the given filter.
	void DeepCopy(const mcFilter *f) {
		for (int i=0; i < mcNUM_ELEMENT_TYPES; i++)			
			m_nMaxInstances[i] = f->m_nMaxInstances[i];		

		m_nType = f->m_nType;
	}



	/////////////////////////////////////////////////////////////////////////////
	//! \name Filter manipulation functions
	//! Functions which changes the allowed elements. @{

	//! (Re)sets one of the entry of the element type include list.
	//! This function changes the instances limit for the given
	//! element type to \c count.
	//! By default, the filter inits its array with zero as maximum number
	//! of instances for all elements. Thus, you should change the maximum
	//! instances allowed for the elements you want, before using this filter.
	void SetMaxInstancesFor(mcElementType t, 
		int count = mcFILTER_NO_INSTANCES_LIMIT);

	//! \todo code this->..
	void math_AddAllowedMathData() {}

	//! Changes the type of this filter.
	void SetFilterType(mcFilterType type) {
		m_nType = type;
	}

	//@}



	/////////////////////////////////////////////////////////////////////////////
	//! \name Check functions
	//! Functions which can be used to check if an element is allowed
	//! to be inserted in the object using this filter. @{

	//! Returns TRUE if the given element type can be instanced one time more.
	//! \param t The type of element which should be created.
	//! \param num The actual number of instances for \c t.
	bool isElementTypeAllowed(mcElementType t, int num);

	//@}
};





// this defines all the classes declared in this header file
mcDEFINE_HELPER_CLASSES(mcElementArray)


//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
class mcElementArrayHelpers : public mcElementHelpers
{
	friend class mcArrayEntry;

#ifdef mcENABLE_DATA
private:		// DATA variables

	//! Pointer to the array of mcElements; double deference is required
	//! for special operations like array enlargment.
	mcElement *mdata_pElemArray;
	
	//! Used to remember the upper bound of the
	//! m_pElem array and thus to avoid the use of non-allocated memory.
	int mdata_nUpperBound;
	
protected:

	//! Number of mcElements currently inserted in the mdata_pElemArray array.
	int mdata_nElements;

	//! The filter which is attached to this array.
	//! If NULL, filtering is disabled.
	mcFilter *mdata_pFilter;

#endif




#ifdef mcENABLE_GUI
protected:		// GUI variables

	//! Index of the element which contains the cursor (mgui_nCursorPos 
	//! cannot index an operator of the array, it must always 
	//! index a non-operator element).
	int mgui_nCursorPos;	


public:				// customizable variables getters

	//! The pen used to draw the selection rectangle. By default,
	//! mcElementArrayGUI constructor builds a black pen.
	static wxPen *sgui_pSelectionPen;

	//! If TRUE, then the selection will be drawn inverting the
	//! colours of the selected elements (this is the standard behaviour
	//! in all editing systems). If FALSE, then the #sgui_pSelectionPen
	//! and the mcElementHelpers::sgui_pSelectionBrush will be used
	//! to draw the border and the interior of the selection rectangle.
	static bool sgui_bDrawSelectionInverting;

	//! Returns the costant which is divided by the exponent depth of this element
	//! to get the number of pixels to leave among the elements.
	//! Derived classes should override this function to allow personalized
	//! values of the 'space between' customizable variable. In fact, if we would
	//! have defined here a STATIC INT nSpaceBetween variable, we would have forced
	//! the instance of all mcElementArrayGUI-derived classes to have the same
	//! 'space between' value.
	virtual int gui_GetSpaceBetweenRatio() const 
		{ return 0; }

#ifdef mcELEMENTARRAY_DRAW_DECORATIONS
	//! The bitmaps used to embellish selection rectangle.
	static wxBitmap *pBeginTriBmp, *pEndTriBmp;
#endif

#endif




public:

	mcElementArrayHelpers() {

		// array is empty and still not allocated
		mdata_nElements = mdata_nUpperBound = 0; 
		mdata_pElemArray = NULL;
		mdata_pFilter = NULL;
		mgui_nCursorPos = 0;
	}

	virtual ~mcElementArrayHelpers() {
		data_DeleteAll();
		mcSAFE_DELETE_ARRAY(mdata_pElemArray);		
		mcSAFE_DELETE(mdata_pFilter);
	}


protected:
	
	//! Inits the GUI section of this element.
	void gui_Init()		{ mcElementHelpers::gui_Init(); mgui_nCursorPos = 0; }



#ifdef mcENABLE_DATA
protected:

	virtual int data_GetIdxFromEntry(const mcArrayEntry *p) const = 0;
	virtual mcElement &data_GetRefFromEntry(const mcArrayEntry *p) = 0;
	
public:

	/////////////////////////////////////////////////////////////////////////////
	//! \name MISCELLANEOUS
	//! Some miscellaneous functions. @{

#ifdef __MCDEBUG__

	void data_Check() const;

	//! Returns complete debug informations about all the members stored
	//! in this array.
	wxString data_Debug(long flags) const;

	//! Returns the name of this class.
	wxString data_GetDebugName() const
		{ return wxT("mcElementArray"); }

#endif

	void data_DeepCopy(const mcElementHelpers *p);

	//! \copydoc mcElement::data_AddProperty
	//! mcElementArray overloads this function to handle the
	//! mcElementArray-specific mcEP_ELEMENTARRAY_FILTER property.
	void data_AddProperty(int p, void *val1 = NULL, void *val2 = NULL) {
		if (p == mcEP_ELEMENTARRAY_FILTER)
			data_ChangeFilter((const mcFilter *)val1);
	}

	int data_GetChildrenCount() const {
		return mcElementHelpers::data_GetChildrenCount()+data_GetCount();
	}

	const mcElement &data_GetConstChild(int n) const {
		mcRETURN_ELEMENT_CHILD(n, mcElementHelpers);
		return data_Get(n);
	}
	
	void data_SetChild(int n, const mcElement &newchild) {
		mcSET_ELEMENT_CHILD(n, mcElementHelpers, newchild);
		data_Delete(n);
		data_Set(n, newchild);
 	}

	
	//! Checks if the array is full; makes room if it is (the space
	//! added to the array is mcELEMENTARRAY_DEFAULT_ROOM.
	void data_CheckArrayLimit();

	//! Changes the filter associated with this array. See mcFilter.
	void data_ChangeFilter(const mcFilter *p);

	//! Updates the filter of the given element setting it to \c f.
	void data_UpdateFilterFor(mcElement &p, const mcFilter *f);

	//! Returns TRUE if the given type of element can be created.
	//! Consults the current filter to give the result.
	bool data_isElementAllowed(mcElementType type) const;

	//! Checks all the children.
	bool data_isSameAs(const mcElementHelpers *p) const;

	//@}

	
	
	/////////////////////////////////////////////////////////////////////////////
	//! \name ARRAY HANDLER functions
	//! Fast and safe (in debug build) array handlers functions. @{

	//! Checks that in this array, all the even (or odd) elements are okay.
	//! \param func The function which is called with the even (or odd)
	//!        elements of the array which must be checked.
	//! \param odd If TRUE, the function check the odd elements of the array;
	//!        otherwise it checks the even elements.
	virtual bool data_ScanArray(bool (*func)(const mcElement &), bool odd) const;

	//! math_Adds one or more elements to the array.
	//! \param p A pointer to the array of elements to add
	//! \param num The number of the elements contained in the given array, that this
	//!        function must add to the array
	//! \param bOverwrite If TRUE the old elements won't be moved right: they will
	//!        be deleted and replaced by the new elements
	//! \param bCopy If TRUE the elements contained in the given array won't be
	//!        directly inserted inside the array; the function will insert just a
	//!        copy of the elements (obtained through mcElement::Clone)
	//! \param pos The entry where the first element of the given array will be
	//!        inserted, or -1 to insert the elements at the end of the array.
	//!        Note that if you use -1 for this argument, the bOverwrite argument
	//!        will be ignored.
	virtual int data_AddElements(const mcElement *p, int num, int pos = -1, 
							bool bOverwrite = FALSE, bool bForceCopy = FALSE);

	//! math_Adds a new element to the array and returns a pointer to it.
	//! The new element is initialized with the given string (for more
	//! details, see mcElement::Init() function); it is built as a child
	//! of this element and it is placed in the n-th entry of this array,
	//! eventually overriding the previously present element.
	virtual mcElement &data_AddNewElement(mcElementType t, bool bOverwrite = FALSE, int pos = -1);

	//! Works like the #math_AddNewElement function but the new element
	//! is first wrapped in the appropriate structure: a mcPolynomial
	//! will wrap the new element in a mcMonomial while a mcMonomial
	//! will just call the #math_AddNewElement function.
	virtual mcElement &data_AddNewWrappedElement(mcElementType t, 
								bool bOverwrite = FALSE, int pos = -1) 
		{ return data_AddNewElement(t, bOverwrite, pos); }

	//! Merges the given array with this one, deep-copying the content and
	//! adding the new elements at the end of this array.
	//! \param p The array to merge with this one.
	//! \param n The number of elements (starting from the first of 'p') that
	//!          should not be merged in this array
	//! \param bAddToEnd If TRUE the given array is added to the end of this
	//!                  element, otherwise it is added to the begin.
	void data_Merge(const mcElementArray &p, int n = 0, bool bAddToEnd = TRUE);

	//! math_Adds a new operator in the array. This function is very different from
	//! mcElementArray::gui_math_AddNewOp: this function doesn't follow input rules,
	//! that is, if you try to add a mcAddOp in a mcMonomialData (this wouldn't
	//! be allowed by mcMonomial::gui_math_AddNewOp), no mcASSERTs will fail.
	//! Another example is that if you add a new operator at the end of the
	//! array, this function won't add any mcEmptyBox (which is the typical 
	//! behaviour of mcElementGUI functions).
	virtual void data_AddNewOp(mcElementType type, int pos = -1);

	//! math_Adds a new mcEmptyBox in the array, at the given position.
	virtual void data_AddNewEmptyBox(int pos = -1);
	
	//! Checks if the neighbors of the element n have the mcEP_NOTIFY_NEIGHBOR_CHANGE
	//! property set; if they have this function calls the 
	//! mcElement::data_OnNeighborChange function of the neighbors of the element n.
	virtual void data_UpdateNeighbor(int n);
	
	//! Mark the requested entry as empty, that is, not containing a valid pointer
	//! value to an allocated element. This function frees the memory used by an 
	//! eventually pre-existent element in the specified position.
	virtual void data_Delete(int entry);

	//! math_Finds the given element among the elements contained directly
	//! (that is, without any wrapper: this function is not recursive)
	//! and then deletes it from the array.
	//! This function also shifts the array, thus decrementing the element
	//! counter.
	//! Returns TRUE if everything is okay; FALSE if the given pointer
	//! is not an element contained in this array.
	virtual bool data_Delete(const mcElement &elem);

	//! Deletes the last num elements decrementing also the element counter.
	//! This function differs from Delete() because it also modifies the
	//! mdata_nElements counter.
	virtual void data_DeleteLast(int num);

	//! Deletes the first num elements decrementing also the element counter.
	//! This function differs from Delete() because it also modifies the
	//! mdata_nElements counter.
	virtual void data_DeleteFirst(int num);

	//! Deletes all the occurrences of the elements of the given type,
	//! shifting the array each time...
	virtual void data_DeleteAllElemType(mcElementType);

	//! Detaches (that is, removes from array but doesn't delete) the
	//! n-th entry of the array and returns it. 
	//! Doesn't modify the #mdata_nElements counter, so this function makes
	//! an "hole" in the array: BE CAREFUL not to forget to replace it.
	mcElement data_Detach(int n);

	//! Detaches the last num elements decrementing the #mdata_nElements counter.
	mcElement data_DetachLastElem(int num);

	//! Detaches all the elements of the array.
	//! VERY IMPORTANT: this could cause a very big memory leak if the detached
	//!                 elements are not deleted by something/someone.
	mcElement data_DetachAll();
	
	//! Deletes all the element in the array and marks all entries as empty;
	//! the array of pointers mdata_pElemArray is not deallocated.
	void data_DeleteAll();
	
	//! Shifts elements in the array one position on the right (mdata_pElemArray[start] will 
	//! be shifted right with all the following elements) and adds one unit
	//! from the elements counter (array grows).
	void data_MoveElemRight(int start);
	
	//! Shifts elements in the array one position on the left (mdata_pElemArray[start] will 
	//! be replaced by the element on his right and so on) and subtracts one unit
	//! to the elements counter (array is shrinked).
	//! \warning The mdata_pElemArray[start] entry will be overwritten but the content of
	//!	      that entry won't be deleted; to avoid memory leaks, delete the
	//!	      element located in the start entry and then call this function.
	//!	      A call to #DeleteElement cannot be inserted in this function
	//!	      because sometimes MoveElemLeft is called on dirty entries...
	void data_MoveElemLeft(int start);

	//! Moves the n-th element of the array (and its preceding operator)
	//! to the position k.
	//! VERY IMPORTANT: with mcMonomial you can use this function only
	//!                 when all mcDivOp have been removed.
	void data_Move(int n, int k);

	//! Moves all the elements of type \c t from *this to the given array \c m
	//! (which must be of compatible with elements of type \c t !!).
	//! \return The number of elements which have been moved.
	//! VERY IMPORTANT: with mcMonomial you can use this function only
	//!                 when all mcDivOp have been removed.	
	int data_MoveAllElemType(mcElementType t, mcElementArray &m);
	
	//! Swaps the element placed in index1 with the element placed in index2.
	//! \param index1 The index of the first element to swap.
	//! \param index2 The index of the second element to swap.
	void data_Swap(int index1, int index2);

	//! Places the elements of the given type and subtype in the firstpos entry
	//! and following (swapping the element previously placed in the entries
	//! where the sequence of the t elements is created).
	//! \param t The class ID of the elements to queue.
	//! \param subt The subtype or subclass ID of the elements to queue or -1
	//!	    if the elements to queue have not a subtype ID.
	//! \param firstpos The first entry where the first element to queue will be 
	//!	    placed; all the following elements are placed in the following entries,
	//!	    swapping the array if necessary.
	//! \return The number of entry this function has processed.
	int data_QueueElemType(mcElementType t, int firstpos);

	//! Repairs the array, solving those problems which derive *not* from logic
	//! programming errors but from some assumptions taken in the algorithms.
	//! The default implementation does nothing.
	virtual void data_Repair() {}

	//! Returns TRUE if this array is containing only an element of the given type.
	virtual bool data_isWrappingOnly(mcElementType t) const;

	//! Returns the element of type \c t wrapped by this element, if it does
	//! exist and it is the only element wrapped by this array; returns NULL
	//! otherwise.
	virtual const mcElement &data_GetWrapped(mcElementType t) const;

	//@}
	
	
	
	
	/////////////////////////////////////////////////////////////////////////////
	//! \name GET functions
	//! Fast and safe (in debug build) get functions. @{
	
	//! Use this as quick array and index check function; it stops the program
	//! if the given index is not a valid index for the current array (which must
	//! have been already created).
	inline void data_CheckIndex(int entry) const {
		// in release mode, for perfomance reasons, don't test
		mcASSERT(mdata_pElemArray != NULL, wxT("Array not allocated yet"));
		mcASSERT(entry >= 0 && entry <= mdata_nUpperBound, wxT("Invalid index"));
	}
	
	//! Returns a pointer to the element placed at the entry-th position in the array; 
	//! it's better to use this function rather than accessing directly the array
	//! because this function (in debug mode), ensure some data checks.
	//! If n is too high, an mcASSERT will fail; use #GetCount() to avoid it.
	//! \warning This function can return an invalid pointer value if the requested
	//!	      element is an empty box or an empty entry (see ARR_EMPTYBOX and
	//!	      mcELEMENTARRAY_EMPTYENTRY).
	inline const mcElement &data_GetConst(int entry) const {
		data_CheckIndex(entry);
		return mdata_pElemArray[entry];
	}

	inline const mcElement &data_Get(int entry) const 
		{ return data_GetConst(entry); }

	inline mcElement &data_Get(int entry) {
		data_CheckIndex(entry);
		return mdata_pElemArray[entry];
	}
	
	//! Returns the index of the entry in the array which points to the given element.
	//! Returns -1 if the given pointer does not points to an element in the array.
	int data_GetIndexOf(const mcElement &p) const;

	//! Returns a pointer to the last element of the array.
	inline const mcElement &data_GetLast() const 
		{ return data_Get(data_GetCount()-1); }

	inline mcElement &data_GetLast()
		{ return data_Get(data_GetCount()-1); }
	
	//! Returns a pointer to the memory address where the pointer to the n-th element
	//! is stored. Use with care.
	inline mcElement *data_GetArray(int n = 0) const {
		data_CheckIndex(n);
		return &mdata_pElemArray[n];
	}

	//! Returns the index of the element with the given ID; -1 if no elements
	//! with the given ID are stored in this expression.
	//! \warning This function will check only the elements in this
	//!          array: it won't do any recursive search. If you need to find
	//!          an element nested in the array, use the #data_GetElemFromID() function.
	//! \param id The ID of the element to search
	inline int data_GetIndex(int id) const {
		for (int i=0; i < data_GetCount(); i++)
			if (data_Get(i).data_GetID() == id)
				return i;
			return -1;
	}
	
	//! Returns the number of the elements stored in the mcPolynomial (takes in
	//! count also empty boxes).
	inline int data_GetCount() const { 
		return mdata_nElements; 
	}
	
	//! Returns the number of the entries in the array (the array automatically
	//! grows if needed).
	inline int data_GetAvailableEntryCount() const {
		return mdata_nUpperBound;
	}
	
	//! Returns the number of the elements of type == t.
	//! \param t The type of the element to find
	int data_GetNumOfElemType(mcElementType t) const;
	
	//! Returns the number of the operators stored in the mcPolynomial.
	int data_GetOpCount() const;
	
	//! Returns a pointer to the n-th occurrence of the element of type == t.
	//! \param n The occurrence of the element of type t to return
	//! \param t The type of the elementsto find (-1 if the element to find has
	//!	    no subtype ID because it's derived directly from mcElement or mcExpElement)
	mcElement &data_GetElemOfType(int n, mcElementType t) const;
	
	//! Returns the index of the n-th occurrence of the element of type == t.
	//! \param n The occurrence of the element of type t to return
	//! \param t The type of the element to find
	int data_GetElemIndexOfType(int n, mcElementType t) const;

	//! Returns a pointer to the n-th occurrence of a non mcOperator-derived class.
	//! \param n The occurrence of the non-operator element to return
	mcElement &data_GetNonOpElem(int n) const;

	//! Returns the index of the n-th occurrence of a non mcOperator-derived class.
	//! \param n The occurrence of the non-operator element to return
	int data_GetNonOpElemIndex(int n) const;
	
	//! Returns a pointer to the n-th operator in the array. See the note
	//! of #GetOpIndex for more info.
	//! If n is too high, an mcASSERT will fail; use #GetOpCount() to avoid it.
	mcOperator &data_GetOp(int n) const;
	
	//! Returns the index of the n-th operator in the array. 
	//! \note This function differs from Get() because
	//! the argument is not an index of the array; it must be the
	//! zero-based index of the operators in the array; for example, in the
	//! expression 12+34*567, if you want to get a pointer to the '*' operator, 
	//! use GetOp(2) because the '*' operator is the second in the array.
	//! If n is too high, an mcASSERT will fail; use #GetOpCount() to avoid it.
	int data_GetOpIndex(int n) const;

	//! Returns the element type of the operator between...
	mcElementType data_GetOpTypeBetween(int n1, int n2) const;

	//! Returns the operator type preceding the n-th element.
	mcElementType data_GetOpTypePreceding(int n) const;

	mcOperator &data_GetOpPreceding(int n) const
		{ return (mcOperator &)*data_GetInstanceOf(data_GetOpTypePreceding(n)); }

	//! Returns the element preceding the given pointer; this pointer
	//! must be a pointer to one element of the array of this class.
	//! If the given pointer is not valid, not a child of this class, or the previous
	//! element doesn't exist, returns NULL.
	const mcElement &data_GetElemAtLeftOf(const mcElement &p) const {
		int n = data_GetIndexOf(p);
		return (n < 1) ? mcEmptyElement : data_Get(n-1);
	}
	
	//! Returns a the element following the given pointer; this pointer
	//! must be a pointer to one element of the array of this class.
	//! If the given pointer is not valid, not a child of this class, or the following
	//! element doesn't exist, returns NULL.
	const mcElement &data_GetElemAtRightOf(const mcElement &p) const { 
		int n = data_GetIndexOf(p);
		return (n == -1 || n >= mdata_nElements-1) ? mcEmptyElement : data_Get(n+1);
	}

	//@}
	
	
	
	
	
	/////////////////////////////////////////////////////////////////////////////
	//! \name INLINE CHECK functions
	//! Fast and safe (in debug build) check functions. @{
	
	//! Returns TRUE if the specified entry is an operator.
	bool data_isOp(int entry) const;
	
	//! Returns TRUE if the specified entry is an empty box; an empty box can be placed
	//! anywhere in the array before the end.
	inline bool data_isEmptyBox(int entry) const {
		data_CheckIndex(entry);
		return (mdata_pElemArray[entry].data_GetType() == mcET_EMPTYBOX); 
	}
	
	//! Returns TRUE if the specified entry is an empty entry; empty entries can be
	//! placed only at the end of the array.
	inline bool data_isEmptyEntry(int entry) const {
		data_CheckIndex(entry);
		return (data_Get(entry) == mcEmptyElement);
	}
	
	//! Returns TRUE if the specified entry is an operator.
	inline bool data_isAddSubOp(int entry) const {
		data_CheckIndex(entry);
		return (data_Get(entry).data_GetType() == mcET_ADDOP || 
			data_Get(entry).data_GetType() == mcET_SUBOP);
	}
	
	//! Returns TRUE if the specified entry is an operator.
	inline bool data_isMultDivOp(int entry) const {
		data_CheckIndex(entry);
		return (data_Get(entry).data_GetType() == mcET_MULTOP || 
			data_Get(entry).data_GetType() == mcET_DIVOP);
	}
	
	//! Returns TRUE if the array is empty or non allocated.
	inline bool data_isArrayEmpty() const {
		return (mdata_nElements == 0);
	}
	
	//! Checks if the given pointer is valid (it's not an empty box or
	//! an empty entry).
	//! This function can be used without allocating any mcElementArray:
	//! it's a static function.
	static inline bool data_isValidElem(const mcElement &p) {
		return p != mcEmptyElement;		
	}
	
	//! Returns TRUE if the specified entry is valid.
	inline bool data_isValidElem(int entry) const {
		data_CheckIndex(entry);
		return data_isValidElem(mdata_pElemArray[entry]);
	}
	//@}
	
	
	
	
	/////////////////////////////////////////////////////////////////////////////
	//! \name INLINE SET functions
	//! Fast and safe (in debug build) set functions. @{
	
	//! Sets the i-th entry with the given value. Be careful: this function
	//! does not free any memory used by the i-th element (eventually) present
	//! before the call to this function.
	void data_Set(int i, const mcElement &value);
	
	//! Marks the given entry as empty but this function doesn't delete the 
	//! eventually pre-existent element. If you specify an entry containing
	//! a valid pointer, the memory address of that element will be lost and
	//! the element will never be allocated: be careful !!!!
	//! If you want to set as empty an entry where an allocated element is
	//! currently placed, use DeleteElement() function.
	inline void data_SetAsEmptyEntry(int entry) {
		data_CheckIndex(entry);
		mdata_pElemArray[entry] = mcEmptyElement;
	}
	
	//@}


	///////////////////////////////////////////////////////////////////////////
	//! \name Filter functions
	//! Functions which work on the mcFilter of this class. @{

	//! (Re)sets the filter attached to this object.
	//! If \c p is NULL, then filter is disabled (as default).
	void data_SetFilter(const mcFilter *p);

	//! Returns the filter attached to this array; NULL if there are no
	//! filters set for this object.
	mcFilter *data_GetFilter()					{ return mdata_pFilter; }

	//! The CONST version of the #data_GetFilter
	const mcFilter *data_GetFilter() const		{ return mdata_pFilter; }

	//! Returns TRUE if this class is using a filter.
	bool data_isUsingFilter() const				{ return (mdata_pFilter != NULL); }

	//@}

#endif		// mcENABLE_DATA





#ifdef mcENABLE_GUI
public:
	
	/////////////////////////////////////////////////////////////////////////////
	//! \name Array handlers
	//! Functions which manipulates the mcElementArray. @{

	//! Creates a new element of the type & subtype, sends it the given
	//! start key and adds it to the array in the specified position.
	//! \param type The class ID of the new element to add
	//! \param subtype The subclass ID of the new element to add
	//! \param key The first key the new element will receive
	//! \param pos The position of the new element; the element (eventually) present
	//!        in the pos-th entry of the array is deleted and then replaced by
	//!        this new element. Use -1 to add the new element at the end of the
	//!        array.
	//! \return The index of the new element in the array (this can be different
	//!         from the given 'pos' variable).
	virtual int gui_AddNewElement(mcElementType type, const mcKey &key, int pos = -1);
	
	//! Just to make easier the empty box creation. The eventually pre-existent
	//! element at the given position is deleted. This function just calls
	//! math_AddNewElement with some fixed arguments.
	void gui_AddNewEmptyBox(int pos = -1);

	//! math_Adds a new operator of the specified type at the specified position.
	//! \param nOpType One of the OP_*** flags
	//! \param pos The position of the operator to add
	void gui_AddNewOp(mcElementType nOpType, int pos = -1);

	//! Sets the cursor at the beginning of the n-th element.
	void gui_SetCursorOnElemBegin(int n) {		
		mgui_nCursorPos = n;
		data_Get(n).gui_SetCursorPos(mcCP_BEGIN);
		gui_CheckCursorPos();
	}

	//! Sets the cursor at the end of the n-th element.
	void gui_SetCursorOnElemEnd(int n) {		
		mgui_nCursorPos = n;
		data_Get(n).gui_SetCursorPos(mcCP_END);
		gui_CheckCursorPos();
	}

	//! Returns TRUE if the given element is contained in this monomial
	//! and it has the focus (that is, the cursor is placed on that
	//! element).
	bool gui_isCursorOn(const mcElement &p) const {
		int idx = data_GetIndexOf(p);
		if (idx != -1)
			return (idx == mgui_nCursorPos);
		return FALSE;
	}

	//! Returns the number of the selected elements in the expression.
	int gui_GetSelElemCount() const {
		return gui_GetSelEnd()-gui_GetSelStart();
	}

	//! Returns the number of the selected elements in the array of the given
	//! type and subtype.
	int gui_GetSelElemCountOfType(mcElementType t) const {
		int count = 0;

		for (int i=gui_GetSelStart(), max=gui_GetSelEnd(); i < max; i++)
			if (data_Get(i).data_GetType() == t)
				count++;

		return count;
	}
	
	//! Returns the index of the n-th selected element found inside the expression.
	//! \param n The selected element whose index will be returned; zero means 'return 
	//!	    first selected elementwxT('s index'), 1 means 'return second selected element's
	//!	    index'...
	//! \return The index of the internally stored selected element or -1 if
	//!	     there are no enough selected elements (see #GetSelElemCount).
	int gui_GetSelElemIndex(int n) const {
		int m = -1;	// PLEASE NOTE THAT WE MUST ALWAYS START FROM -1:
					// IN THIS WAY THE PARAMETER n MUST BE EXPRESSED
					// AS ZERO-BASED
		
		for (int i=0; i < data_GetCount(); i++) {
			if (data_Get(i).gui_isSelected()) {
				m++;
				// found ?
				if (m == n)	return i;
			}
		}
		
		// no enough selected elements
		return -1;
	}

	//! Returns the index of the n-th selected element found inside the expression,
	//! which match the given type and subtype.
	//! \param n The selected element whose index will be returned; zero means 'return 
	//!	    first selected elementwxT('s index'), 1 means 'return second selected element's
	//!	    index'...
	//! \return The index of the internally stored selected element or -1 if
	//!	     there are no enough selected elements (see #GetSelElemCount).
	int gui_GetSelElemIndexOfType(int n, mcElementType t) const {
		int m = -1;
		
		for (int i=0, max=gui_GetSelElemCount(); i < max; i++) {
			if (gui_GetSelElem(i).data_GetType() == t) {
				m++;
				// found ?
				if (n == m)	return gui_GetSelElemIndex(i);
			}
		}
		
		// the n-th occurrence of t doens't exist
		return -1;
	}
	
	//! Returns a pointer to the n-th selected element found inside the expression.
	//! \param n The selected element to return; a null value means 'return first
	//!	    selected element', 1 means 'return second selected element'...
	//! \return A pointer to the internally stored selected element or NULL if
	//!	     there are no enough selected elements (see #GetSelElemCount).
	const mcElement &gui_GetSelElem(int n) const {
		int m = gui_GetSelElemIndex(n);
		
		if (m > -1)
			return data_Get(m);
		
		// not enough selected elements
		return mcEmptyElement;
	}

	const mcElement &gui_GetSelElemOfType(int n, mcElementType t) const {
		int m = gui_GetSelElemIndexOfType(n, t);
		
		if (m > -1)
			return data_Get(m);
		
		// not enough selected elements
		return mcEmptyElement;
	}
	
	//! Returns the index of the first selected element contained in the expression,
	//! or -1 if there is no selected element.
	int gui_GetSelStart() const {
		for (int i=0; i < data_GetCount(); i++)
			if (data_Get(i).gui_isSelected())
				return i;
		return -1;
	}
	
	//! Returns the index of the last selected element contained in the expression,
	//! or -1 if there is no selected element.
	int gui_GetSelEnd() const {
		for (int i=data_GetCount()-1; i >= 0; i--)
			if (data_Get(i).gui_isSelected())
				return i+1;
		return -1;
	}

	//! Deletes all the selected elements of the array.
	void gui_DeleteSelection();
	
	//! Returns the currently active element of the array or NULL if there is
	//! no active element under the mouse cursor.
	mcElement &gui_ExGetActiveElem(int x, int y, const wxPoint &pt, int cl) {
		int n = gui_GetActiveElemIndex(x, y, pt, cl);
		if (n != mcDRW_NOACTIVEELEM && n != mcDRW_ONSELECTION)
			return data_Get(n).gui_GetActiveElem(x, y, pt);
		return mcEmptyElement;
	}

	//! Deletes the operator which follows the currently focused element.
	void gui_DeleteNextOp();

	//! Deletes the operator which precedes the currently focused element.
	void gui_DeletePreviousOp();

	//! Moves cursor from the current position one step right, skipping elements
	//! which return FALSE to the gui_LetInCursor() function.
	void gui_MoveCursorRight();
	void gui_MoveCursorLeft();

	//! Sets the exponent depth of all the elements contained to be the same
	//! of the exponent depth of *this.
	void gui_UpdateExpDepth();

	bool gui_Insert(const mcElement &toinsert);

	//@}





	/////////////////////////////////////////////////////////////////////////////
	//! \name mcElement functions
	//! Functions which implements some (the remaining abstract methods must be
	//! implemented by derived classes) abstract functions of mcElement. @{

	virtual bool gui_isEndKey(const mcKey &ev) const;
	//virtual void DeepCopy(const mcElementGUI *p);
	virtual void gui_SetCursorPos(const mcCursorPos &code);
	virtual mcInputRes gui_Input(const mcKey &ev, mcElement *newelem);
	virtual mcMoveCursorRes gui_MoveCursor(mcMoveCursorFlag flag, long modifiers);
	virtual mcElement gui_GetSelection() const;
	virtual void gui_DoRecalcSize();
	virtual void gui_GetCursorPos(mcCursorPos &) const;
	virtual mcCursorPos gui_GetCursorPos() const
		{ return mcElementHelpers::gui_GetCursorPos(); }

	virtual int gui_GetYAnchor() const {
		return gui_GetCenterLine();
	}

	void gui_OnSelect(wxDC &dc, wxRect &rc) {
		// call the overloaded version;
		gui_ExOnSelect(dc, rc, -1);
	}

	int gui_Draw(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const {
		// call the overloaded version
		return gui_ExDraw(dc, x, y, flags, pt, -1);
	}

	int gui_GetRelCursorPos(wxDC &dc, wxPoint *pt) const {
		// call the overloaded version
		return gui_ExGetRelCursorPos(dc, pt, -1);
	}

	int gui_MoveCursorUsingPoint(wxDC &dc, const wxPoint &pt) {
		// call the overloaded version
		return gui_ExMoveCursorUsingPoint(dc, pt, -1);
	}

	mcElement &gui_GetActiveElem(int x, int y, const wxPoint &pt) {
		// call the overloaded version
		return gui_ExGetActiveElem(x, y, pt, -1);
	}

	//@}




	/////////////////////////////////////////////////////////////////////////////
	//! \name Abstract mcElementArray functions
	//! Pure virtual functions which must be implemented in mcElementArray
	//! derived classes to personalize array's behaviour. @{

	//! Replies to the return flag of the gui_Input() function of the subelement
	//! called by the mcElementArray::gui_Input() function.
	virtual mcInputRes gui_BackInput(const mcKey &key, mcElement *pnewelem, int n) = 0;

	//! Returns TRUE if the given key cannot be handled by the array. This
	//! function is called by mcElementArray::gui_isEndChar() function only
	//! if the currently focused element in the array cannot handle it.
	virtual bool gui_isArrEndKey(const mcKey &ev) const = 0;
		
	//! Checks that the cursor is in the right position; this function is
	//! called in critical points of the most complex mcElementArrayGUI
	//! routines to be sure that wrong cursor positions are immediately
	//! catched. If one of the checks fails, this function should stop
	//! program execution.
	virtual void gui_CheckCursorPos() const = 0;

	//! gui_Splits the currently focused elements in two parts, inserting a new
	//! element of the given type and subtype between those two parts 
	virtual void gui_DoSplit(mcElementType type, const mcKey &ev) = 0;


	//! Returns TRUE if the selection rectangle must be drawn. This
	//! function should be implemented in different ways for different
	//! mcElementArray-derived classes because, typically, this is an
	//! aspect which is handled

	//@}




	/////////////////////////////////////////////////////////////////////////////
	//! \name Helper functions
	//! Handy functions related to the graphical output; these functions are
	//! defined as virtual, so derived classes can override them. @{

	//! Returns the number of pixels to leave among the elements of the array.
	//! This functions takes by default the mcElementArray::gui_nSpaceBetween
	//! variables and divides it by the exponent depth of the array.
	virtual int gui_GetSpaceBetween() const;

	//! Returns the y value which is the half of the height of the
	//! tallest element in the array. This function scans all the array and
	//! do a lot of comparisons: use it as less as possible.
	virtual int gui_GetCenterLine() const;

	//! gui_Draws all the elements contained in the array behaving exactly
	//! as mcElement::gui_Draw but requires some additional parameters.
	//! \param dc The DC where the function will draw
	//! \param x The x-coord where the first element will be drawn
	//! \param y The y-coord where the first element will be drawn
	//! \param pt The mouse cursor position or a mcDRW_*** flag
	//! \param cl The center line to use to draw the elements or -1
	//!           to let the function calculate it
	//! \param begin The index of the first element to draw
	//! \param end The index of the last element to draw
	//! \param bgui_DrawSelected TRUE if you want this function to draw also
	//!                      selected elements.
 	virtual int gui_ExDraw(wxDC &dc, int x, int y, long flags, const wxPoint &pt, int cl = -1, 
					int begin = 0, int end = -1, bool bgui_DrawSelected = FALSE) const;

	//! gui_Draws only the selected elements contained in the array. You should
	//! call this function with the same coordinates you gave to the gui_Draw()
	//! function (e.g.: if the first two elements of the array are not selected
	//! (while the third and fourth are), then, this function will draw the 
	//! third and fourth elements beginning at x+CalcSizeOfRange(0, 3)).
	//! This function doesn't draw anything except selected elements; if you
	//! want to draw also the selection rectangle, use #gui_DrawSelection().
	//! \param dc The DC where the function will draw
	//! \param x The x-coord where the first element will be drawn
	//! \param y The y-coord where the first element will be drawn
	//! \param pt The mouse cursor position or a mcDRW_*** flag
	//! \param cl The center line to use to draw the elements or -1
	//!           to let the function calculate it
 	virtual int gui_DrawSelectedElements(wxDC &dc, int x, int y, long flags, 
									const wxPoint &pt, int cl = -1) const;

	//! gui_Draws the selection on the given DC. If there's no selection, the
	//! function returns immediately. You should
	//! call this function with the same coordinates you gave to the gui_Draw()
	//! function (e.g.: if the first two elements of the array are not selected
	//! (while the third and fourth are), then, this function will draw the 
	//! third and fourth elements beginning at x+CalcSizeOfRange(0, 3)).
	//! \param hDC The DC where the selection will be drawn
	//! \param x The x-coord of the entire array
	//! \param y The y-coord of the monomial
	//! \param pt The mouse position or one of the mcDRW_*** flag
	//! \param cl The center line for this monomial, or -1 to let the function
	//!        calculate it
 	//! \return mcDRW_ONSELECTION if the mouse cursor is inside the selection
	//!         rectangle, mcDRW_NOACTIVEELEM otherwise: please note that this
	//!         function could return mcDRW_NOACTIVEELEM even if an active element
	//!         is present (but not inside the selection rectangle).
	virtual int gui_DrawSelection(wxDC &hDC, int x, int y, long flags, const wxPoint &pt, int cl = -1) const;

	//! Behaves as mcElement::gui_OnSelect() function; the only difference is that
	//! this function allows the caller to specify a space to leave (in 
	//! computations: this function doesn't draw anything) among the various 
	//! elements.
	//! \param dc The DC where the user created the selection rectangle
	//! \param x The x position of this array on the given DC
	//! \param y The y position of this array on the given DC
	//! \param rc The selection rectangle created by the user
	//! \param cl The center line used to draw the array on the given DC (or -1)
	virtual void gui_ExOnSelect(wxDC &dc, wxRect &rc, int cl = -1);

	//! Returns the size of the first n elements in the array. Use #gui_GetSize to
	//! get the size of the entire monomial.
	//! \param n The index of the last element whose sie must be added to the
	//!          size of the previous elements
	//! \param cl The center line used to draw the array on the given DC (or -1)
	virtual wxSize gui_CalcSizeOfFirst(int n, int cl = -1) const;

	//! Returns the size of the bouding box of the given range of elements.
	//! When begin = 0 and end = GetCount(), this function returns the size
	//! of the entire element.
	//! \param begin The index of the first element of the range
	//! \param end The index of the last element of the range
	//! \param cl The center line used to draw the array on the given DC (or -1)
	//! \param sb The space to leave between two elements
	virtual wxSize gui_CalcSizeOfRange(int begin, int end, int cl = -1) const;

	//! Returns the selection rectangle in screen coordinates.
	//! \param dc The DC to use to compute the center line if cl is -1
	//! \param x The x-pos of the monomial on dc
	//! \param y The y-pos of the monomial on dc
	//! \param cl The center line for this monomial, or -1 to let the function
	//!        calculate it
	virtual wxRect gui_GetSelectionRect(wxDC &dc, int x, int y, int cl = -1) const;

	//! Returns the position of the upper-left coord. of the n-th element.
	//! \param cl The center line for this monomial, or -1 to let the function
	//!        calculate it
	virtual wxPoint gui_GetOriginOfElem(int n, int cl = -1) const;
	
	//! Returns the width of n-th element and its bounding box (in the pointer)
	//! in coord. relative to the top-left point of the entire monomial.
	//! If you want to use a special center line, use a non-zero value for yCenter;
	//! otherwise let it to zero, the function will compute it using #GetCenterLine.
	//! If you want to specify the width of all the elements preceding the n-th
	//! element, use w with a non-zero value, otherwise the function will calculate it.
	//! \param n The entry of the element whose bounding box will be returned
	//! \param rc The wxRect where the bounding box will be stored
	//! \param yCenter An optional center line to which the element bounding box
	//!	       will be centered vertically
	//! \param w An optional value: the width of all preceding elements (if you use
	//!	       this function in a loop, just compute center line once and reuse widths)
	//! \return The width of the n-th element.
	virtual int gui_GetBB(int n, wxRect *rc, int yCenter = 0, int w = 0) const;

	//! Asks the currently focused element of the array for the cursor position;
	//! then, offsets it to the right position and return it in the given point
	//! pointer. This function works as mcElement::gui_GetRelCursorPos function should
	//! work, but it can accept a center line and the space to leave among the
	//! elements, given by caller. 
	virtual int gui_ExGetRelCursorPos(wxDC &hDC, wxPoint *pt, int cl = -1) const;
	
	//! This function is typically used in response to a gui_MoveCursorUsingPoint() calls	
	//! \param hDC The DC where the element was drawn last time.
	//! \param pt The mouse cursor position, in ElementArray coord., that is,
	//!        in coord. relative to the upper-left point of the bounding box
	//!        of this element.
	//! \param cl The center line for this monomial, or -1 to let the function
	//!        calculate it
	virtual int gui_ExMoveCursorUsingPoint(wxDC &hDC, const wxPoint &pt, int cl = -1);

	//! Returns the index of the currently active element, or the index of
	//! the element which contains the current active element (this function
	//! cannot recursively search the index of the active element and it
	//! must stop to the first element, even if it's a container one).
	//! \return The index of the active element. If there is no element 
	//!         containing the mouse cursor, the function returns 
	//!         mcDRW_NOACTIVEELEM. If the active element is selected, then
	//!         this function returns mcDRW_ONSELECTION.
	virtual int gui_GetActiveElemIndex(int x, int y, const wxPoint &pt, int cl = -1);

	//@}
#endif		// mcENABLE_GUI




#ifdef mcENABLE_MATH

//! This class is very important because it contains a lot of 
//! useful mathematical functions required by an array of elements.
//!
//! Unlike mcElementArrayData, mcElementArrayMath does not use
//! for all its functions the index of the element's entry in 
//! mcElementArray::data_m_pElem array to specify the element
//! to operate on.
//! mcElementArrayMath uses another type of index, referred to as
//! wxT("math index") opposed to the "data index". This is a value
//! that refers to the monomials in the array, completely
//! discarding the operators placed between them, which can be
//! accessed only through the #GetOpBetween & co functions.
//! The math index zero means "the first monomial of the array"
//! (and this means that, if the array begins with a mcSubOp/mcAddOp
//! the relative data index is 1; otherwise they coincide).
//! The math index one means "the second monomial of the array"
//! and so on...
//! 
//! The advantages of such a system are evident. Anyway, conversion
//! between the two types of indexing system is required: this is
//! why mcElementArrayMath requires derived classes to provide
//! the implementation of #math_DataToMathIdx and #math_MathToDataIdx.


protected:

	//! Handles the given simplification flag returned by the 
	//! i-th element of this array.
	//! \note Unlike almost all math functions, the given
	//!       index \c i must be in DATA coord and not in MATH
	//!       coordinates...
	//! \return The index of the next element to simplify
	//!         (used by caller if simplifying the element
	//!          in a cycle).
	virtual mcExpSimRes math_HandleExpSimFlag(mcExpSimRes r, mcElement *newelem, int i);

	//! Handles the mcBasicOpRes flag returned by one of mcElementMath
	//! basic operations (+,-,*,/).
	//! \param res The flag to handle
	//! \param r The replacement element for the element which was processed
	//! \param ridx The math index of \c r
	virtual void math_HandleBasicOpRes(mcBasicOpRes res, mcElement r, int ridx);

	//! Reorders the given array of the given size using the 
	//! mcElement::math_isListedBeforeOf to perform comparisons.
	//! \param start The start index for the reordering.
	//! \param toskip The number of operators interleaved with the various
	//!        elements of the array (could be one or two).
	//! \return The number of movements performed to reorder the array.
	virtual int math_ReorderElements(mcElement *arr, int sz, int start = 0, int toskip = 1);


	//! Expands all the elements of the array.
	virtual mcExpSimRes math_ExpandAll(long flags, mcElement *pp);


	//! Simplifies all the elements of the array.
	virtual mcExpSimRes math_SimplifyAll(long flags, mcElement *pp);

	//! Expands those elements which contain symbols in common.
	virtual mcExpSimRes math_SimplifyExp(long flags, mcElement *pp);

	//! Removes all the neutral elements from the array, using the #GetNeutralValue
	//! and the #GetNeutralOpType functions.
	virtual mcExpSimRes math_SimplifyRemoveNeutrals(long flags);

	//! Applies all the operators which can be applied on the elements of the array.
	virtual mcExpSimRes math_SimplifySolveOp(long flags);

	//! Reorders the array.
	//! Tipically this operation requires one step only anth thus this function
	//! should return always mcESR_DONE, even if this is not required.
	virtual mcExpSimRes math_Reorder() = 0;

	//! Returns TRUE if the n-th element must be expanded in order to simplify
	//! this array. This function works in data coord.
	virtual bool math_SimplifyNeedExp(long flags, int n) const;

	virtual mcExpSimRes math_BeginSimSteps()		{ return mcESR_DONE; }
	virtual mcExpSimRes math_FinalSimSteps()		{ return mcESR_DONE; }

	//! This routine is called when we're exiting from the math_Simplify(long flags) function
	//! (it's called even if we're exiting with a code different from mcESR_DONE).
	virtual void math_EndSimSteps() {}

	//! Like #EndSimSteps() just for expansion.
	virtual void math_EndExpSteps() {}

public:

	///////////////////////////////////////////////////////////////////////////
	//! \name mcElementMath abstract functions implementation
	//! mcElementArrayMath implements various mcElementMath functions. @{

	mcExpSimRes math_Simplify(long flags, mcElement *newelem);
	mcExpSimRes math_Expand(long flags, mcElement *newelem);

	//! A mcElementArray will never use the argument of the standard
	//! mcElement::math_Simplify(long flags) function, because it will never
	//! return mcESR_REPLACE_THIS flag... this little function
	//! helps to write readable code.
 	mcExpSimRes math_Simplify(long flags)	{ return math_Simplify(flags, NULL); }
 	mcExpSimRes math_Expand(long flags)		{ return math_Expand(flags, NULL); }

	mcMathType math_GetMathType() const;

	bool math_CanBeAddedWith(const mcElement &p) const;
	bool math_CanBeDivBy(const mcElement &p) const;
	bool math_CanBeMultWith(const mcElement &p) const;

	//void math_SetExp(const mcPolynomial &p);
	mcBasicOpRes math_MakeReciprocal(mcElement *);

	virtual mcRealValue math_Evaluate() const;
	
	//! An empty array must not be evaluated zero: in this way, the
	//! simp/exp algorithms know that an array, even if empty, adds
	//! a little bit of complexity...
	virtual mcRealValue math_GetLenght() const		{ return 0.5; }

	//@}




	/////////////////////////////////////////////////////////////////////////////
	//! \name Array access functions
	//! Functions which work on the array in a similar way to mcElementArrayData. @{

	//! This should not be virtual because mcPolynomial will use its own
	//! Get() function with a different return type.
	const mcElement &math_Get(int n) const 
		{ return data_Get(math_MathToDataIdx(n)); }
	mcElement &math_Get(int n) 
		{ return data_Get(math_MathToDataIdx(n)); }

	virtual int math_GetCount() const 
		{ return data_GetCount() - data_GetOpCount(); }

	//! Even if this function could seem similar to 
	//! mcElementArray::data_GetIndexOf, it's not: while data_GetIndexOf
	//! works using for comparisons XX, math_GetIndexOf uses the mcElement
	//! math_NonRecursiveFindInChildren function which uses math_Compare
	//! and math_CompareThisOnly for comparisons.
	//! Last difference: math_GetIndex returns a math index instead of a data index.
	int math_GetIndexOf(int occ, const mcElement &tofind) const;		

	mcElementType math_GetOpTypeBetween(int n1, int n2) const 
		{ return data_GetOpTypeBetween(math_MathToDataIdx(n1), math_MathToDataIdx(n2)); }

	mcElementType math_GetOpTypePreceding(int n) const 
		{ return data_GetOpTypePreceding(math_MathToDataIdx(n)); }

	mcOperator &math_GetOpBetween(int n1, int n2) const;
	mcOperator &math_GetOpPreceding(int n) const;

	//@}



	/////////////////////////////////////////////////////////////////////////////
	//! \name mcElementArrayMath-specific math functions
	//! Functions implemented on the top of the abstract ones. @{
	
	//! Remove the n-th element from the array, creating a mcNumber init with zero
	//! (if \c bmath_AddToZero == TRUE) or with one (if \c bmath_AddToZero == FALSE),
	//! if the expression would remain empty after the removal of the n-th element.
	//! This function also removes the (eventually present) operator which follows
	//! the element.
	//! This function is designed to be used by math engine functions; if you
	//! want a function which simply removes an element, see #DeleteElement.
	//! This function, in fact, not only removes the n-th element; it shifts array
	//! left one position, too.
	//! \return TRUE if a mcNumber intialized with zero or one was added to the
	//!         array to avoid to leave it empty; FALSE otherwise.	
	bool math_Remove(int n, bool bAddZero = TRUE);

	bool math_RemoveFactor(const mcArrayEntry &toremove);

	//! math_Finds the given element among the elements contained directly
	//! (that is, without any wrapper: this function is not recursive)
	//! and then deletes it from the array.
	//! Unlike mcElementArray::data_Delete that matches only the elements
	//! with the same identity (the same ID or, it's the same, with the
	//! same pointer values), this function uses the #math_CompareWith() function
	//! to find the n-th element to delete.
	//! This function also shifts the array, thus decrementing the element
	//! counter.
	//! Returns TRUE if everything is okay; FALSE if the given pointer
	//! is not an element contained in this array.
	virtual bool math_Delete(int n, const mcElement &elem, bool bAddZero = TRUE);

	//! Moves the n-th element of the array (and its preceding operator)
	//! to the position k.
	//! VERY IMPORTANT: with mcMonomial you can use this function only
	//!                 when all mcDivOp have been removed.
	//virtual void Move(int n, int k);

	//! Creates a temporary operator of the given type and then applies it
	//! to the given two elements.
	void math_ApplyOp(mcElementType optype, mcElement res, const mcElement &factor, mcElement *);

	//! Works like #ApplyOp but uses the mcOperator::ApplySimple instead of the
	//! mcOperator::Apply function.
	void math_ApplyOpSimple(mcElementType optype, mcElement res, const mcElement &factor);

	virtual bool math_CompareThisOnly(const mcElement &p, long flags) const;
	virtual bool math_Compare(const mcElement &p, long flags) const;	

	//! Returns TRUE if this array is containing only an element of the given type.
	virtual bool math_isWrappingOnly(mcElementType t) const;

	//! Returns the element of type \c t wrapped by this element, if it does
	//! exist and it is the only element wrapped by this array; returns NULL
	//! otherwise.
	virtual const mcElement &math_GetWrapped(mcElementType t) const;

	virtual mcArrayEntry math_WrapMonomial(const mcMonomial &) = 0;
	virtual mcArrayEntry math_WrapSimple(const mcElement &) = 0;

	virtual mcArrayEntry math_WrapSymbol(const mcSymbolProperties *sym);
	virtual mcArrayEntry math_WrapNumber(const mcRealValue &val);

	//! Returns TRUE if this array is containing only a mcNumber.
	bool math_isWrappingNumOnly() const		{ return math_isWrappingOnly(mcET_NUMBER); }

	//! Returns TRUE if this array is containing only a mcSymbol.
	bool math_isWrappingSymOnly() const		{ return math_isWrappingOnly(mcET_SYMBOL); }

	//! Returns the number which is wrapped by this array or NAN if
	//! this array does not contain a single mcNumber.
	mcRealValue math_GetWrappedNumber() const;

	//! Returns the symbol  which is wrapped by this array or NULL if
	//! this array does not contain a single mcSymbol.
	const mcSymbolProperties *math_GetWrappedSymbol() const;

	bool math_isWrappingOnly(const mcSymbolProperties *sym) const
		{ return math_GetWrappedSymbol() == sym; }
	bool math_isWrappingOnly(const mcRealValue &val) const
		{ return math_GetWrappedNumber() == val; }

	virtual void math_ResetToOne();
	virtual void math_ResetToZero();

	virtual void math_SimpleAdd(const mcElement &p, bool add = TRUE);
	virtual void math_SimpleMultiplyBy(const mcElement &p);
	virtual void math_SimpleDivideBy(const mcElement &p);

	mcArrayEntry math_EmbedInFraction(bool bAsDenominator = FALSE);	
	mcArrayEntry math_EmbedInBracket();
	mcArrayEntry math_EmbedInRadical();

	//@}



	/////////////////////////////////////////////////////////////////////////////
	//! \name mcElementArrayMath abstract functions
	//! Functions which must be implemented by derived classes. @{

	//! Converts from data to math index. See #mcElementArrayMath
	//! general description for more info.
	virtual int math_DataToMathIdx(int dataindex) const = 0;

	//! Converts from math to data index. See #mcElementArrayMath
	//! general description for more info.
	virtual int math_MathToDataIdx(int mathindex) const = 0;

	virtual void math_PrepareForComparison(mcElementArray &p) const;
	virtual void math_PrepareForMathOperations(mcElementArray &p) const;

	virtual mcElementType math_GetNeutralOpType() const = 0;
	virtual double math_GetNeutralValue() const = 0;
	
	//! Returns a new mcPolynomial containing *this array.
	//! mcPolynomial returns a copy of themselves, mcMonomial returns
	//! a mcPolynomial containing itself.
	virtual mcPolynomial math_GetPolynomialWrapper() const = 0;

	//! Returns a new mcElementArray element which wraps the given element.
	virtual mcElementArray math_CreateWrapperFor(const mcElement &p) const;

	virtual mcMonomial math_GetFactors() const;

	//! Factores-out of the contained elements everything is possible.
	//! For mcPolynomials, this means that the GCD among all the factors
	//! of the monomials (get through #math_GetFactors) will be picked up.
	//! For mcMonomials, this means that all the children will be divided
	//! by the factors they returned with the #math_GetFactors function.
	//virtual void math_FactoreOut() = 0;

	//virtual void math_RaiseChildrenTo(const mcRealValue &n) = 0;
	//virtual void math_RaiseChildrenTo(const mcPolynomial &p) = 0;

	void math_EmbedInBracketAndRaiseTo(const mcRealValue &n);		
	void math_EmbedInBracketAndRaiseTo(const mcPolynomial &p);
	void math_EmbedInRadicalAndRaiseTo(const mcPolynomial &p);

	//@}

#endif		// mcENABLE_MATH



#ifdef mcENABLE_IO
public:

	//! Checks the given tag and, if it is an MROW, returns a
	//! null value (the ID of the first mcElementArray-derived class).
	bool io_isBeginTag(const wxXml2Node &tag) const { 
		if (tag.GetName() == wxT("mrow"))
			return TRUE;
		return FALSE;
	}

	//! Element arrays do not have a special begin/end character in
	//! inlined expressions... they are allocated using special
	//! rules and thus do not follow the standard mcElement rules.
	//! This is why this function always returns FALSE.
	bool io_isBeginChar(const wxString &str) const {
		return FALSE;
	}

	wxString io_GetInlinedExpr() const;

#endif		// mcENABLE_IO
};





//! A general array of mcElements; this class has properties
//! similar to wxObjArray class but it allows to perform
//! some complex operations like shift.
class mcElementArray : public mcElement
{
	mcDEFINE_ABSTRACT_CLASS(ElementArray, mcElement);
	
public:

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_MONOMIAL || t == mcET_POLYNOMIAL; }


#ifdef mcENABLE_DATA
public:

	mcWRAPPER void data_CheckIndex(int entry) const
		{ hlp()->data_CheckIndex(entry); }
	mcWRAPPER void data_CheckArrayLimit()
		{ hlp()->data_CheckArrayLimit(); }
	mcWRAPPER void data_ChangeFilter(const mcFilter *p)
		{ hlp()->data_ChangeFilter(p); }

	mcWRAPPER void data_UpdateFilterFor(mcElement &p, const mcFilter *f)
		{ hlp()->data_UpdateFilterFor(p, f); }

	mcWRAPPER bool data_isElementAllowed(mcElementType type) const
		{ return hlp()->data_isElementAllowed(type); }

	mcWRAPPER int data_AddElements(const mcElement *p, int num, int pos = -1, 
						bool bOverwrite = FALSE, bool bForceCopy = FALSE)
		{ return hlp()->data_AddElements(p, num, pos, bOverwrite, bForceCopy); }

	mcWRAPPER mcElement &data_AddNewElement(mcElementType t, bool bOverwrite = FALSE, int pos = -1)
		{ return hlp()->data_AddNewElement(t, bOverwrite, pos); }

	mcWRAPPER mcElement &data_AddNewWrappedElement(mcElementType t, 
								 bool bOverwrite = FALSE, int pos = -1) 
		{ return hlp()->data_AddNewWrappedElement(t, bOverwrite, pos); }

	mcWRAPPER void data_Merge(const mcElementArray &p, int n, bool bmath_AddToEnd = TRUE)
		{ hlp()->data_Merge(p, n, bmath_AddToEnd); }

	mcWRAPPER void data_AddNewOp(mcElementType type, int pos = -1)
		{ hlp()->data_AddNewOp(type, pos); }

	mcWRAPPER void data_AddNewEmptyBox(int pos = -1)
		{ hlp()->data_AddNewEmptyBox(pos); }


	mcWRAPPER bool data_Delete(const mcElement &elem)
		{ return hlp()->data_Delete(elem); }
	mcWRAPPER void data_Delete(int entry)
		{ hlp()->data_Delete(entry); }
	mcWRAPPER void data_DeleteLast(int num)
		{ hlp()->data_DeleteLast(num); }
	mcWRAPPER void data_DeleteFirst(int num)
		{ hlp()->data_DeleteFirst(num); }
	mcWRAPPER void data_DeleteAllElemType(mcElementType t)
		{ hlp()->data_DeleteAllElemType(t); }
	mcWRAPPER void data_DeleteAll()
		{ hlp()->data_DeleteAll(); }

	mcWRAPPER mcElement data_Detach(int n)
		{ return hlp()->data_Detach(n); }
	mcWRAPPER mcElement data_DetachLastElem(int num)
		{ return hlp()->data_DetachLastElem(num); }
	mcWRAPPER mcElement data_DetachAll()
		{ return hlp()->data_DetachAll(); }

	mcWRAPPER void data_MoveElemRight(int start)
		{ hlp()->data_MoveElemRight(start); }
	mcWRAPPER void data_MoveElemLeft(int start)
		{ hlp()->data_MoveElemLeft(start); }
	mcWRAPPER void data_Move(int n, int k)
		{ hlp()->data_Move(n, k); }
	mcWRAPPER int data_MoveAllElemType(mcElementType t, mcElementArray &m)
		{ return hlp()->data_MoveAllElemType(t, m); }

	mcWRAPPER void data_Swap(int index1, int index2)
		{ hlp()->data_Swap(index1, index2); }

	mcWRAPPER int data_QueueElemType(mcElementType t, int firstpos)
		{ return hlp()->data_QueueElemType(t, firstpos); }

	mcWRAPPER void data_Repair()
		{ hlp()->data_Repair(); }



	mcWRAPPER bool data_isWrappingOnly(mcElementType t) const
		{ return hlp()->data_isWrappingOnly(t); }

	mcWRAPPER const mcElement &data_GetWrapped(mcElementType t) const
		{ return hlp()->data_GetWrapped(t); }


	mcWRAPPER const mcElement &data_Get(int entry) const 
		{ return hlp()->data_Get(entry); }
	mcWRAPPER mcElement &data_Get(int entry)
		{ return hlp()->data_Get(entry); }


	mcWRAPPER int data_GetIndexOf(const mcElement &p) const
		{ return hlp()->data_GetIndexOf(p); }

	mcWRAPPER const mcElement &data_GetLast() const 
		{ return hlp()->data_GetLast(); }
	mcWRAPPER mcElement &data_GetLast()
		{ return hlp()->data_GetLast(); }


	mcWRAPPER mcElement *data_GetArray(int n = 0) const
		{ return hlp()->data_GetArray(n); }

	mcWRAPPER int data_GetIndex(int id) const
		{ return hlp()->data_GetIndex(id); }

	mcWRAPPER int data_GetCount() const 
		{ return hlp()->data_GetCount(); }

	mcWRAPPER int data_GetAvailableEntryCount() const
		{ return hlp()->data_GetAvailableEntryCount(); }

	mcWRAPPER int data_GetNumOfElemType(mcElementType t) const
		{ return hlp()->data_GetNumOfElemType(t); }

	mcWRAPPER int data_GetOpCount() const
		{ return hlp()->data_GetOpCount(); }

	mcWRAPPER mcElement &data_GetElemOfType(int n, mcElementType t) const
		{ return hlp()->data_GetElemOfType(n, t); }

	mcWRAPPER int data_GetElemIndexOfType(int n, mcElementType t) const
		{ return hlp()->data_GetElemIndexOfType(n, t); }

	mcWRAPPER mcElement &data_GetNonOpElem(int n) const
		{ return hlp()->data_GetNonOpElem(n); }

	mcWRAPPER int data_GetNonOpElemIndex(int n) const
		{ return hlp()->data_GetNonOpElemIndex(n); }

	mcWRAPPER mcOperator &data_GetOp(int n) const
		{ return hlp()->data_GetOp(n); }

	mcWRAPPER int data_GetOpIndex(int n) const
		{ return hlp()->data_GetOpIndex(n); }

	mcWRAPPER mcElementType data_GetOpTypeBetween(int n1, int n2) const
		{ return hlp()->data_GetOpTypeBetween(n1, n2); }

	mcWRAPPER mcElementType data_GetOpTypePreceding(int n) const
		{ return hlp()->data_GetOpTypePreceding(n); }

	mcWRAPPER const mcElement &data_GetElemAtLeftOf(const mcElement &p) const
		{ return hlp()->data_GetElemAtLeftOf(p); }

	mcWRAPPER const mcElement &data_GetElemAtRightOf(const mcElement &p) const 
		{ return hlp()->data_GetElemAtRightOf(p); }

	mcWRAPPER bool data_isOp(int entry) const
		{ return hlp()->data_isOp(entry); }
	mcWRAPPER bool data_isEmptyBox(int entry) const
		{ return hlp()->data_isEmptyBox(entry); }
	mcWRAPPER bool data_isEmptyEntry(int entry) const
		{ return hlp()->data_isEmptyEntry(entry); }
	mcWRAPPER bool data_isAddSubOp(int entry) const
		{ return hlp()->data_isAddSubOp(entry); }
	mcWRAPPER bool data_isMultDivOp(int entry) const
		{ return hlp()->data_isMultDivOp(entry); }

	mcWRAPPER bool data_isArrayEmpty() const
		{ return hlp()->data_isArrayEmpty(); }
	mcWRAPPER bool data_isValidElem(int entry) const
		{ return hlp()->data_isValidElem(entry); }

	mcWRAPPER void data_Set(int i, const mcElement &value)
		{ hlp()->data_Set(i, value); }

	mcWRAPPER void data_SetAsEmptyEntry(int entry)
		{ hlp()->data_SetAsEmptyEntry(entry); }

	mcWRAPPER void data_SetFilter(const mcFilter *p)
		{ hlp()->data_SetFilter(p); }

	mcWRAPPER mcFilter *data_GetFilter()
		{ return hlp()->data_GetFilter(); }

	mcWRAPPER const mcFilter *data_GetFilter() const		
		{ return hlp()->data_GetFilter(); }

	mcWRAPPER bool data_isUsingFilter() const				
		{ return hlp()->data_isUsingFilter(); }

#endif		// mcENABLE_DATA




#ifdef mcENABLE_GUI
public:

	mcWRAPPER int gui_AddNewElement(mcElementType type, const mcKey &key, int pos = -1)
		{ return hlp()->gui_AddNewElement(type, key, pos); }
	mcWRAPPER void gui_AddNewEmptyBox(int pos = -1)
		{ hlp()->gui_AddNewEmptyBox(pos); }
	mcWRAPPER void gui_AddNewOp(mcElementType nOpType, int pos = -1)
		{ hlp()->gui_AddNewOp(nOpType, pos); }

	mcWRAPPER void gui_SetCursorOnElemBegin(int n)		
		{ hlp()->gui_SetCursorOnElemBegin(n); }
	mcWRAPPER void gui_SetCursorOnElemEnd(int n)		
		{ hlp()->gui_SetCursorOnElemEnd(n); }

	mcWRAPPER bool gui_isCursorOn(const mcElement &p) const
		{ return hlp()->gui_isCursorOn(p); }

	mcWRAPPER int gui_GetSelElemCount() const
		{ return hlp()->gui_GetSelElemCount(); }
	mcWRAPPER int gui_GetSelElemCountOfType(mcElementType t) const
		{ return hlp()->gui_GetSelElemCountOfType(t); }
	mcWRAPPER int gui_GetSelElemIndex(int n) const
		{ return hlp()->gui_GetSelElemIndex(n); }
	mcWRAPPER int gui_GetSelElemIndexOfType(int n, mcElementType t) const
		{ return hlp()->gui_GetSelElemIndexOfType(n, t); }

	mcWRAPPER const mcElement &gui_GetSelElem(int n) const
		{ return hlp()->gui_GetSelElem(n); }
	mcWRAPPER const mcElement &gui_GetSelElemOfType(int n, mcElementType t) const
		{ return hlp()->gui_GetSelElemOfType(n, t); }

	mcWRAPPER int gui_GetSelStart() const
		{ return hlp()->gui_GetSelStart(); }
	mcWRAPPER int gui_GetSelEnd() const
		{ return hlp()->gui_GetSelEnd(); }

	mcWRAPPER void gui_DeleteSelection()
		{ hlp()->gui_DeleteSelection(); }

	mcWRAPPER mcElement &gui_ExGetActiveElem(int x, int y, const wxPoint &pt, int cl)
		{ return hlp()->gui_ExGetActiveElem(x, y, pt, cl); }

	mcWRAPPER void gui_DeleteNextOp()
		{ hlp()->gui_DeleteNextOp(); }
	mcWRAPPER void gui_DeletePreviousOp()
		{ hlp()->gui_DeletePreviousOp(); }

	mcWRAPPER void gui_MoveCursorRight()
		{ hlp()->gui_MoveCursorRight(); }
	mcWRAPPER void gui_MoveCursorLeft()
		{ hlp()->gui_MoveCursorLeft(); }

/*
	mcWRAPPER mcInputRes gui_BackInput(const mcKey &key, mcElement *pnewelem, int n)

	mcWRAPPER bool gui_isArrEndKey(const mcKey &ev) const

	mcWRAPPER void gui_CheckCursorPos() const

	mcWRAPPER void gui_DoSplit(mcElementType type, const mcKey &ev)

	mcWRAPPER int gui_GetSpaceBetween() const
*/
	mcWRAPPER int gui_GetCenterLine() const
		{ return hlp()->gui_GetCenterLine(); }

	mcWRAPPER int gui_ExDraw(wxDC &dc, int x, int y, long flags, const wxPoint &pt, int cl = -1, 
					int begin = 0, int end = -1, bool bDrawSelected = FALSE) const
		{ return hlp()->gui_ExDraw(dc, x, y, flags, pt, cl, begin, end, bDrawSelected); }
/*
 	mcWRAPPER int gui_DrawSelectedElements(wxDC &dc, int x, int y, long flags, 
									mcWRAPPER const wxPoint &pt, int cl = -1) const

	mcWRAPPER int gui_DrawSelection(wxDC &hDC, int x, int y, long flags, const wxPoint &pt, int cl = -1) const

	mcWRAPPER void gui_ExOnSelect(wxDC &dc, int x, int y, wxRect &rc, int cl = -1)

	mcWRAPPER wxSize gui_CalcSizeOfFirst(int n, int cl = -1) const

	mcWRAPPER wxSize gui_CalcSizeOfRange(int begin, int end, int cl = -1) const

	mcWRAPPER wxRect gui_GetSelectionRect(wxDC &dc, int x, int y, int cl = -1) const

	mcWRAPPER wxPoint gui_GetOriginOfElem(int n, int cl = -1) const

	mcWRAPPER int gui_GetBB(int n, wxRect *rc, int yCenter, int w) const
*/
	mcWRAPPER int gui_ExGetRelCursorPos(wxDC &dc, wxPoint *pt, int cl = -1) const
		{ return hlp()->gui_ExGetRelCursorPos(dc, pt, cl); }
/*
	mcWRAPPER int gui_ExMoveCursorUsingPoint(wxDC &hDC, const wxPoint &pt, int cl = -1)

	mcWRAPPER int gui_GetActiveElemIndex(int x, int y, const wxPoint &pt, int cl = -1)
*/
#endif		// mcENABLE_GUI


#ifdef mcENABLE_MATH
public:

 	mcWRAPPER mcExpSimRes math_Simplify(long flags)
		{ return hlp()->math_Simplify(flags); }
 	mcWRAPPER mcExpSimRes math_Expand(long flags)
		{ return hlp()->math_Expand(flags); }
 	mcWRAPPER mcExpSimRes math_Simplify(long flags, mcElement *p)
		{ return hlp()->math_Simplify(flags, p); }
 	mcWRAPPER mcExpSimRes math_Expand(long flags, mcElement *p)
		{ return hlp()->math_Expand(flags, p); }

	mcWRAPPER const mcElement &math_Get(int n) const
		{ return hlp()->math_Get(n); }
	mcWRAPPER mcElement &math_Get(int n)
		{ return hlp()->math_Get(n); }
	mcWRAPPER int math_GetCount() const
		{ return hlp()->math_GetCount(); }

	mcWRAPPER int math_GetIndexOf(int occ, const mcElement &tofind) const
		{ return hlp()->math_GetIndexOf(occ, tofind); }

	mcWRAPPER mcElementType math_GetOpTypeBetween(int n1, int n2) const
		{ return hlp()->math_GetOpTypeBetween(n1, n2); }
	mcWRAPPER mcElementType math_GetOpTypePreceding(int n) const
		{ return hlp()->math_GetOpTypePreceding(n); }
	mcWRAPPER mcOperator &math_GetOpBetween(int n1, int n2) const
		{ return hlp()->math_GetOpBetween(n1, n2); }
	mcWRAPPER mcOperator &math_GetOpPreceding(int n) const
		{ return hlp()->math_GetOpPreceding(n); }

	mcWRAPPER bool math_Remove(int n, bool bAddZero = TRUE)
		{ return hlp()->math_Remove(n, bAddZero); }
	mcWRAPPER bool math_Delete(int n, const mcElement &elem, bool bAddZero = TRUE)
		{ return hlp()->math_Delete(n, elem, bAddZero); }
	mcWRAPPER void math_ApplyOp(mcElementType optype, mcElement res, const mcElement &factor, mcElement *pp)
		{ hlp()->math_ApplyOp(optype, res, factor, pp); }
	mcWRAPPER void math_ApplyOpSimple(mcElementType optype, mcElement res, const mcElement &factor)
		{ hlp()->math_ApplyOpSimple(optype, res, factor); }

	mcWRAPPER bool math_isWrappingOnly(mcElementType t) const
		{ return hlp()->math_isWrappingOnly(t); }

	mcWRAPPER const mcElement &math_GetWrapped(mcElementType t) const
		{ return hlp()->math_GetWrapped(t); }
	mcWRAPPER mcArrayEntry math_WrapMonomial(const mcMonomial &m)
		{ return hlp()->math_WrapMonomial(m); }
	mcWRAPPER mcArrayEntry math_WrapSimple(const mcElement &e)
		{ return hlp()->math_WrapSimple(e); }
	mcWRAPPER mcArrayEntry math_WrapSymbol(const mcSymbolProperties *sym)
		{ return hlp()->math_WrapSymbol(sym); }
	mcWRAPPER mcArrayEntry math_WrapNumber(const mcRealValue &val)
		{ return hlp()->math_WrapNumber(val); }

	mcWRAPPER bool math_isWrappingNumOnly() const		
		{ return hlp()->math_isWrappingNumOnly(); }
	mcWRAPPER bool math_isWrappingSymOnly() const		
		{ return hlp()->math_isWrappingSymOnly(); }

	mcWRAPPER mcRealValue math_GetWrappedNumber() const
		{ return hlp()->math_GetWrappedNumber(); }

	mcWRAPPER const mcSymbolProperties *math_GetWrappedSymbol() const
		{ return hlp()->math_GetWrappedSymbol(); }
	mcWRAPPER bool math_isWrappingOnly(const mcSymbolProperties *sym) const
		{ return hlp()->math_isWrappingOnly(sym); }
	mcWRAPPER bool math_isWrappingOnly(const mcRealValue &val) const
		{ return hlp()->math_isWrappingOnly(val); }

	mcWRAPPER void math_ResetToOne()
		{ hlp()->math_ResetToOne(); }
	mcWRAPPER void math_ResetToZero()
		{ hlp()->math_ResetToZero(); }

	mcWRAPPER mcArrayEntry math_EmbedInFraction(bool bAsDenominator = FALSE)	
		{ return hlp()->math_EmbedInFraction(bAsDenominator); }
	mcWRAPPER mcArrayEntry math_EmbedInBracket()
		{ return hlp()->math_EmbedInBracket(); }
	mcWRAPPER mcArrayEntry math_EmbedInRadical()
		{ return hlp()->math_EmbedInRadical(); }

	mcWRAPPER int math_DataToMathIdx(int dataindex) const
		{ return hlp()->math_DataToMathIdx(dataindex); }
	mcWRAPPER int math_MathToDataIdx(int mathindex) const
		{ return hlp()->math_MathToDataIdx(mathindex); }

/*	mcElementType math_GetNeutralOpType() const
	mcWRAPPER double math_GetNeutralValue() const

	mcWRAPPER mcPolynomial math_GetPolynomialWrapper() const

	mcWRAPPER mcElementArray math_CreateWrapperFor(const mcElement &p) const

	mcWRAPPER mcPolynomial math_GetFactors() const*/
/*
	mcWRAPPER void math_RaiseChildrenTo(const mcPolynomial &p)
		{ hlp()->math_RaiseChildrenTo(p); }
	mcWRAPPER void math_RaiseChildrenTo(const mcRealValue &p)
		{ hlp()->math_RaiseChildrenTo(p); }
*/
	mcWRAPPER void math_EmbedInBracketAndRaiseTo(const mcRealValue &n)
		{ hlp()->math_EmbedInBracketAndRaiseTo(n); }
	mcWRAPPER void math_EmbedInBracketAndRaiseTo(const mcPolynomial &p)
		{ hlp()->math_EmbedInBracketAndRaiseTo(p); }

#endif		// mcENABLE_MATH
};


#endif	// ELEMENTARRAY_H

