///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   ExpElement.h
//! \brief  Defines the mcExpElementData, mcExpElementGUI, 
//!         mcExpElementMath, mcExpElementIO, mcExpElement classes.
//!
//! See ExpElement.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef EXPELEMENT_H
#define EXPELEMENT_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "ExpElement.h"
#endif

// required includes
#include "mc/Element.h"
#include "mc/Polynomial.h"
#include "mc/Text.h"


// this defines all the classes declared in this header file
mcDEFINE_HELPER_CLASSES(mcExpElement)


//! Cursor position values handled by mcExpElement
//! Some of the possible values of mgui_nCursorPos variable 
//! (derived classes must use different values to indicate different positions).
enum mcExpElemCursorLocation {

	//! When the exponent is placed on the left of the element, there are
	//! two values allowed for the mgui_nCursorPos variable: mcECL_EXPONENTBEGIN
	//! and mcECL_INSIDEEXPONENT:
	//!                  |exp
	//!                  |  base        mgui_nCursorLoc = mcECL_EXPONENTBEGIN
	//!
	mcECL_EXPONENTBEGIN,

	//! This value is always valid (if data_mdata_bExpEnabled == TRUE): 
	//! both when the exponent is on the right and
	//! when the exponent is placed on the left...
	//!       e|xp       e|xp
	//!    base      or     base        mgui_nCursorLoc = mcECL_INSIDEEXPONENT
	//!
	mcECL_INSIDEEXPONENT,
	
	//! When the exponent is placed on the right of the element, there are
	//! two values allowed for the mgui_nCursorPos variable: mcECL_EXPONENTEND
	//! and mcECL_INSIDEEXPONENT:
	//!                     exp|
	//!                  base  |        mgui_nCursorLoc = mcECL_EXPONENTEND
	//!
	mcECL_EXPONENTEND,

	//! When the subscript is placed on the left of the element, there are
	//! two values allowed for the mgui_nCursorPos variable: mcECL_SUBSCRIPTBEGIN
	//! and mcECL_INSIDESUBSCRIPT:
	//!                  |  base
	//!                  |sub           mgui_nCursorLoc = mcECL_SUBSCRIPTBEGIN
	//!
	mcECL_SUBSCRIPTBEGIN,

	//! This value is always valid (if data_mdata_bSubEnabled == TRUE):
	//!                  base
	//!                     s|ub        mgui_nCursorLoc = mcECL_INSIDESUBSCRIPT
	//!	
	mcECL_INSIDESUBSCRIPT,

	//! This value is always valid (if data_mdata_bSubEnabled == TRUE); it
	//! is handled exactly as mcECL_EXPONENTEND: that's why they have exactly
	//! the same value.
	//!                  base  |
	//!                     sub|        mgui_nCursorLoc = mcECL_SUBSCRIPTEND
	//!	
	mcECL_SUBSCRIPTEND,

	//! The cursor is inside the base and thus is handled by the base functions
	//! which have been implemented in the derived class.
	//!
	//!                       exp
	//!                   ba|se         mgui_nCursorLoc = mcECL_INSIDEBASE
	//!                       sub
	//!
	mcECL_INSIDEBASE
};



//! If you try to use a EXPONENT function which manipulates the cursor
//! when the cursor is not inside the exponent, this will be the return
//! value of that function !
#define mcECL_NOTINSIDEEXPONENT		0x77777777

//! Same as mcECL_NOTINSIDEEXP, just for the subscript.
#define mcECL_NOTINSIDESUBSCRIPT	0x88888888




//! \name Exponent/subscript choosers
//! Some very useful routines used to choose between the exponent/subscript
//! version of a mcExpElement function. @{

inline int mcEP_HAS(bool exp)
{ return (exp ? mcEP_HASEXPONENT : mcEP_HASSUBSCRIPT); }

inline mcExpElemCursorLocation mcECL_BEGIN(bool exp)
{ return (exp ? mcECL_EXPONENTBEGIN : mcECL_SUBSCRIPTBEGIN); }

inline mcExpElemCursorLocation mcECL_INSIDE(bool exp)
{ return (exp ? mcECL_INSIDEEXPONENT : mcECL_INSIDESUBSCRIPT); }

inline mcExpElemCursorLocation mcECL_END(bool exp)
{ return (exp ? mcECL_EXPONENTEND : mcECL_SUBSCRIPTEND); }

inline mcKey *mcGetEditKey(bool exp)
{ return (exp ? mcMathCore::Get()->m_pEditExpKey : 
			mcMathCore::Get()->m_pEditSubscriptKey); }

inline int mcNotInside(bool exp)
{ return (exp ? mcECL_NOTINSIDEEXPONENT : mcECL_NOTINSIDESUBSCRIPT); }

//@}








//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
class mcExpElementHelpers : public mcElementHelpers
{
#ifdef mcENABLE_DATA
protected:		// DATA variables

	//! The exponent handled by this class. 
	//! It is allocated only when required (performance optimization).
	//! This class is able to handle with the GUI functions any type of
	//! mcElement-derived class but MATH functions expect this to be
	//! a mcPolynomial.
	mcElement mdata_pExp;

	//! The subscript handled by this class. 
	//! It is allocated only when required (performance optimization).
	//! This is usually a mcText even if GUI and IO functions should
	//! work with each type of mcElement-derived class. MATH functions
	//! will simply ignore it.
	mcElement mdata_pSub;

	//! If TRUE, the user can edit the exponent associated; if FALSE, the
	//! mcMathCore::Get()->m_pgui_EditExpKey will be rejected.
	bool mdata_bExpEnabled;

	//! If TRUE, the user can edit the subscript associated; if FALSE, the
	//! mcMathCore::Get()->m_pgui_EditSubscriptKey will be rejected.
	bool mdata_bSubEnabled;

#endif


#ifdef mcENABLE_GUI
protected:		// GUI variables

	//! Defines the position of the 'exponent': this class, in fact, can be used not 
	//! only to implement exponents, but also to implement the indexes of the radicals;
	//! just set to FALSE mgui_bExpRight (default is TRUE) to make the exponent placed
	//! on the left of the element.
	bool mgui_bExpRight;

	//! Defines the position of the 'subscript'.
	//! See #mgui_bExpRight for more info.
	bool mgui_bSubRight;
	
	//! The size of the base only.
	//! \note None of the base functions should modify the m_sz variable; they
	//!	      can modify only this variable.
	wxSize mgui_szBase;

	//! The cursor position. mcExpElement derived classes must track cursor
	//! position using this variable; derived classes should define other own
	//! values for the other possible cursor positions.
	//int mgui_nCursorPos;
	mcExpElemCursorLocation mgui_nCursorLoc;

#endif


#ifdef mcENABLE_MATH
public:			// customizable variables

	//! If TRUE, then the #math_SimplifyExp function will convert
	//! *this element to a mcRadical when the exponent is in the
	//! (0,1) range.
	static bool smath_bConvertToRadicalWhenPossible;

#endif


public:

	mcExpElementHelpers() : mdata_pExp(NULL), mdata_pSub(NULL) {		
		// exponent & subscribt do not exist yet...
		mdata_bExpEnabled = mdata_bSubEnabled = TRUE;
	}

	virtual ~mcExpElementHelpers() {
		data_DestroyExpSub(TRUE);
		data_DestroyExpSub(FALSE);
	}

protected:

	void gui_Init() {
		mcElementHelpers::gui_Init();

		// this value can be changed by derived classes' constructors
		mgui_nCursorLoc = mcECL_INSIDEBASE;		// default: not inside exponent nor subscript
		mgui_bExpRight = mgui_bSubRight = TRUE;
	}




#ifdef mcENABLE_DATA
public:

#ifdef __MCDEBUG__

	void data_Check() const;

	//! Returns debug info about the base (always) and the exponent (only if
	//! present).
	virtual wxString data_Debug(long flags) const;

	//! This function should work exactly as mcElement::data_Debug function but just
	//! for the base; all exponent info are provided by #data_ExpDebug.
	virtual wxString data_BaseDebug(long flags) const		{ return wxT("invalid"); }
	
	//! Returns debug info about the exponent.
	wxString data_ExpDebug(long flags) const;

	//! Returns debug info about the subscript.
	wxString data_SubDebug(long flags) const;

#endif

	int data_GetChildrenCount() const {
		int b1 = data_GetExp() != NULL;
		int b2 = data_GetSub() != NULL;
		return mcElementHelpers::data_GetChildrenCount() + b1 + b2;
	}

	const mcElement &data_GetConstChild(int n) const {
		mcRETURN_ELEMENT_CHILD(n, mcElementHelpers);
		bool b1 = data_GetExp() != NULL;
		bool b2 = data_GetSub() != NULL;
		
		if (b1 && n == 0)
			return data_GetExp();
		if (b2 && (n == 0 || n == 1))
			return data_GetSub();

		return mcEmptyElement;
	}
	
	void data_SetChild(int n, const mcElement &newchild) {
		mcSET_ELEMENT_CHILD(n, mcElementHelpers, newchild);
		bool b1 = data_GetExp() != NULL;
		bool b2 = data_GetSub() != NULL;

		if (b1 && n == 0)
			data_SetExpSub(TRUE, newchild);
		if (b2 && (n == 0 || n == 1))
			data_SetExpSub(FALSE, newchild);
 	}


	//! Deep copies the given mcExpElementHelpers, removing the exponent from 
	//! *this if it is not present in \c p or deep-copying it otherwise.
	//! The same checks are performed on the subscript.
	void data_DeepCopy(const mcElementHelpers *p);

	//! An helper function of data_DeepCopy: works deep-copying only the
	//! exp/sub of the given mcExpElementHelpers class.
	void data_DeepCopyExpSub(bool exp, const mcExpElementHelpers *);
	
	//! The exponent/subscript is mcEmptyElement as default (to avoid wasting memory 
	//! allocating exp/sub which won't never be used...). This function creates it
	//! (deleting the eventually present old exp/sub) and sets correctly
	//! its exponent level.
	virtual void data_CreateExpSub(bool exp);
	
	//! Destroys the exponent if present. This function is called very few times:
	//! in fact, exponents, once created are usually hidden, not destroyed.
	virtual void data_DestroyExpSub(bool exp);

	//! Returns a pointer to the current exponent: the return value could be NULL.
	mcElement &data_GetExp()								{ return mdata_pExp; }
	const mcElement &data_GetExp() const					{ return mdata_pExp; }

	//! Returns a pointer to the current subscript: the return value could be NULL.
	mcElement &data_GetSub()								{ return mdata_pSub; }	
	const mcElement &data_GetSub() const					{ return mdata_pSub; }	

	mcElement &data_Get(bool bExp)
		{ return (bExp ? mdata_pExp : mdata_pSub); }
	const mcElement &data_Get(bool bExp) const
		{ return (bExp ? mdata_pExp : mdata_pSub); }

	//! Destroys the old exponent (if present) and replace it with the given element.
	virtual void data_SetExpSub(bool exp, const mcElement &p);

#endif		// mcENABLE_DATA





#ifdef mcENABLE_GUI
public:

	///////////////////////////////////////////////////////////////////////
	//! \name Exponent & subscript handlers
	//! The following are the functions implemented by mcExpElement class.
	//! They shouldn't be used by derived-classes except for special
	//! reasons: mcExpElement will use them as appropriate when necessary. @{

	//! Returns TRUE if the given char is the end char for the exponent and
	//! the cursor is actually editing the exponent.
	//! If cursor is not inside the exponent, if will always return FALSE.
	bool gui_isExpSubEndKey(bool, const mcKey &) const;
	//bool gui_isSubEndKey(const mcKey &) const;		//!< See #gui_isExpEndKey

	bool gui_isExpSubEndKeyHelper(bool exp, const mcKey &ev) const;
	
	//! Works exactly as mcElementHelpers::sgui_Draw, just on the exponent, checking, 
	//! before drawing, if the exponent is visible.
	int gui_ExpSubDraw(bool, wxDC &, int, int, long flags, const wxPoint &) const;
	//int gui_DrawSub(wxDC &, int, int, long flags, const wxPoint &) const;		//!< See #gui_DrawExp.
	
	//! This function returns only the exponent size (which should be
	//! different from element size).
	/*wxSize gui_GetExpSize();
	wxSize gui_GetSubSize();			//!< See #gui_GetExpSize.
	wxSize gui_GetExpSize() const;
	wxSize gui_GetSubSize() const;*/
	wxSize gui_GetExpSubSize(bool);
	wxSize gui_GetExpSubSize(bool) const;
	
	//! Forces the recalculation of exponent's size.
	void gui_RecalcExpSubSize(bool);
	//void gui_RecalcSubSize();					//!< See #gui_RecalcExpSize.
	
	//! Returns the y-offset of the exponent (usually zero).
	virtual int gui_GetExpSubOffsety(bool) const;
	
	//! Returns the y-offset of the subscript: it's the value of the
	//! base offsety (see #gui_GetBaseOffsety) + mcMathCore::gui_GetSubscriptOffsety().
	//virtual int gui_GetSubOffsety() const;

	//! Returns the offset (respect to the top-left point of the bounding box of
	//! the element) for the x coord. used to draw the exponent.
	//! This function usually returns the width of the base (and it's exactly what
	//! standard implementation does); if you need to do something else, override it.
	virtual int gui_GetExpSubOffsetx(bool) const;
	//virtual int gui_GetSubOffsetx() const;					//!< See #gui_GetExpOffsetx.
	
	//! Returns the height of the cursor (or mcECL_NOTINSIDEEXP flag) and
	//! its position (using the given pointer) in coordinates relative to 
	//! the top-left point of the bounding rect of the entire element.
	virtual int gui_GetExpSubRelCursorPos(bool, wxDC &, wxPoint *) const;
	//virtual int gui_GetSubRelCursorPos(wxDC &, wxPoint *) const;		//!< See #gui_GetExpRelCursorPos.
	
	//! Handles the movements of the cursor when it's inside the exponent 
	//! (if the cursor is not in the exponent, the function returns 
	//! mcECL_NOTINSIDEEXP).
	virtual mcMoveCursorRes gui_ExpSubMoveCursor(bool, mcMoveCursorFlag, long);
	//virtual mcMoveCursorRes gui_SubMoveCursor(mcMoveCursorFlag, long);	//!< See #gui_ExpMoveCursor.

	int gui_ExpSubMoveCursorUsingPoint(bool, wxDC &dc, const wxPoint &p);
	//int gui_SubMoveCursorUsingPoint(wxDC &dc, const wxPoint &p);
	
	//! Returns the Cursor Position flag (mcCP_***)
	//! if the cursor is inside the exponent.
	void gui_GetExpSubCursorPos(bool, mcCursorPos &) const;	
	//void gui_GetSubCursorPos(mcCursorPos &) const;			//!< See #gui_GetExpCursorPos.
	
	//! If the exponent is placed on the right of the element, handles 
	//! the mcCP_END flag, setting cursor position at the end of the exponent;
	//! if the exponent is placed on the left, handles the mcCP_BEGIN flag.
	void gui_SetExpSubCursorPos(bool, const mcCursorPos &code);

	//! Handles the mcCP_END flag, when the cursor is inside the subscript.
	//void gui_SetSubCursorPos(mcCursorPos code);
	
	//! Handles input when cursor is inside the exponent; this function returns
	//! mcECL_NOTINSIDEEXP if cursor is not inside exponent, mcIR_OKAY otherwise.
	//! You can override this function to intercept exponent changes.
	virtual mcInputRes gui_ExpSubInput(bool, const mcKey &ev, mcElement *pnew);
	//virtual mcInputRes gui_SubInput(const mcKey &ev, mcElement *pnew);	//!< See #gui_ExpInput.
	
	//! Sets the cursor inside the exponent (if the exponent doesn't exist or
	//! it is not visible, the function will create it or make it visible).
	//! This function will set cursor position correctly only if the
	//! exponent was hidden before this call; if the exponent was already
	//! visible the cursor position inside the exponent is untouched.
	void gui_EditExpSub(bool);
	//void gui_EditSub();				//!< See #gui_EditExp().
	
	//! Deletes the exponent from this element as if it has never been created.
	void gui_DeleteExpSub(bool);
	//void gui_DeleteSub();			//!< See #gui_DeleteExp.
	
	//! Checks the contents of the exponent and eventually reset them. 
	//! This function is called by #gui_EditExp(). If you do not use as
	//! exponent a mcPolynomial, you must override this function
	//! (see mcExpElementData).
	virtual void gui_CheckExpSub(bool);
	//virtual void gui_CheckSub();			//!< See #gui_CheckExp.

	//! Creates the exponent and inits it with an empty monomial.
	virtual void gui_CreateExpSub(bool);
	///virtual void gui_CreateSub();			//!< See #CreateExp.
	
	//@}




	//////////////////////////////////////////////////////////////
	//! \name Base handlers
	//! These functions must be provided by derived class; they must work as
	//! their respective mcElement versions, but only for the base of the element. 
	//! Derived classes must also implement the abstract functions not covered by
	//! mcExpElement (because they do not apply to exponents): mcElement::gui_Split(),
	//! mcElement::io_isBeginChar... @{
	
	//! This function should work as mcElement::isEndChar function, but it should care 
	//! only for the base; this means that the element should not handle the
	//! mcEXPELEM_**** cursor positions, but it will probably still need to check
	//! for exp/sub presence to choose if the given keyevent is the end key.
	//! When the cursor is inside the exp/sub, this function is
	//! never called. Derived classes must also implement the 
	//! mcElementHelpers::gui_isBeginChar function.
	virtual bool gui_isBaseEndKey(const mcKey &ev) const = 0;
	
	//! Identical to mcElement::gui_Draw, but just for the base. You have to 
	//! draw it at the given point (x,y) without take care of the exp/sub, 
	//! just do everything as they wouldn't exist. The given pointer to
	//! a wxPoint class can be mcDRW_NONACTIVE or a valid pointer
	//! value (in this case, the cursor is always inside the base). You can
	//! be sure that it will never be mcDRW_ALLACTIVE: this flag is completely 
	//! handled by mcExpElement. If all the base must be drawn as active, the
	//! subscript should be too... you can use the #GetBaseAndSubSize function
	//! to make the task easier.
	//! \param dc The DC where the base must be drawn
	//! \param x, y The position on the DC where the base must be drawn
	//! \param pt mcDRW_NONACTIVE or a valid pointer to the mouse position;
	//! \return The ID of the element which is placed under the given cursor
	//!         position (this is typically hlp()->data_GetID(), but in some cases,
	//!         like for mcBracket, it can be different) or mcDRW_NOACTIVELEM, if
	//!         the mouse cursor is placed over empty space.
	virtual int gui_DrawBase(wxDC &dc, int x, int y, long flags, 
								const wxPoint &pt) const = 0;
	
	//! Calculates (in the m_szBase variable) the size of the base only.
	//! It's very important not to modify the m_sz variable: m_sz is handled
	//! by mcExpElement functions: the base must modify ONLY *m_szBase*.
	virtual void gui_DoRecalcBaseSize() = 0;
	
	//! Sets the cursor position of the element.
	//! When this function is called with the mcCP_BEGIN flag, it must set the
	//! cursor at the leftmost point of the base; when this function is called
	//! with mcCP_END flag, the function must set the cursor at the rightmost
	//! point of the base. It should never check exponent/subscript presence:
	//! everything is handled by mcExpElement.
	virtual void gui_SetBaseCursorPos(const mcCursorPos &) = 0;
	
	//! Called when the cursor is switching from the exp/sub to the base; it
	//! should set the cursor inside the base: the correct position will be
	//! set using the #gui_SetBaseCursorPos.
	virtual void gui_EditBase() = 0;
	
	//! Works as #gui_EditBase() but this function also sets the cursor position
	//! inside the base with the given flag.
	virtual void gui_EditBaseAndSetPos(const mcCursorPos &cp);
	
	//! Returns one of the cursor position flags (called only when cursor is 
	//! inside the base).
	virtual void gui_GetBaseCursorPos(mcCursorPos &) const = 0;

	virtual mcCursorPos gui_GetBaseCursorPos() const
		{ mcCursorPos ret; gui_GetBaseCursorPos(ret); return ret; }
	
	//! Handles the input when the cursor is editing the base; an useful
	//! function, to avoid repetitions inside gui_BaseInput(), could be
	//! the #gui_HandleSubExpEditKeys() function...
	virtual mcInputRes gui_BaseInput(const mcKey &ev, mcElement *pnew) = 0;
	
	virtual mcInsertRes gui_BaseInsert(const mcElement &, mcElement *) = 0;

	//! Handles cursor movements when the cursor is editing the base.
	//! This function should work ABSOLUTELY without care about the
	//! exponent or subscript presence or absence: if the cursor
	//! is placed on the rightmost point of the base, and the function
	//! is called with the mcMCF_RIGHT flag, then the function should return
	//! mcMCR_SETFOCUS_NEXT without any check: the mcExpElement::gui_MoveCursor
	//! function will check the return value of this function and will 
	//! corerctly handle it !!!
	virtual mcMoveCursorRes gui_BaseMoveCursor(mcMoveCursorFlag, long modifiers) = 0; 

	//! Moves the cursor inside the base using the given point and,
	//! eventually the given DC for calculations.
	virtual int gui_BaseMoveCursorUsingPoint(wxDC &, const wxPoint &) = 0;
	
	//! Returns the height of the cursor and it's position (in the given pointer
	//! and in coordinates relatives to the top-left point of the base), when the 
	//! cursor is editing the base.
	virtual int gui_GetBaseRelCursorPos(wxDC &, wxPoint *) const = 0;

	//! Returns TRUE if the mcExpElement functions must draw as active the entire
	//! element when the cursor is over the base of the element. If this function
	//! returns FALSE, then the exponent is not drawn as active when cursor is
	//! inside the base.
	//!
	//! This function should always return TRUE if the base is *not* a container
	//! of other elements (e.g. mcSymbol and mcNumber always return TRUE); in this
	//! way when the cursor is inside the base, the mcExpElement::gui_DrawActivationRect()
	//! will always be called and the gui_DrawBase() function will never be called 
	//! with a pt != mcDRW_NONACTIVE.
	//! On the other hand, if the base contains other elements (e.g. mcBracket), this
	//! function should return FALSE, so that the gui_DrawBase() function is called with
	//! a valid pointer to the cursor position, if the mouse is inside the base.
	//! In this way, the container element can send the mouse cursor position to the
	//! contained elements, highligthing them only.
	virtual bool gui_DrawAsActiveOverBase() const = 0;

	//! Uses mcMathCore::gui_GetBaseOffsety() function to get the amount of y-pixels
	//! to leave before drawing the base: the y-offset of the base.
	virtual int gui_GetBaseOffsety() const;

	//! Returns the offset of the x-coord. of the base: override if you want
	//! some special effect (default value is zero).
	virtual int gui_GetBaseOffsetx() const;
		
	//! Returns the size of the base only. The standard implementation of
	//! this function checks if the m_szBase variable is okay; if not it 
	//! calls the #gui_DoRecalcBaseSize. Override only if you need.
	virtual wxSize gui_GetBaseSize() { 
		if (mgui_szBase.GetWidth() == 0)
			gui_DoRecalcBaseSize();
		return mgui_szBase;
	}

	//! Returns the size of the base only (see the difference between the
	//! const and non-const version of mcElementHelpers::sgui_GetSize).
	virtual wxSize gui_GetBaseSize() const		{ return mgui_szBase; }

	//! Handles selections for the base only. Called when the base (or the empty
	//! space) bounding box only intersects the selection rectangle, this function
	//! should call the mcElement::gui_Select() function or, for container classes,
	//! call the appropriate gui_OnSelect function of the children elements.
	//! The standard implementation if or basic classes and just calls the
	//! mcElement::gui_Select() function. Override only if you need.
	virtual void gui_OnBaseSelect(wxDC &dc, wxRect &rc) {
		gui_SelectAll();
	}

	//! Returns TRUE if the exponent or the subscribt is visible on the
	//! right side of the element. Default implementation checks it the
	//! exponent is placed on the right and the exponent or the subscript
	//! are visible...
	virtual bool gui_hasBaseSthOnRightSide() const;

	//! Returns TRUE if the exponent is visible on the left side of the
	//! element. Default implementation checks if the exponent is placed
	//! on the left of the element and it's visible.
	virtual bool gui_hasBaseSthOnLeftSide() const;

	//! Returns TRUE if the exponent/subscript if placed on the right.
	virtual bool gui_isOnRight(bool exp) const
		{ return (exp ? mgui_bExpRight : mgui_bSubRight); }

	//! Returns the selected part of the base.
	virtual mcElement gui_GetBaseSelection() const
		{ return mcEmptyElement; }

	//@}
	




	//////////////////////////////////////////////////////////////
	//! \name Utility functions
	//! Functions which can be used both in base, exp and 
	//! subscript handler functions. @{

	//! Handles the mcMathCore::Get()->m_pgui_EditExpKey and the mcMathCore::Get()->
	//! m_pgui_EditSubscriptKey keypresses, calling gui_EditExp() or gui_EditSub().
	//! If the given keypress is not the EDITEXP or EDITSUBSCRIPT key, then
	//! the function returns a value which is not mcIR_OKAY.
	int gui_HandleSubExpEditKeys(const mcKey &ev);

	//! Returns the size of the base summed with the size of the subscript,
	//! if present. The function doesn't really sum up the two sizes: it
	//! must use the #gui_GetSubOffsety() and #gui_GetSubOffsetx() functions....
	wxSize gui_GetBaseAndSubSize() const;
	
	//! gui_Draws as non active both the base, the exponent and the subscript.
	virtual void gui_DrawNonActive(wxDC &dc, wxPoint &base, 
									wxPoint &exp, wxPoint &sub) const;
	
	//! gui_Draws the activation rectangle of the entire element (base + exp + sub).
	//! This function should be called BEFORE you draw the base and the
	//! exponent, because it fills the background with the activation brush.
	virtual void gui_DrawActivationRect(wxDC &dc, int x, int y) const;

	//! gui_Draws the activation rectangle under the element and then
	//! draws both the base and the exponent (they should be both
	//! drawn as transparent).
	//! \param dc The DC where the base and the exp will be drawn
	//! \param x, y The position of this element on the given DC
	//! \param w The exponent offsetx, or -1 to let this function calculate it
	//! \param h The exponent offsety, or -1 to let this function calculate it
	virtual void gui_DrawAllActive(wxDC &dc, wxPoint &orig, wxPoint &base, 
		wxPoint &exp, wxPoint &sub) const;
	
	//! Returns TRUE if the cursor is inside exp (and if exp is allocated 
	//! and visible).
	//! \param bCheckBeginEndExp If TRUE, the function will return TRUE even if
	//!                          the cursor is set to mcEXPELEM_EXPBEGIN or
	//!                          mcEXPELEM_EXPEND.
	bool gui_isCursorInExpSub(bool, bool bCheckBeginEndExp = TRUE) const;

	//! Returns TRUE if the cursor is inside the subscript.
	//bool gui_isCursorInSub(bool bCheckBeginEndExp = TRUE) const;

	//! Returns TRUE if the cursor is inside the base.
	bool gui_isCursorInBase() const;

	//@}





	//////////////////////////////////////////////////////////////
	//! \name Standard mcElement functions (overridables)
	//! Override these functions only if you really need it; they
	//! call in the right order and with the right arguments the
	//! base and the exponent versions of the various functions.
	//! Note that all the documentation reported is copied by
	//! Doxygen from mcElement functions. @{

	virtual mcMoveCursorRes gui_MoveCursor(mcMoveCursorFlag flag, long modifiers);
	virtual mcInputRes gui_Input(const mcKey &ev, mcElement *newelem);
	virtual mcInsertRes gui_Insert(const mcElement &, mcElement *);
	virtual void gui_GetCursorPos(mcCursorPos &) const;

	virtual void gui_OnSelect(wxDC &dc, wxRect &rc);	
	virtual void gui_SetCursorPos(const mcCursorPos &code);
	virtual void gui_DoRecalcSize();
	virtual void gui_UpdateExpDepth();

	virtual int gui_Draw(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const;
	virtual int gui_GetRelCursorPos(wxDC &dc, wxPoint *pt) const;
	virtual int gui_MoveCursorUsingPoint(wxDC &dc, const wxPoint &p);
	virtual int gui_GetYAnchor() const;

	virtual bool gui_isEndKey(const mcKey &ev) const;
	virtual mcElement gui_GetSelection() const;

	//@}

#endif		// mcENABLE_GUI





#ifdef mcENABLE_MATH
protected:

	//! Called by math_Simplify() when the exponent is ready
	//! to be simplified. If it can be evaluated, then this
	//! functions tries to apply some default simplification
	//! (e.g. if the exponent is in the 0;1 range, converts
	//!  *this into a mcRadical...).
	//! If the exponent cannot be evaluate, calls #math_SimplifyBaseExp.
	mcExpSimRes math_SimplifyExp(long flags, mcElement *);

public:

	//! Returns TRUE if this element has a working exponent.
	bool math_hasExp() const;
	bool math_hasSub() const;

	//! Returns a pointer to the exponent of this element.
	//! If this class has an exponent set to NULL, then the
	//! exponent is created and set to 1.0.
	//! See mcExpElementMath description for details about
	//! the mcElement.mcPolynomial conversion.


	//! Returns a CONST pointer to the exponent of this element.
	//! This function has a different name from #GetExp() since
	//! not only the returned type is different, also the way
	//! it works is very different: if this element has an exponent,
	//! then this function returns a pointer to it like #GetExp;
	//! but if this element does not have an exponent, then it
	//! does not create anything but returns mcPolynomial::math_pOne.
	const mcPolynomial &math_GetConstExp() const;
	mcPolynomial &math_GetExp();
	const mcPolynomial &math_GetExp() const
		{ return math_GetConstExp(); }	

	
	//! See #math_GetExp().
	mcText &math_GetSub() const
		{ return (mcText &)(data_GetSub()); }
	const mcText &math_GetConstSub() const;		//!< See #math_GetExp().


	void math_SetExp(const mcPolynomial &pol)
		{ data_SetExpSub(TRUE, pol); }

	mcBasicOpRes math_RaiseTo(const mcPolynomial &p);

	mcRealValue math_EvaluateExp() const;
	bool math_CompareExp(const mcExpElement &, long flags) const;
	bool math_CompareSub(const mcExpElement &, long flags) const;
	mcBasicOpRes math_MultOrDiv(const mcElement &, mcElement *, bool mult);
	
	

	//////////////////////////////////////////////////////////////
	//! \name Math engine functions
	//! Basic implementation. @{

	//virtual //bool hasSameContentOf(const mcElement &) const;
	virtual bool math_Compare(const mcElement &p, long flags) const;
	
	//! The raise-to-power operation has a little degree of complexity.
	//! In this way we can force our simplify routines to transform an
	//! expression like:
	//!            9^2          into           81
	//!       lenght=2.5                    lenght=2  
	//!
	//! The exponent/subscript of this element will be taken in count
	//! by the #math_GetTotalLenght() function.
	mcRealValue math_GetLenght() const	
		{ mcRealValue l=math_GetBaseLenght(); return (math_hasExp() ? l+0.5 : l); }
	
	virtual mcRealValue math_Evaluate() const;
	virtual mcBasicOpRes math_MakeReciprocal(mcElement *);

	virtual mcBasicOpRes math_MultiplyBy(const mcElement &, mcElement *);
	virtual mcBasicOpRes math_DivideBy(const mcElement &, mcElement *);
	
	virtual mcExpSimRes math_Simplify(long flags, mcElement *newelem);
	virtual mcExpSimRes math_Expand(long flags, mcElement *newelem);

	virtual mcMathType math_GetMathType() const;

	//@}



	//////////////////////////////////////////////////////////////
	//! \name Abstract BASE functions
	//! Math functions regarding the base and not the exponent. @{
	
	virtual mcMathType math_GetBaseMathType() const = 0;

	virtual mcRealValue math_GetBaseLenght() const { return 0; }
	virtual mcRealValue math_EvaluateBase() const = 0;

	virtual mcExpSimRes math_ExpandBase(long flags, mcElement *newelem) = 0;

	virtual mcExpSimRes math_SimplifyBase(long flags, mcElement *newelem) = 0;
	virtual mcExpSimRes math_SimplifyBaseExp(long flags, mcElement *pnew) { return mcESR_DONE; }

	virtual mcBasicOpRes math_MultiplyBaseOnlyBy(const mcElement &, mcElement *) = 0;
	virtual mcBasicOpRes math_DivideBaseOnlyBy(const mcElement &, mcElement *) = 0;

	virtual mcBasicOpRes math_MultiplyBaseBy(const mcElement &, mcElement *)
		{ return mcBOR_REMOVE_OPERAND; }
	virtual mcBasicOpRes math_DivideBaseBy(const mcElement &, mcElement *) 
		{ return mcBOR_REMOVE_OPERAND; }

	mcMonomial math_GetLCM(const mcElement &) const;
	mcMonomial math_GetGCD(const mcElement &) const;

	virtual mcMonomial math_GetBaseLCM(const mcElement &p) const;
	virtual mcMonomial math_GetBaseGCD(const mcElement &p) const;

	//@}


#endif		// mcENABLE_MATH




#ifdef mcENABLE_IO
public:

	//! Returns a MathML tree with the presentation math markup language regarding
	//! the base. This function should work exactly as the hononymous mcElement
	//! function.	
	//! \param bGetPresentation If TRUE, this function returns the presentation
	//!        MathML, otherwise it returns the content markup.
	virtual wxXml2Node io_GetBaseMathML(bool bGetPresentation) const = 0;

	//! Returns the MathML tree which describes this element and, eventually,
	//! its exponent and its subscript.
	virtual wxXml2Node io_GetMathML(bool bGetPresentation) const;

	virtual wxString io_GetInlinedExpr() const;
	virtual wxString io_GetBaseInlinedExpr() const = 0;
	
	/*virtual void GetContentMathML(wxXml2Node *pTag) {
		
		// if an exponent is present, create a POWER section
		if (data_mdata_pExp != NULL && this->data_hasProperty(mcEP_HASEXPONENT)) {
			
			wxXml2Node *base = new wxXml2Node(), *exp = new wxXml2Node();
			
			// save base and exponent Math ML
			GetBaseContentMathML(base);
			data_mdata_pExp.io_GetMathML(exp);
			
			// concatenate all the string
			dest += wxString(wxT(' '), indent) + wxT("<apply>\n") + wxString(wxT(' '), indent+mcMathCore::Get()->m_nIndentationStep) + 
				wxT("<power/>\n") + base + exp + wxString(wxT(' '), indent) + wxT("</apply>\n");
			
			return dest;
		}
		
		// we don't need to create a MSUP tag
		return GetBaseContentMathML(indent);
	}*/
	static bool io_ImportExpSub(const wxString &str, int *count, wxString &pErr);

	virtual bool io_ImportInlinedExpr(const wxString &str, int *count, wxString &pErr);
	virtual bool io_ImportBaseInlinedExpr(const wxString &, int *, wxString &) = 0;

	//! Returns the token containing the string to use to import the
	//! exponent/subscript found while importing an inlined expression.
	static void io_GetExpSubInlinedToken(wxString &subexp);

#endif		// mcENABLE_IO
};


//! Extends the mcElement class, providing functions to handle
//! a polynomial as exponent. 
//! Defines an extension to mcElement base class; mcExpElement provides 
//! functions to help derived classes to manage an exponent;
//! thanks to this class a lot of code isn't repeated among all the
//! elements with exponents...
class mcExpElement : public mcElement
{
	mcDEFINE_ABSTRACT_CLASS(ExpElement, mcElement);

public:

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_SYMBOL || t == mcET_NUMBER ||
				t == mcET_FUNCTION || t == mcET_BRACKET ||
				t == mcET_RADICAL; }

#ifdef mcENABLE_DATA
public:

	mcWRAPPER mcElement &data_GetExp()					{ return hlp()->data_GetExp(); }
	mcWRAPPER const mcElement &data_GetExp() const		{ return hlp()->data_GetExp(); }

	mcWRAPPER mcElement &data_GetSub()					{ return hlp()->data_GetSub(); }	
	mcWRAPPER const mcElement &data_GetSub() const		{ return hlp()->data_GetSub(); }	
	
	mcWRAPPER void data_SetExpSub(bool exp, const mcElement &p)
		{ hlp()->data_SetExpSub(exp, p); }
	mcWRAPPER void data_DestroyExpSub(bool exp)
		{ hlp()->data_DestroyExpSub(exp); }
	mcWRAPPER void data_CreateExpSub(bool exp)
		{ hlp()->data_CreateExpSub(exp); }

#endif


#ifdef mcENABLE_MATH
public:

	mcWRAPPER bool math_hasExp() const
		{ return hlp()->math_hasExp(); }
	mcWRAPPER bool math_hasSub() const
		{ return hlp()->math_hasSub(); }

	mcWRAPPER mcPolynomial &math_GetExp()
		{ return hlp()->math_GetExp(); }
	mcWRAPPER const mcPolynomial &math_GetConstExp() const
		{ return hlp()->math_GetConstExp(); }

	mcWRAPPER mcText &math_GetSub()	
		{ return hlp()->math_GetSub(); }
	mcWRAPPER const mcText &math_GetSub() const	
		{ return hlp()->math_GetSub(); }
#endif
};


#endif	// EXPELEMENT_H

