///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   MathAndSystem.h
//! \brief  Defines the mcMathAndSystemHelpers and mcMathAndSystem classes.
//!
//! See MathAndSystem.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef MATHANDSYSTEM_H
#define MATHANDSYSTEM_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "MathAndSystem.h"
#endif

// required includes
#include "mc/MathSystem.h"


// defined later
class mcMathAndSystem;


//! Defines the render mode of mcMathAndSystem. 
//! See mcMathAndSystemHelpers::sgui_nRenderMode.
enum mcMathAndSystemRenderMode {

	//! Draws a curly bracket on the left of all the mcMathLines
	//! contained. In this mode, sgui_bHorizOrientation is ignored.
	mcMASRM_DRAW_CURLY_BRACKET,

	//! Draws the AND mathematically symbol: ^
	mcMASRM_DRAW_AND_SYMBOLS,

	//! Draws the text "AND" among the mcMathLines...
	mcMASRM_DRAW_AND_TEXT,
};



//! A set of mcMathLines connected with an AND logical operator.
//! \note In the following documentation with the term "main system
//!	      symbol" I mean the great '{' symbol on the left, which groups
//!	      all the equations\\inequalities of the system.
class mcEXPORT mcMathAndSystemHelpers : public mcMathSystemHelpers
{
	mcDEFINE_REFERENCE_DATA(mcMathAndSystem, mcET_MATHANDSYSTEM);

protected:		// functions

	//! Sets the n-th line as the line where the cursor is currently over.
	//! This requires some manipulation of mgui_nLastActiveLine and
	//! mgui_nCurrentActiveLine to optimize redrawing with the #isIDChanged()
	//! function. This function is protected because its use should be
	//! allowed only to mcMathOrSystem functions. The users of this class should
	//! call the #data_UpdateSystemBmp() function (which calls this function
	//! with the right parameters).
	int gui_UpdateActiveLine(int n, wxMemoryDC &hDC, int x, int y, 
								long flags, const wxPoint &p);

	//! Returns the index of the math-line which contains the given point
	int gui_GetLineContainingPt(int x, int y, const wxPoint &pt);

	//! Updates the font which is used to draw the main system symbol.
	void gui_UpdateSystemSym();

	//! This is an overload of mcAbstractArray::data_OnArrayChange function.
	//void data_OnArrayChange();


protected:		// mcAbstractArray functions

	//! Removes the n-th line of the array.
	void data_Delete(int n)
		{ delete ((mcMathLine*)data_GetItem(n)); }

	//! Returns a copy of the n-th line of the array.
	mcAbstractItem *data_GetCopyOf(int n) const	
		{ return new mcMathLine(data_GetLine(n)); }


public:
	
	mcMathAndSystemHelpers() { data_Init(); }
	virtual ~mcMathAndSystemHelpers() { data_Clear(); }



#ifdef mcENABLE_DATA
public:

#ifdef __MCDEBUG__

	wxString data_Debug(long flags) const;

#endif

	/////////////////////////////////////////////////////////////////////////////////
	//! \name DATA functions
	//! Functions that work on the content but which do not belong to the
	//! general GUI, MATH and IO sections. @{

	//! Adds the given line to the array taking ownership of it.
	void data_AddLine(mcMathLine *ptoadd)
		{ mcAbstractArray::data_Add(ptoadd); }

	//! Adds an empty line to the array.
	void data_AddEmptyLine()
		{ data_AddLine(new mcMathLine()); }
	
	mcMathLine &data_GetLine(int n)
		{ return (mcMathLine &)data_GetChild(n); }
	const mcMathLine &data_GetLine(int n) const
		{ return (const mcMathLine &)data_GetConstChild(n); }
	mcMathLine &data_GetLastLine()
		{ return (mcMathLine &)data_GetLast(); }
	const mcMathLine &data_GetLastLine() const
		{ return (const mcMathLine &)data_GetLast(); }

	//@}

	//! Sets the filter associated with all the elements of the array.
	//! See mcFilter and mcPolynomial for more info.
	//! \param p The new mcFilter to set (can be NULL)
	//! \param n The line to set the filter for or -1 to set it for
	//!          all the lines of this array
	void data_SetFilter(const mcFilter *p, int n = -1);

#endif		// mcENABLE_DATA



#ifdef mcENABLE_GUI
public:				// customizable variables

	//! The space left on the left and the right of the math lines.
	static int sgui_nSpaceLeftRight;

	//! The space left above and below the math lines.
	static int sgui_nSpaceAboveBelow;

	//! The space between two math lines when sgui_bUseCurlyBracket is
	//! TRUE and thus lines are drawn one above the other.
	static int sgui_nSpaceBetween;


	//! If TRUE, then the lines of this system will be drawn
	//! horizontally, if FALSE, then they will be drawn vertically.
	static bool sgui_bHorizOrientation;

	//! If TRUE, then sgui_bHorizOrientation will be ignored (the lines
	//! will always ve drawn in vertical order) and a curly bracket
	//! on the left side of the system.
	static mcMathAndSystemRenderMode sgui_nRenderMode;

protected:

	virtual bool gui_DrawHorizontally() const { return sgui_bHorizOrientation; }
	virtual bool gui_DrawVertically() const { return !sgui_bHorizOrientation; }

	virtual int gui_GetSpaceAboveBelow() const { return sgui_nSpaceAboveBelow; }
	virtual int gui_GetSpaceLeftRight() const { return sgui_nSpaceLeftRight; }
	virtual int gui_GetSpaceBetween() const { return sgui_nSpaceBetween; }

public:

	mcMathLine &gui_GetFocusLine()
		{ return (mcMathLine &)gui_GetFocus(); }
	const mcMathLine &gui_GetFocusLine() const
		{ return (const mcMathLine &)gui_GetFocus(); }

	void gui_DrawSystemSymbols(wxDC &dc, int x, int y, long flags) const;
	void gui_RecalcSymbolSize();

	//! Returns a pointer to the portion of mathsystem currently selected;
	//! NULL if there are no parts selected.
	mcElement gui_GetSelection() const;

	//! Returns TRUE if the last call to #data_UpdateBmp() set as active
	//! element (the element under mouse cursor) an element which is
	//! different from the element which was active before that call.
	bool gui_isIDChanged() const;

	//! Returns the ID of the element which was marked as active
	//! in the last bitmap update.
	//! \note This function can be much faster than calling
	//!            gui_GetActiveElem(...).data_GetID()
	//!       because mcMathLine holds a variable containing directly
	//!       the ID, while gui_GetActiveElem() requires a search
	//!       into the math & gui tree.
	virtual int gui_GetActiveElemID() const;

#endif		// mcENABLE_GUI



#ifdef mcENABLE_MATH
public:
	
	//mcMathAndSystem *math_Merge(const mcMathAndSystem *) const;
	void math_Merge(const mcMathAndSystem &arr1,
					const mcMathAndSystem &arr2);

	mcMathSystemType math_GetMathSystemType() const;

#endif		// mcENABLE_MATH



#ifdef mcENABLE_IO
public:

	//! Returns as inlined expression all the current system.
	//wxString io_GetInlinedExpr() const;

	wxString io_GetInlinedExprOfLogicalOperator() const
		{ return wxT("AND"); } 

	bool io_ImportToken(const wxString &str, wxString *perr);

#endif		// mcENABLE_IO
};


//! Container of a mcMathAndSystemHelpers-derived class which uses COW.
//! Never use mcMathAndSystemHelpers directly: it must be accessed
//! through this class. For more info see mcElement class details.
class mcMathAndSystem : public mcMathSystem
{
	mcDEFINE_MAIN_CLASS(MathAndSystem, mcMathSystem);

public:

	mcMathAndSystem(const mcMathLine &l1, const mcMathLine &l2)
	{ data_SetRefData(new mcMathAndSystemHelpers()); 
	  data_AddLine(new mcMathLine(l1)); data_AddLine(new mcMathLine(l2)); }

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_MATHANDSYSTEM; }


#ifdef mcENABLE_DATA
public:

	mcWRAPPER void data_AddLine(mcMathLine *ptoadd)
		{ hlp()->data_AddLine(ptoadd); }
	mcWRAPPER mcMathLine &data_GetLine(int n) 
		{ return hlp()->data_GetLine(n); }
	mcWRAPPER const mcMathLine &data_GetLine(int n) const
		{ return hlp()->data_GetLine(n); }
	mcWRAPPER void data_SetFilter(const mcFilter *p, int n = -1)
		{ hlp()->data_SetFilter(p, n); }

#endif		// mcENABLE_DATA


#ifdef mcENABLE_GUI
public:

	mcWRAPPER int gui_GetActiveElemID() const
		{ return hlp()->gui_GetActiveElemID(); }
	mcWRAPPER bool gui_isIDChanged() const
		{ return hlp()->gui_isIDChanged(); }

	mcWRAPPER mcMathLine &gui_GetFocusLine()
		{ return hlp()->gui_GetFocusLine(); }
	mcWRAPPER const mcMathLine &gui_GetFocusLine() const
		{ return hlp()->gui_GetFocusLine(); }

#endif		// mcENABLE_GUI


#ifdef mcENABLE_MATH
public:

	mcWRAPPER void math_Merge(const mcMathAndSystem &arr1,
								const mcMathAndSystem &arr2)
		{ hlp()->math_Merge(arr1, arr2); }

#endif		// mcENABLE_MATH
};

#endif		// mcMATHANDSYSTEM
