///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   MathCore.h
//! \brief  Defines the mcMathCore class.
//!
//! See MathCore.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef MATHCORE_H
#define MATHCORE_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "MathCore.h"
#endif

// required includes
#include "mc/MathHelpers.h"
#include <wx/xml2.h>				// a wxWidgets extension



// the following classes will be defined later; we need to define
// them here to let the compiler know about their existence
class mcElement;
class mcElementHelpers;
class mcOperator;
class mcFunction;
class mcPolynomial;
class mcMonomial;
class mcNumber;
class mcFraction;
class mcBracket;
class mcSymbol;
class mcText;
class mcElementArray;
class mcExpContainer;
class mcSymbolArray;
class mcKey;
class mcStyle;
class mcSolverArray;


//! The default step threshold. See mcMathCore::m_fStepThreshold for more info.
#define mcDEFAULT_STEP_THRESHOLD							1.0


//! The mcMathCore class: this static class contains global variables,
//! functions and defines used by almost all MathCore classes.
//!
//! This class has the following roles:
//! - global allocator of MathCore classes (through #NewElem).
//! - manager of mcElement-derived classes' behaviour (through all
//!   the functions which work on the #m_pElem array, like
//!   #isKeyBeginKey, #....).
//! - miscellaneous functions collector
//!
class mcMathCore
{
protected:		// member variables

	//! The global instance of the class shared everywhere in programs
	//! which use this library.
	static mcMathCore *g_pTheInstance;

	bool m_bGUIEnabled;		//!< TRUE if the GUI section of the library is enabled.
	bool m_bIOEnabled;		//!< TRUE if the IO section of the library is enabled.
	bool m_bMathEnabled;	//!< TRUE if the MATH section of the library is enabled.

	//! The current step threshold.
	//! This important value determines how many operations should be
	//! performed between two simplification/expansion/resolution
	//! (through the mcSolvers) steps.
	//! If this number is very high, then you'll see complex data solved
	//! in very few steps (the steps could then be quite difficult to
	//! check by hand...); if this number is very low, then you'll see
	//! a lot of steps even for simple data (all those steps could be
	//! quite boring...).
	mcRealValue m_fStepThreshold;

protected:		// functions

	void Init(bool bIO = TRUE, bool bMath = TRUE, bool bGUI = TRUE);
	void SetupGUI(void (*OnError)(const wxString &) = NULL);
	void SetupIO();
	void SetupMath();

	void CleanupGUI();
	void CleanupIO();
	void CleanupMath();


public:

	//! Main constructor: inits all the variables contained in this class.
	mcMathCore();

	//! The Destructor: frees all the memory allocated by this object.
	virtual ~mcMathCore();

	//! Returns the global instance of mcMathCore.
	//! This is the way all MathCore functions access the mcMathCore
	//! instance; of course, it should be also the way the user of
	//! the library access mcMathCore... just write
	//!
	//! \code
	//! mcMathCore::Get()->Setup();
	//! ... the code which uses the MathCore library ...
	//! mcMathCore::Get()->Cleanup();
	//! \endcode
	static mcMathCore *Get();

	//! math_Replaces the current instance of mcMathCore class with the given one.
	//! This function should be used only in the case that, for some special
	//! reasons, the user wants to create a class derived from mcMathCore
	//! and he wants to use it instead of the base class.
	static void Set(mcMathCore *);

	//! Returns TRUE if the mcMathCore global instance is ready to work.
	static bool isOk()		{ return g_pTheInstance != NULL; }




	///////////////////////////////////////////////////////////////////////////
	//! \name GENERAL SECTION
	//! Defines functions and values for general purposes. @{

	//! Call this function with a description of the error to
	//! notify the user about a syntax error; this function will
	//! return immediately and the caller should continue data
	//! processing as if nothing happened.
	//! \param str The description of the error
	void SyntaxError(wxString str);

	//! Allocates MathCore stuff. This functions should be called
	//! only once at the beginning and at the end of the program; if you need
	//! to change MathCore settings after its initialization, you
	//! will have to modify manually MathCore variables.
	//! \param cfg The wxConfig object which will be used by all MathCore
	//!        functions to load & save customizable parameters; the application
	//!        which uses MathCore library must also provide a //mcMathCore::Get()->GetConfigPath()
	//!        function which will be used to retrieve the group of the wxConfig object
	//!        where all settings will be saved & loaded.
	//! \param OnError Pointer to the function which will be called by
	//! 	   #mcMathCore::Get()->SyntaxError() function with the description of the error occurred
	//void SetUpMathCore(wxConfig *cfg, void (*OnError)(const wxString &) = NULL);

	//! Builds all the objects with a constructor (here declared as pointers)
	//! using the default values; initializes the specified subsystems;
	//! prepares everything to work.
	static void Setup(bool bIO = TRUE, bool bMath = TRUE, bool bGUI = TRUE);

	//! Deallocates MathCore stuff.
	static void Cleanup();

	//! Returns TRUE if mcElementGUI are being allocated when new elements
	//! are created: that is, GUI classes of MathCore are being used.
	bool isGUIEnabled() const;

	//! Like #isGUIEnabled() but for math section.
	bool isMathEnabled() const;

	//! Like #isGUIEnabled() but for input/output section.
	bool isIOEnabled() const;

	//@} END OF GENERAL SECTION




	///////////////////////////////////////////////////////////////////////////
	//! \name MATH SECTION
	//! Holds the array of solvers. @{

	//! The main mcSolver array.
	static mcSolverArray arrSolvers;

	//! Loads the default mcSolvers in the array.
	static void LoadDefaultSolvers();

	//! Returns the step threshold.
	mcRealValue GetStepThreshold() const			{ return m_fStepThreshold; }

	//! Changes the step threshold.
	void SetStepThreshold(const mcRealValue &n)		{ m_fStepThreshold = n; }

	//@}




	///////////////////////////////////////////////////////////////////////////
	//! \name KEYS SECTION
	//! The virtual keycodes customizable and non-customizable used for
	//! editing. @{

	//! Edits/inserts the exponent.
	//! Pressing this key, if the current element allows it, an exponent is created
	//! and the cursor is moved there. The user should be able to insert any mathdata
	//! as exponent.
	mcKey *m_pEditExpKey;

	//! Edits/inserts the subscript.
	//! The same as mcMathCore::m_pgui_EditExpKey, but this key creates a subscript
	//! and allows the user to edit it. Currently, only mcSymbols allow subscripts.
	mcKey *m_pEditSubscriptKey;

	mcKey *m_pDeleteKey;			//!< Deletes the char at the left of the cursor.
	mcKey *m_pCancelKey; 			//!< Deletes the char at the right of the cursor.
	mcKey *m_pNewLineKey;			//!< Creates a new line (see mcText).

	//! Returns TRUE if the given keypress matches one of the combinations associated
	//! with the given key.
	//! \param ke The keypress event to check.
	//! \param keyentry The mcKey class to compare with the keypress event.
	//bool MatchKeyEntry(wxKeyEvent &ke, mcKey &keyentry);

	//! Returns TRUE if the given keypress matches one of the following
	//! combinations: m_pgui_EditExpKey, m_pgui_EditSubscriptKey, m_pDeleteKey,
	//! m_pCancelKey->
	bool MatchEditKeys(const mcKey &ke);

	//@} END OF KEYS SECTION





	///////////////////////////////////////////////////////////////////////////
	//! \name DEBUG SECTION
	//! General debug variables and functions used by MathCore. @{

	//! The maximum number of elements which shared the same data in
	//! the current session (works as COW performance meter).
	int m_nMaxSharing;

	//! Indentation step for debug output only.
	int m_nIndentationStep;

	//! The function called by mcMathCore::Get()->SyntaxError()
	//! with the description of the error.
	static void (*OnError)(const wxString &);

	//! @} END OF DEBUG SECTION
};


#endif	// MATHCORE_H

