///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   MathHelpers.h
//! \brief  Defines some helper CLASSes used by MathCore.
//! \author Luciano Cattani, Francesco Montorsi
//! \date   3-2-2004
//!
//! Defines following classes:
//!  - mcKeyEntry
//!  - mcKey                        
//!  - mcTextSettings
//!  - mcStyle                      
//!  - mcStyleArray
//!  - mcSymbolProperties
//!  - mcSymbolArray
//! 
///////////////////////////////////////////////////////////////////////



#ifndef MATHHELPERS_H
#define MATHHELPERS_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "MathHelpers.h"
#endif



//  Includes
// ----------

#include <wx/dynarray.h>	// for wxSymbolArray
#include <wx/event.h>		// for wxKeyEvent
#include <wx/font.h>
#include <wx/colour.h>
#include <wx/dc.h>

#include "mc/MathUtils.h"
#include "mc/MathTypes.h"
#include "mc/Value.h"

class mcElement;


//! An abstract item can be anything; it is thus another way to call "void".
typedef void		mcAbstractItem;


//! Defines an array class for abstract classes.
//! This class should be used as parent class for specialized arrays.
class mcAbstractArray
{
	//! The array containing the pointers to the abstract items.
	wxArrayPtrVoid m_arr;

protected:

	//! Deletes the n-th entry of the array.
	virtual void data_Delete(int n) = 0;

	//! Copies the n-th object of the array and returns a pointer to it.
	//! Those arrays which do not need to use #DeepCopy can safely ignore
	//! function; a default implementation which returns NULL is provided.
	virtual mcAbstractItem *data_GetCopyOf(int) const	{ return NULL; }

	//! This is a callback-like function called each time the array is
	//! modified (this does not mean *each* time a non-CONST function is
	//! called: RemoveAt(0, 0) would not trigger this function).
	virtual void data_OnArrayChange() {}

public:

	//! The constructor; does nothing.
	mcAbstractArray() {}

	//! The destructor.
	//! \note: the Clear() function must be called from the derived-class 
	//!        destructor, since its a pure virtual function and thus it
	//!        cannot be called from *this* destructor.
	virtual ~mcAbstractArray() {}

	//! Checks the array integrity.
#ifdef __MCDEBUG__
	virtual void data_Check() const;
#else
	inline void data_Check() const {}
#endif



	//////////////////////////////////////////////////////////////////////////////
	//! \name Insert/Remove functions
	//! Core array-management functions. @{

	//! math_Adds a pointer to the array. The caller should never delete that
	//! pointer (it's owned from the array).
	void data_Add(mcAbstractItem *pointer);

	//! Inserts a pointer immediately before the n-th entry.
	void data_Insert(mcAbstractItem *pointer, int n);

	//! Removes the n-th object of the array (using #data_Delete) and replaces
	//! it with the given pointer.
	//! This function is a sort of #Insert which overwrites destination entry.
	void data_SetItem(mcAbstractItem *pointer, int n);

	//! Removes \c count elements starting from the idx-th entry of the array.
	//! Unlike wxArray does, this function also deletes, using the #data_Delete()
	//! function the memory associated with the pointers removed.
	void data_RemoveAt(int idx, int count = 1);

	//! Removes the given element if the #data_Find() function returns a valid index.
	void data_Remove(const mcAbstractItem *p);

	//! Detaches the i-th pointer and then returns it to the caller.
	//! \note The caller will have to handle its destruction.
	mcAbstractItem *data_Detach(int idx);

	//! Removes all the elements of the array and free the memory occupied.
	void data_Clear();

	//! Copies all the pointers in the given array; uses the #data_GetCopyOf
	//! function to populate this array.
	void data_DeepCopy(const mcAbstractArray &arr);

	//! Returns a non-CONST pointer to the idx-th element of the array.
	mcAbstractItem *data_GetItem(int idx)				{ return m_arr.Item(idx); }

	//! Returns a CONST pointer to the idx-th element of the array.
	const mcAbstractItem *data_GetItem(int idx) const	{ return m_arr.Item(idx); }

	//@}



	/////////////////////////////////////////////////////////////////////////////
	//! \name Query function
	//! Miscellaneous aray-access functions. @{
	
	//! Returns TRUE if the array does not contain pointers.
	bool data_isEmpty() const							{ return m_arr.IsEmpty(); }

	//! Returns the number of objects contained.
	int data_GetCount() const							{ return m_arr.GetCount(); }

	//! Returns the index of the given pointer in this array or -1 if the
	//! given pointer is not stored in the array.
	int data_Find(const mcAbstractItem *pointer) const;

	//@}
};



//! An helper class for the GUI section of MathCore.
//! Like for mcMathHelper, this class is used mainly to group
//! together the GUI helper classes: it does not provide any
//! important function.
class mcGUIHelper
{
public:
	mcGUIHelper() {}
	virtual ~mcGUIHelper() {}
};



//! An helper class for the MATH section of MathCore.
//! Like for mcGUIHelper, this class is used mainly to group
//! together the MATH helper classes: it does not provide any
//! important function.
class mcMathHelper
{
public:
	mcMathHelper() {}
	virtual ~mcMathHelper() {}
};



//! An helper class for the DATA section of MathCore.
//! Like for mcGUIHelper, this class is used mainly to group
//! together the DATA helper classes: it does not provide any
//! important function.
class mcDataHelper
{
public:
	mcDataHelper() {}
	virtual ~mcDataHelper() {}
};




//! Defines the number of possible combinations the user can use to
//! activate a command.
#define mcKEY_MAX_COMBINATIONS						2

//! A simple key entry structure: it contains the information required to
//! recognize a combination of CTRL, META, SHIFT, or ALT + one standard key.
class mcKeyEntry : public mcGUIHelper
{
protected:
	
	//! \name mcKeyEntry flags
	//! Some values which can be combined with the OR operator
	//! in the #m_nFlags variable. @{
#define	SHIFT_KEY		1
#define CTRL_KEY		2
#define META_KEY		4
#define ALT_KEY			8	//!< WARNING: this flag interferes with menu handling.
	//@}
	
	//! The flags which enable/disable the modifiers of this mcKeyEntry.
	int m_nFlags;
	
	//! The keycode for this key entry.
	//! See "wxWidgets keycodes" in the wxWidgets manual for more info.
	int m_nKeyCode;
	
	//! TRUE if this is a 'special' keypress. This information has no
	//! special meaning for mcKeyEntry but it has for the mcElementGUI
	//! functions. See for example, mcFraction::gui_isBeginKey or
	//! mcRadical::gui_isBeginKey->
	bool m_bSpecial;
	
	
protected:
	
	//! Initializes this mcKeyEntry's flags with the given string.
	//! The SHIFT_KEY flag is set if the string contains the "s" character,
	//! the CTRL_KEY if it contains wxT("c") and META_KEY if it contains "m".
	void InitFlags(const wxString &str) {
		wxString s = GetModifiersFrom(str);
		m_nFlags = 0;
		
		m_nFlags |= (s.Contains(wxT("c")) ? CTRL_KEY : 0);
		m_nFlags |= (s.Contains(wxT("m")) ? META_KEY : 0);
		m_nFlags |= (s.Contains(wxT("a")) ? ALT_KEY : 0);
		m_nFlags |= (s.Contains(wxT("s")) ? SHIFT_KEY : 0);
		m_bSpecial = s.Contains(wxT("x"));
	}
	
	//! Initializes with the given string (which should contain a number)
	//! the main keycode of this class.
	void InitKeyCode(const wxString &str) {
		long m;
		wxString num = str.Right(str.Len()-GetModifiersFrom(str).Len());
		if (num.GetChar(0) == wxT('+') || num.GetChar(0) == wxT('-'))
			num.Remove(0, 1);		// remove first character
		num.ToLong(&m);
		m_nKeyCode = (int)m;
	}
	
	//! Returns the modifiers contained in the given string.
	wxString GetModifiersFrom(const wxString &def) {
		
		// return everything before first '+'
		wxString modifiers = def.BeforeFirst(wxT('+'));
		modifiers.MakeLower();
		return modifiers;
	}
	
public:
	
	//! Inits the customizable variables of this key entry.
	//! \param def The default key combination contained in this object; the
	//!            string should encode it using a string like these:
	//!                  wxT("ca+65")        wxT("s+88")        wxT("+66")      "x+F1"
	//!            that mean:
	//!                  CTRL+ALT+A     SHIFT+X       B          F1 (special)
	//!            To build a correct init string, use a '+' to divide
	//!            modifiers (CTRL, ALT, SHIFT, META, SPECIAL = c, a, s, m, x) 
	//!            from the wxWidgets keycode.
	mcKeyEntry() {
		Reset();
	}
	
	mcKeyEntry(const wxString &str, long keycode) {
		Reset();
		InitFlags(str);
		m_nKeyCode = keycode;
	}
	
	mcKeyEntry(const wxString &str) {
		Reset();
		InitFlags(str);
		InitKeyCode(str);
	}
	
	//! Constructs a mcKeyEntry structure from a wxKeyEvent structure.
	mcKeyEntry( const wxKeyEvent &keyEvent )
	{
		Reset();
		if ( keyEvent.m_altDown )
			m_nFlags += ALT_KEY;
		if ( keyEvent.m_shiftDown )
			m_nFlags += SHIFT_KEY;
		if ( keyEvent.m_controlDown )
			m_nFlags += CTRL_KEY;
		if ( keyEvent.m_metaDown )
			m_nFlags += META_KEY;
		m_nKeyCode = keyEvent.m_keyCode;
	}

	virtual ~mcKeyEntry() {}
	
	
	//! Resets the variables of this key entry. After this call,
	//! this key entry won't match any key event.
	void Reset() {
		m_nKeyCode = m_nFlags = 0;
		m_bSpecial = FALSE;
	}
	
	//! Encodes this mcKeyEntry in a wxString which can be used to
	//! init another mcKeyEntry object with the #InitFlags or
	//! #InitKeyCode functions (or in the constructor).
	wxString Encode() const {
		wxString str;
		
		// adds the (eventually) present modifiers
		if (m_nFlags & CTRL_KEY) str += wxT("c");
		if (m_nFlags & SHIFT_KEY) str += wxT("s");
		if (m_nFlags & META_KEY) str += wxT("m");
		if (m_nFlags & ALT_KEY) str += wxT("a");
		
		// save main keycode
		str.Printf(wxT("%s+%d"), (const wxChar *)str, m_nKeyCode);
		
		// finished...
		return str;
	}
	
	//! Returns TRUE if the given mcKeyEntry matches the key
	//! entry defined by this class.
	bool MatchKey(const mcKeyEntry &ke) const {
		bool bTheSame = TRUE;
		
		// if one of this pair of values are different,
		// the two key objects represents different key
		// combinations
		bTheSame &= (m_nFlags == ke.m_nFlags);
		bTheSame &= (m_nKeyCode == ke.m_nKeyCode);
		bTheSame &= (m_bSpecial == ke.m_bSpecial);
		
		return bTheSame;
	}
	
	//! Returns TRUE if the given key event matches the key
	//! entry defined by this class.
	bool MatchKey(const wxKeyEvent &ke) const {
		bool bTheSame = TRUE;
		
		// if one of this pair of values are different,
		// the two key objects represents different key
		// combinations
		bTheSame &= (ke.ControlDown() == ((m_nFlags & CTRL_KEY) != 0));
		bTheSame &= (ke.ShiftDown() == ((m_nFlags & SHIFT_KEY) != 0));
		bTheSame &= (ke.MetaDown() == ((m_nFlags & META_KEY) != 0));
		bTheSame &= (ke.AltDown() == ((m_nFlags & ALT_KEY) != 0));
		bTheSame &= (ke.GetKeyCode() == m_nKeyCode);
		
		return bTheSame;
	}
	
	//! Constructs a wxKeyEvent that matches this mcKeyEntry.
	wxKeyEvent GetEvent() {
		wxKeyEvent res;
		
		// copy attributes ( the '!= 0' at the end is required to avoid warnings)
		res.m_altDown = (m_nFlags & ALT_KEY) != 0;
		res.m_shiftDown = (m_nFlags & SHIFT_KEY) != 0;
		res.m_controlDown = (m_nFlags & CTRL_KEY) != 0;
		res.m_metaDown = (m_nFlags & META_KEY) != 0;
		res.m_keyCode = m_nKeyCode;
		
		// return result
		return res;
	}
	
	//! Returns the keycode stored in this object.
	wxChar GetKeyCode() {
		if ((m_nKeyCode >= 0 && m_nKeyCode <= 255) || m_bSpecial)
			return (wxChar)m_nKeyCode;
		return 0;
	}
	
	//! Returns the modifiers associated with this key entry.
	int GetModifiers() {
		return m_nFlags;
	}
	
	//! Sets this keypress as a "special" keypress (see #m_bSpecial).
	//! This can also be done directly when initializing this object
	//! using as key modifier also the wxT("x") character: "x+C" is 
	//! recognized as a special keypress of "C".
	void SetAsSpecialKey() {
		m_bSpecial = TRUE;
	}
	
	//! Returns TRUE if this is set as a special keypress.
	bool isSpecialKey() {
		return m_bSpecial;
	}
};




//! The structure containing all the possible key combinations for a
//! command. See CUSTOMIZABLE VALUES section for further info.
class mcKey : public mcGUIHelper
{
protected:
	
	//! Sets to NULL all the variables of this class.
	//! To be used only in mcKey constructors to avoid the
	//! repetitions of the initialization loop.
	//! \warning DO NOT USE THIS FUNCTION TO REMOVE THE
	//!          mcKeyEntry CURRENTLY ASSOCIATED: USE
	//!          #Cleanup() INSTEAD !!!
	void Reset() {
		for (int i=0; i < mcKEY_MAX_COMBINATIONS; i++)
			m_pMatch[i] = NULL;
		m_strDescription = wxT("");
	}

public:		// these variables are public so we can avoid a lot of
			// getters/setters which would provide full access anyway
	
	//! The possible key matches for this command.
	mcKeyEntry *m_pMatch[mcKEY_MAX_COMBINATIONS];
	
	//! The description of this key combination.
	wxString m_strDescription;
	
public:		// special routines

	//! Deep copies the given object into this one.
	void DeepCopy( const mcKey& other ) {

		// avoid to copy itself
		if ( &other == this )
			return;

		Cleanup();  // deallocates actual content

		for (int i=0; i < mcKEY_MAX_COMBINATIONS; i++)
			if (other.m_pMatch[i])
				m_pMatch[i] = new mcKeyEntry(*other.m_pMatch[i]);
	}
	
	//! Deletes the contents of this object.
	void Cleanup() {
		for (int i=0; i < mcKEY_MAX_COMBINATIONS; i++)
			mcSAFE_DELETE(m_pMatch[i]);
	}

	
public:
	
	//! Init this key combination using the given strings.
	//! \param def The string to use to init the key combination. This string
	//!            should be a concatenation (through semicolons) of init
	//!            strings of mcKeyEntry. See mcKeyEntry::mcKeyEntry for more info.
	mcKey(const wxString &def)	{
		wxString str = def;
		
		// create the mcKeyEntries and init them
		for (int i=0; i < mcKEY_MAX_COMBINATIONS; i++) {
			m_pMatch[i] = new mcKeyEntry(str.BeforeFirst(wxT(';')));
			str = str.AfterFirst(wxT(';'));
		}
	}
	
	mcKey(const wxChar c, bool bSpecial = FALSE) {
		Reset();
		m_pMatch[0] = new mcKeyEntry(wxEmptyString, c);
		if (bSpecial) m_pMatch[0]->SetAsSpecialKey();
	}
	
	mcKey(const wxKeyEvent &key, bool bSpecial = FALSE) {
		Reset();
		m_pMatch[0] = new mcKeyEntry(key);
		if (bSpecial) m_pMatch[0]->SetAsSpecialKey();
	}
	
	mcKey(const mcKey &other) {
		Reset();
		DeepCopy( other );
	}
	
	mcKey() {
		Reset();
	}
	
	mcKey& operator = ( const mcKey& other )  {
		DeepCopy( other );
		return *this;
	}
	
	//! Deallocates the mcKeyEntries contained in this object.
	virtual ~mcKey() {
		Cleanup();
	}

	
	/*	void Decode(const wxString &str) {
	wxString match;
	
	 // load each key entry enconding string using semicolon as tokenizer
	 for (int c=0; c < mcKEY_MAX_COMBINATIONS; c++) {
	 match = str.Left(str.math_Find(wxT(";")));
	 kMatch[c]->Decode(match);
	 }
	 }
	 
	  wxString Encode() const {
	  wxString str;
	  
	   // divide each key entry enconding string with a semicolon
	   for (int c=0; c < mcKEY_MAX_COMBINATIONS; c++)
	   str += kMatch[c]->Encode() + wxT(";");
	   
		return str;
        }
	*/
	//! Returns TRUE if the given key event matches one of the key entries
	//! stored in this class.
	bool MatchKey(const wxKeyEvent &ke) const {
		bool bTheSame = FALSE;
		for (int c=0; c < mcKEY_MAX_COMBINATIONS; c++)
			if (m_pMatch[c])
				bTheSame |= m_pMatch[c]->MatchKey(ke);
			return bTheSame;
	}
	
	//! Returns TRUE if the given mcKey matches one of the key entries
	//! stored in this object.
	bool MatchKey(const mcKey &ke) const {
		bool bTheSame = FALSE;
		for (int c=0; c < mcKEY_MAX_COMBINATIONS; c++)
			for (int d=0; d < mcKEY_MAX_COMBINATIONS; d++)
				if (m_pMatch[c] && ke.m_pMatch[d])
					bTheSame |= m_pMatch[c]->MatchKey(*ke.m_pMatch[d]);
                return bTheSame;
	}
	/*
	bool operator== (const mcKey &key) const {
	return MatchKey(key);
        }*/
	
	//! Returns TRUE if the n-th combination contained in this object
	//! is set as a special keypress.
	bool isSpecialKey(int n = 0) const {
		return m_pMatch[n]->isSpecialKey();
	}
	
	//! Sets the keypress associated with the n-th combination, associated
	//! to this object, as a special keypress: see mcKeyEntry::m_bSpecial.
	void SetAsSpecialKey(int n = 0) {
		m_pMatch[n]->SetAsSpecialKey();
	}
	
	//! Builds the wxKeyEvent associated with the n-th combination associated
	//! to this object.
	wxKeyEvent GetEvent(int n = 0) const {
		return m_pMatch[n]->GetEvent();
	}
	
	//! Returns the keycode associated with the n-th combination associated
	//! to this object.
	wxChar GetKeyCode(int n = 0) const {
		return m_pMatch[n]->GetKeyCode();
	}
	
	//! Returns the modifiers associated with the n-th combination associated
	//! to this object.
	int GetModifiers(int n = 0) const {
		return m_pMatch[n]->GetModifiers();
	}
};





//! A simple structure containing the classification of a math container.
class mcMathType : public mcMathHelper
{
public:		// public to avoid a lot of getters/setters
	
	mcMathTypeLevel1 m_tMath1;
	mcMathTypeLevel2 m_tMath2;
	mcMathTypeLevel3 m_tMath3;

	//! The container of flags regarding the last level of classification.
	//! See mcMTL4_LOGARITHMIC, mcMTL4_EXPONENTIAL...
	long m_tMath4;

public:

	mcMathType(mcMathTypeLevel1 t1 = mcMTL1_NOT_RECOGNIZED, 
 				mcMathTypeLevel2 t2 = mcMTL2_NOT_RECOGNIZED, 
 				mcMathTypeLevel3 t3 = mcMTL3_NOT_RECOGNIZED, 
     			long t4 = mcMTL4_NOT_RECOGNIZED) :
	  			m_tMath1(t1), m_tMath2(t2), m_tMath3(t3), m_tMath4(t4) {}

	virtual ~mcMathType() {}

	//! Returns a descriptive string of the classification info holded
	//! inside this class.
	wxString io_GetDescString() const;

	//! Sets the current object as the math type resulting from the
	//! multiplication of an object of *this type with an object
	//! of the given type, and then returns the result.
	//! For example, multiplying a non parametric data with a
	//! parametric object, will result in a parametric object
	//! (since the parameter will sure be present in the result).
	mcMathType math_MultiplyBy(const mcMathType &);
	mcMathType math_Add(const mcMathType &);
	mcMathType math_DivideBy(const mcMathType &);
	
	mcMathType math_RaiseTo(const mcMathType &);

	//! Merges the given mcMathType with this one and stores the result.
	//! The merging-algorithm is chosen using the given operator type.
	mcMathType math_ApplyOp(mcElementType optype, const mcMathType &);
};


//! A simple class containing the classification of a mcMathSystem-derived
//! class; this class is like mcMathType;
class mcMathSystemType : public mcMathHelper
{
public:
	mcMathSystemTypeLevel1 m_tMath1;

public:

	mcMathSystemType(mcMathSystemTypeLevel1 t1 = mcMSTL1_NOT_RECOGNIZED) :
	  			m_tMath1(t1) {}
	virtual ~mcMathSystemType() {}

	//! Returns a descriptive string of the classification info holded
	//! inside this class.
	//! \param neq The number of equations/inequalities contained
	wxString io_GetDescString(int neq) const;

	//! Sets the current object as the math type resulting from 
	//! the union of this mathsystem type with the given one.
	//! The result is identic both for a join with the AND and with
	//! the OR logical operator.
	mcMathSystemType math_CombineWith(const mcMathSystemType &);
};


#define mcSYM_PARAMETER			0		//!< The mcSymbol::arrParameters array ID.
#define mcSYM_CONSTANT			1		//!< The mcSymbol::arrConstants array ID.
#define mcSYM_UNKNOWN 			2		//!< The mcSymbol::arrUnknowns array ID.
#define mcSYM_UNREGISTERED		3		//!< The mcSymbol::arrUnregistered array ID.

#define mcSYM_NOTSET			-1		//!< Used to mark as uninitialized mcSymbolProperties::m_nSymbolType.
#define mcSYM_NOTFOUND			-1		//!< Used by some mcSymbolProperties functions.

// this will be defined later
class mcSymbol;
class mcSymbolHelpers;
class mcSymbolArray;
class mcExtRange;

//! A little class which holds informations about a specific symbol.
class mcSymbolProperties : public mcMathHelper
{
	friend class mcSymbolArray;

protected:

	//! The inlined expression name of this symbol.
 	//! This string can be safely empty and in this case, the 
	//! #m_strSymbol string is used.
	//! Use #GetInlinedSym() to get this variable.
	wxString m_strInlinedSymbol;
	
	//! The list of mcSymbols linked to this class.	
	//! The first entry is reserved for the "safe symbol" associated
	//! to this mcSymbolProperties class.
	wxArrayLong m_arrLinkedSymbol;

public:			// to avoid a LOT of getters/setters
				// (all members must be modifiable)

	//! A mcSYM_XXX ID that identifies this symbol as a parameter, a
	//! constant or an unknown...
	int m_nSymbolType;

	//! The encoding of the font which should be used to render
	//! this symbol. If the renderer cannot set this font encoding
	//! then it should use the string returned by #GetInlinedSym().
	wxFontEncoding m_fEncoding;

	//! The name of the symbol represented by this object.
	//! VERY IMPORTANT: this string should be an ANSI string without
	//!                 any special character and it *must* be UNIQUE
	//!                 among all mcSymbolArrays (this task should be
	//!                 performed by mcSymbolArray::math_AddSymbol).
	wxString m_strSymbol;

	//! The subscript associated with this symbol.
	//! This string can be safely empty.
	wxString m_strSubscript;

public:			// math properties

	//! The value of this symbol if it is a constant.
	//! This variable can be used temporary also when this symbol
	//! is an unknown or a parameter... see #m_bEvaluating.
	mcRealValue m_fValue;
	
	//! The domain of this symbol (if a parameter or unknown).
	//! This object also contains info about the set of numbers
	//! considered (Z, N, R, C).
	//! This is a pointer (and not directly a mcExtRange) for
	//! performance reasons (and to save memory for constants).
	mcExtRange *m_pDomain;

	//! If TRUE, then this unknown/parameter (this variable
	//! is useless for constants) is being used with a specific
	//! value (set into #m_fValue) and thus it must be considered
	//! temporary as a constant.
	//! \note This system allows to evaluate a math expression
	//!       containing unknowns/parameters at a fixed value very
	//!       quickly. It would be much slower to move all unknowns/
	//!       parameters of an expression to mcSymbol::arrConstants
	//!       just for a call to math_Evaluate() and then undo the
	//!       change, moving them all back to arrUnknowns/arrParameters.
	bool m_bEvaluating;
	

public:

	//! Inits this class with default values or user-supplied values.
	mcSymbolProperties(int symtype = mcSYM_NOTSET, 
						const wxString &name = wxEmptyString,
						wxFontEncoding encoding = wxFONTENCODING_DEFAULT,
						const mcRealValue &value = 0.0, 
						const wxString &inlinedsym = wxEmptyString, 
						const wxString &subscript = wxEmptyString,
       					mcExtRange *domain = NULL,
						bool evaluating = FALSE);

	virtual ~mcSymbolProperties();


	//! Clones this object returning an identic copy.
	//! Remember that mcSymbol arrays cannot contain duplicates.
	mcSymbolProperties *data_Clone() const {
		mcSymbolProperties *copy = new mcSymbolProperties(m_nSymbolType, m_strSymbol, 
			m_fEncoding, m_fValue, m_strInlinedSymbol, m_strSubscript, m_pDomain,
			m_bEvaluating);

		return copy;
	}

	//! Returns #m_strInlinedSymbol if it's not empty.
	//! Returns #m_strSymbol otherwise.
	wxString io_GetInlinedSym() const 
		{ return m_strInlinedSymbol.IsEmpty() ? m_strSymbol : m_strInlinedSymbol; }
	wxString io_GetInlinedExpr() const
		{ return io_GetInlinedSym(); }


	//! Checks if this symbol holds valid info. If something is wrong,
	//! then an assert fails.
	void data_Check() const {
		bool valid = TRUE;
		
		// check that data is correctly init
		valid &= !m_strSymbol.IsEmpty();
		valid &= (m_nSymbolType != mcSYM_NOTSET);
		if (data_isConstant()) valid &= (m_fValue != *mcRealValue::pNAN);
		
		mcASSERT(valid, wxT("This symbol is damaged !!!"));
 	}

	//! Using mcSymbol::math_FindSymbol searches for the occ-th occurrence
	//! of a *this-clone in the given array. 
 	//! If no duplicates are present, the function returns mcSYM_NOTFOUND.
	//! If a duplicate symbol (for comparison, #isSameAs is used)
	//! is found, then the function returns the index of the duplicate entry.
	int math_FindDuplicate(const mcSymbolArray *tosearch, int occ) const;




	//////////////////////////////////////////////////////////////////////
	//! \name LINKING functions
	//! Link/unlink this class to a mcSymbol @{

	//! Links this object with the given symbol.
	void data_Link(const mcSymbolHelpers *p)		{ m_arrLinkedSymbol.Add((long)p); }
	void data_Unlink(const mcSymbolHelpers *p)		{ m_arrLinkedSymbol.Remove((long)p); }

	int data_GetLinkedSymCount() const				{ return m_arrLinkedSymbol.GetCount(); }
	mcSymbolHelpers *data_GetLinkedSym(int n) const	{ return (mcSymbolHelpers*)m_arrLinkedSymbol.Item(n); }	

	//! Returns a "safe symbol" associated with this object.
	//! This is a reserved mcSymbolHelpers class which is always a valid pointer
	//! and is always linked to this mcSymbolProperties.
	const mcSymbolHelpers *data_GetSafeLinkedSym() const
	{ mcASSERT(data_GetLinkedSym(0) != NULL, wxT("Invalid safe index ?"));
	  return (const mcSymbolHelpers *)data_GetLinkedSym(0); }

	//@}


	//////////////////////////////////////////////////////////////////////
	//! \name CHECK functions
	//! Instead of reading m_nSymbolType, you can use these.... @{

	bool data_isRegistered() const	{ return m_nSymbolType != mcSYM_UNREGISTERED; }
	bool data_isParameter() const	{ return m_nSymbolType == mcSYM_PARAMETER; }
	bool data_isUnknown() const		{ return m_nSymbolType == mcSYM_UNKNOWN; }
	bool data_isConstant() const	{ return m_nSymbolType == mcSYM_CONSTANT; }
	bool data_isUnlinked() const	{ return m_arrLinkedSymbol.IsEmpty(); }
	bool data_isSameAs(const mcSymbolProperties &s) const;

	bool math_isBeingEvaluated() const
		{ return m_bEvaluating; }

 	//@}
};




//! The maximum number of mcSymbolArray instances which can be created
//! at the same time. By now, they are 4:
//! - mcSymbol::arrUnregistered
//! - mcSymbol::arrConstants
//! - mcSymbol::arrParameters
//! - mcSymbol::arrUnknowns
#define mcMAX_SYMBOL_ARRAY_NUM					5

// flags used by mcSymbolArray::math_Find function.
#define mcSYMFIND_MATCH_NAME					1
#define mcSYMFIND_MATCH_ENCODING				2
#define mcSYMFIND_MATCH_INLINED					4
#define mcSYMFIND_MATCH_SUBSCRIPT				8
#define mcSYMFIND_MATCH_VALUE					16

#define mcSYMFIND_MATCH_ALL		(mcSYMFIND_MATCH_NAME |	mcSYMFIND_MATCH_ENCODING | \
								mcSYMFIND_MATCH_INLINED | mcSYMFIND_MATCH_SUBSCRIPT | \
								mcSYMFIND_MATCH_VALUE)


//! An array of mcSymbolProperties with some specific function
//! which works on the array.
class mcSymbolArray : public mcAbstractArray
{
protected:

	//! The ID of this array; all the mcSymbolProperties contained
	//! in #m_arrSym should have their mcSymbolProperties::m_nSymbolType
	//! variables all set to the same value of #m_nSymbolArray.
	int m_nSymbolArray;

	//! The list of the arrays to check when trying to add a new symbol.
	mcSymbolArray *m_pArr[mcMAX_SYMBOL_ARRAY_NUM];

protected:

	void data_Delete(int n)
		{ delete data_GetSymbol(n); }
	mcAbstractItem *data_GetCopyOf(int n) const
		{ return data_GetSymbol(n)->data_Clone(); }

	//! Resets to NULL the #m_pArr array of pointers.
	void data_ResetLinkedArr() {
		for (int i=0; i < mcMAX_SYMBOL_ARRAY_NUM; i++)
			m_pArr[i] = NULL;
	}

public:

	//! The callback function which is called when #math_AddSymbol manages
	//! to add a new symbol to the array.
	void (*m_pAddSymbolCallback)(void);

public:

	mcSymbolArray(int type = mcSYM_NOTSET,
				 void (*fnc)(void) = NULL) { 
		data_ResetLinkedArr(); 
		m_nSymbolArray = type; 
		m_pAddSymbolCallback = fnc;
	}

	virtual ~mcSymbolArray() { data_Clear(); }


	//! math_Adds a new mcSymbolProperties entry to the array, iniziatializing it
	//! with the given data and then returning it.
	//! See the other overloading for more info.
	mcSymbolProperties *data_AddSymbol(bool bRemovePrevious,
									const wxString &name,
									wxFontEncoding enc = wxFONTENCODING_DEFAULT, 
									const mcRealValue &value = 0.0, 
									const wxString &inlinedsym = wxEmptyString, 
									const wxString &subscript = wxEmptyString,
									mcExtRange *domain = NULL);

	//! math_Adds a copy of the given mcSymbolProperties object to the array.
	//! If 'bRemovePrevious' == TRUE then all the symbols with the same name
	//! that are found in the array linked with this one (see #LinkWithArray)
	//! are removed. If 'bRemovePrevious' == FALSE and another symbol with the
	//! same name is already present, then the function does nothing and
	//! returns FALSE; this is because all mcSymbolProperties *must* have
	//! different names.
	//! After the symbol has been added, before returning TRUE, also the 
	//! #m_pAddSymbolCallback function, if not null, is called.
	bool data_AddSymbol(const mcSymbolProperties &newsym, bool bRemovePrevious = TRUE);

	//! Moves the i-th symbol to the given mcSymbolArray, removing it from
	//! *this array and preserving data integrity.
	//! Returns a pointer to the new symbol.
	mcSymbolProperties *data_MoveSymbols(int i, mcSymbolArray *parr);

	//! Returns the index of the entry containing a symbol with the same
	//! given features; -1 if no such symbols are registered in this array.
	//! \param str The symbol name to search
	//! \param bGreek TRUE if the symbol to find is greek
	//! \param value If not mcmcSYM_INVALID_VALUE, then this is
	//!        the value of the symbol to search
	//! \param subscript If not mcmcSYM_INVALID_STRING, this is
	//!        the subscript of the symbol to search
	//! \param skip If not mcmcSYM_INVALID_ENTRY, this is the entry to skip
	int data_FindSymbol(long flags,
					const wxString &name, 
					wxFontEncoding enc = wxFONTENCODING_DEFAULT,
					const mcRealValue &value = 0.0, 
					const wxString &subscript = wxEmptyString, 
					const wxString &inlined = wxEmptyString,
					int occurrence = 0) const;

	int data_FindSymbol(long flags, const mcSymbolProperties &symtofind, int occ = 0) const;


	//! Links *this with the given array.
	//! In this way, when you call the #math_AddSymbol function, it will
	//! scan the given array to avoid duplicates of symbols with the
	//! same name. The function will then remove the symbol in the
	//! given array or it will just return FALSE. Anyway, with this 
	//! system, we can easily prevent array corruption.
	void data_LinkWithArray(mcSymbolArray *p) {
		int entry = 0;
		while (m_pArr[entry] != NULL)
			entry++;

		m_pArr[entry] = p;
	}



	//////////////////////////////////////////////////////////////////////////
	//! \name INLINE getters/setters. @{

	mcSymbolProperties *data_GetSymbol(int n)				{ return (mcSymbolProperties *)data_GetItem(n); }
	const mcSymbolProperties *data_GetSymbol(int n) const	{ return (const mcSymbolProperties *)data_GetItem(n); }

	wxString data_GetSymbolName(int n) const		{ return data_GetSymbol(n)->m_strSymbol;}
	wxString data_GetSubscript(int n) const			{ return data_GetSymbol(n)->m_strSubscript; }
	wxString io_GetInlinedSymbol(int n) const		{ return data_GetSymbol(n)->io_GetInlinedSym(); }
	mcRealValue math_GetValue(int n) const			{ return data_GetSymbol(n)->m_fValue; }
	wxFontEncoding data_GetEncoding(int n) const	{ return data_GetSymbol(n)->m_fEncoding; }
	int data_GetArrayId() const						{ return m_nSymbolArray; }

	//@}
};


#define mcCP_MAX_DEPTH		64

#define mcCP_UNDEFINED		-256
#define mcCP_BEGIN			-257
#define mcCP_END			-258
#define mcCP_BEGINEND		-259


//! example:
//!                        d + 12345|
//!               a + b + c
//!
//! for the expression above, the cursor position array will be [4][2][5]:
//! [4] to locate the c^(d + 12345) term
//! [2] to locate the 12345 element
//! [5] to locate the exact cursor position into the mcNumber
//!
//!
//!         _________________________________________
//! less   |     |     |     |     |     |     |     | most
//! nested |  0  |  1  |  2  |  3  |  4  | ... |  n  | nested
//! cursor |_____|_____|_____|_____|_____|_____|_____| cursor position
//! position                                      
//!                                               ^--- m_nUsed-th element
//! Basic usage:
//! - use #gui_Push
//!
class mcCursorPos : public mcGUIHelper
{
protected:

	int m_nPos[mcCP_MAX_DEPTH];
	int m_nUsed;

public:
	
	// constructor
	mcCursorPos() 
		{ m_nUsed=0; }
	mcCursorPos(int pos) 
		{ m_nUsed=1; m_nPos[0] = pos; }

	virtual ~mcCursorPos() {}


public:		// push, pop

	//! Pushes a new cursor position value into the stack,
	//! setting it as nested one level deeper than the last.
	void gui_Push(int cp)
		{ m_nPos[m_nUsed++] = cp; }

	void gui_Push(const mcCursorPos &c);
	void gui_Pop();

	//! Returns the less nested element.
	int gui_Last() const
		{ return m_nPos[0]; }

	int gui_GetLastAndPop()
		{ int n=gui_Last(); gui_Pop(); return n; }


public:		// checkers

	bool isEnd() const
		{ return gui_Last() == mcCP_END; }

	bool isSpecial() const
		{ return isSpecial(gui_Last()); }

	bool isSpecial(int cp) const
		{ return cp <= mcCP_UNDEFINED && cp >= mcCP_BEGINEND; }

	bool isBegin() const
		{ return gui_Last() == mcCP_BEGIN; }

	bool isBeginEnd() const
		{ return gui_Last() == mcCP_BEGINEND; }

	bool isUndefined() const
		{ return gui_Last() == mcCP_UNDEFINED; }

	bool isInside() const
		{ return !isBegin() && !isEnd() && !isBeginEnd() && !isUndefined(); }
};


//! A MATH+GUI command which is handled by mcMathMng.
class mcMathMngCmd : public mcGUIHelper
{
public:

	mcMathMngCmdID m_nID;
	const wxChar *m_strDescription;

public:
	mcMathMngCmd(mcMathMngCmdID id, const wxChar *desc) : m_nID(id), m_strDescription(desc) {}
	virtual ~mcMathMngCmd() {}
};


#endif    // MATHHELPERS_H
