///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   MathLine.h
//! \brief  Defines the mcMathMng class.
//!
//! See MathLine.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef MATHLINE_H
#define MATHLINE_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "MathLine.h"
#endif

// required includes
#include <wx/dcmemory.h>
#include "mc/MathMng.h"



//! \name Bitmap defines
//! Standard bitmap size at the beginning. @{

#define mcBMP_STDWIDTH		60
#define mcBMP_STDHEIGHT		60	

//! When bitmaps become too small, they are resized adding the following value
//! to the previous size; don't set it too big (it would be a waste
//! of memory for small expression) and don't set it too small (it would force
//! mcMathLineHelpers to continuosly destroy and recreate the bitmap).
#define mcBMP_ADDEDWIDTH		30
#define mcBMP_ADDEDHEIGHT		20

//! Space added to the bitmap on the left and on the right of the element, both
//! when blitting from the bitmap to a DC, and when drawing on the bitmap; this
//! space is useful for selection rectangles, for example. In fact, if no space
//! were added, the border of the selection rectangles would often be cut...
#define mcBMP_SPACELEFT			5
#define mcBMP_SPACERIGHT		5

//@}



//! Adds info about the bitmap used to make double buffer when
//! calling the mcMathLineHelpers::data_GetDebug() function.
#define mcMATHLINE_DBG_BITMAP				256


// global objects
class mcMathLine;
extern mcMathLine mcEmptyLine;



//! A simple abstract interface which defines those functions common
//! to all elements which provide a double buffering system or which
//! have children with double buffering.
class mcDoubleBufferedElement
{
public:
	mcDoubleBufferedElement() {}
	virtual ~mcDoubleBufferedElement() {}



	///////////////////////////////////////////////////////////////////////////
	//! \name Double buffering functions
	//! The functions which handle double buffering system for better
	//! GUI performances. @{
	
	//! Updates the bitmap in the mcMathLineHelpers structure, 
	//! calling the mcMathMng::gui_Draw function.
	//! \param memDC The memory DC to use as temporary DC (performance optimization:
	//!	       instead creating each time a new memory DC, allocate one once and then
	//!	       reuse it on each call to data_UpdateLineBmp).
	//! \param x, y The position where this line is drawn.
	//! \param p The mouse cursor position (in line coord.) or mcDRW_NONACTIVE, 
	//!	       mcDRW_ALLACTIVE flags.
	//! \return The ID of the active element
	virtual int gui_UpdateBmp(int x, int y, long flags, const wxPoint &p) = 0;
	
	//! Draws this line on the given DC using the bitmap.
	//! \param dc The DC where the line will be drawn
	//! \param memDC The memory DC to use to blit the bitmap
	//! \param x The x-coord. where the line will be drawn
	//! \param y The y-coord. where the line will be drawn
	//! \return The ID of the element currently under mouse cursor.
	virtual int gui_DrawBmp(wxDC &dc, int x, int y) const = 0;

	//! Forces the update of the internal bitmap used for double buffering.
	//! All the parameters are the same of the #gui_UpdateBmp function.
	virtual int gui_ForceBmpUpdate(int x, int y, long flags, const wxPoint &p) = 0;

	//! Returns TRUE if with the last call to #gui_UpdateBmp or #gui_ForceBmpUpdate,
	//! a redraw is required to keep the output up to date.
	virtual bool gui_NeedsRefresh() const = 0;

	//! Sets the given colour as the mask which will be used when blitting
	//! the bitmap in the given DC in a #gui_DrawBmp call.
	virtual void gui_SetMask(const wxColour &) = 0;

	//! Sets the background colour to use to clear the bitmap before each
	//! redrawing... the typical value is *wxWHITE_BRUSH.
	virtual void gui_SetBmpBackgroundBrush(const wxBrush &) = 0;
	
	//@}
};




//! This class is basically a wrapper for a mcMathMng and a bitmap object;
//! it adds to mcMathMng a new feature: the ability to store in a bitmap 
//! the most updated output of the math object for fast redraws (and this 
//! is very important because a lot of redraws are necessary: for
//! cursor flashing, selection\\activation state changes and so on....).
//!
//! \todo Besides, the ability to use a bitmap for the output, also gives 
//! mcMathLineHelpers the new feature of "equation wrap": when an equation is too
//! long, mcMathLineHelpers can break it up in various pieces...
class mcMathLineHelpers : public mcMathMngHelpers, public mcDoubleBufferedElement
{
#ifdef mcENABLE_GUI
public:			// customizable variables

	//! The width of the cursor.
	//! This variable is used only to set the width of the rectangle
	//! returned by the gui_GetCursorPos() function.
	static int sgui_nCursorWidth;

protected:	
	
	//! Last rendering of this line (without cursor, activation state, selection state).
	//! Cannot use wxImage because of the high number of conversions between wxImage and
	//! wxBitmap that would be necessary.
	wxBitmap mgui_bmpMath;

	//! The brush used to clear the bitmap before each update.
	wxBrush mgui_brushBmpBg;

	//! TRUE if we are using a mask when blitting the bitmap.
	bool mgui_bUsingMask;
	
	//! The ID of the active element before last redraw.
	int mgui_nLastID;
	
	//! The ID of the current active element.
	int mgui_nCurrentID;
	
	//! The height of the math obj before last change.
	//int mgui_nLastHeight;

#endif


public:
	
	//! Constructor of the structure.
	//! Automatically calls the mcMathMng::Init() function and automatically creates
	//! the bitmap used to hold the last rendering of the math obj.
	mcMathLineHelpers() { data_Init(); }
	virtual ~mcMathLineHelpers() {}


	//! Inits the GUI variables of this class.
	void gui_Init();



#ifdef mcENABLE_DATA
public:
	
	//! Deep copy this object in the given pointer.
	void data_DeepCopy(const mcElementHelpers *mbl);
	
#ifdef __MCDEBUG__

	//! Returns the debug info taken by math object.
	wxString data_Debug(long flags) const;
	
#endif
#endif		// mcENABLE_DATA



#ifdef mcENABLE_GUI
public:

	///////////////////////////////////////////////////////////////////////////
	//! \name Double buffering functions
	//! The functions which handle double buffering system for better
	//! GUI performances. @{
	
	int gui_UpdateBmp(int x, int y, long flags, const wxPoint &p);
	int gui_DrawBmp(wxDC &dc, int x, int y) const;

	void gui_SetMask(const wxColour &);
	
	int gui_ForceBmpUpdate(int x, int y, long flags, const wxPoint &p)
	{ return gui_UpdateBmp(x, y, flags, p); }

	bool gui_NeedsRefresh() const
	{ return gui_isIDChanged(); }

	//! Changes the brush used to clear the bitmap.
	//! No updates are done after the change since this could clear out
	//! the current active element. It's caller's duty to call the
	//! #gui_ForceBmpUpdate function with the right parameters.
	void gui_SetBmpBackgroundBrush(const wxBrush &br)
	{ mgui_brushBmpBg = br; }
	
	//! Expands line's bitmap if it's too small.
	//! Use this function to expand the bitmap size at least to the math
	//! object size; if the given size is smaller than bmp size, nothing is done;
	//! if the bitmap size is updated, previous contents of the bitmap are lost.
	//! This function is automatically called when size changes.
	void gui_ExpandBmp();
	
	//! Returns TRUE if the ID of the active element has changed between the last
	//! two calls to the gui_Draw() function of the math object.
	bool gui_isIDChanged() const { 
		return (mgui_nLastID != mgui_nCurrentID); 
	}

	//! Returns the ID of the currently active element.
	int gui_GetActiveElemID() const {
		return mgui_nCurrentID;
	}
	
	//@}



	///////////////////////////////////////////////////////////////////////////
	//! \name mcElement GUI overloaded functions
	//! mcMathLine needs to modify some mcMathMng GUI behaviour to adjust
	//! coordinates and to keep bitmap's size right for this element. @{

	mcElement gui_GetSelection() const;		
	int gui_GetRelCursorPos(wxDC &dc, wxPoint *) const;
	int gui_MoveCursorUsingPoint(wxDC &, const wxPoint &);
	void gui_OnSelect(wxDC &dc, wxRect &);

	//! This is the most important overload of mcMathLine.
	//! This is the function infact, which calls #gui_ExpandBmp()
	//! which is a very important thing to do to keep the double
	//! buffering system active.
	void gui_DoRecalcSize();
	
	//@}

#endif		// mcENABLE_GUI
};


//! Implements a simple fraction in the program, working as a simple 
//! container class of two mcPolynomial classes (to hold numerator and
//!	denominator). The user can edit the fraction as any other element,
//! switching from numerator to denominator using the up or right keys.
class mcMathLine : public mcMathMng
{
	// We cannot use the standard mcDEFINE_MAIN_CLASS(MathLine, mcMathMng)
	// macro because mcMathLine is based upon a non-abstract parent class
	// and thus we need to create some customized constructors.
	mcDEFINE_CLASS(MathLine);

public:

	// VERY IMPORTANT:
	// since we don't want mcMathMng() to be called (since it would call
	// data_SetRefData(new mcMathMngHelpers()), we must always call another
	// mcMathMng constructor, maybe passing to it a NULL pointer to wrap...
	// Thus NEVER CALL IMPLICITLY THE DEFAULT mcMathMng CONSTRUCTOR
	mcMathLine(const mcElementHelpers *p) : mcMathMng(p) { data_CheckContainer(); }
	mcMathLine(const mcElement &v) : mcMathMng(v.hlp()) { data_CheckContainer(); }
	mcMathLine(const mcMathLine &toref) : mcMathMng(NULL) { data_Ref(toref); }
	mcMathLine() : mcMathMng(NULL) { data_SetRefData(new mcMathLineHelpers); }

	// math constructors
	mcMathLine(const mcPolynomial &left, const mcPolynomial &right) : mcMathMng(NULL) 
		{ data_SetRefData(new mcMathLineHelpers); data_Init(left, right); }
	mcMathLine(const mcMathMng &p) : mcMathMng(NULL) {		
		mcMathLineHelpers *h = new mcMathLineHelpers();
		h->mcMathMngHelpers::data_DeepCopy(p.chlp());
		data_SetRefData(h);
	}

	virtual ~mcMathLine() {}



	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_MATHMNG; }

	mcWRAPPER int gui_GetActiveElemID() const
		{ return hlp()->gui_GetActiveElemID(); }
	mcWRAPPER int gui_isIDChanged() const
		{ return hlp()->gui_isIDChanged(); }
	
	mcWRAPPER int gui_UpdateBmp(int x, int y, long flags, const wxPoint &p)
		{ return hlp()->gui_UpdateBmp(x, y, flags, p); }	
	mcWRAPPER int gui_ForceBmpUpdate(int x, int y, long flags, const wxPoint &p)
		{ return hlp()->gui_ForceBmpUpdate(x, y, flags, p); }	
	mcWRAPPER int gui_DrawBmp(wxDC &dc, int x, int y) const
		{ return hlp()->gui_DrawBmp(dc, x, y); }
};


#endif	// MATHLINE_H


