///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   MathOrSystem.h
//! \brief  Defines the mcMathOrSystemHelpers and mcMathOrSystem classes.
//!
//! See MathOrSystem.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef MATHORSYSTEM_H
#define MATHORSYSTEM_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "MathOrSystem.h"
#endif

// required includes
#include "mc/MathSystem.h"
#include "mc/MathAndSystem.h"


// defined later
class mcMathOrSystem;
class mcMathAndSystem;
class mcMathLine;


//! A set of mcMathLines connected with an OR logical operator.
class mcEXPORT mcMathOrSystemHelpers : public mcMathSystemHelpers
{
	mcDEFINE_REFERENCE_DATA(mcMathOrSystem, mcET_MATHORSYSTEM);

public:
	mcMathOrSystemHelpers() { data_Init(); }
	virtual ~mcMathOrSystemHelpers() { data_Clear(); }



#ifdef mcENABLE_DATA
protected:		// mcAbstractArray functions

	//! Removes the n-th line of the array.
	void data_Delete(int n);

	//! Returns a copy of the n-th line of the array.
	mcAbstractItem *data_GetCopyOf(int n) const;

public:

#ifdef __MCDEBUG__

	wxString data_Debug(long flags) const;

	//! Returns a string with memory debug info.
	wxString data_GetMemoryInfoStr() const;

#endif

	mcMathAndSystem &data_GetSys(int n)
		{ return (mcMathAndSystem &)data_GetChild(n); }
	const mcMathAndSystem &data_GetSys(int n) const
		{ return (const mcMathAndSystem &)data_GetConstChild(n); }
	mcMathAndSystem &data_GetLastSys()
		{ return (mcMathAndSystem &)data_GetLast(); }
	const mcMathAndSystem &data_GetLastSys() const
		{ return (const mcMathAndSystem &)data_GetLast(); }

	void data_AddSys(mcMathAndSystem *toadd)
		{ data_Add(toadd); }

	void data_AddLineToNewAndSystem(mcMathLine *toadd);
	void data_AddLineToLastAndSystem(mcMathLine *toadd);

	//! Returns the total count of the lines contained into all our mcMathAndSystems.
	int data_GetLineCount() const;

	//! Returns a string with the description (ie, classification) of
	//! the current math data.
	wxString data_GetDescString() const;

	//! Sets the filter associated with all the elements of the array.
	//! See mcFilter and mcPolynomial for more info.
	//! \param p The new mcFilter to set (can be NULL)
	//! \param n The line to set the filter for or -1 to set it for
	//!          all the lines of this array
	void data_SetFilter(const mcFilter *p, int n = -1);

#endif		// mcENABLE_DATA



#ifdef mcENABLE_GUI
public:		// customizable variables

	static bool sgui_bHorizOrientation;
	static bool sgui_bDrawOrLabels;
	static int sgui_nSpaceAboveBelow;
	static int sgui_nSpaceLeftRight;
	static int sgui_nSpaceBetween;

protected:

	virtual bool gui_DrawHorizontally() const { return sgui_bHorizOrientation; }
	virtual bool gui_DrawVertically() const { return !sgui_bHorizOrientation; }

	virtual int gui_GetSpaceAboveBelow() const { return sgui_nSpaceAboveBelow; }
	virtual int gui_GetSpaceLeftRight() const { return sgui_nSpaceLeftRight; }
	virtual int gui_GetSpaceBetween() const { return sgui_nSpaceBetween; }

public:

	mcMathAndSystem &gui_GetFocusSys()
		{ return (mcMathAndSystem &)gui_GetFocus(); }
	const mcMathAndSystem &gui_GetFocusSys() const
		{ return (const mcMathAndSystem &)gui_GetFocus(); }

	mcMathLine &gui_GetFocusLine();
	const mcMathLine &gui_GetFocusLine() const;

	void gui_AddEmptyLineToNewAndSystem();
	void gui_AddEmptyLineToLastAndSystem();

	void gui_DrawSystemSymbols(wxDC &dc, int x, int y, long flags) const;
	void gui_RecalcSymbolSize();

	//! Returns a pointer to the portion of mathsystem currently selected;
	//! NULL if there are no parts selected.
	mcElement gui_GetSelection() const;

	//! Returns the ID of the element which was marked as active
	//! in the last bitmap update.
	//! \note This function can be much faster than calling
	//!            gui_GetActiveElem(...).data_GetID()
	//!       because mcMathLine holds a variable containing directly
	//!       the ID, while gui_GetActiveElem() requires a search
	//!       into the math & gui tree.
	virtual int gui_GetActiveElemID() const;

	//! Returns TRUE if the last call to #data_UpdateBmp() set as active
	//! element (the element under mouse cursor) an element which is
	//! different from the element which was active before that call.
	bool gui_isIDChanged() const;

#endif		// mcENABLE_GUI



#ifdef mcENABLE_MATH
public:

	//! Merges the two given mcMathOrSystems and puts the result in *this.
	void math_AndMerge(const mcMathOrSystem &, const mcMathOrSystem &);

	void math_OrMerge(const mcMathOrSystem &, const mcMathOrSystem &);

	mcMathSystemType math_GetMathSystemType() const;

#endif		// mcENABLE_MATH



#ifdef mcENABLE_IO
public:

	wxString io_GetInlinedExprOfLogicalOperator() const
		{ return wxT("OR"); } 

	bool io_ImportToken(const wxString &str, wxString *perr);

#endif		// mcENABLE_IO
};


//! Container of a mcMathOrSystemHelpers-derived class which uses COW.
//! Never use mcMathOrSystemHelpers directly: it must be accessed
//! through this class. For more info see mcElement class details.
class mcMathOrSystem : public mcMathSystem
{
	mcDEFINE_MAIN_CLASS(MathOrSystem, mcMathSystem);

public:

	mcMathOrSystem(const mcMathAndSystem &towrap)
	{ data_SetRefData(new mcMathOrSystemHelpers()); data_AddSys(new mcMathAndSystem(towrap)); } 
	
	mcMathOrSystem(const mcMathLine &l1, const mcMathLine &l2)
	{ data_SetRefData(new mcMathOrSystemHelpers()); 
	  data_AddLineToNewAndSystem(new mcMathLine(l1)); 
	  data_AddLineToNewAndSystem(new mcMathLine(l2)); }

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_MATHORSYSTEM; }


#ifdef mcENABLE_DATA
public:

	mcWRAPPER void data_AddSys(mcMathAndSystem *p)
		{ hlp()->data_AddSys(p); }
	mcWRAPPER void data_AddLineToNewAndSystem(mcMathLine *p)
		{ hlp()->data_AddLineToNewAndSystem(p); }
	mcWRAPPER void data_AddLineToLastAndSystem(mcMathLine *p)
		{ hlp()->data_AddLineToLastAndSystem(p); }
	mcWRAPPER mcMathAndSystem &data_GetSys(int n) 
		{ return hlp()->data_GetSys(n); }
	mcWRAPPER const mcMathAndSystem &data_GetSys(int n) const
		{ return hlp()->data_GetSys(n); }
	mcWRAPPER void data_SetFilter(const mcFilter *p, int n = -1)
		{ hlp()->data_SetFilter(p, n); }
	mcWRAPPER wxString data_GetDescString() const
		{ return hlp()->data_GetDescString(); }

	mcWRAPPER int gui_GetActiveElemID() const
		{ return hlp()->gui_GetActiveElemID(); }

#ifdef __MCDEBUG__

	mcWRAPPER wxString data_GetMemoryInfoStr() const
		{ return hlp()->data_GetMemoryInfoStr(); }

#endif
#endif		// mcENABLE_DATA


#ifdef mcENABLE_GUI
public:

	mcWRAPPER bool gui_isIDChanged() const
		{ return hlp()->gui_isIDChanged(); }

	mcWRAPPER mcMathLine &gui_GetFocusLine()
		{ return hlp()->gui_GetFocusLine(); }
	mcWRAPPER const mcMathLine &gui_GetFocusLine() const
		{ return hlp()->gui_GetFocusLine(); }

	mcWRAPPER void gui_AddEmptyLineToLastAndSystem()
		{ hlp()->gui_AddEmptyLineToLastAndSystem(); }
	mcWRAPPER void gui_AddEmptyLineToNewAndSystem()
		{ hlp()->gui_AddEmptyLineToNewAndSystem(); }

#endif


#ifdef mcENABLE_MATH
public:

	mcWRAPPER void math_AndMerge(const mcMathOrSystem &arr1, const mcMathOrSystem &arr2)
		{ hlp()->math_AndMerge(arr1, arr2); }
	mcWRAPPER void math_OrMerge(const mcMathOrSystem &arr1, const mcMathOrSystem &arr2)
		{ hlp()->math_OrMerge(arr1, arr2); }

#endif		// mcENABLE_MATH
};


#endif		// mcMATHORSYSTEM

