///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   MathSystem.h
//! \brief  Defines the mcMathSystemHelpers and mcMathSystem classes.
//!
//! See MathSystem.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef MATHSYSTEM_H
#define MATHSYSTEM_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "MathSystem.h"
#endif

// required includes
#include "mc/MathLine.h"


//! The maximum number of lines which can be stored in a mcMathOrSystem.
#define mcMATHSYSTEM_MAXLINES				32

// global objects
class mcMathSystem;
extern mcMathSystem mcEmptySystem;


//! The base class of mcMathAndSystem and mcMathOrSystem: provides
//! all the facilities for a class which acts as logical container
//! of mcMathLine objects.
class mcMathSystemHelpers : public mcAbstractArray, public mcElementHelpers,
							public mcDoubleBufferedElement
{
#ifdef mcENABLE_GUI
protected:		// member variables

	//! The currently selected element of the array.
	//! This is the element which receives keyboard input.
	int mgui_nSel;

	//! The index of the element containing the element which was marked
	//! as active before last call to the #gui_UpdateBmp() function.
	//! If this is set to -1, then there were no active elements.
	int mgui_nLastActive;

	//! The index of the element containing the currently active element.
	//! If this is set to -1, then there are no currently active elements.
	int mgui_nCurrentActive;

	//! The size of one of the connector symbols.
	wxSize mgui_szSymbol;


	//! The font which is used to draw the main system symbol.
	wxFont mgui_fLastFont;

	//! The width before last size change.
	int mgui_nLastWidth;

#endif


public:
	mcMathSystemHelpers() {}
	virtual ~mcMathSystemHelpers() {}

	void gui_Init() {
		mgui_nSel = mgui_nLastWidth = 0;
		mgui_nLastActive = -1;
		mgui_nCurrentActive = -1;
		mcElementHelpers::gui_Init();
	}



#ifdef mcENABLE_DATA
public:

#ifdef __MCDEBUG__

	//! Checks the array.
	void data_Check() const;

#else
	
	inline void data_Check() const {}

#endif		// __MCDEBUG__

	//! Performs a deep copy of another mcMathOrSystem object. The two
	//! copies will be totally indipendent.
	void data_DeepCopy(const mcElementHelpers *p);

	int data_GetChildrenCount() const
		{ return mcAbstractArray::data_GetCount(); }
	const mcElement &data_GetConstChild(int n) const
		{ return *((const mcElement *)(data_GetItem(n))); }	

	// these functions are used to link the children functions
	// defined above with the mcAbstractArray functions/terminology.
	mcElement &data_Get(int n)
		{ return data_GetChild(n); }
	const mcElement &data_Get(int n) const
		{ return data_GetConstChild(n); }	
	mcElement &data_GetLast()
		{ return data_Get(data_GetCount()-1); }
	const mcElement &data_GetLast() const
		{ return data_Get(data_GetCount()-1); }
	int data_GetCount() const
		{ return data_GetChildrenCount(); }

#endif		// mcENABLE_DATA



#ifdef mcENABLE_GUI
protected:

	//! Returns TRUE if the elements of the array should be oriented
	//! horizontally.
	virtual bool gui_DrawHorizontally() const = 0;
	virtual bool gui_DrawVertically() const = 0;

	virtual int gui_GetSpaceAboveBelow() const = 0;
	virtual int gui_GetSpaceLeftRight() const = 0;
	virtual int gui_GetSpaceBetween() const = 0;	

public:	

	/////////////////////////////////////////////////////////////////////////////////
	//! \name GUI getters
	//! Specialized GET functions which returns elements casted to
	//! mcDoubleBufferedElements. Thanks to this cast, many functions
	//! related to double buffering can be implemented directly in mcMathSystem. @{

	mcElement &gui_GetFocus()
		{ return data_Get(mgui_nSel); }
	const mcElement &gui_GetFocus() const
		{ return data_Get(mgui_nSel); }

	mcDoubleBufferedElement *gui_Get(int n)
		{ return dynamic_cast<mcDoubleBufferedElement*>(data_Get(n).hlp()); }
	const mcDoubleBufferedElement *gui_Get(int n) const
		{ return dynamic_cast<const mcDoubleBufferedElement*>(data_Get(n).chlp()); }

	//@}



	///////////////////////////////////////////////////////////////////////////
	//! \name Double buffering functions
	//! The functions which handle double buffering system for better
	//! GUI performances. @{
	
	int gui_UpdateBmp(int x, int y, long flags, const wxPoint &p);
	int gui_ForceBmpUpdate(int x, int y, long flags, const wxPoint &p);
	int gui_DrawBmp(wxDC &dc, int x, int y) const;

	void gui_SetMask(const wxColour &col);
	void gui_SetBmpBackgroundBrush(const wxBrush &);

	bool gui_NeedsRefresh() const;
	
	//@}



	/////////////////////////////////////////////////////////////////////////////////
	//! \name GUI ABSTRACT functions
	//! Functions that change the graphical aspects of the math system. @{

	//! Draws the connector symbols of this system *without* drawing
	//! the elements contained.
	virtual void gui_DrawSystemSymbols(wxDC &dc, int x, int y, long flags) const = 0;

	//! Recalculates the size of one of the connectors (they must have all
	//! the same size for mcMathSystem algorithms) and puts the result into
	//! the #mgui_szSymbol variable.
	virtual void gui_RecalcSymbolSize() = 0;

	//@}



	/////////////////////////////////////////////////////////////////////////////////
	//! \name mcMathSystem-specific GUI functions
	//! Functions implemented over mcElement GUI functions. @{

	//! Slow function used to retrieve the x relative coord. of the n-th line.
	//! \param n The number of the line for which the x coord. is returned
	//! \return The x coord (in pixel) of n-th line, relative to the window
	virtual int gui_GetXOf(int n) const;

	//! Slow function used to retrieve the y relative coord. of the n-th line
	//! \param n The number of the line for which the y coord. is returned
	//! \return The y coord (in pixel) of n-th line, relative to the window
	virtual int gui_GetYOf(int n) const;

	//! Returns the X relative coordinate of the n-th connector symbol.
	virtual int gui_GetXOfSymbol(int n) const;

	//! Returns the Y relative coordinate of the n-th connector symbol.
	virtual int gui_GetYOfSymbol(int n) const;

	//! Returns the bounding box of the n-th line of the system.
	wxRect gui_GetRectOf(int n) const;

	//! Returns the index of the element containing the given point
	//! (which must be in relative coordinates).
	int gui_GetElemContaining(const wxPoint &pt) const;

	//! Returns the size of a connector.
	virtual wxSize gui_GetSymbolSize() const
		{ return mgui_szSymbol; }

	//! Returns the top-left position of the cursor.
	//! \param dc The device context where the math system was drawn last time.
	//! \param x The x-coord. where the system has been drawn
	//! \param y The y-coord where the system has been drawn
	//! \param height The pointer where the cursor's height will be stored.
	wxPoint gui_GetRelCursorPos(wxDC &dc, int x, int y, int *height) const;

	//! Checks if the selected line is valid or not; in the last case,
	//! selects the last valid line.
	void gui_CheckSelLineIdx();

	//! Returns a bitmap containing the image of the current system.
	//! The width and the height of the bitmap are set to the bounding
	//! box's width and height; the color depth is set to the screen one.
	wxBitmap gui_GetBmp() const;

	//! Returns the index of the currently selected line.
	int gui_GetFocusIdx() const 
		{ return mgui_nSel;	}

	//! Selects the n-th math element of this math system: sets it as the
	//! destination of the user input & as default input for math operations.
	void gui_SetFocusIdx(int n)
		{ mgui_nSel = n; }

	//! Returns TRUE if this system should be drawn with its special
	//! symbol (that represents the logical operation AND or OR) or
	//! it can be drawn as a simple mcMathMng object.
	bool gui_isSymbolToDraw() const 
	{ return data_GetCount() >= 2; }

	//@}




	/////////////////////////////////////////////////////////////////////////////////
	//! \name mcElement GUI functions
	//! Implements mcElementHelpers abstract GUI functions. @{
	
	// these does not apply to a mcMathSystem
	bool gui_isBeginKey(const mcKey &key) const 
		{ return FALSE; }
	bool gui_isEndKey(const mcKey &key) const
		{ return FALSE; }

	void gui_DoRecalcSize();
	void gui_SetCursorPos(const mcCursorPos &code);
	void gui_OnSelect(wxDC &dc, wxRect &rc);
	int gui_GetRelCursorPos(wxDC &dc, wxPoint *pt) const;
	int gui_MoveCursorUsingPoint(wxDC &dc, const wxPoint &pos);
	int gui_Draw(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const;	
	void gui_GetCursorPos(mcCursorPos &) const;
	mcInputRes gui_Input(const mcKey &ev, mcElement *newelem);
	mcInsertRes gui_Insert(const mcElement &, mcElement *);
	mcMoveCursorRes gui_MoveCursor(mcMoveCursorFlag flag, long modifiers);
	
/*
	mcElement &gui_GetActiveElem(int x, int y, const wxPoint &pt);
	mcElement data_GetElemFromID(int id);
*/

	//! Deletes the selected portion of the line currently selected.
	//void gui_DeleteSelection();

	//! Sets to zero the exponent of all our children.
	void gui_UpdateExpDepth();

	//@}



	///////////////////////////////////////////////////////////////////////////
	//! \name GUI+MATH functions
	//! Math functions which work on selected parts of the system. @{

	/*void gui_MultiplyBySel() {
		gui_GetSelLine()->gui_MultiplyBySel();
	}

	void gui_MoveSel() {
		gui_GetSelLine()->gui_MoveSel();
	}

	wxArrayString gui_GetPossibleOpOnSel(mcMathMngCmd *pID) {
		return gui_GetSelLine()->gui_GetPossibleOpOnSel(pID);
	}

	wxArrayString math_GetPossibleOp(mcMathMngCmd *pID, int n = 0) {
		return data_GetLine(n)->gui_GetPossibleOp(pID);
	}

	void gui_ExecCmdOnSel(mcMathMngCmd n) {
		gui_GetSelLine()->gui_ExecCmdOnSel(n);
	}*/

	//! Returns as inlined expression the current selection.
	wxString gui_GetSelInlinedExpr() const;

	//@}

#endif		// mcENABLE_GUI



#ifdef mcENABLE_MATH
protected:
		
	//! Calls math_Simplify() or math_Expand() on all the lines of this system
	//! and then returns the global result for the entire array.
	mcExpSimRes math_ExpSimAllLines(bool simp = TRUE);

public:

	///////////////////////////////////////////////////////////////////////////
	//! \name MATH functions
	//! mcMathOrSystem-specific math functions. @{

	//! Multiplies all the mcMathLine contained in this object by 
	//! the given polynomial.
	mcBasicOpRes math_MultiplyBy(const mcElement &, mcElement *);

	//! Divides all the mcMathLine contained in this object by
	//! the given polynomial.
	mcBasicOpRes math_DivideBy(const mcElement &, mcElement *);
	mcBasicOpRes math_Add(const mcElement &, mcElement *, bool);
	mcBasicOpRes math_Subtract(const mcElement &, mcElement *);

	mcBasicOpRes math_RaiseTo(const mcPolynomial &);

	mcExpSimRes math_Simplify(long flags, mcElement *);
	mcExpSimRes math_Expand(long flags, mcElement *);

	//! Evaluates the n-th line and returns the result.
	//! If the given index is -1, then the selected line is used.
	mcRealValue math_Evaluate(int n = -1) const;

	bool math_isMaxSimplified() const;
	//bool math_isConstant() const;

/*	mcElement math_Find(int n, const mcElement &tofind, long flags,
 						int *pos = NULL, int *entry = NULL) const;
	int math_GetCountOf(const mcElement &tocount, long flags) const;
	int math_Replace(const mcElement &tofind, int n, long flags,
		const mcElement &rep, bool bAttachChildren);*/

	mcIntegerValue math_GetMathSystemDegreeFor(const mcSymbolProperties *unk) const;
	
	mcRealValue math_GetLenght() const;
	//mcRealValue math_GetTotalLenght() const;

	//bool math_isComplete() const;

	//! Returns the type of this math system as a mcMathOrSystemType enumeration value.
	virtual mcMathSystemType math_GetMathSystemType() const = 0;

	//! Returns the mcMathType resulting from the *addition* of the mcMathType
	//! returned by each mcMathMng stored in this array.
	mcMathType math_GetMathType() const;

	bool math_Compare(const mcElement &, long flags) const;	


	// these does not have sense for a mcMathSystem
	mcRealValue math_Evaluate() const		{ return 0.0; }
	int math_GetOrderPos() const			{ return 0; }

	bool math_CanBeAddedWith(const mcElement &p) const	{ return FALSE; }
	bool math_CanBeMultWith(const mcElement &p) const	{ return FALSE; }
	bool math_CanBeDivBy(const mcElement &p) const		{ return FALSE; }

	mcMonomial math_GetLCM(const mcElement &) const		{ return mcEmptyMonomial; }
	mcMonomial math_GetGCD(const mcElement &) const		{ return mcEmptyMonomial; }

	//@}

#endif		// mcENABLE_MATH



#ifdef mcENABLE_IO
protected:

	//! Filter out the characters not allowed by MathCore inlined expression
	//! parsers and returns the number of characters filtered out.
	int io_FilterInlinedExpr(wxString &);

public:

	///////////////////////////////////////////////////////////////////////////
	//! \name IO functions
	//! mcMathOrSystem input/output functions. @{

	//! Imports a mcMathOrSystem from the given string.
	//! The words OR, AND can be used to separe different lines:
	//!     "4ax+b^2>0 AND b=1/2x"	
	bool io_ImportInlinedExpr(const wxString &str, int *count, wxString &perr);

	//! Works exactly like the standard mcElement #io_ImportInlinedExpr but
	//! this function does not take the useless "count" argument since a
	//! mcMathSystem always import all the given string.
	bool io_ImportInlinedExpr(const wxString &str)
	{ wxString tmp; return io_ImportInlinedExpr(str, NULL, tmp); }

	//! Returns the inlined expression for this system.
	wxString io_GetInlinedExpr() const;

	//! Returns the MathML tree which encodes this current mathsystem.
	wxXml2Node io_GetMathML(bool bGetPresentation) const;

	//! Imports a mcMathOrSystem from the given XML tree.
	bool io_ImportPresentationMathML(wxXml2Node pDoc, wxString &pErr);

	virtual wxString io_GetInlinedExprOfLogicalOperator() const = 0;
	virtual bool io_ImportToken(const wxString &str, wxString *perr) = 0;


	// USELESS ??
	
	//! Returns the given logical operator encoded as a string.
	wxString io_GetInlinedExprOfLogicalOperator(mcLogicOperator op, 
									bool shortversion = FALSE) const;

	//! Returns a string separed by
	wxString io_GetInlinedExprOfAllLogicalOp() const;


	// these does not have sense for a mcMathSystem
	bool io_isBeginTag(const wxXml2Node &tag) const		{ return FALSE; }
	bool io_isBeginChar(const wxString &str) const 		{ return FALSE;	}

	//@}

#endif		// mcENABLE_IO
};


//! Container of a mcMathSystemHelpers-derived class which uses COW.
//! Never use mcMathSystemHelpers directly: it must be accessed
//! through this class. For more info see mcElement class details.
class mcMathSystem : public mcElement
{
	mcDEFINE_ABSTRACT_CLASS(MathSystem, mcElement);

public:

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_MATHORSYSTEM || t == mcET_MATHANDSYSTEM; }


#ifdef mcENABLE_DATA
public:

	mcWRAPPER mcElement &data_Get(int n)
		{ return hlp()->data_Get(n); }
	mcWRAPPER const mcElement &data_Get(int n) const
		{ return hlp()->data_Get(n); }
	mcWRAPPER mcElement &data_GetLast()
		{ return hlp()->data_GetLast(); }
	mcWRAPPER const mcElement &data_GetLast() const
		{ return hlp()->data_GetLast(); }
	
	mcWRAPPER int data_GetCount() const
		{ return hlp()->data_GetCount(); }	
	mcWRAPPER void data_Insert(mcAbstractItem *pointer, int n)
		{ hlp()->data_Insert(pointer, n); }
	mcWRAPPER void data_SetItem(mcAbstractItem *pointer, int n)
		{ hlp()->data_SetItem(pointer, n); }
	mcWRAPPER void data_RemoveAt(int idx, int count = 1)
		{ hlp()->data_RemoveAt(idx, count); }
	mcWRAPPER void data_Remove(const mcAbstractItem *p)
		{ hlp()->data_Remove(p); }
	mcWRAPPER void data_Clear()
		{ hlp()->data_Clear(); }
	mcWRAPPER bool data_isEmpty() const	
		{ return hlp()->data_isEmpty(); }	
	mcWRAPPER int data_Find(const mcAbstractItem *pointer) const
		{ return hlp()->data_Find(pointer); }	
	mcWRAPPER mcAbstractItem *data_Detach(int idx)
		{ return hlp()->data_Detach(idx); }	

#endif		// mcENABLE_DATA


#ifdef mcENABLE_GUI
public:

	mcWRAPPER int gui_UpdateBmp(int x, int y, long flags, const wxPoint &p)
		{ return hlp()->gui_UpdateBmp(x, y, flags, p); }	
	mcWRAPPER int gui_ForceBmpUpdate(int x, int y, long flags, const wxPoint &p)
		{ return hlp()->gui_ForceBmpUpdate(x, y, flags, p); }	
	mcWRAPPER int gui_DrawBmp(wxDC &dc, int x, int y) const
		{ return hlp()->gui_DrawBmp(dc, x, y); }

	mcWRAPPER bool gui_NeedsRefresh() const
		{ return hlp()->gui_NeedsRefresh(); }
	mcWRAPPER void gui_SetMask(const wxColour &c)
		{ hlp()->gui_SetMask(c); }
	mcWRAPPER void gui_SetBmpBackgroundBrush(const wxBrush &c)
		{ hlp()->gui_SetBmpBackgroundBrush(c); }

	mcWRAPPER void gui_SetFocusIdx(int n)
		{ hlp()->gui_SetFocusIdx(n); }
	mcWRAPPER int gui_GetFocusIdx() const
		{ return hlp()->gui_GetFocusIdx(); }

	mcWRAPPER wxPoint gui_GetRelCursorPos(wxDC &dc, int x, int y, int *height) const
		{ return hlp()->gui_GetRelCursorPos(dc, x, y, height); }
	
	mcWRAPPER wxBitmap gui_GetBmp() const
		{ return hlp()->gui_GetBmp(); }
	
	mcWRAPPER wxString gui_GetSelInlinedExpr() const
		{ return hlp()->gui_GetSelInlinedExpr(); }

#endif		// mcENABLE_GUI


#ifdef mcENABLE_IO
public:

	mcWRAPPER bool io_ImportInlinedExpr(const wxString &str)
		{ return hlp()->io_ImportInlinedExpr(str); }
	mcWRAPPER bool io_ImportInlinedExpr(const wxString &str, int *count, wxString &err)
		{ return hlp()->io_ImportInlinedExpr(str, count, err); }

#endif		// mcENABLE_IO


#ifdef mcENABLE_MATH
public:

	mcWRAPPER mcMathSystemType math_GetMathSystemType() const
		{ return hlp()->math_GetMathSystemType(); }

#endif		// mcENABLE_MATH
};


#endif		// MATHSYSTEM_H

