///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   MathTypes.h
//! \brief  Defines various ENUMs used by MathCore.
//!
//! See MathTypes.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef MATHTYPES_H
#define MATHTYPES_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "MathTypes.h"
#endif



// FLAGS\RETURN VALUES SECTION
// ----------------------------
// Defines the values returned by some MathCore and mcElement functions;
// use these defines to write more readable code.


//! Tells to the mcElementHelpers::gui_Draw() function to draw everything
//! as usual: no special requirements.
#define mcDRW_NOFLAGS					1

//! Tells to the mcElementHelpers::gui_Draw() function to draw everything
//! as *not* highlighted.
//! This flag cannot be combined with mcDRW_ALLACTIVE or mcDRW_USEPOINT
#define mcDRW_NONACTIVE					1

//! Tells to the mcElementHelpers::gui_Draw() function to draw everything
//! as highlighted.
//! This flag cannot be combined with mcDRW_NONACTIVE or mcDRW_USEPOINT
#define mcDRW_ALLACTIVE					2

//! Tells to the mcElementHelpers::gui_Draw() function to use the given
//! point to highlight the contents.
//! This flag cannot be combined with mcDRW_NONACTIVE or mcDRW_ALLACTIVE
#define mcDRW_USEPOINT					4

//! This can be used to told to the children of an element that they
//! are allowed to draw the XXXXXXX
#define mcDRW_ALLOW_TOTAL_SELECTION		8



//! Value which can be returned by a gui_Draw() function instead of the ID for
//! the active element if the cursor is not over an element.
#define mcDRW_NOACTIVEELEM				(-1)

//! Value which can be returned by a gui_Draw() function instead of the ID for
//! the active element if the cursor is placed over the selection.
#define mcDRW_ONSELECTION 				(-2)



// The math_Expand/math_Simplify possible flags:
#define mcEXPSIM_NOFLAGS				0

#define mcEXPSIM_KEEP_FACTORIZATION		1




//  Enumerations
// --------------

//! REALLY IMPORTANT: if you add a new type of class, update
//! the following define, add a new define after mcET_RADICAL,
//! modify mcMathCore::Newhlp() function, (eventually) modify
//! the mcMathCore::Setup() function.
#define mcNUM_ELEMENT_TYPES				15

//! The mcElement class identifiers.
enum mcElementType {
	
	// Class identifiers
	mcET_INVALID = -1,		//!< When the mcElement::m_nType variable must
							//! still be set. This value can also be used
							//! as return value by some functions, like
							//! mcMathCore::isCharBeginChar...
		
	// Container classes
	mcET_PARENTHESIS,		//!< The mcParenthesis class ID.
	mcET_BRACKET,			//!< The mcExpContainer class ID.
	mcET_RADICAL,			//!< The mcRadical class ID.
	mcET_FRACTION,			//!< The mcFraction class ID.
	mcET_FUNCTION,			//!< The mcFunction class ID.
		
	// Basic classes
	mcET_NUMBER,			//!< Basic class mcNumber.
	mcET_SYMBOL,			//!< The mcSymbol class ID.
	
	// Operator classes
	mcET_ADDOP,				//!< The mcAddSubOp class ID (when working as +).
	mcET_SUBOP,				//!< The mcAddSubOp class ID (when working as -).
	mcET_MULTOP,			//!< The mcMultDivOp class ID (when working as *).
	mcET_DIVOP,				//!< The mcMultDivOp class ID (when working as /).
	
	// Special classes: classes that cannot be created (directly) by the user.
	mcET_TEXT,				//!< The mcText class ID.
	mcET_EMPTYBOX,			//!< The mcEmptyBox class ID.
	mcET_POLYNOMIAL,		//!< The mcPolynomial class ID.
	mcET_MONOMIAL,			//!< The mcMonomial class ID.

	mcET_MATHMNG,
	mcET_MATHANDSYSTEM,
	mcET_MATHORSYSTEM
};


//! First raw implementation of a list of math data types.
enum mcMathTypeLevel1 {
	mcMTL1_NOT_RECOGNIZED = -1,

		//! If one or more unknowns are contained in denominators or (it's the same),
		//! are contained in expressions elevated to an exponent < 0.
		mcMTL1_RATIONAL,
		
		//! If all the unknowns have positive exponents.
		//! Something like: ax^n + bx^(n-1) + .... + nx^0 = 0		
		mcMTL1_POLYNOMIAL
};

enum mcMathTypeLevel2 {
	mcMTL2_NOT_RECOGNIZED = -1,

		//! If one or more unknowns are contained in one or more types of 
		//! transcendent functions: trigonometric, logaritmic, exponential.
		mcMTL2_TRANSCENDENT,

		//! If all the unknowns are contained in algebraic functions only:
		//! polynomials, rational polynomials....
		mcMTL2_ALGEBRAIC
};

enum mcMathTypeLevel3 {
	mcMTL3_NOT_RECOGNIZED = -1,

		//! If one or more mcSymbol registered as parameters is present.
		mcMTL3_PARAMETRIC,

		//! If there are mcSymbols registered as unknowns but there are
		//! no symbols registered as parameters.
		mcMTL3_UNKNOWN,

		//! If there are no mcSymbols registered as unknowns/parameters.
		//! If the data is classified as mcMTL1_CONSTANT it does not
		//! mean that the mcElement::math_isConstant() function will return TRUE:
		//! mcMathType-classification system is *always* referred to
		//! the unknowns contained in the data, while the mcElement::math_isConstant
		//! function is referred also to the mcSymbols registered as parameters.
		//! 
		//! Note that writing:
		//! \code
		//!     bool b = mypol.math_GetMathType() == mcMTL1_CONSTANT;
		//! \endcode
		//! or:
		//! \code
		//!     bool b = !mypol.math_ContainsUnknowns();
		//! \endcode
		//! should be exactly the same.
		mcMTL3_CONSTANT
};

//! If the data has been set as mcMTL2_ALGEBRAIC, then it should
//! be set also as mcMTL4_NOT_RECOGNIZED.
/*enum mcMathTypeLevel4 {
	mcMTL4_NOT_RECOGNIZED = -1,

		//! If one or more unknowns are contained in one of:
		//! SIN, COS, TAN, COTAN, SEC, COSEC or one of their inverse functions.
		mcMTL4_TRIGONOMETRIC,

		//! If one or more unknowns are contained inside an exponent.
		mcMTL4_EXPONENTIAL,

		//! If one or more unknowns are contained inside a LOG function.
		mcMTL4_LOGARITMIC
};*/

#define mcMTL4_NOT_RECOGNIZED		0

//! If one or more unknowns are contained in one of:
//! SIN, COS, TAN, COTAN, SEC, COSEC or one of their inverse functions.
#define mcMTL4_TRIGONOMETRIC		1

//! If one or more unknowns are contained inside an exponent.
#define mcMTL4_EXPONENTIAL			2

//! If one or more unknowns are contained inside a LOG function.
#define mcMTL4_LOGARITHMIC			4


//! The possible types of a mcMathOrSystem.
//! See mcMathOrSystem::math_GetMathSystemType for more info.
enum mcMathSystemTypeLevel1 {
	mcMSTL1_NOT_RECOGNIZED = -1,
		
		//! The mcMathOrSystem contains only line containing an expression.
		mcMSTL1_EXPRESSION,
		
		//! The mcMathOrSystem contains only equations.
		mcMSTL1_EQUATIONS,
		
		//! The mcMathOrSystem contains only inequalities.
		mcMSTL1_INEQUALITIES,
		
		//! The mcMathOrSystem contains both equations and inequalities.
		mcMSTL1_MIXED
};

//! Values which can be used as flag argument when calling
//! mcElement::gui_MoveCursor() function.
enum mcMoveCursorFlag {
	mcMCF_RIGHT,		//!< Cursor must be moved right.
	mcMCF_LEFT,			//!< Cursor must be moved left.
	mcMCF_UP,			//!< Cursor must be moved up.
	mcMCF_DOWN			//!< Cursor must be moved down.
};

//! Values returned by gui_MoveCursor() and gui_MoveCursorUsingPoint() functions.
enum mcMoveCursorRes {
	mcMCR_OKAY,					//!< Everything was okay.
	mcMCR_SETFOCUS_PREVIOUS,	//!< Cursor must be moved on previous element.
	mcMCR_SETFOCUS_NEXT,		//!< Cursor must be moved on next element.
	mcMCR_SETFOCUS_BELOW,		//!< Cursor must be moved on element lying below.
	mcMCR_SETFOCUS_ABOVE,		//!< Cursor must be moved on element lying above.
	mcMCR_CANNOT_SETFOCUS		//!< There is no elements under the given point
								//! (eventually returned by gui_MoveCursorUsingPoint).
};

//! Values which can be returned by gui_Input() functions.
enum mcInputRes {
	mcIR_OKAY,				//!< Everything was okay.
	mcIR_DELETE_PREVIOUS,	//!< The previous element must be deleted.
	mcIR_DELETE_NEXT,		//!< The next element must be deleted.

	mcIR_DELETE_THIS,		//!< This element must be deleted.
	mcIR_REPLACE_THIS,		//!< This element must be replaced.
	mcIR_DIRECT_DELETE,		//!< The deleted element was an empty box.

	mcIR_DISTRIBUTE
};

//! The values which can be returned by the gui_Insert() function.
enum mcInsertRes {
	mcINSR_OKAY,				//!< Everything was okay.
	mcINSR_REPLACE_THIS			//!< This element must be replaced.
};

// Values returned by math_Simplify() or math_Expand() functions.
enum mcExpSimRes {
	mcESR_INVALID_DATA,		//!< The simplification/expansion could not take place.

	mcESR_DONE,				//!< No further simplification can be applied...
	mcESR_NOTFINISHED,		//!< The element can be simplified even more through math_Simplify(long flags).

	mcESR_REPLACE_THIS,		//!< This element must be replaced by the parent.
	mcESR_DELETE_THIS,		//!< This element can be removed from parent (it's empty).

	//! This element contains something (typically a mcPolynomial or a mcMonomial)
	//! which needs to be distributed by the parent with mathematical consistency.
	//! Eg:
	//!    Consider "ax(bx+c)"; if the mcBracket returns mcESR_DISTRIBUTE 
	//!    (with the math_Simplify special pointer pointing to its contents) to 
	//!    its mcMonomial parent, the mcMonomial must return mcESR_DISTRIBUTE too,
	//!    to its mcPolynomial parent (it cannot distribute bracket's contents since
	//!    the bracket's contents contains a mcAddOp) which will finally set itself
	//!    to something like "axbx+axc".
	mcESR_DISTRIBUTE,

	//! This element has changed the signs of a mcPolynomial it contains somewhere
	//! (maybe inside one of its children): the parent must change its sign too,
	//! in order to keep the result mathematically equivalent.
	mcESR_CHANGE_SIGN

};

//! Values returned by mcElementMath basic functions.
enum mcBasicOpRes {
	mcBOR_INVALID,			//!< The operation could not be performed.

	mcBOR_REMOVE_OPERAND,	//!< The operation left the second operand empty: 
							//!< it can be removed.

	mcBOR_REPLACE_OPERAND,	//!< The operation could not make second operand empty: 
							//!< it must be replaced with the element stored in the 
							//!< second argument given to the function call.
	mcBOR_REPLACE_OPERAND_AND_SET_MULTOP,
	mcBOR_REPLACE_OPERAND_AND_SET_DIVOP,
	mcBOR_REPLACE_OPERAND_AND_SET_ADDOP,
	mcBOR_REPLACE_OPERAND_AND_SET_SUBOP,

	mcBOR_REPLACE_BOTH,		//!< The operation required the creation of a completely
							//!< new element: both first and second operands must
							//!< be removed and replaced.
};


//! Possible data types.
//! Values that m_nDataType can assume.
enum mcMathMngType {

	mcMMT_NOT_SET = -1,			//!< Something is wrong.
	mcMMT_EXPRESSION,			//!< example:	56t^2+22x-12t.
	mcMMT_EQUATION,				//!< example:	34x^2+5x = 55.
	mcMMT_INEQUALITY_NE,		//!< example:	34xy+11x != 55.
	mcMMT_INEQUALITY_G,			//!< example:	34xy+11x > 55.
	mcMMT_INEQUALITY_GE,		//!< example:	34xy+11x >= 55.
	mcMMT_INEQUALITY_L,			//!< example:	34xy+11x < 55.
	mcMMT_INEQUALITY_LE			//!< example:	34xy+11x <= 55.
};

//! The possible relations accepted by a mcMathMng class.
//! To create them through user input routines, send a mcKey marked
//! as "special" keypress to the mcMathMng::gui_Input() function,
//! containing the mcMathMngRelationalOp keycode you want:
//!
//! \code
//! mcKey mykey(mcMMRO_GREATER, // the keycode of the mcMMRO_GREATER coincides
//!								// with mcMMRO_GREATER enumeration value
//!             TRUE);			// set the key as special...
//! mymathmng.gui_Input(mykey);
//! \endcode
//!
//! \note The keycodes are stored as Unicode characters.
//!       When building in ANSI mode, the special character codes are
//!       set to -1; don't use them in ANSI mode !!!
enum mcMathMngRelationalOp {

	mcMMRO_GREATER = 0x003E,				//!< The ">" relation.
	mcMMRO_GREATER_OR_EQUAL = 0x2265,		//!< The ">=" relation.

	mcMMRO_LESS = 0x003C,					//!< The "<" relation.
	mcMMRO_LESS_OR_EQUAL = 0x2264,			//!< The "<=" relation.

	mcMMRO_EQUAL = 0x003D,					//!< The "=" relation.
	mcMMRO_NOT_EQUAL = 0x2260				//!< The "!=" relation.
};

//! The sets of numbers supported by MathCore functions.
//! Each set has its own mcValue-derived class: mcIntegerValue,
//! mcRationalValue, mcRealValue, mcComplexValue.
enum mcSet {
	
	//! The set of positive integer numbers, zero comprised.
	mcSET_N,
	
	//! The set of positive & negative integer numbers, zero comprised.
	mcSET_Z,
	
	//! The set of real numbers.
	mcSET_R,
	
	//! The set of complex numbers.
	mcSET_C
};

//! The different modes to consider boundaries: included or excluded.
enum mcBoundaryMode {

	//! Both boundaries are included: conventionally this is
	//! represented writing the set as [1;2].
	mcBM_INCLUDE_BOTH,
	
	//! Only left boundary is included: [1;2) or [1;2[
	mcBM_INCLUDE_LEFT,
	
	//! Only right boundary is included: (1;2] or ]1;2]
	mcBM_INCLUDE_RIGHT,
	
	//! None of the two boundaries is included: (1;2) or ]1;2[
	mcBM_INCLUDE_NONE
};

//! The types of filters which can be set by mcFilter::SetFilterType.
enum mcFilterType {

	//! The filter works allowing the creation only of the element 
	//! types contained in the list associated to mcFilter.
	//! Each element type has an associated maximum number of
	//! instances which can be allowed for that element type.
	mcFILTER_INCLUDE_ELEMENT_LIST,

	//! The filter works allowing the creation only of the math
	//! data types (radicals, polynomial of n-th degree...)
	//! listed in the mcFilter.
	//! \todo STILL NOT WORKING !!!
	mcFILTER_INCLUDE_MATH_LIST
};

//! The types of mcSolver available in MathCore.
//! They should be loaded in the right order in mcMathCore::arrSolvers.
enum mcSolverType {

	mcST_SIMPLIFY,
	mcST_EXPAND,
	mcST_POLYNOMIAL,
	mcST_BISECTION
};

//! The type of logical operators which can be used to link together
//! the equations/inequalities of a mcMathOrSystem.
enum mcLogicOperator {

	mcLO_NOTSET = -1,

	mcLO_AND,
	mcLO_OR,

	mcLO_MAX
};


//! Possible command ids.
//! These are the values you can pass to the #ExecCmd() function.
enum mcMathMngCmdID {

	mcMMC_MOVE_SEL = 0,		// operations on selection
	mcMMC_MULT_SEL,
	mcMMC_DIV_SEL,
	mcMMC_ADDSUB_SEL,

	mcMMC_SIMPLIFY,		// global operations
	mcMMC_EXPAND,

	mcMMC_LAST
};


#endif		// MATHTYPES_H
