///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   MathUtils.h
//! \brief  Defines MACROs & FUNCTIONs used by MathCore.
//! \author Luciano Cattani, Francesco Montorsi, Vinay Doma
//! \date   20-9-2002
//!
//! Defines some macros and global functions used in MathCore classes.
///////////////////////////////////////////////////////////////////////



#ifndef MATHUTILS_H
#define MATHUTILS_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "MathUtils.h"
#endif


// -----------
// INCLUDES
// -----------

#include "mc/Setup.h"		// contains all mcENABLE_XXX defines
#include <wx/bitmap.h>
#include <wx/log.h>			// for the mcLOG macro
#include "wx/xml2.h"

// these will be defined later... we just need them for mcStyle::gui_Select function
class mcElement;
class mcRealValue;




// -----------
// DEFINES
// -----------

#if defined(mcENABLE_LOG)

	// MathCore uses the wxWidgets' logging system:
	// it allows to exclude selectively some type of logs.
	// our logs are divided in some main types:
	enum {
		mcLOG_Generic = 1024,		// MathCore log level start here
		mcLOG_Math,
		mcLOG_GUI,
		mcLOG_IO
	};


	//! The log class used by the MathCore log functions defined above.
	//! It works exactly as wxLogStderr but it does not include the timestamp
	//! of the log message in the output.
	class mcLog : public wxLog
	{
		FILE *m_fp;

	public:
		mcLog(FILE *p = NULL) { m_fp = p; if (!m_fp) m_fp = stderr; }
		virtual ~mcLog() {}

		void DoLogString(const wxChar *msg, time_t timestamp);
	};

#endif

#if defined(mcENABLE_LOG) && \
	!(defined(__GNUG__) && defined(__WXMSW__))	// actually there is an inexplicable bug when using MinGW and mcENABLE_LOG is defined...


	#define mcDECLARE_MATHCORE_LOG_FUNCTION(level)						  \
		extern void WXDLLEXPORT mcVLog##level(const wxChar *szFormat,     \
										  va_list argptr);				  \
		extern void WXDLLEXPORT mcLog##level(const wxChar *szFormat,      \
										 ...) ATTRIBUTE_PRINTF_1


	mcDECLARE_MATHCORE_LOG_FUNCTION(Generic);
	mcDECLARE_MATHCORE_LOG_FUNCTION(Math);
	mcDECLARE_MATHCORE_LOG_FUNCTION(GUI);
	mcDECLARE_MATHCORE_LOG_FUNCTION(IO);


	//! The log macro of generic functions of MathCore.
	#define mcLOG							mcLogGeneric

	//! The log macro of MathCore GUI functions.
	#define mcGUILOG						mcLogGUI

	//! The log macro of MathCore MATH functions.
	#define mcMATHLOG						mcLogMath

	//! The log macro of MathCore IO functions.
	#define mcIOLOG							mcLogIO

	//! The log macro of MathCore SOLVER functions.
	#define mcSOLVERLOG						mcLogMath

#else

	#ifdef __GNUG__
	
		// GCC accepts variadic macros and this avoids us a *lot* of
		// warnings about "statements with no effects" that it ouputs
		// if mcLOG is defined to nothing...
		#define mcLOG(...)								{;}
		#define mcMATHLOG(...)							{;}
		#define mcGUILOG(...)							{;}
		#define mcIOLOG(...)							{;}
		#define mcSOLVERLOG(...)						{;}
	#else

		// If all compilers supported the variadic macros (macros with a variable
		// number of arguments, declared using ... and __VA_ARGS__), then it would
		// be simpler to write these macros !!
		#define mcLOG								/* expand to nothing */
		#define mcMATHLOG							/* expand to nothing */
		#define mcGUILOG							/* expand to nothing */
		#define mcIOLOG								/* expand to nothing */
		#define mcSOLVERLOG							/* expand to nothing */
	#endif

#endif

//! Gets the C inlined string for the given mcElement-derived object.
#define mcTXT(x)				((x).io_GetInlinedExpr().c_str())

//! Same as #mcTXT but this macro works with a pointer.
#define mcTXTP(x)				((x)->io_GetInlinedExpr().c_str())	

//! Returns the inlined expression of a mcValue-derived class.
#define mcTXTV(x)				((x).GetSmartStr().c_str())

//! Same as #mcTXTP but fixed on *this.
//! \note Using wxT("hlp()") instead of "this" this macro works in mcElementIO-derived
//!       classes but it does not work on the mcElement-derived classes.
#define mcTXTTHIS				(this->io_GetInlinedExpr().c_str())	

// define some useful UNICODE & ANSI support functions: if you want to
// avoid #ifdefs in your source code, do not use strlen() or wcslen();
// use wxStrlen(), wxStrcpy() or wxStrcat()
#ifndef wxStrcpy
	#ifdef __UNICODE__
		#define	wxStrcpy		wcscpy
	#else
		#define wxStrcpy		strcpy
	#endif
#endif

#ifndef wxStrcpy
	#ifdef __UNICODE__
		#define	wxStrcat		wcscat
	#else
		#define wxStrcat		strcat
	#endif
#endif



// define the mcEXPORT & mcIMPORT macros which must be put everywhere
// something must be exported in the DLL, when we are building a DLL
// instead of a simple LIB
#if defined(__MCMSW__)
    /*
       __declspec works in BC++ 5 and later, Watcom C++ 11.0 and later as well
       as VC++ and gcc
     */
#    if defined(__VISUALC__) || defined(__BORLANDC__) || defined(__GNUC__) || defined(__WATCOMC__)
#        define mcEXPORT __declspec(dllexport)
#        define mcIMPORT __declspec(dllimport)
#    else /* compiler doesn't support __declspec() */
#        define mcEXPORT
#        define mcIMPORT
#    endif
#elif defined(__MCPM__)
#    if defined (__WATCOMC__)
#        define mcEXPORT __declspec(dllexport)
        /*
           __declspec(dllimport) prepends __imp to imported symbols. We do NOT
           want that!
         */
#        define mcIMPORT
#    elif (!(defined(__VISAGECPP__) && (__IBMCPP__ < 400 || __IBMC__ < 400 )))
#        define mcEXPORT _Export
#        define mcIMPORT _Export
#    endif
#elif defined(__MCMAC__) || defined(__MCCOCOA__)
#    ifdef __MWERKS__
#        define mcEXPORT __declspec(export)
#        define mcIMPORT __declspec(import)
#    endif
#endif

/* for other platforms/compilers we don't anything */
#ifndef mcEXPORT
#    define mcEXPORT
#    define mcIMPORT
#endif





// ------------------------
// MISCELLANEOUS MACROS
// ------------------------

// function-like macros used to create new empty elements...
#define mcNewAddOp()			mcElementHelpers::data_NewElem(mcET_ADDOP)
#define mcNewSubOp()			mcElementHelpers::data_NewElem(mcET_SUBOP)
#define mcNewMultOp()			mcElementHelpers::data_NewElem(mcET_MULTOP)
#define mcNewDivOp()			mcElementHelpers::data_NewElem(mcET_DIVOP)
#define mcNewSymbol()			mcElementHelpers::data_NewElem(mcET_SYMBOL)
#define mcNewEmptyBox()			mcElementHelpers::data_NewElem(mcET_EMPTYBOX)
#define mcNewNumber()			mcElementHelpers::data_NewElem(mcET_NUMBER)
#define mcNewFunction()			mcElementHelpers::data_NewElem(mcET_FUNCTION)
#define mcNewFraction()			mcElementHelpers::data_NewElem(mcET_FRACTION)
#define mcNewBracket()			mcElementHelpers::data_NewElem(mcET_BRACKET)
#define mcNewMonomial()			mcElementHelpers::data_NewElem(mcET_MONOMIAL)
#define mcNewPolynomial()		mcElementHelpers::data_NewElem(mcET_POLYNOMIAL)


// just to give a standard naming to macros...

//! math_Adds the given classname to the wxWidgets RTTI system.
#define wxDEFINE_CLASS(x)			/*DECLARE_CLASS(x)*/

//! Implements the wxWidgets RTTI support for the class \c x
//! (which must derive from the class \c y).
#define wxIMPLEMENT_CLASS(x, y)		/*IMPLEMENT_CLASS(x, y)*/





// ----------------------------------------------------
// MACROS FOR FUNCTION DEFINITION & IMPLEMENTATION 
// ----------------------------------------------------

//! Defines and implements a "custom" clone function: a function which
//! has a different name for each class and, thanks to this, returns
//! a pointer of the same type of this class.
//! When you can, you should write:
//! \code
//!     mcPolynomial &p = mypol.PolynomialClone();
//! \encode
//! instead of:
//! \code
//!     mcPolynomial &p = (mcPolynomial &)mypol.Clone();
//! \encode
//! 
//! \param x The name of the class where you put this macro 
//!        without the MC suffix
//! \note This macro can be used within both a mcElement- and a
//!       mcElementHelpers-derived class.
//! 
#define mcDEFINE_CUSTOM_CLONE_FNC(x)								\
	mc##x x##Clone() const;

//! Defines and implements the mcElementHelpers::data_GetMemoryInfo() function:
//! this function just gets the size of helper classes (and the 
//! mcElementHelpers-derived class used to contain them) through 
//! the sizeof() operator.
//! \param x The name of the class where you put this macro.
#define mcDEFINE_HELPER_GETMEMORYINFO_FNC(x)						\
	int data_GetMemoryInfo() const									\
		{ return sizeof(x##Helpers); }

//! Returns the name to show in the debug informations returned by
//! the data_Debug() function.
#define mcDEFINE_DEBUGNAME_FNC(x)									\
	wxString data_GetDebugName() const								\
		{ return wxT(#x); }

//! Defines & implements the mcElementHelpers::data_Clone function which performs
//! a DeepCopy() on a mcElementHelpers of the same type of *this
#ifdef mcENABLE_COW_DEEPDEBUG
	#define mcDEFINE_HELPER_CLONE_FNC(x)								\
		virtual mcElementHelpers *data_Clone() const {					\
			mcLOG(#x wxT("Helpers::Clone"));							\
			x##Helpers *clone = new x##Helpers();						\
			clone->data_DeepCopy(this);									\
			return clone;												\
		}
#else
	#define mcDEFINE_HELPER_CLONE_FNC(x)								\
		virtual mcElementHelpers *data_Clone() const {					\
			x##Helpers *clone = new x##Helpers();						\
			clone->data_DeepCopy(this);									\
			return clone;												\
		}
#endif

//! Defines & implements the mcElementHelpers::data_GetType which returns a
//! mcElementType identifier for *this class.
#define mcDEFINE_HELPER_GETTYPE_FNC(x)								\
	mcElementType data_GetType() const		{ return x; }

//! Defines the operator= for the given class.
//! This simple operator just reference the data which is referenced
//! by the given element (and thus this is very fast).
//! Example:
//! \code
//!     mcPolynomial x = y;
//! \encode
//! The polynomial wxT("x") will reference the same data of "y".
//!
//! \note This macro has sense only for mcElement-derived classes.
#define mcDEFINE_ASSIGNMENT_OPERATOR(x)									\
	x &operator=(const x &e)											\
		{ if (this != &e) { data_UnRef(); data_Ref(e); } return *this; }

//! Defines the equal (==) and not-equal (!=) operators.
//! The not-equal operator is just the equal one filtered through a NOT
//! logical operator. The comparison operator first checks if the 
//! two pointers to the reference data are identic; if not it calls the 
//! mcElementHelpers::isSameAs function to perform a deep comparison.
#define mcDEFINE_COMPARISON_OPERATORS(x)													\
    bool operator==(const x &e) const 														\
        { const mcElementHelpers *data = (const mcElementHelpers *)e.data_GetRefData();		\
        return data_GetRefData() ? (data && chlp()->data_isSameAs(data)) : !data;	}		\
    bool operator!=(const x &e) const		{ return !(*this == e); }

//! Defines & implements the copy constructor and the wrapper constructor
//! for the given \c x class which derives from \c baseclass.
#define mcDEFINE_ABSTRACT_CONSTRUCTORS(x, baseclass)						\
	x(const mcElementHelpers *p) : baseclass(p) { data_CheckContainer(); }	\
	x(const mcElement &v) : baseclass(v.hlp()) { data_CheckContainer(); }	\
	x(const x &toref) { data_Ref(toref); }									\
	x() { /* cannot create our helpers since we're abstract... */ }			\
	virtual ~x() {}

#define mcDEFINE_MAIN_CONSTRUCTORS(x, baseclass)							\
	x(const mcElementHelpers *p) : baseclass(p) { data_CheckContainer(); }	\
	x(const mcElement &v) : baseclass(v.hlp()) { data_CheckContainer(); }	\
	x(const x &toref) { data_Ref(toref); }									\
	x() { data_SetRefData(new x##Helpers); }								\
	virtual ~x() {}

//! Defines & partially implements a mcElement-derived class.
//! I would have like to make this through templates but I don't think
//! it's possible; anyway, the log macros I put inside should help
//! any eventually-required debug...
#define mcDEFINE_REFERENCE_DATA(x, type)								\
	public:																\
		mcDEFINE_HELPER_GETMEMORYINFO_FNC(x)							\
		mcDEFINE_HELPER_CLONE_FNC(x)									\
		mcDEFINE_HELPER_GETTYPE_FNC(type)								\
		mcDEFINE_DEBUGNAME_FNC(x)

//@}






////////////////////////////////////////////////////////////////////////////////////
//! \name MACROs which _defines_ GETTERs
//! Use these macros in header files to define getters.
//! for some strange reasons, we cannot declare all the following functions INLINE:
//! with MSVC++ (inline version has not been tested with other compilers yet)
//! it would create a LOT of linker errors... anyway, any good compiler should
//! be able to inline them, when full optimization is turned on. @{

#ifdef __MCDEBUG__
	#define mcGETTER			/* expand to nothing */
	#define mcWRAPPER			/* expand to nothing */
#else
    // FIXME: by now, leave everything as if in debug mode: defining MCGETTER to
    // inline makes the release build fail due to a lot of unresolved externals...
    //#define mcGETTER			/* expand to nothing */
	#define mcGETTER			inline		// should be a very important optimization
	#define mcWRAPPER			inline		// should be a very important optimization
#endif

#define mcDEFINE_HLP_GETTER(x)											\
	/*! Returns a pointer to the main section of this element. */		\
	mcGETTER x##Helpers *hlp() 		{ return (x##Helpers *)mcElement::phlp(); }											\
	/*! Returns a CONST pointer to the main section of this element. */	\
 	mcGETTER const x##Helpers *hlp() const	{ return (const x##Helpers *)mcElement::chlp(); }

#define mcDEFINE_HLP_CP_GETTER(x)										\
	mcGETTER x##Helpers *phlp()		{ return (x##Helpers *)mcElement::phlp(); }		\
 	mcGETTER const x##Helpers *chlp() const	 { return (const x##Helpers *)mcElement::chlp(); }

//! Defines & implements various functions for a generic mcElement-derived class:
#define mcDEFINE_CLASS(x)						\
	mcDEFINE_HLP_GETTER(mc##x)					\
	mcDEFINE_HLP_CP_GETTER(mc##x)				\
	mcDEFINE_ASSIGNMENT_OPERATOR(mc##x)			\
	mcDEFINE_COMPARISON_OPERATORS(mc##x)		\
	mcDEFINE_DEBUGNAME_FNC(mc##x)

//! Use this macro to define the helper classes associated with the
//! main element 'x'. For example, mcDEFINE_HELPER_CLASSES(mcNumber),
//! defines the existence of: mcNumber, mcNumberGUI, mcNumberMath,
//! mcNumberIO, mcNumberData. This is usually required before defining
//! the classes because compiler needs to know the existence of all
//! other classes in advance.
//! \param x The class name whose helper classes need to be defined.
#define mcDEFINE_HELPER_CLASSES(x)		\
	class x;							\
	class x##Helpers;

//! This macro should be used instead of #mcDEFINE_MAIN_CLASS
//! for those classes which contains directly or in the related helper
//! classes abstract (i.e. pure virtual) methods and thus cannot have
//! a Clone() function.
#define mcDEFINE_ABSTRACT_CLASS(x, baseclass)			\
public:													\
	mcDEFINE_CLASS(x)									\
	mcDEFINE_ABSTRACT_CONSTRUCTORS(mc##x, baseclass)

//! This macro puts together a lot of other macros resulting in
//! a super-short form for:
//! - definition of all getters of an ELEM section
//! - definition (and implementation) of Clone and "custom" Clone functions.
//! - definition (and implementation) of data_GetMemoryInfo() function.
//! - wxWidgets definition for the RTTI system
//! - definition (and implementation) of assignment operator.
//! - definition (and implementation) of comparison operators.
//! \note Use this macro in a PUBLIC section of the class.
#define mcDEFINE_MAIN_CLASS(x, baseclass)			\
public:												\
	mcDEFINE_CLASS(x)								\
	mcDEFINE_MAIN_CONSTRUCTORS(mc##x, baseclass)

//@}




///////////////////////////////////////////////////////////////////////////////////
//! \name MACROs which _implements_ GETTERs
//! Use these macros in CPP files to implements the getters defined
//! with mcDEFINE_xxx_GETTERS. @{

#define mcIMPLEMENT_HLP_GETTER(x)																	\
	mcGETTER x##Helpers *x::hlp()				{ return (x##Helpers *)mcElement::phlp(); }			\
	mcGETTER const x##Helpers *x::hlp() const	{ return (const x##Helpers *)mcElement::chlp(); }	\

#define mcIMPLEMENT_HLP_CP_GETTER(x)																\
	mcGETTER x##Helpers *x::phlp()				{ return (x##Helpers *)mcElement::phlp(); }			\
	mcGETTER const x##Helpers *x::chlp() const	{ return (const x##Helpers *)mcElement::chlp(); }

#define mcIMPLEMENT_MAIN_CLASS(x, baseclass)		\
	/*mcIMPLEMENT_HLP_GETTER(x)						\
	mcIMPLEMENT_HLP_CP_GETTER(x)*/

#define mcIMPLEMENT_ABSTRACT_CLASS(x, baseclass)	\
	/*mcIMPLEMENT_HLP_GETTER(x)						\
	mcIMPLEMENT_HLP_CP_GETTER(x)*/


//! Use this macro to create the helper classes associated with class 'x'.
//! This macro should be used in the mcElement-derived classes' constructors.
//!
//! VERY IMPORTANT: DO NOT FORGET TO CALL mcFREE_HELPER_CLASSES() MACRO
//! IN THE DESTRUCTOR OF THE CLASS USING THIS MACRO.
//!
//! \param x The class name whose helper classes need to be allocated.
//! \todo the OnChangeParent() is really required ?
#define mcCREATE_HELPER_CLASSES(x)		data_SetRefData(new x##Helpers());

//@}





///////////////////////////////////////////////////////////////////////////////////
//! \name Miscellaneous macros.
//! Still other macros... @{

//! Deletes the given pointer and reset it. This macro is 'safe' because
//! it checks the argument before deleting it: if it's already NULL,
//! nothing is done.
//! \param x The pointer to the object which must be deleted.
#define mcSAFE_DELETE(x)					{ if (x) delete x; x = NULL; }

//! Like mcSAFE_DELETE() but uses the "delete [] x" istruction.
#define mcSAFE_DELETE_ARRAY(x)				{ if (x) delete [] x; x = NULL; }

//! Returns the children belonging to the mcElement structure.
#define mcRETURN_ELEMENT_CHILD(n, baseclass)		\
	if (n < baseclass::data_GetChildrenCount())		\
		return baseclass::data_GetConstChild(n);	\
	n -= baseclass::data_GetChildrenCount();

#define mcSET_ELEMENT_CHILD(n, baseclass, p)		\
	if (n < baseclass::data_GetChildrenCount()) {	\
		baseclass::data_SetChild(n, p);				\
		return;										\
	} 												\
 	n -= baseclass::data_GetChildrenCount();


//! Returns -1 if the the argument is negative, 1 otherwise.
//! \param x The object to test with the '<' operator
#define mcSIGN(x)				(((x) < 0) ? -1 : 1)

//! Returns the highest value between the two arguments supplied.
#define mcMAX(x, y)				(((x) < (y)) ? (y) : (x))

//! Returns the lowest value between the two arguments supplied.
#define mcMIN(x, y)				(((x) < (y)) ? (x) : (y))

//! Swaps the contents of the variables; the type of the two arguments
//! must support the assignment operator =.
#define mcSWAP(type, x, y)		{ type tmp = x; x = y; y = tmp; }

//! The maximimum value for a "double" variable. Taken from standard headers.
#define mcDBL_MAX				1.7976931348623157e+308

//! The maximimum value for a "signed int" variable. Taken from standard headers.
#define mcINT_MAX				2147483647

//! Returns the RED component of the integer argument supplied.
#define mcGETRED(c)				(((wxInt32)(c)) & 255)

//! Returns the GREEN component of the integer argument supplied.
#define mcGETGREEN(c)			((((wxInt32)(c)) >> 8) & 255)

//! Returns the BLUE component of the integer argument supplied.
#define mcGETBLUE(c)			((((wxInt32)(c)) >> 16) & 255)

//! Initializes a wxColour object with the given dword.
#define mcINITWX(x)				(wxColour(mcGETRED(x), mcGETGREEN(x), mcGETBLUE(x)))

//! Returns an integer value containing the encoded for of the given
//! red, green & blue values.
#define mcRGB_ENCODE(r, g, b)  (wxUint32) ((wxUint8)(r) |  \
				((wxUint16)(g) << 8)) | (((wxUint32)((b) << 16))

//! Avoids compiler warnings about unused arguments.
#if wxCHECK_VERSION(2, 5, 1)
	#define mcUNUSED(x)			wxUnusedVar(x)
#else
	#define mcUNUSED(x)			{ x == x; }
#endif

//! Enlarge an array, reallocating it, using new and delete operators.
//! This is not a function because of its flexibility: it can reallocate
//! any type of array, without using any complex C++ syntax; just new and
//! delete.
//! The given array must have been allocated using new() and the macro
//! will assign another memory address to the array; it works exactly
//! like realloc() C function.
//! \param type The type of the array to extend
//! \param parray The pointer to the array to extend, supplied by new at
//!		    the time of the creation
//! \param oldsize The old size of the array (used to copy the old array
//!		     entry in the new extended one
//! \param newsize The size of the new array
#define mcEXTEND_ARRAY(type, parray, oldsize, newsize)			\
	{															\
		type *pnew = new type[newsize];							\
		for (int e=0; e < oldsize; e++)							\
			pnew[e] = parray[e]; 								\
		delete [] parray;										\
		parray = pnew;											\
	}


//! Custom mcASSERT macro used by all MathCore classes.
//! This macro simply calls the wxASSERT_MSG macro adding some newlines to the
//! given description line.
//! The dialog box which will pop up if the mcASSERT fails, contains also
//! the references to the line and the file where the error was found.
//! \param x The expression to test; if it's TRUE (!= 0), nothing will be done;
//!	       otherwise, mcASSERTion will fail and execution halted.
//! \param desc The description of the mcASSERTion failure (the description that
//!	       will be displayed in the case that the mcASSERTion fails)
#define mcASSERT(x, desc)			wxASSERT_MSG(x, wxT("\n\n") desc wxT("\n\n"));

//@}





// ------------------
// GLOBAL FUNCTIONS
// ------------------

//! Returns the index of the string \c "tofind" when it's found inside "str".
//! Returns wxNOT_FOUND (-1) when the second string is not inside the first.
int wxFindOneOf(const wxString &str, const wxString &tofind);

//! Indents a copy of the given string with the given indentation value and then
//! returns it. The indentation is added at the beginning of each line.
wxString wxIndent(const wxString &str, int indent);

//! Returns a pointer to a bitmap initialized with the XPM data and with a mask
//! set to the given colour.
wxBitmap *wxLoadMaskForXPM(char **xpm, const wxColour &mask);



#endif	// MATHUTILS_H

