///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Monomial.h
//! \brief  Defines the mcMonomial class.
//!
//! See Monomial.cpp for more info.
///////////////////////////////////////////////////////////////////////




#ifndef MONOMIAL_H
#define MONOMIAL_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "Monomial.h"
#endif

// required includes
#include "mc/ElementArray.h"


// defined later
class mcExpElement;
class mcRadical;
class mcBracket;
class mcSymbol;

mcDEFINE_HELPER_CLASSES(mcMonomial)
extern mcMonomial mcEmptyMonomial;



//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
class mcMonomialHelpers : public mcElementArrayHelpers
{
	mcDEFINE_REFERENCE_DATA(mcMonomial, mcET_MONOMIAL);

#ifdef mcENABLE_GUI
public:				// customizable variables

	//! If TRUE, when a monomial which doesn' contain an empty box,
	//! is deleted, then it is replaced with a monomial containing
	//! an empty box. If FALSE, the monomial is immediately deleted
	//! with his previous operator.
	static bool sgui_bEmptyBoxReplacing;

	//! If TRUE, when the user edits the contents of a mcMonomial class,
	//! all mcSymbol sequences are automatically scanned searching for
	//! function names; for example, if three mcSymbol containing "s",
	//! wxT("i"), "n" are found to be contiguos or separated only by mcMultOps,
	//! then those three mcSymbol are replaced by a mcFunction representing
	//! the SIN function.
	static bool sgui_bFunctionScanEnabled;

	//! If TRUE, when the user creates a mcDivOp, all the factors which are
	//! placed *before* the operator, become the numerator of a new fraction
	//! whose denominator are the elements placed *after* the operator.
	//! The new fraction then replaces the entire content of this element.
	static bool sgui_bTransformDivInFractions;

	//! A value that, divided by the exponent depth of this element, gives
	//! the number of pixels to leave between all elements.
	static int sgui_nSpaceBetween;

	//! mcPolynomials have different space between ratio from mcPolynomials...
	int gui_GetSpaceBetweenRatio() const	{ return sgui_nSpaceBetween; }

#endif


#ifdef mcENABLE_MATH
public:			// customizable variables

	static mcMonomial *smath_pZero;
	static mcMonomial *smath_pOne;

#endif


#ifdef mcENABLE_IO
public:			// customizable variables

	//! If TRUE, just *after* (see #io_PostProcess) each import process, 
	//! all the div ops will be
	//! converted to fractions using mcMonomial::data_TransformDivOp().
	static bool sio_bAutoConvertDivOpToFractions;

#endif



public:

	mcMonomialHelpers() { data_Init(); }
	virtual ~mcMonomialHelpers() {}



#ifdef mcENABLE_DATA
protected:

	int data_GetIdxFromEntry(const mcArrayEntry *p) const
	{ return p->mdata_idx; }

	mcElement &data_GetRefFromEntry(const mcArrayEntry *p)
	{ return data_Get(p->mdata_idx); }

public:

#ifdef __MCDEBUG__

	//! Checks all the array. In some cases an mcASSERT is used (when the array
	//! incongruences should be caused by programmer's errors), in some others 
	//! the function will repair the error and let everything goes on...
	void data_Check() const;

#endif

	//! Repairs the array, adding the mcMultOp where they are required
	//! (for example, between two mcNumbers), removing them where they must
	//! not be present (for example, at the beginning/end of the array),
	//! and so on...
	//! \note Use this function rarely not only because it's quite slow,
	//!       but also because all the manipulations on the array should
	//!       be done in a controlled way and no errors should be introduced:
	//!       if you call often this function, you will auto-correct those
	//!       errors, thus leaving incorrect code scattered in the library.
	void data_Repair();

	//! Transforms mcDivOps to mcFractions:
	//! 24ax/2*3   becomes   ((24ax)/2)*3    (brackets have been added only
	//!                                       to increase readability)
	//! Unlike mcMonomial::math_TransformDivOp(), this function does not
	//! use the mcElement::math_MakeReciprocal() function; it creates 
	//! directly mcFractions that are then filled with the elements
	//! placed before & after the old mcDivOp.
	void data_TransformDivOp();

#endif		// mcENABLE_DATA



#ifdef mcENABLE_GUI
public:


	/////////////////////////////////////////////////////////////////////////////
	//! \name Array handlers
	//! Functions which manipulates the mcElementArray. @{

	int gui_AddNewElement(mcElementType type, const mcKey &key, int pos = -1);

	//! Checks the cursor position. If the cursor is not in the right position,
	//! the program stops.
	void gui_CheckCursorPos() const;
	
	//! Scans the array for mcSymbol sequences that matches the name of a mcFunction.
	void gui_ScanArrayForFunctions();

	//! After a function has been recognized inside a sequence of mcSymbols,
	//! the container of the mcSymbol classes should call the following function
	//! to create the recognized mcFunction.
	void gui_ReplaceSymbolsWithFunction(int idx, int fnc);
	
	//@}



	/////////////////////////////////////////////////////////////////////////////
	//! \name mcElement overloaded functions
	//! Implements abstract mcElement functions. @{

	//! This class cannot be allocated (directly) by the user; this is
	//! why it *always* returns a FALSE value...
	bool gui_isBeginKey(const mcKey &) const		{ return FALSE;  }

	//! See #io_isBeginChar function.
	bool gui_isArrEndKey(const mcKey &key) const { 
		mcElementType m = mcElementHelpers::gui_isKeyBeginKey(key);

		if (m == mcET_ADDOP || m == mcET_SUBOP)
			return TRUE;
		return FALSE;
	}

	//! Like mcElement::gui_Draw, but you can specify a center line
	//! to use to calculate the cursor position.
	//! \param dc The dc where this element will be drawn
	//! \param x,y The coordinates where this element will be drawn
	//! \param pt The mouse cursor position or one of the mcDRW_*** flags
	//! \param cl The center line to use, or -1 to let the function
	//!        calculate it
	int gui_Draw(wxDC &dc, int x, int y, long flags, const wxPoint &pt, int cl) const;

	//! A little helper funcitons to be used in some rare situations.
	//! You should NEVER use it: it's just some code which would be
	//! repeated 2-3 times in the gui_BackInput() function.
	int gui_DeleteFirstOp();

	mcInputRes gui_Input(const mcKey &ev, mcElement *newelem);
	mcInputRes gui_BackInput(const mcKey &key, mcElement *pnewelem, int n);

	mcInsertRes gui_Insert(const mcElement *toinsert, int n);
	mcInsertRes gui_Insert(const mcElement &toinsert, mcElement *);

	void gui_Replace(mcElement *);

	bool gui_Split(mcElement *);
	void gui_DoSplit(mcElementType type, const mcKey &key);
	bool gui_MergeWith(const mcElement &);

	int gui_Draw(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const {
		// call the overloaded version
		return gui_Draw(dc, x, y, flags, pt, -1);
	}

	//@}

#endif		// mcENABLE_GUI




#ifdef mcENABLE_MATH
protected:

	//! Handles a little mcMonomial-specific exception.
	virtual mcExpSimRes math_HandleExpSimFlag(mcExpSimRes r, mcElement *newelem, int i);
	
	//! Reorders the elements contained in this monomial collapsing all the 
	//! elements which can be simplified and reordering them. The function
	//! puts the element in this order:
	//! - mcNumber(s)
	//! - mcFraction(s)
	//! - mcBracket(s)
	//! - mcRadical(s)
	//! - mcFunctions(s)
	//!
	//! VERY IMPORTANT: this function can be called only if mcDivOp
	//!                 are not present in the array.
	mcExpSimRes math_Reorder();

	//! Reorders all the symbols of the given list of the given size.
	//! The comparison function used is mcSymbol::math_isBeforeInOrderListOf.
	//void ReorderSymbols(mcSymbol **arr, int nsymbols);

	//! Removes all the mcDivOp using the TransformDivOp function.
	virtual mcExpSimRes math_BeginSimSteps();

	//! math_Adds eventually missing mcMultOp.
	virtual void math_EndSimSteps();

public:


	/////////////////////////////////////////////////////////////////////////////
	//! \name mcElementArrayMath abstract functions
	//! Implementation of those functions defined as pure virtual in parent class. @{

	int math_MathToDataIdx(int dataindex) const;
	int math_DataToMathIdx(int mathindex) const;
	
	void math_PrepareForComparison(mcElementArray &p) const;

	mcElementType math_GetNeutralOpType() const		{ return mcET_MULTOP; }
	double math_GetNeutralValue() const				{ return 1.0; }
	
	mcPolynomial math_GetPolynomialWrapper() const;
	mcArrayEntry math_WrapMonomial(const mcMonomial &);
	mcArrayEntry math_WrapSimple(const mcElement &);
	//void math_GetFactors();

//	void math_FactoreOut();
/*	const mcElement &math_Get(int n) const 
		{ return (const mcElement &)mcElementArrayHelpers::math_Get(n); }
	mcElement &math_Get(int n) 
		{ return (mcElement &)mcElementArrayHelpers::math_Get(n); }
*/
	//void math_RaiseChildrenTo(const mcRealValue &n);
	mcBasicOpRes math_RaiseTo(const mcPolynomial &p);
	
	//@}



	/////////////////////////////////////////////////////////////////////////////
	//! \name Math engine functions
	//! Implements math calculus. @{
	
	//! Modifies this monomial replacing the first operator with a mcAddOp;
	//! in this way, the monomial's sign is changed if the monomial was
	//! negative; it will remain untouched if it was positive.
	//! This function needs a mcET_POLYNOMIAL as parent to work.
	void math_Abs();

	//! Sets the operator preceding this operator to a mcAddOp if the given
	//! number is positive, to a mcSubOp if it is negative.
	//bool SetSign(int n);

	//! Changes the sign of this monomial: if the monomial is preceded by
	//! a mcAddOp, a mcSubOp is created and used to replace the old op and
	//! viceversa. This function needs a mcET_POLYNOMIAL as parent to work.
	//bool ChangeSign();

	//! Returns the sign of this monomial.
	//! This function needs a mcET_POLYNOMIAL as parent to work.
	//int GetSign() const;

	//! Returns TRUE if the parent of this element is a mcET_POLYNOMIAL
	//! and in this case the function stores in the given pointers the
	//! parent address and the math-index of this monomial in its parent.
	//! Returns FALSE if the parent of this monomial is not a mcPolynomial.
	//bool GetParentPos(const mcPolynomial &*pol = NULL, int *idx = NULL) const;
	//bool GetParentPos(mcPolynomial &*pol = NULL, int *idx = NULL);

	//! Returns TRUE if the given monomial can be summed with this monomial;
	//! that is, both the two monomials have the same factors which contains
	//! symbols or functions.
	bool math_isSimilarTo(const mcMonomial &) const;

	//! Using the mcElement::math_MakeReciprocal function, replaces mcDivOp
	//! with mcMultOp and then transform the element(s) following it
	//! in their reciprocals. A fast and safe way to remove mcDivOps.
	void math_TransformDivOp();

	//! Removes all mcDivOp using #TransformDivOp and then removes all
	//! remaining mcMultOp from the array.
	void math_RemoveAllOp();	
	
	//! Returns a new monomial containing the factors non-common to *this
	//! and the given monomial.
	mcMonomial math_DetachNonCommonFactors(const mcMonomial &);

	//! Returns the maximum degree of the given symbol for this monomial.
	//! If this monomial does not contain the given symbol, then the return
	//! value is zero; if this monomial contains a mcSymbol linked with the
	//! given mcSymbolProperties but not containing a simple mcNumber, then
	//! the return value is mcIntegerValue::pNAN.
	mcIntegerValue math_GetMaxDegreeFor(const mcSymbolProperties *) const;

	//! Returns the sum of the maximum degrees for all the symbols in the
	//! given array.
	mcIntegerValue math_GetMaxDegreeForSymbolArray(const mcSymbolArray *p) const;

	//@}

	


	/////////////////////////////////////////////////////////////////////
	//! \name Coefficient-related functions
	//! Functions working on the coefficient of this monomial. @{
		
	void math_SimplifyCoeff();
	void math_RemoveCoeff();
	void math_SetCoeff(const mcNumber &);
	void math_ChangeCoeffSign();
	mcRealValue math_GetCoeff() const;

	//@}



	/////////////////////////////////////////////////////////////////////
	//! \name mcElementMath abstract functions
	//! Implements some common math functions. @{

	bool math_CanBeAddedWith(const mcElement &p) const;

	mcBasicOpRes math_Add(const mcElement &, mcElement *p, bool add);
	mcBasicOpRes math_MultiplyBy(const mcElement &, mcElement *p);
	mcBasicOpRes math_DivideBy(const mcElement &, mcElement *p);

	//! Computes a value which is used to reorder this monomial.
	int math_GetOrderPos() const;

	mcMonomial math_GetLCM(const mcElement &) const;
	mcMonomial math_GetGCD(const mcElement &) const;

	//@}

#endif		// mcENABLE_MATH



#ifdef mcENABLE_IO
public:

	wxXml2Node io_GetMathML(bool bGetPresentation) const;

	bool io_ImportPresentationMathML(wxXml2Node tag, wxString &pErr);
	bool io_ImportInlinedExpr(const wxString &, int *, wxString &pErr);

	//! If the 'auto convert mcDivOp(s) to mcFraction(s)' feature is
	//! enabled (through #bAutoConvertDivOpToFractions) this function
	//! will remove all divop replacing them with mcFraction...
	void io_PostProcess();

#endif		// mcENABLE_IO
};


//! Implements methods useful to manipulate the monomials contained in
//! an external mcPolynomial. This class is really light because it
//! doesn't copy anything from the given expression; it just links them.
//! Thus, you can create/delete mcMonomials everywhere you need; you
//! can also reuse them recalling Init().
//! This class is somewhat slow: don't make useless calls because each
//! function often uses complex algorithms and almost always will result
//! in a lot of recursive calls.
//! \warning All the entry values required by mcMonomial functions must be
//!	         in 'monomial coord.': if you call mcMonomial::GetElem(0), you
//!	         will NOT get the first element of the attached expression, but
//!	         the first element which composes this monomial.
class mcMonomial : public mcElementArray 
{
	friend class mcPolynomial;
	mcDEFINE_MAIN_CLASS(Monomial, mcElementArray);

public:

	mcMonomial(const mcSymbol &towrap);	
	mcMonomial(const mcNumber &towrap);
	mcMonomial(const mcRadical &towrap);
	mcMonomial(const mcBracket &towrap);
	mcMonomial(const mcFraction &towrap);


	mcMonomial operator*(const mcMonomial &m) const
	{ mcMonomial ret(*this); ret.math_SimpleMultiplyBy(m); return ret; }
	mcMonomial operator/(const mcMonomial &m) const
	{ mcMonomial ret(*this); ret.math_SimpleDivideBy(m); return ret; }
	mcMonomial operator-(const mcMonomial &m) const
	{ mcMonomial ret(*this); ret.math_SimpleSubtract(m); return ret; }
	mcMonomial operator+(const mcMonomial &m) const
	{ mcMonomial ret(*this); ret.math_SimpleAdd(m); return ret; }

	mcMonomial operator+=(const mcMonomial &m)		{ *this = *this+m;return *this; }
	mcMonomial operator-=(const mcMonomial &m)		{ *this = *this-m;return *this; }
	mcMonomial operator*=(const mcMonomial &m)		{ *this = *this*m;return *this; }
	mcMonomial operator/=(const mcMonomial &m)		{ *this = *this/m;return *this; }



	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_MONOMIAL; }

	mcWRAPPER void math_RemoveAllOp()
		{ hlp()->math_RemoveAllOp(); }


	mcWRAPPER void math_Abs()
		{ hlp()->math_Abs(); }
/*	mcWRAPPER bool math_isSimilarTo(const mcMonomial &) const;
	mcWRAPPER void math_TransformDivOp();

	mcWRAPPER mcMonomial math_DetachNonCommonFactors(const mcMonomial &);*/
/*
	mcWRAPPER const mcElement &math_Get(int n) const
		{ return hlp()->math_Get(n); }
	mcWRAPPER mcElement &math_Get(int n)
		{ return hlp()->math_Get(n); }*/

	mcWRAPPER mcIntegerValue math_GetMaxDegreeFor(const mcSymbolProperties *p) const
		{ return hlp()->math_GetMaxDegreeFor(p); }
	mcWRAPPER mcIntegerValue math_GetMaxDegreeForSymbolArray(const mcSymbolArray *p) const
		{ return hlp()->math_GetMaxDegreeForSymbolArray(p); }

	mcWRAPPER void math_SimplifyCoeff()
		{ hlp()->math_SimplifyCoeff(); }
	mcWRAPPER void math_SetCoeff(const mcNumber &n)
		{ hlp()->math_SetCoeff(n); }
	mcWRAPPER void math_RemoveCoeff()
		{ hlp()->math_RemoveCoeff(); }
	mcWRAPPER mcRealValue math_GetCoeff() const
		{ return hlp()->math_GetCoeff(); }

	mcWRAPPER void math_ChangeCoeffSign()
		{ hlp()->math_ChangeCoeffSign(); }
};


#endif	// MONOMIAL_H

