///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Number.h
//! \brief  Defines the mcNumberData, mcNumberGUI, mcNumberMath,
//!         mcNumberIO, mcNumber classes.
//!
//! See Number.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef NUMBER_H
#define NUMBER_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "Number.h"
#endif

// required includes
#include "mc/ExpElement.h"



//! The point used to separe the integer part from the fractional one.
//! This one cannot be chosen differently because MAPM recognizes only '.'
#define mcNUMBER_DECIMAL_POINT			wxT('.')



mcDEFINE_HELPER_CLASSES(mcNumber)

// defined later
extern mcNumber mcEmptyNumber;



//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
class mcNumberHelpers : public mcExpElementHelpers
{
	mcDEFINE_REFERENCE_DATA(mcNumber, mcET_NUMBER);

#ifdef mcENABLE_DATA
protected:		// DATA variables

	//! The number represented by this class.
	mcRealValue mdata_n;

#endif



#ifdef mcENABLE_GUI
protected:		// GUI variables

	//! mcNumberGUI allows the user to input decimal numbers (of any lenght)
	//! so it cannot just read the input through #gui_BaseInput() and add the
	//! typed digits to mcNumber::data_m_n because if the user wants to
	//! add a zero when the cursor is placed in the rightmost digit after
	//! the comma, in this way it wouldn't work:
	//!
	//!        1234.5678|         the user types "0"
	//!        1234.56780         must be the result
	//!
	//! Using a string (which is always kept sync with mcNumber::data_m_n),
	//! we can avoid such problems...
	wxString mgui_strTrailer;

	//! The cursor position.
	//! Note on the cursor positions available for mcExpElement::mgui_nCursorPos
	//! in the mcNumber class.
	//!
	//! Positive numbers of mgui_nCursorPos indicates the digit where
	//! the cursor is next to...
	//! example: 
	//! 	  mgui_nCursorPos=0, m_n=123	  .	   |123
	//! 	  mgui_nCursorPos=3, m_n=335	  .	   335|
	//! 	  mgui_nCursorPos=2, m_n=1024	  .	   10|24
	//!	
	//! example:
	//!    mgui_nCursorPos=mcECL_INSIDEEXP, m_n=123   .	 123^(...|...)
	//!    mgui_nCursorPos=mcECL_ENDEXP,	m_n=335   .	 335^(...)|
	int mgui_nCursorPos;

public:			// customizable variables

	//! The number of digits of #m_n which will be shown when exporting through
	//! the mcNumberIO functions or when viewing the graphical representation
	//! through the mcNumberGUI functions.
	//! If this variable is set to -1, then all the significant digits will be
	//! shown; if set to n, then only the first n digits of the number will be
	//! shown (padding with zeroes if significant digits are missing).
	static int sgui_nDigitToShow;

	//! Contains the characters recognized as floating point.
	static wxString sgui_strFloatingPoint;


public:		// utilities
		
	//! Returns TRUE if the given keycode is a digit 
	//! (0, 1, 2, 3, 4, 5, 6, 7, 8 or 9).
	//! This utility does not belong to any specific subsection of the
	//! program because it is used almost in each of them.
	static bool gui_isDigit(int vk);

	//! Returns TRUE if the given character is considered as a decimal point.
	static bool gui_isDecimalPoint(wxChar totest)
		{ return sgui_strFloatingPoint.Find(totest) != wxNOT_FOUND; }

#endif



#ifdef mcENABLE_MATH
public:			// customizable variables

	//! If TRUE, then fractions with integer coefficients will be used
	//! when possible, instead of decimal numbers:
	//!
	//!    24/15  will be simplified to  8/5  instead of  1.6
	//!
	static bool smath_bUseIntegersWhenPossible;

public:			// static utilities

	//! A mcNumber with "1.0" as content...
	static mcNumber *smath_pOne;
	static mcNumber *smath_pTwo;
	static mcNumber *smath_pFour;
	static mcNumber *smath_pMinusOne;
	static mcNumber *smath_pZero;

	static bool math_isZero(const mcElement &);

#endif



public:

	mcNumberHelpers() {
		mdata_n = 0;		    // zero by default
		//m_n.Set(0);

		mdata_bExpEnabled = TRUE;
		mdata_bSubEnabled = FALSE;	// do not allow subscripts

		data_Init();
	}

	virtual ~mcNumberHelpers() {}

protected:

	void gui_Init() {
		mcExpElementHelpers::gui_Init();
		mgui_nCursorPos = 0;	    // cursor is placed on first digit
	}




#ifdef mcENABLE_DATA
public:

#ifdef __MCDEBUG__

	//! Reports the data contained in this class: an integer value.
	wxString data_BaseDebug(long flags) const {

		wxString exp = data_Get().GetStr();
		wxString smart = data_Get().GetSmartStr();

		// choose shortest
		return wxT("mcNumber     ") + (exp.Len() > smart.Len() ? smart : exp) + wxT("\n");
	}

#endif

	void data_DeepCopy(const mcElementHelpers *p) {
		const mcNumberHelpers *n = (const mcNumberHelpers *)p;
		mdata_n = n->mdata_n;
		mgui_strTrailer = n->mgui_strTrailer;
		mcExpElementHelpers::data_DeepCopy(p);
	}

	bool data_isSameAs(const mcElementHelpers *p) const {
		if (!mcElementHelpers::data_isSameAs(p))
			return FALSE;

		const mcNumberHelpers *ed = (const mcNumberHelpers *)p;
		if (mdata_n != ed->mdata_n) return FALSE;
		return TRUE;
	}

	bool data_isAllowedBefore(mcElementType type) const {
		if (type == mcET_NUMBER)
			return FALSE;
		return TRUE;
	}

	bool data_isAllowedAfter(mcElementType type) const {
		// mcET_PARENTHESIS is a special case: when it's a left
		// bracket, then there must be *no* operators between it
		// and other elements: (*4x) is invalid !!
		if (type == mcET_NUMBER || type == mcET_BRACKET || 
			type == mcET_FUNCTION || type == mcET_SYMBOL || 
			type == mcET_RADICAL || type == mcET_FRACTION)
			return FALSE;
		return TRUE;
	}

	void data_Set(const mcRealValue &d)				{ mdata_n = d; }
	mcRealValue data_Get() const					{ return mdata_n; }

#endif		// mcENABLE_DATA





#ifdef mcENABLE_GUI
public:

	void gui_Set(const wxString &toset);
	wxString gui_GetStr() const
		{ return data_Get().GetSmartStr()+mgui_strTrailer; }


	/////////////////////////////////////////////////////////////////////////////////
	//! \name mcExpElement overloaded BASE functions
	//! These functions handle the base of the symbol; the exponent is handled by
	//! mcExpElement functions. @{

	bool gui_isBeginKey(const mcKey &ev) const;
	bool gui_isBaseEndKey(const mcKey &ev) const;

	//! mcNumber is not a container element, thus when mouse cursor is placed
	//! over the base, all the element should be drawn as active.
	bool gui_DrawAsActiveOverBase() const { return TRUE; }
	
	mcInsertRes gui_BaseInsert(const mcElement &, mcElement *);
	mcInputRes gui_BaseInput(const mcKey &key, mcElement *pnew);
	mcMoveCursorRes gui_BaseMoveCursor(mcMoveCursorFlag flag, long modifiers);
	int gui_DrawBase(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const;	
	int gui_BaseMoveCursorUsingPoint(wxDC &, const wxPoint &);
	int gui_GetBaseRelCursorPos(wxDC &dc, wxPoint *pt) const;
	void gui_GetBaseCursorPos(mcCursorPos &) const;

	void gui_SetBaseCursorPos(const mcCursorPos &);
	void gui_DoRecalcBaseSize();
	void gui_EditBase();

	//! mcNumber is one of the few elements which supports splitting.
	//! In fact, if the user types something which is not a digit
	//! inside the number, the mcNumber is splitted in two parts.
	bool gui_Split(mcElement *);
	bool gui_MergeWith(const mcElement &);

	bool gui_isSplittable() const		{ return TRUE; }

	//@}

#endif		// mcENABLE_GUI





#ifdef mcENABLE_MATH
public:

	void math_Abs()							{ data_Set(data_Get().abs()); }
	bool math_isValid() const				{ return data_Get().isValid(); }

	bool math_GetNumDen(mcIntegerValue &num, mcIntegerValue &den);
	mcFraction math_TransformInFraction();

	void math_RaiseBaseTo(mcRealValue n);
	virtual void math_RaiseTo(const mcIntegerValue &n);



	//////////////////////////////////////////////////////////////////////////////
	//! \name mcExpElementMath and mcElementMath abstract functions
	//! Implementation of pure virtual common math functions. @{

	bool math_CanBeAddedWith(const mcElement &p) const;
	bool math_CanBeMultWith(const mcElement &p) const;
	bool math_CanBeDivBy(const mcElement &p) const;

	mcBasicOpRes math_Add(const mcElement &, mcElement *, bool add);

	mcBasicOpRes math_MultiplyBaseOnlyBy(const mcElement &, mcElement *);
	mcBasicOpRes math_DivideBaseOnlyBy(const mcElement &, mcElement *);
	mcBasicOpRes math_MultiplyBaseBy(const mcElement &, mcElement *);
	mcBasicOpRes math_DivideBaseBy(const mcElement &, mcElement *) 
		{ return mcBOR_REMOVE_OPERAND; }

	mcRealValue math_GetBaseLenght() const		{ return data_Get().GetNumOfDigits(); }
	mcRealValue math_EvaluateBase() const		{ return data_Get(); }
	int math_GetOrderPos() const				{ return 0; }
	//mcBasicOpRes math_MakeReciprocal(mcElement *);
	
	mcMathType math_GetBaseMathType() const 
		{ return mcMathType(mcMTL1_POLYNOMIAL, mcMTL2_ALGEBRAIC, mcMTL3_CONSTANT); }

	mcBasicOpRes math_MakeReciprocal(mcElement *);

	mcExpSimRes math_ExpandBase(long flags, mcElement *newelem);
	mcExpSimRes math_SimplifyBase(long flags, mcElement *newelem);
	mcExpSimRes math_SimplifyBaseExp(long flags, mcElement *pnew);

	//bool hasSameContentOf(const mcElement &p) const;
	bool math_CompareThisOnly(const mcElement &p, long flags) const;

	mcMonomial math_GetBaseLCM(const mcElement &p) const;
	mcMonomial math_GetGCD(const mcElement &p) const;

	//@}

#endif		// mcENABLE_MATH




#ifdef mcENABLE_IO
public:

	bool io_isBeginTag(const wxXml2Node &tag) const {
		if (tag.GetName() == wxT("mn"))
			return TRUE;
		return FALSE;
	}

	bool io_isBeginChar(const wxString &str) const;

	// MATH ML export functions
	wxXml2Node io_GetBaseMathML(bool bGetPresentation) const;
	wxString io_GetBaseInlinedExpr() const;

	bool io_ImportPresentationMathML(wxXml2Node tag, wxString &pErr);
	bool io_ImportBaseInlinedExpr(const wxString &, int *count, wxString &pErr);

#endif		// mcENABLE_IO
};




//! One of the most used class of MathCore: a simple number (and optionally
//! its exponent). This class allows the user to insert coefficients in the
//! monomials.
//! \todo math_Add range checking: try to enter too high numbers should generate
//!       a warning - IS THIS REQUIRED SINCE WE USE MAPM ?
class mcNumber : public mcExpElement
{
	mcDEFINE_MAIN_CLASS(Number, mcExpElement);

public:

	//! An handy constructor.
	mcNumber(const mcRealValue &n)
		{ data_SetRefData(new mcNumberHelpers); hlp()->data_Set(n); }

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_NUMBER; }

	mcWRAPPER void data_Set(const mcRealValue &d)			{ hlp()->data_Set(d); }
	mcWRAPPER mcRealValue data_Get() const					{ return hlp()->data_Get(); }

	mcWRAPPER void math_Abs()
		{ hlp()->math_Abs(); }

	mcWRAPPER bool math_isValid() const
		{ return hlp()->math_isValid(); }

	mcWRAPPER bool math_GetNumDen(mcIntegerValue &num, mcIntegerValue &den)
		{ return hlp()->math_GetNumDen(num, den); }

	// this needs to be in the source file
	mcFraction math_TransformInFraction();
};

#endif	// NUMBER_H


