///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Object.h
//! \brief  Defines the mcObject class.
//!
//! See Object.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef OBJECT_H
#define OBJECT_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "Object.h"
#endif

// defined later
class mcObjectRefData;


//! The base class for mcElement. Provides basic COW services.
//! This class is almost identic to wxObject.
class mcObject
{
protected:

	//! The data shared by this mcObject.
    mcObjectRefData *mdata_refData;

public:

	//! Default constructor.
    mcObject() : mdata_refData(NULL) {}

	//! Copy constructor.
    mcObject(const mcObject& other) : mdata_refData(NULL) { data_Ref(other); }

	//! Destructor: automatically unreferences the shared data.
    virtual ~mcObject()	{ data_UnRef(); }
	
	
	//! The assignment operator: it just references the given object.
    mcObject &operator=(const mcObject& other) {
        if (this != &other) {
            data_UnRef();
            data_Ref(other);
        }
        return *this;
    }
	    
	
    //! Returns the shared data.
    mcObjectRefData *data_GetRefData() const			{ return mdata_refData; }

	//! Changes the shared data.
    void data_SetRefData(mcObjectRefData *data)			{ mdata_refData = data; }
	
    //! Makes a clone of the given object referencing its shared data.
    virtual void data_Ref(const mcObject &clone);
	
    //! Decrements the reference count of the currently shared data,
	//! eventually deleting it, and sets the reference data to NULL.
    virtual void data_UnRef();

	//! Makes a private copy of the currently shared data.
	virtual mcObjectRefData *data_MakePrivateCopy() = 0;
};



//! Reference counted data meant to be stored in mcObject.
//! This class has an important difference with wxObjectRefData:
//! it provides the #IncRefCount() and #DecRefCount() as public
//! methods, thus allowing mcElement-derived class to implement
//! their wrapper constructors.
class mcObjectRefData
{
public:
    mcObjectRefData() : mdata_count(1) {}
    virtual ~mcObjectRefData() {}
	
    int data_GetRefCount() const		{ return mdata_count; }
	void data_SetRefCount(int n)		{ mdata_count = n; }
	
	void data_IncRefCount()				{ mdata_count++; }
	void data_DecRefCount()				{ mdata_count--; }
	
private:
    int mdata_count;
};


#endif	// OBJECT_H

