///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Operator.h
//! \brief  Defines the mcOperator class.
//!
//! See Operator.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef OPERATOR_H
#define OPERATOR_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "Operator.h"
#endif

// required includes
#include "mc/Decoration.h"



mcDEFINE_HELPER_CLASSES(mcOperator)


//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
class mcOperatorHelpers : public mcDecorationHelpers
{
#ifdef mcENABLE_DATA
public:		// DATA variables

	////////////////////////////////////////////////////////////////////
	//! \name OPERATORS SECTION
	//! Declares class IDs functions and values used by the subclasses 
	//! of mcOperator. @{

	//! Class identifier (to avoid RTTI use); every derived class should
	//! set this variable to its specific #define in his constructor.
	mcElementType mdata_nType;
	
	//! mcOperator::m_nSubType possible values.
	#define mcNUM_OPERATORS			4
	
	//! The keys used by mcOperator to recognize the subclass to allocate when
	//! the user types an operator symbol. See #MatchOpSymbol
	mcKey kOpBeginChar[mcNUM_OPERATORS];

	//! This little array is not placed in mcOperatorGUI because it is not
	//! related to the GUI system only: the entire mcOperator creation and
	//! destruction system uses this array.
	static wxString strOpSymbol[mcNUM_OPERATORS];
	
	//! The name of each mcOperator subclass in class ID order (first mcET_ADDOP, then
	//! mcET_SUBOP and so on) used by mcOperator MathML export functions.
	//wxString strMathMLOp[mcNUM_OPERATORS];
	
	//@}

#endif
	


#ifdef mcENABLE_GUI
protected:		// GUI variables

	//! Real size of this operator (m_sz is set to the maximum width
	//! and height found among all the operators).
	//! This member is static since it can be safely shared among all
	//! the operators instances.
	//! It is updated in the #gui_DoRecalcSize() calls only if the
	//! style used for drawing operators has changed since last
	//! recalculation (and #sgui_pLastStyle is used for this).
	static wxSize sgui_szReal[mcNUM_OPERATORS][mcELEMENTMATH_MAX_EXPDEPTH];

	//! The maximum size.
	static wxSize sgui_sz[mcELEMENTMATH_MAX_EXPDEPTH];

	//! The style array used for last recalculation of the
	//! #sgui_sz and #sgui_szReal arrays.
	static const mcStyleArray *sgui_pLastStyle;

#endif




#ifdef mcENABLE_MATH
protected:		// MATH variables

	//! Priority level of this operator. Not used.
	int m_nPriority;

#endif



public:

	mcOperatorHelpers() {
		mdata_nType = mcET_INVALID;
	}

	virtual ~mcOperatorHelpers() {}



#ifdef mcENABLE_DATA
public:


	//! Operators are generally allowed only in some specific cases.
	bool data_isAllowedBefore(mcElementType type) const;

	// shortcuts to STATIC versions
	int data_GetOpIndex() const			{ return data_GetOpIndex(data_GetType()); }
	wxString data_GetOpSymbol() const	{ return data_GetOpSymbol(data_GetType()); }


	void data_DeepCopy(const mcElementHelpers *p) {
		const mcOperatorHelpers *o = (const mcOperatorHelpers *)p;
		mdata_nType = o->mdata_nType;
		mcDecorationHelpers::data_DeepCopy(p);
	}


	//! Returns the index in mcOperator arrays for the given operator type.
	static int data_GetOpIndex(mcElementType t) {
		mcASSERT(t != mcET_INVALID, wxT("Invalid operator type"));
		return t-mcET_ADDOP;
	}

	//! Returns the symbol to use for the given operator type.
	static wxString data_GetOpSymbol(mcElementType t) {
		mcASSERT(t != mcET_INVALID, wxT("Invalid operator type"));
		return strOpSymbol[data_GetOpIndex(t)];
	}
	
	//! Returns TRUE if the specified entry is an operator.
	static bool data_isOp(mcElementType t) {
		return (t == mcET_ADDOP || t == mcET_SUBOP || 
				t == mcET_MULTOP || t == mcET_DIVOP);
	}

#endif		// mcENABLE_DATA




#ifdef mcENABLE_GUI
public:

	///////////////////////////////////////////////////////////////////////////////
	//! \name mcElement overloaded functions
	//! Overloads of the abstract mcElement functions. @{
	
	bool gui_isBeginKey(const mcKey &ev) const;
	
	// operators are defined by only 1 char; thus every character
	// user presses after the begin char of the operator, is an end char
	bool gui_isEndKey(const mcKey &ev) const		{ return TRUE; }
	
	mcMoveCursorRes gui_MoveCursor(mcMoveCursorFlag flag, long modifiers);
	mcInputRes gui_Input(const mcKey &ev, mcElement *newelem);
	int gui_MoveCursorUsingPoint(wxDC &dc, const wxPoint &p);
	int gui_Draw(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const;
	int gui_GetRelCursorPos(wxDC &dc, wxPoint *pt) const;	
	void gui_DoRecalcSize();
	
	// these doesn't apply to an operator...
	void gui_GetCursorPos(mcCursorPos &p) const		{ p.gui_Push(mcCP_BEGINEND); }
	void gui_SetCursorPos(const mcCursorPos &)		{ ; }
	bool gui_LetInCursor(mcCursorPos code)			{ return FALSE;	}

	//! Returns the real size of this operator.
	//! See #sgui_szReal for more info.
	wxSize gui_GetRealSize() const
		{ return sgui_szReal[data_GetType()-mcET_ADDOP][gui_GetExpDepth()]; }
	
	//@}

#endif		// mcENABLE_GUI




#ifdef mcENABLE_MATH
public:

	//! All operators have a lenght/complexity of 1.5
	mcRealValue math_GetLenght() const				{ return 1.5; }


	////////////////////////////////////////////////////////////////////////////////
	//! \name mcOperator specific functions
	//! Functions defined for mcOperators only. @{
	
	//! Returns TRUE if this operator can be applied on the two given elements.
	//! For non-commutative operators, the order is important: the first given
	//! pointer is the left operand while the second is the right one.
	virtual bool math_CanBeApplied(const mcElement &l, const mcElement &r) const = 0;

	//! Applies this operator on the left and right monomials.
	//! \return TRUE if the right monomial can be deleted.
	virtual mcBasicOpRes math_Apply(mcElement &, const mcElement &, mcElement *) const = 0;

	//! Works like #Apply but uses the mcElement::math_SimpleAdd/Mult/Div
	//! functions and thus does not take the pointer-to-pointer argument.
	virtual mcBasicOpRes math_ApplySimple(mcElement &, const mcElement &) const = 0;

	//! Applies the operator on the two given values and then returns it.
	virtual mcRealValue math_Evaluate(mcRealValue val, mcRealValue val2) const = 0;
	
	// apply this operator between pLeft and pRight elements
	//virtual void SolveFirst(mcElement pLeft, mcElement pRight) = 0;
	bool math_Compare(const mcElement &p, long flags) const	
		{ return math_CompareThisOnly(p, flags); }
	//bool hasSameContentOf(const mcElement &p) const			{ return FALSE; }
	
	//@}
#endif		// mcENABLE_MATH



#ifdef mcENABLE_IO
public:

	bool io_isBeginTag(const wxXml2Node &tag) const;
	bool io_isBeginChar(const wxString &str) const;

	// mathML functions	
	wxXml2Node io_GetMathML(bool bGetPresentation) const;
	wxString io_GetInlinedExpr() const;

	bool io_ImportPresentationMathML(wxXml2Node tag, wxString &pErr);
	bool io_ImportInlinedExpr(const wxString &, int *count, wxString &pErr);

#endif		// mcENABLE_IO
};




//! A basic class for MathCore operators.
//! This class derives directly from mcElement.
class mcOperator : public mcDecoration 
{
	mcDEFINE_ABSTRACT_CLASS(Operator, mcDecoration);
	
public:

	static wxString data_GetOpSymbol(mcElementType t)
		{ return mcOperatorHelpers::data_GetOpSymbol(t); }

	wxString data_GetOpSymbol() const
		{ return hlp()->data_GetOpSymbol(); }

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_ADDOP || t == mcET_MULTOP ||
				t == mcET_SUBOP || t == mcET_DIVOP; }

	mcWRAPPER bool math_CanBeApplied(const mcElement &l, const mcElement &r) const
		{ return hlp()->math_CanBeApplied(l, r); }

	mcWRAPPER mcBasicOpRes math_Apply(mcElement &l, const mcElement &r, mcElement *pp) const
		{ return hlp()->math_Apply(l, r, pp); }

	mcWRAPPER mcBasicOpRes math_ApplySimple(mcElement &l, const mcElement &r) const
		{ return hlp()->math_ApplySimple(l, r); }

	mcWRAPPER mcRealValue math_Evaluate(mcRealValue val, mcRealValue val2) const
		{ return hlp()->math_Evaluate(val, val2); }
};


#endif	// OPERATOR_H


