///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Parenthesis.h
//! \brief  Defines the mcParenthesisData, mcParenthesisGUI, mcParenthesisMath,
//!         mcParenthesisIO, mcParenthesis classes.
//!
//! See Parenthesis.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef PARENTHESIS_H
#define PARENTHESIS_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "Parenthesis.h"
#endif

// required includes
#include "mc/Bracket.h"




mcDEFINE_HELPER_CLASSES(mcParenthesis)

class mcParenthesis;
extern mcParenthesis mcEmptyParenthesis;




//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
class mcParenthesisHelpers : public mcDecorationHelpers
{
	mcDEFINE_REFERENCE_DATA(mcParenthesis, mcET_PARENTHESIS);

#ifdef mcENABLE_DATA
protected:

	//! If TRUE, this class represents a left parenthesis;
	//! otherwise it is a right one.
	bool mdata_bLeftBracket;

public:

	//! The exponent is temporary stored in a mcParenthesis if
	//! it is required to keep it somewhere before creating
	//! a mcBracket.
	mcPolynomial mdata_pTmpExp;

#endif



public:

	mcParenthesisHelpers() { 
		mdata_bLeftBracket = FALSE;
		data_AddProperty(mcEP_NOTIFY_NEIGHBOR_CHANGE);
		data_Init(); 
	}

	virtual ~mcParenthesisHelpers() { /*mcSAFE_DELETE(mdata_pTmpExp) */}



#ifdef mcENABLE_DATA
public:

#ifdef __MCDEBUG__
	wxString data_Debug(long flags) const {
		wxString str = data_GetSymbol();
		if (mcMathCore::Get()->isGUIEnabled())
			str = data_GetSymbol();
		return wxT("mcParenthesis    ") + str + wxT("\n");
	}
#endif

	void data_DeepCopy(const mcElementHelpers *p) {
		const mcParenthesisHelpers *t = (const mcParenthesisHelpers *)p;
		mdata_bLeftBracket = t->mdata_bLeftBracket;
		mdata_pTmpExp.data_DeepCopy(t->mdata_pTmpExp);
		mcDecorationHelpers::data_DeepCopy(p);
	}

	//! Returns the #m_bLeftBracket value.
	bool data_isLeftBracket() const {
		return mdata_bLeftBracket;
	}

	//! Returns the wxT("(") symbol if m_bLeftBracket == TRUE, ")" otherwise.
	//! This function differs from mcParenthesis::gui_GetSymbol() because
	//! the latter can return also wxT("["), wxT("{") or wxT("]"), "}"....
	wxString data_GetSymbol() const {
		return (data_isLeftBracket() ? wxT("(") : wxT(")"));
	}

	void data_SetAsLeftBracket()				{ mdata_bLeftBracket = TRUE; }
	void data_SetAsRightBracket()				{ mdata_bLeftBracket = FALSE; }

	void data_OnNeighborChange();

#endif		// mcENABLE_DATA




	
#ifdef mcENABLE_GUI
protected:

	//! Font used to draw the parentheses (it's updated each time
	//! the content changes its height).
	wxFont mgui_fLastFont;

	//! 
	bool gui_SearchInMonomial(mcMonomial &m, int *h, bool left, int idx);
	int gui_GetMaxHeightToNextBracket(bool left);
	void gui_RebuildFont();
	
public:


	///////////////////////////////////////////////////////////////////
	//! \name mcElementGUI abstract functions.
	//! Some of the functions which are provided by mcDecorationGUI
	//! must be overloaded... @{
	
	bool gui_isBeginKey(const mcKey &ev) const {
		wxChar c = ev.GetKeyCode();
		if (mcBracketHelpers::sgui_bCreateBothSymbols)		// build only mcBrackets
			return FALSE;
		if (mcBracketHelpers::gui_isLeftParenthesis(c) || 
			mcBracketHelpers::gui_isRightParenthesis(c))
			return TRUE;
		return FALSE;
	}

	int gui_Draw(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const;
	mcInputRes gui_Input(const mcKey &key, mcElement *pnew);
	void gui_DoRecalcSize();

	//@}

#endif		// mcENABLE_GUI




#ifdef mcENABLE_IO
public:

	bool io_isBeginTag(const wxXml2Node &tag) const { return FALSE; }
	bool io_isBeginChar(const wxString &str) const;

	bool io_ImportPresentationMathML(wxXml2Node tag, wxString &pErr) { return FALSE;}
	bool io_ImportInlinedExpr(const wxString &str, int *count, wxString &pErr);

	wxString io_GetInlinedExpr() { return data_GetSymbol(); }

#endif		// mcENABLE_IO

};


//! Implements the non-closed parentheses which makes GUI input very
//! easier for the user: if using only mcBracket, then the left
//! and the right parentheses would be built immediately after the
//! the "(" user keypress.
//! mcParenthesis allows the user to type both the wxT("(") and the ")"
//! placing them in the place he wants.
class mcParenthesis : public mcDecoration
{
	mcDEFINE_MAIN_CLASS(Parenthesis, mcDecoration);
	
public:

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_PARENTHESIS; }

	mcWRAPPER bool data_isLeftBracket() const
		{ return hlp()->data_isLeftBracket(); }
	mcWRAPPER wxString data_GetSymbol() const
		{ return hlp()->data_GetSymbol(); }

	mcWRAPPER void data_SetAsLeftBracket()
		{ hlp()->data_SetAsLeftBracket(); }
	mcWRAPPER void data_SetAsRightBracket()
		{ hlp()->data_SetAsRightBracket(); }
};


#endif // PARENTHESIS_H



