///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   PolySolver.h
//! \brief  Defines the mcPolySolver class.
//!
//! See PolySolver.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef POLYSOLVER_H
#define POLYSOLVER_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "PolySolver.h"
#endif

// required includes
#include "mc/Solver.h"
#include "mc/Monomial.h"



// defines
#define mcPOLYSOLVER_MAXDEGREE				10

// defined later
class mcMathLine;


//! The base class for the polynomial solvers.
//! The classes derived from this one, are able to isolate the
//! unknowns from equations which can be written in this form:
//!
//!  a_1*x^n + a_2*x^(n-1) + a_3*x^(n-2) ..... + a_n*x^0 = 0
//!
class mcPolySolver : public mcSolver
{
protected:

	//! The array of the coefficients of the unknowns.
	//! The #m_pCoeff[0] entry contains the coefficient
	//! of #m_pCurrUnk raised to one.
	mcMonomial m_pCoeff[mcPOLYSOLVER_MAXDEGREE];

protected:		// functions

	void math_ResetCoeffArr();
	void math_DeleteCoeffArr();

public:

	mcPolySolver();
	virtual ~mcPolySolver() { math_DeleteCoeffArr(); }


	//! Prepares the given line for real solving, determines the degree
	//! of the given symbol and calls the right solver for the supported
	//! equations degrees.
	bool math_SolveLine(mcMathLine &p, const mcSymbolProperties *unk, mcSystemStepArray &);

	//! Solves equations of the first degree.
	bool math_SolveLineFirstDegree(mcMathLine &p, const mcSymbolProperties *unk,
									mcSystemStepArray &steps);

	//! Solves equations of the second degree.
	bool math_SolveLineSecondDegree(mcMathLine &p, const mcSymbolProperties *unk,
									mcSystemStepArray &steps);

	//! Checks that the given data contains a polynomial equation.
	bool math_WorksOn(const mcMathOrSystem &m) const;


	//! Fills the #m_pCoeff array with the coefficient for the 
	//! \c unk symbol, up to the n-th degree.
	void math_FindPolyCoeff(const mcMathLine &data, const mcSymbolProperties *unk, int n);

	//! A powerful version of #math_FindPolyCoeff: simplifies, moves the data
	//! and calls automatically math_FindPolyCoeff.
	void math_GetPolyCoeff(mcMathLine &p, const mcSymbolProperties *unk,
						 mcSystemStepArray &steps);


	//! A mcPolySolver is always ready to work...
	bool math_isReady() const					{ return TRUE; }

	//! A mcPolySolver does not need any special setting.
	wxPanel *gui_GetOptionsPanel() const		{ return NULL; }
};


#endif		// POLYSOLVER_H
