///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Polynomial.h
//! \brief  Defines the mcPolynomial class.
//!
//! See Polynomial.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef POLYNOMIAL_H
#define POLYNOMIAL_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "Polynomial.h"
#endif

// required includes
#include "mc/ElementArray.h"
#include "mc/Monomial.h"


// defined later
class mcMonomialHelpers;
class mcMonomial;
class mcRadical;

mcDEFINE_HELPER_CLASSES(mcPolynomial)
extern mcPolynomial mcEmptyPolynomial;



//! A useful macro to create mcArrayEntry for the mcPolynomial class.
//! \param midx The DATA index of the monomial to localize
//! \param eidx The DATA index of the element inside the monomial to point
#define mcMAKEPOL_IDX(midx, eidx)		(((midx) << 16) | (eidx))


//! A general polynomial: a container of mcElements.
//! With this class, the user can type an infinite (up to RAM 
//! memory limits) sequence of mcElements. The class is also capable
//! to manage empty boxes...
//! mcPolynomial is one of the most important class of
//! MathCore; in short, what mcPolynomial does, is
//! managing an array of mcElement derived classes
//! letting user to add, remove or modify them...
class mcPolynomialHelpers : public mcElementArrayHelpers
{
	mcDEFINE_REFERENCE_DATA(mcPolynomial, mcET_POLYNOMIAL);

#ifdef mcENABLE_GUI
public:				// customizable variables

	//! A value that, divided by the exponent depth of this element, gives
	//! the number of pixels to leave between all elements.
	static int sgui_nSpaceBetween;

	//! mcPolynomials have different space between ratio from mcMonomials...
	int gui_GetSpaceBetweenRatio() const		
		{ return sgui_nSpaceBetween/(gui_GetExpDepth()+1); }

	//! A special flag used by gui_Input() to search & remove parentheses
	//! only when new ones are added...
	bool mgui_bRemoveParenthesis;
	bool mgui_bMoveCursorAtBracketBegin;

	mcElement mgui_pBracketToReplace;
	bool mgui_bRemoveLeftParenthesis;

#endif


#ifdef mcENABLE_MATH
public:			// static utilities

	//! A mcPolynomial containing a mcNumber with "1.0" as content...
	static mcPolynomial *smath_pOne;

	//! An empty mcPolynomial.
	static mcPolynomial *smath_pEmpty;

#endif



public:

	mcPolynomialHelpers() {	data_Init(); }
	virtual ~mcPolynomialHelpers() {}

protected:

	void gui_Init() {
		mgui_bRemoveParenthesis = FALSE;
		mgui_bMoveCursorAtBracketBegin = FALSE;
		mgui_bRemoveLeftParenthesis = FALSE;
		mcElementArrayHelpers::gui_Init();
	}




#ifdef mcENABLE_DATA
protected:

	int data_GetIdxFromEntry(const mcArrayEntry *p) const
	{ return p->mdata_idx & ~(0xFFFF<<16); }
	int data_GetMonomialIdxFromEntry(const mcArrayEntry *p) const
	{ return (p->mdata_idx >> 16); }

	mcElement &data_GetRefFromEntry(const mcArrayEntry *p)
	{
		int midx = data_GetMonomialIdxFromEntry(p), eidx = data_GetIdxFromEntry(p);
		if (eidx != -1) {
			mcASSERT(data_Get(eidx).data_GetType() == mcET_MONOMIAL, wxT("idx is not valid"));
			return ((mcMonomial &)data_Get(midx)).data_Get(eidx);
		}

		return data_Get(midx);
	}

public:

	int math_GetIdxFromEntry(const mcArrayEntry *p) const
	{ return math_DataToMathIdx(data_GetIdxFromEntry(p)); }
	int math_GetMonomialIdxFromEntry(const mcArrayEntry *p) const
	{ return math_DataToMathIdx(data_GetMonomialIdxFromEntry(p)); }


public:
#ifdef __MCDEBUG__

	//! Checks all the array. In some cases an mcASSERT is used (when the array
	//! incongruences should be caused by programmer's errors), in some others 
	//! the function will repair the error and let everything goes on...
	void data_Check() const;
#endif

	void data_DeepCopy(const mcElementHelpers *e) {
		const mcPolynomialHelpers *p = (const mcPolynomialHelpers *)e;
		mgui_bMoveCursorAtBracketBegin = p->mgui_bMoveCursorAtBracketBegin;
		mgui_bRemoveLeftParenthesis = p->mgui_bRemoveLeftParenthesis;
		mgui_bRemoveParenthesis = p->mgui_bRemoveParenthesis;
		mcElementArrayHelpers::data_DeepCopy(e);
	}




	///////////////////////////////////////////////////////////////////////////////
	//! \name Monomial utilities
	//! mcElementArrayHelpers::data_XXXX-like functions specialized for mcMonomials. @{

	//! Creates a new monomial containing the given array of elements, and then
	//! adds it to this polynomial. Returns the index of the new monomial.
	//! \param contents   The array of the mcElements to add
	//! \param count      The size of the array of mcElement to put in the mcMonomial
	//! \param pos        The position of the new monomial in the array
	//! \param bOverwrite TRUE if you want to overwrite the old mcMonomial which is
	//!                   (eventually) placed in the given position
	int data_AddNewMonomialContaining(mcElement *contents, int count = 1, 
										int pos = -1, bool bOverwrite = FALSE);

	//! Creates a new monomial containing only the given pointer.
	//! This function is exactly like the previous one, just specialized for the
	//! creation of a monomial containing a single element.
	//! \return The index of the new monomial
	int data_AddNewMonomialContaining(mcElement &monomialinit, 
									int pos, bool bOverwrite = FALSE)
		{ return data_AddNewMonomialContaining(&monomialinit, 1, pos, bOverwrite); }

	//! Inserts at \c pos (a copy of) the given monomial.
	//! This is a specialized version of data_AddElements.
	//! \return The index of the just inserted monomial.
	int data_AddNewMonomial(const mcMonomial &toinsert, int pos = -1, 
							bool bOverwrite = FALSE)
		{ return data_AddElements((mcElement*)(&toinsert), 1, pos, bOverwrite); }

	//! Exactly like data_AddNewEmptyMonomial.
	void data_AddNewEmptyBox(int pos)
		{ data_AddNewEmptyMonomial(pos); }

	//! Adds an empty monomial to the array in the given position.
	//! \return The index of the new monomial.
	int data_AddNewEmptyMonomial(int pos = -1);

	//! Inserts at \c pos an element of type \c t, eventually overwriting
	//! (if \c bOverwrite == TRUE) the previous element.
	//! \return The reference to the just created & added element.
	mcElement &data_AddNewWrappedElement(mcElementType t, 
				bool bOverwrite = FALSE, int pos = -1);

	//! Returns the index of the monomial containing the given pointer.
	//! This function works like data_GetIndexOf but it does not search
	//! directly inside the array of this polynomial: it searchs inside
	//! the monomials contained in this array...
	//! \return The index of the monomial found or -1.
	int data_FindMonomialContaining(int occ, const mcElement &elem) const;

	//! Detaches the n-th element of the array, which *must* be a monomial,
	//! with its associated sign (if present).
	//! This function also shifts the array filling the holes left by
	//! the detachment.
	void data_DetachAndShiftMonomial(int n);

	virtual bool data_isWrappingOnly(mcElementType t) const;
	virtual const mcElement &data_GetWrapped(mcElementType t) const;

	//@}



	///////////////////////////////////////////////////////////////////////////////
	//! \name Special functions
	//! Function which highly specialized tasks used by mcPolynomial algorithms. @{

	//! Creates, if necessary, the first operator of this array.
	//! Usually, when the first monomial is positive, then it is not present
	//! in the array which begins immediately with that monomial.
	//! This function forces the array to have an operator as first entry,
	//! even if it is a mcAddOp.
	//! \return TRUE if the operator was added.
	bool data_CreateFirstOp();

	//! Deletes, if present, the first operator of this array.
	//! This function removes the first operator even if it is not a mcAddOp;
	//! so be careful to call this function !!
	void data_DeleteFirstOp();
	
	//! Searches a pairs of mcParenthesis and, if possible, replaces them
	//! with mcBrackets.
	//! \param bMoveAtBracketEnd If TRUE, then the cursor of the monomial
	//!        containing the new brackets (if GUI is enabled) is moved at
	//!        the end of the bracket; otherwise, it is placed at the
	//!        beginning of the bracket.
	//! \return TRUE if a pair of mcParenthesis have been replaced;
	//!         FALSE if there are no valid pairs of mcParenthesis.
	//!         This happens, for example, in the following cases:
	//!            ((...+....-...(....       or
	//!            ((...)+...    [inner parentheses are hold by a mcBracket]
	bool data_ReplaceParentheses(bool bMoveAtBracketEnd);
	bool data_ReplaceParentheses(int, int, int, int, bool);

	//! Searches *all* the pairs of mcParentheses which can be replaced
	//! by mcBrackets and replaces them.
	//! If this function is called when GUI section is active, a call
	//! to gui_SetCursorPos(mcCP_END) is strongly suggested.
	void data_ReplaceAllParentheses();

	//@}

#endif		// mcENABLE_DATA



#ifdef mcENABLE_GUI
public:

	/////////////////////////////////////////////////////////////////////////////
	//! \name mcElement overloaded functions
	//! Implements abstract mcElement functions. @{

	//! This function should never be called (except by MathCore::isCharBeginChar()) 
	//! because mcMonomials should always
	//! be a member variable of some other class; this class cannot be
	//! allocated by the user.
	bool gui_isBeginKey(const mcKey &) const		{ return FALSE; }

	//! Like mcElement::gui_Draw, but you can specify a center line
	//! to use to calculate the cursor position.
	//! \param dc The dc where this element will be drawn
	//! \param x,y The coordinates where this element will be drawn
	//! \param pt The mouse cursor position or one of the mcDRW_*** flags
	//! \param cl The center line to use, or -1 to let the function
	//!        calculate it
	int gui_ExDraw(wxDC &dc, int x, int y, long flags, const wxPoint &pt, int cl) const;

	//! Like mcElement::gui_OnSelect, but you can specify a center line
	//! to use to calculate the cursor position.
	//! \param dc The dc where this element is drawn
	//! \param x,y The coordinates where this element is drawn
	//! \param rc The selection rectangle created by the user
	//! \param cl The center line to use, or -1 to let the function
	//!        calculate it
	void gui_ExOnSelect(wxDC &dc, wxRect &rc, int cl);

	//! Handles the returned codes of the mcMonomial::gui_Input() function:
	//! mcIR_DELETE_THIS, mcIR_REPLACE_THIS and mcIR_DELETE_PREVIOUS,
	//! mcIR_DELETE_NEXT.
	mcInputRes gui_BackInput(const mcKey &key, mcElement *pnewelem, int n);

	//! Handles two different mcPolynomialGUI-specific problems:
	//! 1) first operator 
	//! 2) mcParenthesis replacement with mcBracket
	mcInputRes gui_Input(const mcKey &key, mcElement *pnewelem);

	mcInsertRes gui_Insert(const mcElement &toinsert, mcElement *);
	
	
	void gui_DoSplit(mcElementType type, const mcKey &ev);
	void gui_SetCursorPos(const mcCursorPos &code);

	void gui_OnSelect(wxDC &dc, wxRect &rc) {
		// call the overloaded version;
		gui_ExOnSelect(dc, rc, -1);
	}

	int gui_Draw(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const {
		// call the overloaded version
		return gui_ExDraw(dc, x, y, flags, pt, -1);
	}
	
	//@}



	/////////////////////////////////////////////////////////////////////////////////
	//! \name mcPolynomial functions
	//! Functions which are overridden by mcElementArray or are just new. @{

	//! See #io_isBeginChar function.
	bool gui_isArrEndKey(const mcKey &) const			{ return FALSE; }

	//! Returns TRUE if the array doesn't contain elements or it contains only
	//! the first operator without any elements following it....
	//! This function is different from mcElementArray::data_isArrayEmpty():
	//! it also checks for the first operator presence/absence.
	bool gui_isArrayEmpty();

	//! Checks the cursor position. If the cursor is not in the right position,
	//! the program stops.
	void gui_CheckCursorPos() const;

	//! This functions sets the #m_bRemoveParenthesis flag and thus allow
	//! the mcPolynomial::gui_Input() function to remove the parentheses
	//! the next time the function is run...
	void gui_OnNewParenthesis(bool leftpar) { 
		mgui_bRemoveParenthesis = TRUE;
		mgui_bMoveCursorAtBracketBegin = leftpar;
	}

	void gui_SetBracketToReplace(const mcBracket &p, bool leftpar) {
		mgui_pBracketToReplace = (const mcElement &)p;
		mgui_bRemoveLeftParenthesis = leftpar;
	}

	//! math_Finds the given mcBracket and then replaces it with a left
	//! (if \c left == TRUE) mcParenthesis or right (if \c left == FALSE)
	//! mcParenthesis...
	void gui_ReplaceBracket(const mcBracket &, bool left);

	// these two are very different from data_XXXX corrispondents.
	int gui_AddNewElement(mcElementType type, const mcKey &key, int pos = -1);	

	void gui_AddNewEmptyMonomial(int pos = -1);

	//@}

#endif		// mcENABLE_GUI




#ifdef mcENABLE_MATH
protected:

	//! Reorders (tries to reorder) the array for the first unknown 
	//! this polynomial contains (if possible):
	//! 
	//! BEFORE:   5a + 2 - x^4 + x
	//! AFTER:    - x^4 + x + 5a + 2
	//!
	//! where x is the only unknown registered.
	mcExpSimRes math_Reorder();

	//! Expands the bracket placed in the \c "bridx" math entry of the
	//! monomial placed in the \c "mdataidx" entry (expressed in data-coords).
	//! The expansion takes place considering all elements placed before and
	//! after the bracket and thus generating a mathematically correct result.
	void math_DistributeBracket(int mdataidx, int bridx);

	//! See the topic "Sign management in mcPolynomial".
	virtual void math_EndSimSteps() {/* math_SyncCoeffWithSigns();*/ }

	//! See the topic "Sign management in mcPolynomial".
	virtual void math_EndExpSteps() { /*math_SyncCoeffWithSigns();*/ }

	//! See the topic "Sign management in mcPolynomial".
	mcExpSimRes math_SimplifySolveOp(long flags);


public:

	//////////////////////////////////////////////////////////////////////////
	//! \name Monomial-specific functions
	//! Functions which work with the monomials of the array. @{

	//! Returns the number of the monomials contained in the array.
	//! This function should always return a different
	//! value from the number of the elements contained in the array
	//! (in fact, the array contains also the + or - operators).
	//! \note This is a fastest implementation of the general one
	//!       given by mcElementArray::math_GetCount()
	virtual int math_GetCount() const 
		{ return data_GetNumOfElemType(mcET_MONOMIAL); }

	//! Returns the n-th monomial of the array.
	const mcMonomial &math_Get(int n) const 
		{ return (const mcMonomial &)(data_GetElemOfType(n, mcET_MONOMIAL)); }
	mcMonomial &math_Get(int n)
		{ return (mcMonomial &)data_GetElemOfType(n, mcET_MONOMIAL); }

	//const mcMonomialHelpers *math_GetHlp(int n) const;
	//mcMonomialHelpers *math_GetHlp(int n);

	virtual bool math_isWrappingOnly(mcElementType t) const;
	virtual const mcElement &math_GetWrapped(mcElementType t) const;

	//! Returns the math-index of the monomial containing the given pointer.
	//! This function is different from mcPolynomial::data_math_FindMonomialContaining
	//! because it uses the mcElementArray::math_FindElemIdx() function.
	//! \param n If not NULL, in this pointer is stored the 
	int math_FindMonomialContaining(const mcElement &elem, int occ = 0, int *n = NULL) const;

	//! Returns the monomial containing the occ-th occurrence of the given element.
	//! If not NULL, stores in \c n the math-index of that element in the returned
	//! monomial.
	const mcMonomial &math_GetMonomialContaining(const mcElement &elem, 
										int occ = 0, int *n = NULL) const;

	//! Returns the mathidx-th monomial without all the elements of the
	//! \c toremove array (which must be of size \c ntoremove) that are
	//! #Delete()d. If the sign of the monomial is negative, then the
	//! functions packs it inside the returned monomial (i.e. multiplies
	//! by -1 the monomial).
	mcMonomial math_GetCoeffOf(int mathidx, const mcElement *toremove, int ntoremove) const;

	//! Returns the coefficient of the occ-th occurrence of the monomial containing
	//! a mcSymbol linked to the given mcSymbolProperties object and with the given
	//! polynomial as exponent.
	mcMonomial math_GetCoeffOf(const mcSymbolProperties *sym, const mcPolynomial &exp, int occ = 0) const;

	//! Works like #GetCoeffOf(mcSymbolProperties *, const mcPolynomial &, int) but
	//! automatically creates the polynomial to use as exponent initializing it with
	//! the given mcRealValue.
	mcMonomial math_GetCoeffOf(const mcSymbolProperties *sym, const mcRealValue &exp, int occ = 0) const;

	//void math_AddMonomial(const mcMonomial &p);
	//void math_Monomial(const mcMonomial &p);

	//! Returns the math-index of the given monomial, if it is contained directly
	//! (this function differs from mcElement::math_Find() because it is not recursive)
	//! in the array.
	//int GetIndexOf(const mcMonomial &tofind) const;

	//! Returns TRUE if this polynomial contains only a single monomial
	//! and thus it contains only factors.
	bool math_isFactorized() const		{ return math_GetCount() == 1; }

	//! Returns TRUE if the sign of the first monomial is negative.
	//! This is typically used by containers of mcPolynomials to
	//! decide if the sign should be changed or not.
	bool math_isFirstMonomialNegative() const
		{ if (data_isArrayEmpty()) return FALSE; else return (math_GetSign(0)==-1); }


	//! Acts so 
	void math_Factorize(int *n, int count);

	void math_FactoreOut(int *n, int count, bool bForceUseless);
	void math_FactoreOutAll(bool bForceUseless) 
		{ math_FactoreOut(NULL, -1, bForceUseless); }

	void math_FactoreOut(const mcSymbolProperties *unk, const mcPolynomial &pol, bool bForceUseless);
	void math_FactoreOutFreeOf(const mcSymbolProperties *unk, const mcPolynomial &pol, bool bForceUseless);

	void math_FactoreOut(const mcSymbolArray *arr);
	void math_FactoreOutUnknowns();
	void math_FactoreOutParameters();
	void math_FactoreOutConstants();


	mcArrayEntry math_EmbedInBracket(int *n, int count);

	//! This function works like mcElementArrayHelpers::math_GetIndexOf but
	//! accepts another parameter, "positivesign" which allows the caller
	//! to search for a monomial with the specified sign.
	mcArrayEntry math_GetIndexOf(int occ, const mcMonomial &tofind, bool positivesign) const;

	//! 
	//void math_ExpandContaining(
	mcExpSimRes math_HandleExpSimFlag(mcExpSimRes r, mcElement *pnew, int i);

	//@}



	////////////////////////////////////////////////////////////////////////////
	//! \name mcElementArrayMath abstract function implementation
	//! Implements those functions defined as pure virtual by parent class. @{

	//! Converts from data-index to math-index.
	int math_DataToMathIdx(int dataindex) const;

	//! Converts from math-index to data-index.
	int math_MathToDataIdx(int mathindex) const;

	mcElementType math_GetNeutralOpType() const		{ return mcET_ADDOP; }
	double math_GetNeutralValue() const				{ return 0.0; }

	mcPolynomial math_GetPolynomialWrapper() const;
	mcArrayEntry math_WrapMonomial(const mcMonomial &);
	mcArrayEntry math_WrapSimple(const mcElement &);
	mcElementArray math_CreateWrapperFor(const mcElement &p) const;

	//void &math_GetFactors();

	//! Returns the maximum degree for the given symbol retrieved by all the
	//! monomials contained in this array. See mcMonomial::GetMaxDegreeFor
	//! for more info.
	mcIntegerValue math_GetMaxDegreeFor(const mcSymbolProperties *) const;

	//@}



	////////////////////////////////////////////////////////////////////////////
	//! \name Sign-management
	//! Functions working on the operators preceding mcMonomials. @{

	//! Returns the sign of the n-th monomial (-1 if negative, 1 if positive).
	int math_GetSign(int n) const;

	//! Returns mcNumber::math_pOne/mcNumber::math_pMinusOne if
	//! the occ-th occurrence of \c m is positive/negative.
	//! Returns NULL if the given monomial could not be found.
	const mcNumber &math_GetNumSign(const mcMonomial &m, int occ = 0) const;

	//! Sets the sign of the n-th monomial to sgn.
	void math_SetSign(int n, int sgn);

	//! Changes the sign of the n-th monomial.
	//! \param n The zero-based math-index of monomial whose sign must be changed.
	void math_ChangeSign(int n)		{ math_SetSign(n, -math_GetSign(n)); }

	//! Changes the sign of all the monomials of the array.
	void math_ChangeAllSigns();

	//! Synchronizes the coefficients of all monomials with the mcAddSubOp
	//! placed immediately before them.
	void math_SyncCoeffWithSigns();

	//! Packs the sign of the given monomial reading the sign of the
	//! occ-th monomial identic to it belonging to this array.
	void math_PackSign(mcMonomial &m, int occ = 0) const;
	
	//! Deletes the first operator of the array, if present and if it is a mcAddOp.
	//! Returns TRUE if the operator was deleted or FALSE otherwise.
	bool math_DeleteFirstOp();

	//@}



	//////////////////////////////////////////////////////////////////////////
	//! \name gui_Selection-related math functions
	//! Function working on the selected portions of elements only. @{

	int math_GetSelCount() const 
		{ return gui_GetSelElemCountOfType(mcET_MONOMIAL); }

	int math_GetSelIndex(int n) const 
		{ return gui_GetSelElemIndexOfType(n, mcET_MONOMIAL); }

	const mcMonomial &math_GetSel(int n) const 
		{ return (const mcMonomial &)gui_GetSelElemOfType(n, mcET_MONOMIAL); }

	mcMonomial &math_GetSel(int n) 
		{ return (mcMonomial &)gui_GetSelElemOfType(n, mcET_MONOMIAL); }

	void math_DeleteSelection();

	//@}



	//////////////////////////////////////////////////////////////////////////////
	//! \name mcElementMath abstract functions
	//! Implementation of common math functions. @{
	
	mcBasicOpRes math_Add(const mcElement &, mcElement *p, bool add);
	mcBasicOpRes math_Subtract(const mcElement &, mcElement *p);
	mcBasicOpRes math_MultiplyBy(const mcElement &, mcElement *p);
	mcBasicOpRes math_DivideBy(const mcElement &, mcElement *p);
	
	mcMonomial math_GetLCM(const mcElement &p) const;
	mcMonomial math_GetGCD(const mcElement &p) const;

	mcMonomial math_GetGCDFor(int *n, int count) const;
	
	mcExpSimRes math_Simplify(long flags, mcElement *newelem);
 	mcExpSimRes math_Simplify(long flags)
		{ return math_Simplify(flags, NULL); }

	mcIntegerValue math_RaiseGetCoeff(int *indexes, int m);
	bool math_CanBeRaisedTo(const mcPolynomial &) const;
	
	void math_RaiseTo(const mcRealValue &n);
	mcBasicOpRes math_RaiseTo(const mcPolynomial &);

	//! This function is used by mcPolynomial on general elements
	//! but it does not make sense for mcPolynomial itself: they
	//! should never be "reordered".
	virtual int math_GetOrderPos() const			{ return -1; }
	
	//@}
#endif		// mcENABLE_MATH



#ifdef mcENABLE_IO
public:

	wxXml2Node io_GetMathML(bool bGetPresentation) const;
	wxString io_GetInlinedExpr() const;

	bool io_ImportPresentationMathML(wxXml2Node tag, wxString &pErr);
	bool io_ImportInlinedExpr(const wxString &todecode, int *, wxString &pErr, int mlimit);
	bool io_ImportInlinedExpr(const wxString &todecode, int *p, wxString &pErr)
		{ return io_ImportInlinedExpr(todecode, p, pErr, -1); }

#endif		// mcENABLE_IO
};



//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
class mcPolynomial : public mcElementArray
{
	mcDEFINE_MAIN_CLASS(Polynomial, mcElementArray);

public:

	// these constructors must use only data_**** functions
	mcPolynomial(const mcMonomial &towrap)
	{ data_SetRefData(new mcPolynomialHelpers); data_AddElements(&towrap, 1); }

	// math constructors
	mcPolynomial(const mcSymbol &towrap)
	{ data_SetRefData(new mcPolynomialHelpers); data_AddNewMonomialContaining((mcElement *)&towrap, 1); }
	mcPolynomial(const mcNumber &towrap)
	{ data_SetRefData(new mcPolynomialHelpers); data_AddNewMonomialContaining((mcElement *)&towrap, 1); }
	mcPolynomial(const mcSymbolProperties *towrap)
	{ data_SetRefData(new mcPolynomialHelpers); math_WrapSymbol(towrap); }
	mcPolynomial(const mcRealValue &towrap)
	{ data_SetRefData(new mcPolynomialHelpers); math_WrapNumber(towrap); }

	mcPolynomial(const mcRadical &towrap)
	{ data_SetRefData(new mcPolynomialHelpers); data_AddNewMonomialContaining((mcElement *)&towrap, 1); }
	mcPolynomial(const mcBracket &towrap)
	{ data_SetRefData(new mcPolynomialHelpers); data_AddNewMonomialContaining((mcElement *)&towrap, 1); }
	mcPolynomial(const mcFraction &towrap)
	{ data_SetRefData(new mcPolynomialHelpers); data_AddNewMonomialContaining((mcElement *)&towrap, 1); }


	// unary negation
	mcPolynomial &operator-()
		{ this->math_ChangeAllSigns(); return *this; }


#ifdef mcENABLE_DATA
public:

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_POLYNOMIAL; }

	mcWRAPPER bool data_CreateFirstOp()
		{ return hlp()->data_CreateFirstOp(); }
	mcWRAPPER void data_DeleteFirstOp()
		{ hlp()->data_DeleteFirstOp(); }

	mcWRAPPER int data_AddNewMonomialContaining(mcElement *contents, 
								int n = 1, int pos = -1, bool bOverwrite = FALSE)
		{ return hlp()->data_AddNewMonomialContaining(contents, n, pos, bOverwrite); }

	mcWRAPPER int data_AddNewMonomialContaining(mcElement &monomialinit, 
								int pos = -1, bool bOverwrite = TRUE)
		{ return hlp()->data_AddNewMonomialContaining(monomialinit, pos, bOverwrite); }

	mcWRAPPER int data_AddNewMonomial(const mcMonomial &toinsert, 
								int pos = -1, bool bOverwrite = FALSE)
		{ return hlp()->data_AddNewMonomial(toinsert, pos, bOverwrite); }

	mcWRAPPER int data_AddNewEmptyMonomial(int pos = -1)
		{ return hlp()->data_AddNewEmptyMonomial(pos); }
/*
	mcWRAPPER mcElement &data_AddNewWrappedElement(mcElementType t, bool bOverwrite = FALSE, int pos = -1);
	mcWRAPPER int data_FindMonomialContaining(const mcElement &elem, int occ = 0) const;
	mcWRAPPER void data_DetachAndShiftMonomial(int n);*/	

#endif

	 
#ifdef mcENABLE_GUI
public:

	mcWRAPPER void gui_AddNewEmptyMonomial(int pos = -1)
		{ hlp()->gui_AddNewEmptyMonomial(pos); }

	mcWRAPPER int gui_ExDraw(wxDC &dc, int x, int y, long flags, const wxPoint &pt, int cl) const
		{ return hlp()->gui_ExDraw(dc, x, y, flags, pt, cl); }
	
/*	
	int gui_ExDraw(wxDC &dc, int x, int y, long flags, const wxPoint &pt, int cl) const;
	void gui_ExOnSelect(wxDC &dc, int x, int y, wxRect &rc, int cl);
	mcInputRes gui_BackInput(const mcKey &key, mcElement *pnewelem, int n);
	mcInputRes gui_Input(const mcKey &key, mcElement *pnewelem);
	void gui_DoSplit(mcElementType type, const mcKey &ev);
	void gui_SetCursorPos(const mcCursorPos &code);

	void gui_OnSelect(wxDC &dc, wxRect &rc) {
		// call the overloaded version;
		gui_ExOnSelect(dc, x, y, rc, -1);
	}

	int gui_Draw(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const {
		// call the overloaded version
		return gui_ExDraw(dc, x, y, flags, pt, -1);
	}
	bool gui_isArrEndKey(const mcKey &) const			{ return FALSE; }
	bool gui_isArrayEmpty();
	void gui_CheckCursorPos() const;
	void gui_OnNewParenthesis(bool leftpar) { 
		mgui_bRemoveParenthesis = TRUE;
		mgui_bMoveCursorAtBracketBegin = leftpar;
	}
	void gui_SetBracketToReplace(const mcBracket &p, bool leftpar) {
		mgui_pBracketToReplace = (const mcElement &)p;
		mgui_bRemoveLeftParenthesis = leftpar;
	}
	void gui_ReplaceBracket(const mcBracket &, bool left);
	int gui_AddNewElement(mcElementType type, const mcKey &key, int pos = -1);	
	void gui_AddNewEmptyMonomial(int pos = -1);
		//{ mgui_nCursorPos = data_AddNewEmptyMonomial(pos); }
*/
#endif


	mcWRAPPER const mcMonomial &math_Get(int n) const 
		{ return hlp()->math_Get(n); }
	mcWRAPPER mcMonomial &math_Get(int n)
		{ return hlp()->math_Get(n); }


	mcWRAPPER int math_FindMonomialContaining(const mcElement &elem, int occ = 0, int *n = NULL) const
		{ return hlp()->math_FindMonomialContaining(elem, occ, n); }
	mcWRAPPER const mcMonomial &math_GetMonomialContaining(const mcElement &elem, int occ = 0, int *n = NULL) const
		{ return hlp()->math_GetMonomialContaining(elem, occ, n); }
	mcWRAPPER mcMonomial math_GetCoeffOf(int mathidx, const mcElement *toremove, int ntoremove) const
		{ return hlp()->math_GetCoeffOf(mathidx, toremove, ntoremove); }
	mcWRAPPER mcMonomial math_GetCoeffOf(const mcSymbolProperties *sym, const mcPolynomial &exp, int occ = 0) const
		{ return hlp()->math_GetCoeffOf(sym, exp, occ); }
	mcWRAPPER mcMonomial math_GetCoeffOf(const mcSymbolProperties *sym, const mcRealValue &exp, int occ = 0) const
		{ return hlp()->math_GetCoeffOf(sym, exp, occ); }


	mcWRAPPER bool math_isFactorized() const
		{ return hlp()->math_isFactorized(); }
	mcWRAPPER bool math_isFirstMonomialNegative() const
		{ return hlp()->math_isFirstMonomialNegative(); }
	mcWRAPPER mcMonomial math_GetFactors() const
		{ return ((mcElementArrayHelpers*)hlp())->math_GetFactors(); }

	mcWRAPPER void math_Factorize(int *n, int count)
		{ hlp()->math_Factorize(n, count); }
	mcWRAPPER void math_FactoreOut(int *n, int count, bool bForceUseless)
		{ hlp()->math_FactoreOut(n, count, bForceUseless); }
	mcWRAPPER void math_FactoreOutAll(bool bForceUseless)
		{ hlp()->math_FactoreOutAll(bForceUseless); }


	mcWRAPPER void math_FactoreOut(const mcSymbolProperties *unk, const mcPolynomial &pol, bool bForceUseless)
		{ hlp()->math_FactoreOut(unk, pol, bForceUseless); }
	mcWRAPPER void math_FactoreOutFreeOf(const mcSymbolProperties *unk, const mcPolynomial &pol, bool bForceUseless)
		{ hlp()->math_FactoreOutFreeOf(unk, pol, bForceUseless); }

	mcWRAPPER void math_FactoreOut(const mcSymbolArray *arr)
		{ hlp()->math_FactoreOut(arr); }
	mcWRAPPER void math_FactoreOutUnknowns()
		{ hlp()->math_FactoreOutUnknowns(); }
	mcWRAPPER void math_FactoreOutParameters()
		{ hlp()->math_FactoreOutParameters(); }
	mcWRAPPER void math_FactoreOutConstants()
		{ hlp()->math_FactoreOutConstants(); }


	mcWRAPPER mcIntegerValue math_GetMaxDegreeFor(const mcSymbolProperties *s) const
		{ return hlp()->math_GetMaxDegreeFor(s); }


	mcWRAPPER int math_GetSign(int n) const
		{ return hlp()->math_GetSign(n); }
	mcWRAPPER const mcNumber &math_GetNumSign(const mcMonomial &m, int occ = 0) const
		{ return hlp()->math_GetNumSign(m, occ); }

	mcWRAPPER void math_SetSign(int n, int sgn)
		{ hlp()->math_SetSign(n, sgn); } 
	mcWRAPPER void math_ChangeSign(int n)
		{ hlp()->math_ChangeSign(n); } 
	mcWRAPPER void math_ChangeAllSigns()
		{ hlp()->math_ChangeAllSigns(); } 
	mcWRAPPER void math_PackSign(mcMonomial &m, int occ = 0) const
		{ hlp()->math_PackSign(m, occ); } 
	mcWRAPPER bool math_DeleteFirstOp()
		{ return hlp()->math_DeleteFirstOp(); } 

	mcWRAPPER bool math_CanBeRaisedTo(const mcPolynomial &n)
		{ return hlp()->math_CanBeRaisedTo(n); } 

	mcWRAPPER mcArrayEntry math_GetIndexOf(int occ, const mcMonomial &tofind, bool b) const
	{ return hlp()->math_GetIndexOf(occ, tofind, b); }

	mcWRAPPER int math_GetMonomialIdxFromEntry(const mcArrayEntry &p) const
		{ return hlp()->math_GetMonomialIdxFromEntry(&p); }
	mcWRAPPER int math_GetIdxFromEntry(const mcArrayEntry &p) const
		{ return hlp()->math_GetIdxFromEntry(&p); }
};


#endif	// POLYNOMIAL_H

