///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Radical.h
//! \brief  Defines the mcRadicalData, mcRadicalGUI, mcRadicalMath,
//!         mcRadicalIO classes.
//!
//! See Radical.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef RADICAL_H
#define RADICAL_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "Radical.h"
#endif

// required includes
#include "mc/ExpContainer.h"
#include "mc/Polynomial.h"
#include "mc/Function.h"




mcDEFINE_HELPER_CLASSES(mcRadical)


//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
class mcRadicalHelpers : public mcExpContainerHelpers
{
	mcDEFINE_REFERENCE_DATA(mcRadical, mcET_RADICAL);

#ifdef mcENABLE_GUI
public:				// customizable variables

	//! The key to use to create this element.
	static mcKey *sgui_pNewRadical;

	static int sgui_nSpaceAboveBelow;
	static int sgui_nSpaceLeftRight;

	virtual int gui_GetSpaceAboveBelow() const	 { return sgui_nSpaceAboveBelow; }
	virtual int gui_GetSpaceLeftRight() const	 { return sgui_nSpaceLeftRight; }
#endif


#ifdef mcENABLE_IO
public:		// customizable variables

	static wxString sio_strSquareRoot;
	static wxString sio_strGenericRoot;
#endif



public:

	mcRadicalHelpers() {

		// mcRadicals uses the mcExpElement's exponent as
		// an index, placed on the left.
		mgui_bExpRight = FALSE;
		data_Init();
	}

	virtual ~mcRadicalHelpers() {}
	


#ifdef mcENABLE_DATA
public:

#ifdef __MCDEBUG__

	wxString data_BaseDebug(long flags) const {
		return wxT("mcRadical:\n") + data_GetContent().data_GetDebug(
						mcMathCore::Get()->m_nIndentationStep, flags);
	}

#endif
#endif		// mcENABLE_DATA



#ifdef mcENABLE_GUI
public:

	//! Returns the width of the 'V' of the radical symbol.
	int gui_GetVWidth() const;


	////////////////////////////////////////////////////////////////////
	//! \name mcExpContainer overloaded functions
	//! Functions which implements the mcExpContainer abstract functions
	//! modifyng the default behaviour of the exponent, transforming it
	//! in the index of the radical. @{

	bool gui_isBeginKey(const mcKey &key) const;
	void gui_DrawContainer(wxDC &, int x, int y, long flags, const wxPoint &) const;
	//int gui_DrawBase(wxDC &, int x, int y, const wxPoint &);
	//int gui_GetRelCursorPos(wxDC &dc, wxPoint *);
	//int gui_GetYAnchor() const;

	//! The left position of a mcRadical depends from the presence
	//! of the index...
	bool gui_isLeftPosEnabled() const;

	//! To insert something outside a mcRadical, you need to move
	//! the cursor out of the root symbol.
	bool gui_isRightPosEnabled() const		 { return TRUE; }

	//mcMoveCursorRes gui_BaseMoveCursor(mcMoveCursorFlag flag, long modifiers);
	//int gui_BaseMoveCursorUsingPoint(wxDC &, const wxPoint &);
	void gui_DoRecalcBaseSize();

	int gui_GetExpOffsetx() const;
	int gui_GetContentOffsetX() const;
	int gui_GetContentOffsetY() const;

	//! VERY IMPORTANT: a mcRadical doesn't draw the base as mcExpElement
	//! normally expect; it draws the.
	int gui_GetBaseOffsety() const	{ return 0; }

	//@}

#endif		// mcENABLE_GUI



#ifdef mcENABLE_MATH
public:

	// the exp of a mcRadicalMath is called index
	bool math_hasIdx() const							{ return math_hasExp(); }
	mcPolynomial &math_GetIdx()							{ return math_GetExp(); }
	const mcPolynomial &math_GetIdx() const				{ return math_GetExp(); }

	mcRealValue math_Evaluate() const;
	mcMathType math_GetBaseMathType() const;
	//mcBasicOpRes math_MakeReciprocal(mcElement *);

	mcExpSimRes math_SimplifyContents(long flags, mcElement *newelem);
	mcExpSimRes math_ExpandContents(long flags, mcElement *newelem);
	
	//! A mcRadical has a complexity/lenght of 1.0 that is, twice the
	//! lenght of an element with a 
	//mcRealValue math_GetBaseLenght() const { return mcExpContainer::math_GetBaseLenght()+1; }

	int math_GetOrderPos() const		{ return 3; }

#endif		// mcENABLE_MATH



#ifdef mcENABLE_IO
public:

	bool io_isBeginTag(const wxXml2Node &tag) const {
		if (tag.GetName() == sio_strSquareRoot || 
			tag.GetName() == sio_strGenericRoot)
			return TRUE;
		return FALSE;
	}

	static bool io_isRadicalBeginChar(const wxString &str) {
		if (str.StartsWith(sio_strSquareRoot) ||
			str.StartsWith(sio_strGenericRoot))
			return TRUE;
		return FALSE;
	}

	bool io_isBeginChar(const wxString &str) const 
		{ return io_isRadicalBeginChar(str); }
	
	// MATH ML functions
	wxXml2Node io_GetMathML(bool bGetPresentation) const;
	wxXml2Node io_GetBaseMathML(bool bGetPresentation) const;
	bool io_ImportPresentationMathML(wxXml2Node tag, wxString &pErr);

	wxString io_GetInlinedExpr() const;
	wxString io_GetBaseInlinedExpr() const { return wxEmptyString; }
	bool io_ImportBaseInlinedExpr(const wxString &, int *, wxString &);

#endif		// mcENABLE_IO
};


//! mcRadical implements the radicals using mcPolynomial 
//! to hold as content any polynomial
//! the user can input... in addition this class also
//! stores an index modifiable by the user using the KEY_EDITEXP...	
class mcRadical : public mcExpContainer
{
	mcDEFINE_MAIN_CLASS(Radical, mcExpContainer);

public:

	mcRadical(const mcPolynomial &contents)
	{ data_SetRefData(new mcRadicalHelpers()); data_SetContent(contents); }

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_RADICAL; }
};

#endif	// RADICAL_H

