///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Range.h
//! \brief  Defines the mcRange and mcExtRange classes.
//!
//! See Range.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef RANGE_H
#define RANGE_H

// optimization for GCC compiler
#ifdef __GNUG__
//#pragma interface "Range.h"
#endif

// required includes
#include "mc/Polynomial.h"


//! A mathematical range where something (for example a parameter
//! or an unknown) can vary.
class mcRange : public mcMathHelper
{
protected:

	//! The upper limit of this range. This should never be NULL.
	mcPolynomial m_pUpper;
	
	//! The lower limit of this range. This should never be NULL.
	mcPolynomial m_pLower;
	
	//! One of the boundaries-mode for this range. See #mcBoundaryMode.
	mcBoundaryMode m_nBounds;
	
	//! The set to which the upper and lower limits belong.
	mcSet m_nSet;

public:		// static public objects

	static mcRange *pN;		//!< A range which goes from zero to +inf of N.
	static mcRange *pZ;		//!< A range which goes from -inf to +inf of Z.
	static mcRange *pR;		//!< A range which goes from -inf to +inf of R.
	static mcRange *pC;		//!< A range which goes from -inf to +inf of C.	
	
	static mcRange *pEmpty;		//!< A range which goes from zero to zero of N.
	
public:
	
	//! Builds this range object with polynomials containing only the given values.
	mcRange(const mcRealValue &l = *mcIntegerValue::pZero,
		const mcRealValue &u = *mcIntegerValue::pOne,
		mcSet s = mcSET_R,
		mcBoundaryMode m = mcBM_INCLUDE_BOTH)
	{ math_Set(l, u); math_Set(s, m); }

	//! Builds this range object copying the given upper & lower limits.
	mcRange(const mcPolynomial &l,
		const mcPolynomial &u,
		mcSet s = mcSET_R,
		mcBoundaryMode m = mcBM_INCLUDE_BOTH)
	{ math_Set(l, u); math_Set(s, m); }
	
	virtual ~mcRange();


	//! Returns a clone of this class.
	mcRange *data_Clone() const 
		{ return new mcRange(m_pLower, m_pUpper, m_nSet, m_nBounds); }
	
	//! Swaps the two limits, if required (because the #m_pUpper
	//! contains the lower limit and viceversa).
	void math_Order();

	//! Returns TRUE if this range has a finite lower and upper limit.
	bool math_isFinite() const;


	void math_Set(const mcPolynomial &l = mcEmptyElement, 
			const mcPolynomial &u = mcEmptyElement);
	void math_Set(const mcRealValue &l = *mcRealValue::pNAN, 
			const mcRealValue &u = *mcRealValue::pNAN);
	void math_Set(mcSet newset, mcBoundaryMode m) 
		{ m_nSet=newset; m_nBounds=m; }

	mcSet math_GetSet() const							{ return m_nSet; }
	mcBoundaryMode math_GetBoundaryMode() const			{ return m_nBounds; }
	const mcPolynomial &math_GetUpperLimit() const		{ return m_pUpper; }
	const mcPolynomial &math_GetLowerLimit() const		{ return m_pLower; }
	
	mcRealValue math_GetUpperLimitValue() const;
	mcRealValue math_GetLowerLimitValue() const;

	//! Returns the inlined expression which represents this object.
	//! This is typically a string like
	//!          [a,b]   or   [a,b)   or   (a,b]   or   (a,b)
	//! for the different values of #m_nSet.
	wxString io_GetInlinedExpr() const;
};


//! A container of various mcRanges defining a wider range
//! through union or a smaller one through intersection.
class mcExtRange : public mcAbstractArray
{
protected:
	void data_Delete(int n)	
		{ delete data_GetRange(n); }

public:		// static public objects

	static mcExtRange *pN;		//!< A range which goes from zero to +inf of N.
	static mcExtRange *pZ;		//!< A range which goes from -inf to +inf of Z.
	static mcExtRange *pR;		//!< A range which goes from -inf to +inf of R.
	static mcExtRange *pC;		//!< A range which goes from -inf to +inf of C.	
	
	static mcExtRange *pEmpty;		//!< A range which goes from zero to zero of N.
	
public:
	mcExtRange() {}
	mcExtRange(const mcRange &r) { data_Add(r); }
	mcExtRange(const mcExtRange &r) { data_DeepCopy(r); }
	virtual ~mcExtRange() { data_Clear(); }
	

	void data_DeepCopy(const mcExtRange &r) {
		data_Clear();
		for (int i=0; i < r.data_GetCount(); i++)
			data_Add(*r.data_GetRange(i));
	}

	void data_Add(const mcRange &r)			
		{ mcAbstractArray::data_Add(r.data_Clone()); }

	void math_Intersect(const mcRange &r);
	void math_Union(const mcRange &r);

	//! Returns TRUE if this range contains only finite ranges.
	bool math_isFinite() const;

	mcExtRange *data_Clone() const
		{ return new mcExtRange(*this); }

	mcRange *data_GetRange(int n) 					{ return (mcRange *)data_GetItem(n); }
	const mcRange *data_GetRange(int n) const 		{ return (mcRange *)data_GetItem(n); }
};

#endif		// RANGE_H
