///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Solver.h
//! \brief  Defines the mcSolver, mcSolverArray,
//!         mcSystemStepArray and mcLineStepArray classes.
//!
//! See Solver.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef SOLVER_H
#define SOLVER_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "Solver.h"
#endif

// required includes
#include <wx/panel.h>
#include "mc/Range.h"
#include "mc/MathLine.h"


// these will be defined later...
class mcMathOrSystem;
class mcLineStepArray;
class mcSystemStepArray;


//! The max number of lines solved by a mcSolver.
#define mcSOLVER_MAXLINES				16

//! Returns the C-string associated with the given step.
#define mcTXTSTEP(x)					((x).io_GetInlinedExpr().c_str())


//! An algorithm designed to solve a specific type of problem.
//! This is an abstract class which must be used as base class for
//! all the solvers handled by mcMathCore.
class mcSolver 
{
protected:		// member variables

	//! The unique identifier of this solver.
	//! This should be set on the derived-class constructor.
	mcSolverType m_nType;

	//! The name of this solver.
	//! This should be set on the derived-class constructor.
	wxString m_strName;

	//! A short description of this solver. This should be set
	//! on the derived-class constructor.
	wxString m_strDesc;	

	//! The description of the last error (if any).
	wxString m_strLastErr;

	//! The lenght value for the last step.
	mcRealValue m_fLastLenght;


protected:		// handy variables

	//! The unknown respect to the system is being solved.
	const mcSymbolProperties *m_pUnkArr[mcSOLVER_MAXLINES];

	//! Sets to NULL the array containing the unknowns.
	void math_ResetUnkArray();


protected:		// utilities

	//! Creates a new mcMathOrSystem copying the given one; sets the
	//! #m_pCurrLine to point to the first line of the new system
	//! and then returns it.
	//mcMathMng *math_GetNewStep(const mcMathMng *last);

	//! Creates a new step only if the operations performed since
	//! the previous step are big enough to justify the creation of
	//! a new step (see #m_fLastLenght).
	//mcMathMng *math_GetNewSmartStep(const mcMathMng *last);

	//! Returns the n-th unknown symbol found in the given system.
	mcSymbolProperties *math_GetUnknown(const mcMathOrSystem &sys, int n) const;


protected:		// simplify/expand

	//! Performs a simplification step on the given system and puts
	//! the new step in the given mcSystemStepArray.
	//! If \c p is not NULL, then it is set to the result of the
	//! operation. Returns the new step added to the array.
	void math_DoSimplifyStep(long lflags, long rflags, mcMathLine &tosimplify, 
							mcSystemStepArray &steps, mcExpSimRes *p = NULL);

	//! Performs repeated simplification steps until the given system
	//! is not completely simplified.
	//! Returns the last step added to the given mcLineStepArray.
	void math_DoCompleteSimplification(long lflags, long rflags, mcMathLine &tosimplify, 
							mcSystemStepArray &steps);

	//! Like #math_DoSimplifyStep but calls mcElementHelpers::math_Expand.
	void math_DoExpandStep(long lflags, long rflags, mcMathLine &tosimplify, 
							mcSystemStepArray &steps, mcExpSimRes *p = NULL);
	
	//! Like #math_DoCompleteSimplification but calls mcElementHelpers::math_Expand.
	void math_DoCompleteExpansion(long lflags, long rflags, mcMathLine &tosimplify, 
							mcSystemStepArray &steps);

	//! Creates the solution
	void math_SplitInTwoEquations(const mcPolynomial &lfirsteq, const mcPolynomial &lseceq,
				const mcPolynomial &rfirsteq, const mcPolynomial &rseceq, mcSystemStepArray &steps,
				mcLogicOperator conn = mcLO_OR);
	void math_SplitInTwoEquations(mcMathLine *firsteq, mcMathLine *seceq,
				mcSystemStepArray &steps, mcLogicOperator conn = mcLO_OR);
	void math_SplitInTwoEquations(const mcMathLine &firsteq, const mcMathLine &seceq,
				mcSystemStepArray &steps, mcLogicOperator conn = mcLO_OR);


public:
	mcSolver() { math_ResetUnkArray(); }
	virtual ~mcSolver() {}



	/////////////////////////////////////////////////////////////////////////
	//! \name Getters.
	//! Some inlined getters. @{

	//! Returns the description of this solver.
	wxString math_GetDesc() const			{ return m_strDesc; }

	//! Returns the name of this solver.
	wxString math_GetName() const			{ return m_strName; }

	//! Returns the description of the last error after
	//! a call to #Solve().
	wxString math_GetLastErr() const		{ return m_strLastErr; }

	//! Returns the type of this solver.
	mcSolverType math_GetType() const		{ return m_nType; }

	//@}



	/////////////////////////////////////////////////////////////////////////
	//! \name Miscellaneous functions.
	//! Some various functions. @{

	//! Checks this solver; an assert should fail if this function
	//! finds something wrong.
	virtual void data_Check() const;

	virtual bool math_PreSolve(const mcMathOrSystem &, mcSystemStepArray &);
	virtual bool math_PostSolve(const mcMathOrSystem &, mcSystemStepArray &);

	//@}



	/////////////////////////////////////////////////////////////////////////
	//! \name Abstract functions.
	//! These are the core of mcSolver-derived classes. @{

	//! Returns TRUE if this algorithm can be applied on the
	//! given math system solving it for one of the mcSymbol
	//! contained registered as unknowns.
	//! \note This function does not take a mcSymbolProperties
	//!       which indicates the unknown to solve for.because
	//!       the unknown to solve for will be decided by 
	//!       math_PreSolve, _after_ this function has been called.
	virtual bool math_WorksOn(const mcMathOrSystem &) const = 0;

	//! Works like math_WorksOn but this overloaded function also
	//! takes the symbol which will be used as unknown to solve for.
	//! This function is called after the more generic #math_WorksOn
	//! function, when the unknown to be used to solve each specific
	//! line has been decided by #math_PreSolve.
	virtual bool math_WorksOn(const mcMathOrSystem &, 
		const mcSymbolProperties *unk) const	{ return TRUE; }

	//! The most important function: it must apply the algorithm
	//! encapsulated in this class to the given math system.
	//! \todo define solver behaviour for multiple unknowns or equations.
	virtual bool math_Solve(const mcMathOrSystem &, mcSystemStepArray &);

	//! Solves the given mcMathMng object for the given symbol,
	//! filling the given mcSystemStepArray with the intermediate
	//! resolution steps.
	virtual bool math_SolveLine(mcMathLine &, const mcSymbolProperties *unk, 
								mcSystemStepArray &) = 0;

	//! Returns TRUE if this solver is ready to work.
	//! This function can be used to check if all the options/settings for this
	//! solver have been correctly set and the #Solve() function will correctly
	//! work (on a system where #WorksOn returns TRUE).
	virtual bool math_isReady() const = 0;

	//! Returns the panel containing the options for this algorithm.
	//! The returned pointer must be deleted by the caller, when it
	//! is not required anymore...
	//! If the returned value is NULL, no options are available for
	//! this solver.
	virtual wxPanel *gui_GetOptionsPanel() const = 0;

	//@}
};




//! A specialized array containing pointers to mcSolver classes.
//! The contained objects are pointers so that mcSolver-derived classes
//! can be used; however, this technical detail is completely hidden
//! to the user which uses mcSolverArray...
class mcSolverArray : public mcAbstractArray
{
protected:	
	void data_Delete(int n)
		{ delete data_Get(n); }

public:
	mcSolverArray() {}
	virtual ~mcSolverArray() { data_Clear(); }


	//! Adds a new mcSolver-derived class to the array.
	//! Returns TRUE if the operation is performed without problems;
	//! FALSE if another solver with the same name is already present
	//! (and thus the given one could not be added...).
	bool math_Add(mcSolver *solver);

	//! Returns TRUE if the given mcSolver-derived class is already contained
	//! in the array.
	bool math_Contains(const mcSolver *) const;

	//! Returns an array of wxString containing the descriptions for all
	//! entries of this array.
	wxArrayString io_GetDescArray() const;

	//! Checks all the solver registered in this array.
	void data_Check() const;

	mcSolver *data_Get(int i)				{ return (mcSolver *)data_GetItem(i); }
	const mcSolver *data_Get(int i) const	{ return (mcSolver *)data_GetItem(i); }
};



/*
//! An array of mcMathMng objects.
class mcLineStepArray : public mcAbstractArray
{
protected:
	void data_Delete(int n)		{ delete data_Get(n); }

public:
	mcLineStepArray() {}
	virtual ~mcLineStepArray() { data_Clear(); }

	//! Removes from the array those contiguos steps which differ from the
	//! previous ones for a too small lenght, where the lenght is computed
	//! through the mcMathOrSystem::math_GetLenght() function and the threshold used
	//! is mcMathCore::GetThreshold().
	//! \return The number of steps removed from the array.
	int data_RemoveSimpleSteps();

	const mcMathMng *data_Get(int n) const	{ return (const mcMathMng *)data_GetItem(n); }
	mcMathMng *data_Get(int n)				{ return (mcMathMng *)data_GetItem(n); }

	//! Checks all the steps contained in the array.
	void data_Check() const;
};*/


//! No special behaviour will be forced.
#define mcSYSTEMSTEPARR_NOFLAGS				0

//! Forces a mcSystemStepArray to make its mcSystemStepArray::data_AddLine
//! and mcSystemStepArray::data_AddSys as the
//! mcSystemStepArray::data_AddLineOnlyIfComplexEnough and
//! mcSystemStepArray::data_AddSysOnlyIfComplexEnough functions.
//! In this way, the caller of a mcSolver is able to decide at runtime, if he
//! wants _all_ the steps, even if too simple (see #math_RemoveSimpleSteps),
//! or if he wants to solve the system faster, keeping only the most complex steps.
#define mcSYSTEMSTEPARR_DISCARD_SIMPLE		1


//! An array of mcMathOrSystem objects used to store the 'steps' created
//! by mcSolver-derived classes.
class mcSystemStepArray : public mcAbstractArray
{
#ifdef mcENABLE_DATA
protected:

	//! Holds some info about array management.
	//! E.g. see mcSYSTEMSTEPARR_DISCARD_SIMPLE.
	long m_nFlags;

#endif



public:
	mcSystemStepArray(long flags = mcSYSTEMSTEPARR_NOFLAGS) { m_nFlags=flags; }
	virtual ~mcSystemStepArray() { data_Clear(); }


#ifdef mcENABLE_DATA
protected:

	//! Deletes the n-th step of this array.
	void data_Delete(int n);

public:

	//! Removes those contiguos (i.e. one after the other in the array order)
	//! steps which result to be identical through the mcMathOrSystem::math_Compare
	//! function.
	//! \return The number of steps removed from the array.
	int math_RemoveIdenticSteps();

	//! Removes from the array those contiguos steps which differ from the
	//! previous ones for a too small lenght, where the lenght is computed
	//! through the mcMathOrSystem::math_GetLenght() function and the threshold used
	//! is mcMathCore::GetThreshold().
	//! \return The number of steps removed from the array.
	int math_RemoveSimpleSteps();

	//! A specialized version of mcAbstractArray::math_Add.
	void data_AddSys(mcMathOrSystem *pointer);

	//! Works like #data_AddSys() but in the case that the mcSYSTEMSTEPARR_DISCARD_SIMPLE
	//! flag is set, this function forces anyway the addition of the given pointer.
	void data_AddFinalSys(mcMathOrSystem *pointer);

	//! A "smart" version of #data_AddSys(): before adding to the array
	//! the given mcMathOrSystem object, it checks if it is considered
	//! too "simple" to be added comparing it with the previous one.
	//! If it results too simple, it is deleted; otherwise, it is added
	//! to the array normally.
	void data_AddSysOnlyIfComplexEnough(mcMathOrSystem *pointer);

	//! Adds a *copy* given mcMathLine into a new empty mcMathOrSystem
	//! using the data_AddSys function.
	void data_AddLine(const mcMathLine &p);

	//! Works like #data_AddLine but in the case that the mcSYSTEMSTEPARR_DISCARD_SIMPLE
	//! flag is set, this function forces anyway the addition of the given pointer.
	//! This function should be used only to add the *key* steps: those passages
	//! which must be absolutely retained...
	void data_AddKeyLine(const mcMathLine &p);

	//! Like #data_AddLine but uses the #data_AddSysOnlyIfComplexEnough() function.
	void data_AddLineOnlyIfComplexEnough(const mcMathLine &p);

	//! Adds the given array of lines to this array inserting the
	//! all the mcMathMngs of the given array to the mcMathOrSystems of this
	//! array in the j-th position.
	//void data_AddLines(mcLineStepArray &steps, int j);

	//! Merges the given array of mcMathOrSystem.
	void data_MergeSystems(mcSystemStepArray &, int i);

	//! Checks all the steps contained in the array.
	void data_Check() const;

	const mcMathOrSystem *data_Get(int n) const	
		{ return (const mcMathOrSystem *)data_GetItem(n); }
	mcMathOrSystem *data_Get(int n)
		{ return (mcMathOrSystem *)data_GetItem(n); }
	const mcMathOrSystem *data_GetLast() const	
		{ return data_Get(data_GetCount()-1); }
	mcMathOrSystem *data_GetLast()
		{ return data_Get(data_GetCount()-1); }

#endif		// mcENABLE_DATA


#ifdef mcENABLE_MATH
public:

	//! Calls the mcMathOrSystem::math_AndMerge function
	//! (when \c lo == mcLO_AND) or mcMathOrSystem::math_OrMerge
	//! (when \c lo == mcLO_OR),
	//! on each i-th step of the first array with each i-th step
	//! of the second array, and then appends the result to this
	//! array.
	void math_AppendCombinationOf(const mcSystemStepArray &arr1,
								const mcSystemStepArray &arr2,
								mcLogicOperator lo);
#endif		// mcENABLE_MATH


#ifdef mcENABLE_IO
public:
	
	//! Returns a wxString containing the inlined expressions which describe
	//! the system.
	wxString io_GetInlinedExpr() const;

#endif		// mcENABLE_IO
};

#endif		// SOLVER_H
