///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Styles.h
//! \brief  Defines the mcTextSettings, mcStyle, mcStyleArray classes
//! \author Francesco Montorsi
//! \date   12-10-2004
//!
//! See Styles.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef STYLES_H
#define STYLES_H

// optimization for GCC compiler
#ifdef __GNUG__
//#pragma interface "Styles.h"
#endif


// required includes
#include "mc/MathTypes.h"
#include "mc/MathHelpers.h"
#include <wx/dc.h>



// this will be defined in "Element.h" (which needs this file
// for the definition of mcStyle...)
class mcElement;


//! Definition of the structure used to store foreground, background
//! colors and font which are required parameters for drawing text.
class mcTextSettings : public mcGUIHelper
{
public:		// these variables are public so we can avoid a lot of
	// getters/setters which would provide full access anyway
	
	//! The foreground and background colors.
	wxColour m_clrFg, m_clrBk;
	
	//! The font of this style.
	wxFont m_hFont;
	
public:
	
	//! Sets up the text settings using the n-th token in the given strings.
	//! \param fg The foreground color to use.
	//! \param bk The background color to use.
	//! \param fnt The font settings to use; they should be encoded as:
	//!            "font name-font size-font weight"
	mcTextSettings(const wxColour fg, const wxColour bk, const wxString &fnt) {
		Set(fg, bk, fnt);
	}
	
	mcTextSettings() {}

	void DeepCopy(const mcTextSettings *p) {
		m_clrFg = p->m_clrFg;
		m_clrBk = p->m_clrBk;
		m_hFont = p->m_hFont;
	}

	
	//! Sets up this object with the given colours and the given
	//! string; for the string format, see #Decode.
	void Set(const wxColour fg, const wxColour bk, const wxString &fnt) {
		SetColours(fg, bk);
		SetFont(fnt);
	}

	//! Changes the font attributes for this object.
	void SetFont(const wxString &fnt) {
		Decode(fnt);
	}

	//! Changes the colors associated to this object.
	void SetColours(const wxColour fg, const wxColour bk) {
		m_clrFg = fg;
		m_clrBk = bk;
	}
	
	//! Decodes the given string and initializes this object with it.
	//! The string should be in this format:
	//! "font family-face name-font size-font weight-font style".
	void Decode(const wxString &fnt);
	
	//! gui_Selects these text settings in the given DC.
	void gui_Select(wxDC &hDC) const {
		hDC.SetTextForeground(m_clrFg);
		hDC.SetTextBackground(m_clrBk);
		hDC.SetFont(m_hFont);
	}
	
	//! Returns TRUE if the class has been correctly set up.
	bool Ok() const {
		return m_hFont.Ok();
	}

	//! Scales the font size by given value.
	void ScaleSize(float f) {
		m_hFont.SetPointSize((int)(m_hFont.GetPointSize()*f));
	}
};



//! Definition of the structure used to store all the attributes associated
//! with a specific exponent level.
class mcStyle : public mcGUIHelper
{
protected:				// member variables
	
	
	//! The number of entries in the #m_pTextSettings array.
	//! See also the #GetIndexFor() function.
#define mcSTYLE_TEXTSETTINGS_NUM		6
	
	//! The index of the entry in the #m_pTextSettings array reserved
	//! for special uses (like drawing of greek & math symbols).
#define mcSTYLE_SPECIAL_SETTINGS		mcSTYLE_TEXTSETTINGS_NUM-1		// always the last
	
	
	//! The text settings for MathCore basic classes.
	mcTextSettings m_pTextSettings[mcSTYLE_TEXTSETTINGS_NUM];
	
	
public:				// functions
	
	//! Initializes this class with the given strings. Each string (except
	//! for the first) should be divided by '|' to specify the individual
	//! settings respectively for NUMBERS, FUNCTIONS, SYMBOLS, OPERATORS,
	//! SUBSCRIPTS.
	//! \param fg The foreground colors array.
	//! \param bk The background colors array.
	//! \param fnt The font settings array.
	mcStyle(const wxColour *fg, const wxColour *bk, const wxString *fnt) {
		Set(fg, bk, fnt);
	}
	
	mcStyle() {}
	virtual ~mcStyle() {}
	
	
	//! Checks if this style is correctly set up.
	bool Ok();
	
	//! gui_Select this style in the given DC; uses the given pointer to detect
	//! which text settings should be used.
	void gui_Select(wxDC &hDC, const mcElement &e) const {
		m_pTextSettings[GetIndexFor(e)].gui_Select(hDC);
	}

	void DeepCopy(const mcStyle *p) {
		for (int i=0; i < mcSTYLE_TEXTSETTINGS_NUM; i++)
			m_pTextSettings[i].DeepCopy(&p->m_pTextSettings[i]);
	}

	void ScaleSize(float f) {
		for (int i=0; i < mcSTYLE_TEXTSETTINGS_NUM; i++)
			m_pTextSettings[i].ScaleSize(f);
	}
	
	
	
	///////////////////////////////////////////////////////////////////////////
	//! \name GET functions.
	//! The mcStyle getters. @{
	
	//! Returns the index of the #m_pTextSettings array for the given
	//! element type.
	int GetIndexFor(mcElementType t) const;
	
	//! See #GetIndexFor(mcElementType).
	int GetIndexFor(const mcElement &) const;
	
	//! Returns the text-settings for the given element (type).
	const mcTextSettings *GetTextSettingsFor(mcElementType t) const {
		return &m_pTextSettings[GetIndexFor(t)];
	}
	
	//! Returns the text-settings for the given element (type).
	const mcTextSettings *GetTextSettingsFor(const mcElement &p) const {
		return &m_pTextSettings[GetIndexFor(p)];
	}
	
	//! Returns the n-th entry of the text settings array.
	const mcTextSettings *GetTextSettings(int n) const {
		return &m_pTextSettings[n];
	}
	
	//! Returns the special entry of the text settings array.
	const mcTextSettings *GetSpecialTextSettings() const {
		return &m_pTextSettings[mcSTYLE_SPECIAL_SETTINGS];
	}
	
	//@}
	
	
	
	///////////////////////////////////////////////////////////////////////////
	//! \name SET functions.
	//! The mcStyle setters. @{
	
	//! Sets the text-settings for the given element (type).
	//! See mcTextSettings::Set() for more info.
	//! \param str For the format of this string, see mcTextSettings::Decode.
	void SetTextSettingsFor(mcElementType t, const wxColour fg = *wxBLACK,
		const wxColour bk = *wxWHITE, const wxString &str = wxEmptyString) {
		m_pTextSettings[GetIndexFor(t)].Set(fg, bk, str);
	}

	//! Sets the font attributes for the given element (type).
	void SetFontFor(mcElementType t, const wxString &str) {
		m_pTextSettings[GetIndexFor(t)].SetFont(str);
	}
	
	//! Sets the special text-settings entry in #m_pTextSettings array.
	//! \param str For the format of this string, see mcTextSettings::Decode.
	void SetSpecialTextSettings(const wxColour fg = *wxBLACK,
		const wxColour bk = *wxWHITE, const wxString &str = wxEmptyString) {
		m_pTextSettings[mcSTYLE_SPECIAL_SETTINGS].Set(fg, bk, str);
	}

	//! Sets the font attributes for the special text-settings entry.
	void SetSpecialFont(const wxString &str) {
		m_pTextSettings[mcSTYLE_SPECIAL_SETTINGS].SetFont(str);
	}

	//! Sets the text-settings for this class.
	void Set(const wxColour *fg, const wxColour *bk, const wxString *fnt) {
		for (int i=0; i < mcSTYLE_TEXTSETTINGS_NUM; i++)
			m_pTextSettings[i].Set(fg[i], bk[i], fnt[i]);
	}

	//! Changes the font attributes for this class.
	//! \param fnt For the format of this string, see mcTextSettings::Decode.
	void SetFonts(const wxString *fnt) {
		for (int i=0; i < mcSTYLE_TEXTSETTINGS_NUM; i++)
			m_pTextSettings[i].SetFont(fnt[i]);
	}

	//! Changes the colors for this class.
	void SetColours(const wxColour *fg, const wxColour *bk) {
		for (int i=0; i < mcSTYLE_TEXTSETTINGS_NUM; i++)
			m_pTextSettings[i].SetColours(fg[i], bk[i]);
	}
	
	//@}
};



//! Container of n mcStyle classes, where n = MATHCORE_MAX_EXPDEPTH.
//! This class can be used to store a set of styles 
class mcStyleArray : public mcGUIHelper
{
protected:

	//! The array of styles for all the exponent depths of mcElementGUI.
	//! This array has mcELEMENTMATH_MAX_EXPDEPTH entries.
	mcStyle *m_pStyles;

	//! Creates the array of styles if it's not already a valid pointer.
	//! This function is required because this class always need to 
	//! work on a valid array but it cannot just define the #m_pStyles
	//! array as follows:
	//!
	//! \code
	//! mcStyle m_pStyles[mcELEMENTMATH_MAX_EXPDEPTH];
	//! \endcode
	//!
	//! In fact, the mcELEMENTMATH_MAX_EXPDEPTH value is defined in Element.h,
	//! where also mcElementGUI (which requires this class) is defined.
	//! This implies that we cannot use that #define here because we cannot
	//! include in this file "Element.h".
	void CreateArray();

public:

	//! Default constructor.
	mcStyleArray() : m_pStyles(NULL) { CreateArray(); }

	//! Constructs the array of styles, copying the given styles.
	mcStyleArray(const mcStyle *arr) : m_pStyles(NULL) {
		CreateArray();
		Set(arr);
	}

	//! Copy constructor.
	mcStyleArray(const mcStyleArray *arr) : m_pStyles(NULL) {
		CreateArray();
		DeepCopy(arr);
	}

	//! Destroys the contained styles.
	virtual ~mcStyleArray();

	//! Deep copies the given array into this one.
	void DeepCopy(const mcStyleArray *);
	


	///////////////////////////////////////////////////////////////////////////
	//! \name SET functions.
	//! The mcStyleArray setters. @{

	//! (Re)sets all the contents of this class.
	void Set(const mcStyle *arr);

	//! (Re)sets the n-th style of this class.
	void Set(const mcStyle *style, int n) {
		m_pStyles[n].DeepCopy(style);
	}

	//! Scales the size of all the text settings contained inside all
	//! the styles contained in this object,
	//! See mcTextSettings::ScaleSize() for more info.
	void ScaleSize(float f = 1.0);

	//@}



	///////////////////////////////////////////////////////////////////////////
	//! \name GET functions.
	//! The mcStyleArray setters. @{
	
	//! Returns the number of mcStyles stored in this class.
	//! The returned number is always the same because the maximum exponent
	//! depth of the expressions the user inputs is a #define which, obviously, 
	//! cannot change at run-time.
	static int GetCount();

	//! Returns the style for the given element.
	//! \return The index (in the style array) of the style for the current element
	int gui_GetStyleIndexFor(const mcElement &p) const;

	//! Returns the mcStyle associated to the n-th exponent depth.
	const mcStyle *Get(int n) const {
		return &m_pStyles[n];
	}

	//! Returns the style for given element.
	const mcStyle *gui_GetStyleFor(const mcElement &p) const {
		return Get(gui_GetStyleIndexFor(p));
	}

	//@}
};

#endif		// STYLES_H
