///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Symbol.h
//! \brief  Defines the mcSymbolData, mcSymbolGUI, mcSymbolMath,
//!         mcSymbolIO, mcSymbol classes.
//!
//! See Symbol.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef SYMBOL_H
#define SYMBOL_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "Symbol.h"
#endif

// required includes
#include "mc/ExpElement.h"



mcDEFINE_HELPER_CLASSES(mcSymbol)



//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
//! The symbol is always registered in at least one of mcSymbol global arrays
//! (mcSymbol::arrUnknowns, mcSymbol::arrParameters, mcSymbol::arrConstants
//! or mcSymbol::arrUnregistered).
//! This class just holds a pointer to the right entry of one of those arrays
//! (a mcSymbolProperties class), thus this class doesn't contain any info about 
//! the mathematical meaning of the symbol: in fact, 
//! those info are global info which shouldn't be owned by a particular
//! mcSymbolData class).
class mcSymbolHelpers : public mcExpElementHelpers
{
	mcDEFINE_REFERENCE_DATA(mcSymbol, mcET_SYMBOL);

#ifdef mcENABLE_DATA
protected:		// DATA variables

	//! Points to a mcSymbolProperties class, containing the properties
	//! of this mcSymbol.
	mcSymbolProperties *mdata_pEntry;

#endif



#ifdef mcENABLE_GUI
protected:		// GUI variables
	
	// mcSymbol uses the mcExpElementGUI cursor positions + 
	// cursor positions defined here:	 
#define mcSYMBOL_LEFTMOST		0		//!< Cursor is at the left of the symbol.
#define mcSYMBOL_RIGHTMOST		1		//!< Cursor is at the right of the symbol.
	
	int mgui_nCursorPos;

public:			// customizable variables

	//! The color used to draw unregistered symbols.
	//! The parameters, constants and unknowns which are not registered in the
	//! appropriate buffers, are drawn with this colour.
	static wxColour sgui_clrUndefSymbols;

#endif


#ifdef mcENABLE_MATH
public:		// customizable variables

	static int smath_nUnknownOrderPos;
	static int smath_nParameterOrderPos;
	static int smath_nConstantOrderPos;

#endif



public:

	mcSymbolHelpers() {
		mdata_pEntry = NULL;

		// mcSymbol allows both exponents and subscript
		mdata_bSubEnabled = mdata_bExpEnabled = TRUE;
		data_Init();
	}

	virtual ~mcSymbolHelpers() { data_Unlink(); }

protected:

	void gui_Init() {

		// this sets the cursor pos to zero
		mcExpElementHelpers::gui_Init();

		// cursor begins positioned at the end of the symbol; infact, when
		// the mcSymbol class has been allocated by MathCore input system, the user
		// has already pressed the symbol associated with mcSymbol class
		mgui_nCursorPos = mcSYMBOL_RIGHTMOST;
	}




#ifdef mcENABLE_DATA
public:

#ifdef __MCDEBUG__
	//! Returns a plain text representation of this class.
	wxString data_BaseDebug(long flags) const;
#endif

	void data_DeepCopy(const mcElementHelpers *p) {

		// link this symbol to the same mcSymbolProperties of 'p'
		data_LinkWith(((const mcSymbolHelpers *)p)->mdata_pEntry);
		mcExpElementHelpers::data_DeepCopy(p);
	}

	bool data_isSameAs(const mcElementHelpers *p) const {
		if (!mcElementHelpers::data_isSameAs(p))
			return FALSE;
		return mdata_pEntry == ((const mcSymbolHelpers *)p)->mdata_pEntry;
	}



	///////////////////////////////////////////////////////////////////////////////
	//! \name LINKING functions
	//! Link/unlink this symbol to a mcSymbolProperties class. @{

	//! Links this class with the given symbol properties, eventually
	//! unlinking it from its previous mcSymbolProperties.
	void data_LinkWith(mcSymbolProperties *prop)	{
		if (mdata_pEntry) data_Unlink();
		if (prop == NULL) return;
		mdata_pEntry = prop; 
		prop->data_Link(this);
	}

	//! Unlinks this class from the current symbol properties.
	//! VERY IMPORTANT: after the execution of this function, this class
	//! must not be used, because it cannot work unlinked.
	void data_Unlink() {
		if (mdata_pEntry) mdata_pEntry->data_Unlink(this);
		mdata_pEntry = NULL;
	}

	//! Returns TRUE if the class is ready to work.
	bool data_isLinked() const
		{ return mdata_pEntry != NULL; }

	//! Returns TRUE if this class is linked with the given mcSymbolProperties.
	bool data_isLinkedWith(const mcSymbolProperties *p) const
		{ return mdata_pEntry == p; }

	//! data_Updates this symbol, trying to link this element with another
	//! mcSymbolProperties.
	void data_Update();

	//@}



	///////////////////////////////////////////////////////////////////////////////
	//! \name GET functions
	//! Handy getters. @{

	//! Returns the string associated with the eventually present subscript.
	wxString data_GetSubscript() const {
		mcASSERT(mdata_pEntry, wxT("An unlinked symbol ?"));
		return mdata_pEntry->m_strSubscript; // should be in sync with mcExpElement's subscript
	}

	//! Returns the symbol associated with this class.
	wxString data_GetSymbol() const {
		mcASSERT(mdata_pEntry, wxT("An unlinked symbol ?"));
		return mdata_pEntry->m_strSymbol;
	}

	//! Returns the string used to export this symbol as inlined expression.
	wxString data_GetInlinedSymbol() const {
		mcASSERT(mdata_pEntry, wxT("An unlinked symbol ?"));
		return mdata_pEntry->io_GetInlinedSym();
	}

	//! Returns a string containing a single character:
	//! U if this is an unknown, C if this is a constant or P if this is a parameter.
	wxString data_GetEncodedType() const {
		if (data_isParameter()) return wxT("p");
		if (data_isUnknown()) return wxT("u");
		if (data_isConstant()) return wxT("c");
		return wxEmptyString;	// unregistered ?
	}

	//! Returns the numeric value of this symbol if it is a constant.
	mcRealValue data_GetValue() const {
		return mdata_pEntry->m_fValue;
	}

	//! If this is a parameter/unknown, this function returns the domain of this symbol.
	const mcExtRange *data_GetDomain() const {
		return mdata_pEntry->m_pDomain;
	}

	//! Returns the domain range of this symbol.
	mcExtRange *data_GetDomain() {
		return mdata_pEntry->m_pDomain;
	}

	//! Returns the properties of this symbol.
	const mcSymbolProperties *data_GetConstProperties() const {
		return mdata_pEntry;
	}

	mcSymbolProperties *data_GetProperties() {
		return mdata_pEntry;
	}

	//@}



	///////////////////////////////////////////////////////////////////////////
	//! \name CHECK functions
	//! The "isXXXXX" functions. @{

	//! Returns TRUE if this symbol must be drawn with a special font containing
	//! greek symbols because, even if its code if a normal ASCII code.
	bool data_isGreekSymbol() const {
		mcASSERT(mdata_pEntry, wxT("An unlined symbol ?"));
		return mdata_pEntry->m_fEncoding == wxFONTENCODING_ISO8859_7;
	}

	//! Returns TRUE if this symbol is registered in the mcMathCore global
	//! arrays (mcSymbol::arrUnknowns, mcSymbol::arrParameters,
	//! mcSymbol::arrConstants).
	//! Call #data_Update() to be sure to get updated info.
	bool data_isRegistered() const {
		mcASSERT(mdata_pEntry, wxT("An unlinked symbol ?"));
		return mdata_pEntry->data_isRegistered();
	}

	bool data_isParameter() const {
		mcASSERT(mdata_pEntry, wxT("An unlinked symbol ?"));
		return mdata_pEntry->data_isParameter();
	}

	bool data_isUnknown() const {
		mcASSERT(mdata_pEntry, wxT("An unlinked symbol ?"));
		return mdata_pEntry->data_isUnknown();
	}

	bool data_isConstant() const {
		mcASSERT(mdata_pEntry, wxT("An unlinked symbol ?"));
		return mdata_pEntry->data_isConstant();
	}

	//@}



	/////////////////////////////////////////////////////////////////////////////
	//! \name SET functions
	//! The setters of mcSymbolData. @{

	//! Sets the symbol associated with this class. This function checks
	//! if the symbol belongs to the mcSymbol global arrays; if it does
	//! not, then it register to mcSymbol::arrUnregistered, eventually
	//! creating a new entry in that array.
	void data_SetSymbol(const wxString &str);

	//! If this symbol is a constant, this function sets its value.
	void data_SetValue(double d)				{ mdata_pEntry->m_fValue = d; }
	void data_SetAsGreek(bool b = TRUE)			{ mdata_pEntry->m_fEncoding = wxFONTENCODING_ISO8859_7; }
	void data_SetSubscript(const wxString &str);

	//@}
#endif			// mcENABLE_DATA




#ifdef mcENABLE_GUI
public:

	/////////////////////////////////////////////////////////////////////////////////
	//! \name mcExpElement overloaded BASE functions
	//! These functions handle the base of the symbol; the exponent is handled by
	//! mcExpElement functions. @{

	//! Returns zero when the given character is valid but unrecognized
	//! symbol, returns a positive value when the character is a valid and
	//! recognized symbol and returns FALSE when the character
	//! is not a valid symbol.
	bool gui_isBeginKey(const mcKey &ke) const;

	//! A mcSymbol element is always composed by only one character; however,
	//! user could still try to delete it, to enter an exponent or a subscript;
	//! that's why this function doesn't return always TRUE...
	bool gui_isBaseEndKey(const mcKey &key) const;

	//! The mcSymbol class is not a container of elements, so it returns
	//! TRUE (see the mcExpElement::gui_DrawAsActiveOverBase documentation).
	bool gui_DrawAsActiveOverBase() const	{ return TRUE; }
		
	mcInputRes gui_BaseInput(const mcKey &ke, mcElement *pnew);
	mcInputRes gui_SubInput(const mcKey &ev, mcElement *pnew);
	mcInsertRes gui_BaseInsert(const mcElement &, mcElement *);

	mcMoveCursorRes gui_BaseMoveCursor(mcMoveCursorFlag flag, long modifiers);
	void gui_GetBaseCursorPos(mcCursorPos &) const; 
	int gui_BaseMoveCursorUsingPoint(wxDC &, const wxPoint &);
	int gui_GetBaseRelCursorPos(wxDC &dc, wxPoint *pt) const;
	int gui_DrawBase(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const;

	void gui_SetBaseCursorPos(const mcCursorPos &code);
	void gui_EditBase();
	void gui_DoRecalcBaseSize();
	
	//@}

#endif		// mcENABLE_GUI




#ifdef mcENABLE_MATH
public:

	//! Returns TRUE if this symbol should be put before the given one
	//! in an ordered list.
	//! This function takes in count 
	//! 1) the type of symbol (parameter/unknown/constant)
	//! 2) the name of the symbol
	bool math_isListedBeforeOf(const mcElement &s) const;

	//! Returns the order position for this symbol.
	int math_GetOrderPos() const {
		if (data_isParameter()) return smath_nParameterOrderPos;
		if (data_isUnknown()) return smath_nUnknownOrderPos;
		if (data_isConstant()) return smath_nConstantOrderPos;
		return -1;	// unregistered ?
	}



	//////////////////////////////////////////////////////////////////////////
	//! \name mcExpElementMath and mcElementMath abstract functions
	//! Implementation of general math abstract functions. @{

	bool math_CompareThisOnly(const mcElement &, long flags) const;

	mcExpSimRes math_ExpandBase(long flags, mcElement *newelem);
	mcExpSimRes math_SimplifyBase(long flags, mcElement *newelem);
	mcExpSimRes math_SimplifyBaseExp(long flags, mcElement *pnew);

	// these do not apply
	virtual mcBasicOpRes math_Add(const mcElement &, mcElement *p, bool add)		{ return mcBOR_REMOVE_OPERAND; }
	virtual mcBasicOpRes math_MultiplyBaseOnlyBy(const mcElement &, mcElement *p)	{ return mcBOR_REMOVE_OPERAND; }
	virtual mcBasicOpRes math_DivideBaseOnlyBy(const mcElement &, mcElement *p)		{ return mcBOR_REMOVE_OPERAND; }

	//! Symbols can be divided by only other identical symbols:
	//!
	//! a/b       cannot produce a unique simple element like a mcSymbol or mcNumber...
	//! a/a       produces a unique mcNumber set to one
	//! a^3/a^2   produces a unique mcSymbol (that is, a^1)
	//!
	//! This is why in the math_CanBeXXXX() functions we just have to compare the bases
	//! of the symbols.
	bool math_CanBeDivBy(const mcElement &p) const		{ return math_CompareThisOnly(p, TRUE); }
	bool math_CanBeMultWith(const mcElement &p) const	{ return math_CompareThisOnly(p, TRUE); }
	bool math_CanBeAddedWith(const mcElement &p) const	{ return FALSE; }

	mcRealValue math_GetBaseLenght() const	{ return 1; }

	mcRealValue math_EvaluateBase() const;
	mcMathType math_GetBaseMathType() const;

	//@}

#endif		// mcENABLE_MATH



#ifdef mcENABLE_IO
public:

	bool io_isBeginTag(const wxXml2Node &tag) const {
		if (tag.GetName() == wxT("mi"))
			return TRUE;
		return FALSE;
	}

	bool io_isBeginChar(const wxString &str) const {
		if (wxIsalpha(str.GetChar(0)))
			return TRUE;
		return FALSE;
	}
	
	wxXml2Node io_GetBaseMathML(bool bGetPresentation) const;
	wxString io_GetBaseInlinedExpr() const;

	bool io_ImportPresentationMathML(wxXml2Node tag, wxString &pErr);
	bool io_ImportBaseInlinedExpr(const wxString &str, int *count, wxString &pErr);

#endif		// mcENABLE_IO
};




//! Allows the user to input characters as well as digits.
//! In MathCore a symbol can be a parameter, a constant, 
//! or an unknown. Users can type any char they want, but
//! if it is not registered in MathCore parameters, constants
//! or unknowns lists, the symbol will be drawn with a different 
//! color, to underline that it is not recognized by MathCore.
class mcSymbol : public mcExpElement
{
	mcDEFINE_MAIN_CLASS(Symbol, mcExpElement);

public:

	mcSymbol(const mcSymbolProperties *sym)
	{ data_SetRefData(new mcSymbolHelpers()); hlp()->data_LinkWith((mcSymbolProperties *)sym); }

	mcSymbol operator^(const mcPolynomial &m) const
	{ mcSymbol ret(*this); ret.math_RaiseTo(m); return ret; }
	mcSymbol &operator^=(const mcPolynomial &m)
	{ *this = (*this)^m;return *this; }


	/////////////////////////////////////////////////////////////////////////////////
	//! \name Static mcSymbol functions and arrays
	//! The global arrays containing the registered parameters, constants and
	//! unknowns and some related utility functions. @{

	//! List of registered parameters.
	static mcSymbolArray arrParameters;
	
	//! List of registered constants.
	static mcSymbolArray arrConstants;
	
	//! List of registered unknowns.
	static mcSymbolArray arrUnknowns;

	//! List of unregistered symbols, currently present in the math data.
	static mcSymbolArray arrUnregistered;

	// WARNING: A UNREGISTERED mcSYMBOL CANNOT KNOW IF IT IS THE LAST SYMBOL OF
	//          THAT TYPE.... SO, WHEN IT IS DESTROYED IT CANNOT KNOW IF IT HAS
	//          TO DELETE THE UNREGISTERED SYMBOL FROM THE arrUnregistered OR NOT.
	//
	// SOLUTION: mcSymbolProperties HOLDS A LIST OF POINTERS TO THE SYMBOLS
	//           LINKED WITH THAT CLASS AND SO IT ALWAYS KNOW IF IT HAS
	//           INSTANCES OR IT'S UNLINKED.

	//! Inits the symbol arrays. This function must be called before using
	//! any of the following function.
	static void InitSymbols();

	//! Cleanups the symbol arrays.
	static void CleanupSymbols();
	
	//! Checks if a character is a symbol already registered.
	//! If it is, the function returns the identifier code (SYM_***) of the 
	//! symbol representing the given char and fill the pointer with the entry of
	//! that symbol in the respective array; if the given char is not registered in 
	//! any array, it returns SYM_NOTSET; you can also use the last two arguments 
	//! to specify one entry of one array to skip.
	//! \param c The character to test
	//! \param entry A pointer to the variable where the geivn character is registered
	//! \param skipArr The array of registered symbols to skip: use one of SYM_* flags
	//! \param skipEntry The entry to skip in the array to skip
	//! \return A pointer to the array where the symbol has been found
	static mcSymbolArray *math_FindSymbol(
					long flags,
					const wxString &name, 
					wxFontEncoding enc = wxFONTENCODING_DEFAULT,
					const mcRealValue &value = 0.0, 
					const wxString &subscript = wxEmptyString, 
					const wxString &inlined = wxEmptyString,
					int occ = 0,
					int *entry = NULL, 
					int skipArr = -1);

	static mcSymbolArray *math_FindSymbol(long flags,
							const mcSymbolProperties &symtofind, 
							int occurrence = 0, 
							int *entry = NULL,
							int skipArr = -1);

	//! Checks the given symbol array, resolving errors.
	static void CheckArray(mcSymbolArray *);

	//! Checks the 4 symbol arrays for invalid data; it automatically
	//! deletes errors; this function is not very efficient.
	static void CheckSymbols();
	
	//! Parses the given XML tree and loads the symbols stored inside.
	//! If \c removeprevious == TRUE and other symbols with the same
	//! names are found, then they are removed.
	//! The only tags contained in the given node (which must be a
	//! wxXML_ELEMENT_NODE) must be:
	//!
	//! <constant> that has the following accepted properties:
	//!    name = the name of the constant (must be non-empty)
	//!    greek = 1 if the constant is a greek character; 0 otherwise
	//!    subscript = the subscript of the constant (can be empty)
	//!    inlinedoutput = the inlined representation of this constant 
	//!                    (used when exporting as inlined expression)
	//!    value = the numeric value of the constant
	//!
	//! <unknown> that accepts the same properties of <constant>, except for VALUE
	//! <param> that accepts the same properties of <constant>, except for VALUE
	//!
	//! So, a valid example of an XML tree to parse would be:
	//!
	//! <mysymbols>		// the node to give to this fnc (name is not important)
	//!   <param name=wxT("x") greek=wxT("0") subscript=wxT("0") inlinedoutput="a"/>
	//!   <unknown name=wxT("x") greek=wxT("0") subscript=wxT("") inlinedoutput=""/>
	//!   <constant name=wxT("p") greek=wxT("1") subscript=wxT("") inlinedoutput=wxT("PI") value="3.14159"/>
	//! </mysymbols>
	//!
	//! \return TRUE if the given tree was successfully loaded; FALSE otherwise.
	static bool LoadSymbols(const wxXml2Node &main, bool removeprevious = TRUE);

	//! Loads some default parameters, constants, unknowns: pigreco, the 'e'
	//! (base of the natural logarithms), Euler's gamma.... overwriting
	//! (if \c removeprevious == TRUE) eventually present symbols with 
	//! the same names.
	static void LoadDefaultSymbols(bool removeprevious = TRUE);

	//@}



	//! data_Updates this symbol, trying to search the 
	virtual void data_Update() {
		data_Update();
		mcElement::data_Update();
	}

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_SYMBOL; }



	mcWRAPPER void data_LinkWith(mcSymbolProperties *prop)
		{ hlp()->data_LinkWith(prop); }
	mcWRAPPER void data_Unlink()
		{ hlp()->data_Unlink(); }
	mcWRAPPER bool data_isLinked() const
		{ return hlp()->data_isLinked(); }
	mcWRAPPER bool data_isLinkedWith(const mcSymbolProperties *p) const
		{ return hlp()->data_isLinkedWith(p); }

	mcWRAPPER wxString data_GetSubscript() const
		{ return hlp()->data_GetSubscript(); }
	mcWRAPPER wxString data_GetSymbol() const
		{ return hlp()->data_GetSymbol(); }

/*	//! Returns the string used to export this symbol as inlined expression.
	wxString data_GetInlinedSymbol() const {
		mcASSERT(mdata_pEntry, wxT("An unlinked symbol ?"));
		return mdata_pEntry->GetInlinedSym();
	}

	//! Returns a string containing a single character:
	//! U if this is an unknown, C if this is a constant or P if this is a parameter.
	wxString data_GetEncodedType() const {
		if (data_isParameter()) return wxT("p");
		if (data_isUnknown()) return wxT("u");
		if (data_isConstant()) return wxT("c");
		return wxEmptyString;	// unregistered ?
	}

	//! Returns the numeric value of this symbol if it is a constant.
	mcRealValue data_GetValue() const {
		return mdata_pEntry->m_fValue;
	}

	//! If this is a parameter/unknown, this function returns the domain of this symbol.
	const mcExtRange *data_GetDomain() const {
		return mdata_pEntry->m_pDomain;
	}

	//! Returns the domain range of this symbol.
	mcExtRange *data_GetDomain() {
		return mdata_pEntry->m_pDomain;
	}
*/
	//! Returns the properties of this symbol.
	mcWRAPPER const mcSymbolProperties *data_GetConstProperties() const
		{ return hlp()->data_GetConstProperties(); }

	mcWRAPPER mcSymbolProperties *data_GetProperties()
		{ return hlp()->data_GetProperties(); }


/*

	///////////////////////////////////////////////////////////////////////////
	//! \name CHECK functions
	//! The "isXXXXX" functions. @{

	//! Returns TRUE if this symbol must be drawn with a special font containing
	//! greek symbols because, even if its code if a normal ASCII code.
	bool data_isGreekSymbol() const {
		mcASSERT(mdata_pEntry, wxT("An unlined symbol ?"));
		return mdata_pEntry->m_fEncoding == wxFONTENCODING_ISO8859_7;
	}

	//! Returns TRUE if this symbol is registered in the mcMathCore global
	//! arrays (mcSymbol::arrUnknowns, mcSymbol::arrParameters,
	//! mcSymbol::arrConstants).
	//! Call #data_Update() to be sure to get updated info.
	bool data_isRegistered() const {
		mcASSERT(mdata_pEntry, wxT("An unlinked symbol ?"));
		return mdata_pEntry->isRegistered();
	}

	bool data_isParameter() const {
		mcASSERT(mdata_pEntry, wxT("An unlinked symbol ?"));
		return mdata_pEntry->isParameter();
	}

	bool data_isUnknown() const {
		mcASSERT(mdata_pEntry, wxT("An unlinked symbol ?"));
		return mdata_pEntry->isUnknown();
	}

	bool data_isConstant() const {
		mcASSERT(mdata_pEntry, wxT("An unlinked symbol ?"));
		return mdata_pEntry->math_isConstant();
	}

	//@}



	/////////////////////////////////////////////////////////////////////////////
	//! \name SET functions
	//! The setters of mcSymbolData. @{

	//! Sets the symbol associated with this class. This function checks
	//! if the symbol belongs to the mcSymbol global arrays; if it does
	//! not, then it register to mcSymbol::arrUnregistered, eventually
	//! creating a new entry in that array.
	void data_SetSymbol(const wxString &str);

	//! If this symbol is a constant, this function sets its value.
	void data_SetValue(double d)				{ mdata_pEntry->m_fValue = d; }
	void data_SetAsGreek(bool b = TRUE)			{ mdata_pEntry->m_fEncoding = wxFONTENCODING_ISO8859_7; }
	void data_SetSubscript(const wxString &str);
*/

};


#endif	// SYMBOL_H

