///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Text.h
//! \brief  Defines the mcTextData, mcTextGUI, mcTextMath, mcTextIO, 
//!         mcText classes.
//!
//! See Text.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef TEXT_H
#define TEXT_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "Text.h"
#endif


// required includes
#include "mc/MathCore.h"
#include "mc/Decoration.h"
#include "mc/EmptyBox.h"


mcDEFINE_HELPER_CLASSES(mcText)



//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
class mcTextHelpers : public mcDecorationHelpers
{
	mcDEFINE_REFERENCE_DATA(mcText, mcET_TEXT);

#ifdef mcENABLE_DATA
protected:

	//! The text contained by the class.
	wxArrayString mdata_arrStr;

	//! The string used to filter the input; if the string is an
	//! empty string, no filter is used.
	wxString mdata_strFilter;
	
	//! TRUE if this is a multiline control.
	bool mdata_bMultiline;
#endif


#ifdef mcENABLE_MATH
public:		// static

	// An empty mcText object.
	static mcText *smath_pEmpty;
#endif


#ifdef mcENABLE_GUI
protected:

	//! The cursor position: zero when cursor is on the left of the first
	//! character of the string.
	int mgui_nCursorPos;
	
	//! If multiline is TRUE, this variable contains the zero-based
	//! row index where the cursor is set, and mgui_nCursorPos is the
	//! zero-based column index.
	int mgui_nRowPos;
	
#define mcTEXT_ALIGN_LEFT	0
#define mcTEXT_ALIGN_CENTER	1
#define mcTEXT_ALIGN_RIGHT	2

public:

	//! If TRUE, an empty box is drawn when the class is empty.
	bool mgui_bDrawEmptyBoxIfEmpty;

	//! The empty box which is used to draw this element when it's empty
	//! and m_bgui_DrawEmptyBoxIfEmpty == TRUE.
	mcEmptyBox *mgui_pEmptyBox;
#endif




public:

	mcTextHelpers() {
		mdata_bMultiline = FALSE;
		mgui_pEmptyBox = NULL;
		data_Init();
	}

	virtual ~mcTextHelpers() 
	{ mcSAFE_DELETE(mgui_pEmptyBox); }


protected:

	void gui_Init();



#ifdef mcENABLE_DATA
public:

#ifdef __MCDEBUG__
	wxString data_Debug(long flags) const {
		return wxT("mcText         ") + data_GetText();
	}
#endif

	void data_DeepCopy(const mcElementHelpers *p);

	int data_GetChildrenCount() const {
		int n = mcElementHelpers::data_GetChildrenCount();
		if (mcMathCore::Get()->isGUIEnabled())
			n+=(int)(mgui_pEmptyBox != NULL);
		return n;
	}

	const mcElement &data_GetConstChild(int n) const {
		mcRETURN_ELEMENT_CHILD(n, mcDecorationHelpers);
		if (n == 0 && mcMathCore::Get()->isGUIEnabled())
			return *mgui_pEmptyBox;
		return mcEmptyElement;
	}

	void data_SetChild(int n, const mcElement &newchild) {
		mcSET_ELEMENT_CHILD(n, mcDecorationHelpers, newchild);
		if (n == 0 && mcMathCore::Get()->isGUIEnabled()) {
			mcASSERT(newchild.data_GetType() == mcET_EMPTYBOX, wxT("Invalid new child"));
			mcSAFE_DELETE(mgui_pEmptyBox);
			mgui_pEmptyBox = new mcEmptyBox((const mcEmptyBox &)newchild);
		}
 	}

	//! Returns TRUE if the array of this element is empty.
	bool data_isArrayEmpty() const {
		return mdata_arrStr.IsEmpty();
	}

	//! Returns TRUE if the given char must be rejected by
	//! the current filter.
	bool data_isToReject(wxChar c) const {
		wxString str(c, 1);
		if (data_GetFilter() != wxEmptyString && 
			!data_GetFilter().Contains(str))
			return TRUE;
		return FALSE;
	}


	
	///////////////////////////////////////////////////////////////////////////
	//! \name SET functions.
	//! SET functions not overloaded from base class. @{

	//! Sets the text associated with this class.
	void data_SetText(const wxString &str);
	
	//! Enables the multiline mode.
	void data_SetAsMultiline() {
		mdata_bMultiline = TRUE;
	}
	
	//! Disables the multiline mode.
	//! The lines present after the fist, if present, are deleted.
	void data_SetAsSingleline() {
		mdata_bMultiline = FALSE;
		int n = mdata_arrStr.Count()-1;
		if (n > 0) mdata_arrStr.Remove(1, n);
	}

	//! Sets the given string as the current filter; use an empty string
	//! to diable input filtering.
	void data_SetFilter(const wxString &str) {
		mdata_strFilter = str;
	}

	//@}
	


	///////////////////////////////////////////////////////////////////////////
	//! \name GET functions.
	//! GET functions not overloaded from base class. @{

	//! Returns the text stored in this class.
	wxString data_GetText() const {
		wxString tmp;
		for (int i=0; i < data_GetRowCount(); i++)
			tmp += mdata_arrStr[i] + wxT("\n");
		return tmp;
	}
	
	//! Returns the longest string stored in this object.
	wxString data_GetLongestStr() const {
		wxString m;
		for (int i=0; i < data_GetRowCount(); i++)
			if (mdata_arrStr[i].Len() > m.Len())
				m = mdata_arrStr[i];
		return m;
	}

	//! Returns the filter in use.
	wxString data_GetFilter() const {
		return mdata_strFilter;
	}

	//! Returns the number of rows stored in this object; if this
	//! class is working in single-line mode, the result is one.
	int data_GetRowCount() const {
		return mdata_arrStr.GetCount();
	}

	//@}
#endif		// mcENABLE_DATA




#ifdef mcENABLE_GUI
public:

	///////////////////////////////////////////////////////////////////////////
	//! \name mcText specific functions
	//! Implementation of mcElement abstract functions. @{

	//! Returns the height of the i-th row stored in this object.
	int gui_GetRowHeight(wxDC *dc, int i) const {
		wxString str(wxT("1"));
		return gui_GetHeightOf(dc, (mdata_arrStr[i].Len() > 0) ? 
			mdata_arrStr[i] : str);
	}
	
	int gui_GetRowWidth(wxDC *dc, int i) const {
		wxString str(wxT("1"));
		return gui_GetWidthOf(dc, (mdata_arrStr[i].Len() > 0) ? 
			mdata_arrStr[i] : str);
 	}

	//@}



	///////////////////////////////////////////////////////////////////////////
	//! \name mcElement overloaded functions
	//! Implementation of mcElement abstract functions. @{
	
	bool gui_isBeginKey(const mcKey &ev) const {
		// user cannot allocate mcText classes; they can only
		// be member classes of other classes.
		return FALSE;
	}
	
	bool gui_isEndKey(const mcKey &ev) const {
		return FALSE;
	}
	
	void gui_GetCursorPos(mcCursorPos &) const;
		
	void gui_DoRecalcSize();
	void gui_OnSelect(wxDC &dc, wxRect &rc) {}
	void gui_SetCursorPos(const mcCursorPos &code);

	int gui_GetRelCursorPos(wxDC &dc, wxPoint *pt) const;
	mcMoveCursorRes gui_MoveCursor(mcMoveCursorFlag flag, long modifiers);
	int gui_MoveCursorUsingPoint(wxDC &dc, const wxPoint &p);
	mcInputRes gui_Input(const mcKey &ev, mcElement *newelem);
	
	int gui_Draw(wxDC &dc, int x, int y, long flags, int w, wxColour *color) const;
	int gui_Draw(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const 
		{ return gui_Draw(dc, x, y, flags, 0, NULL); }

	void gui_UpdateExpDepth();

	//@}

#endif		// mcENABLE_GUI



	
#ifdef mcENABLE_MATH
public:

	//! This is one of the few math functions which applies on a mcText object...
	bool math_CompareThisOnly(const mcElement &p, long flags) const;

#endif		// mcENABLE_MATH



	
#ifdef mcENABLE_IO
public:

	bool io_isBeginTag(const wxXml2Node &tag) const			{ return FALSE; }
	bool io_isBeginChar(const wxString &str) const			{ return FALSE; }

	wxXml2Node io_GetMathML(bool bGetPresentation) const;
	wxString io_GetInlinedExpr() const;

	bool io_ImportPresentationMathML(wxXml2Node tag, wxString &pErr);
	bool io_ImportInlinedExpr(const wxString &, int *count, wxString &pErr);

#endif		// meNABLE_IO
};


//! A simple class using MathCore mcElement functions to create a 
//! simple editable text box. Used for subscripts.
class mcText : public mcDecoration
{
	mcDEFINE_MAIN_CLASS(Text, mcDecoration);


#ifdef mcENABLE_DATA
public:

	mcWRAPPER bool data_isArrayEmpty() const
		{ return hlp()->data_isArrayEmpty(); }

	mcWRAPPER bool data_isToReject(wxChar c) const
		{ return hlp()->data_isToReject(c); }

	mcWRAPPER void data_SetText(const wxString &str)
		{ hlp()->data_SetText(str); }

	mcWRAPPER void data_SetAsMultiline()
		{ hlp()->data_SetAsMultiline(); }

	mcWRAPPER void data_SetAsSingleline()
		{ hlp()->data_SetAsSingleline(); }

	mcWRAPPER void data_SetFilter(const wxString &str)
		{ hlp()->data_SetFilter(str); }

	mcWRAPPER wxString data_GetText() const
		{ return hlp()->data_GetText(); }

	mcWRAPPER wxString data_GetLongestStr() const
		{ return hlp()->data_GetLongestStr(); }

	mcWRAPPER wxString data_GetFilter() const
		{ return hlp()->data_GetFilter(); }

	mcWRAPPER int data_GetRowCount() const
		{ return hlp()->data_GetRowCount(); }

	mcWRAPPER bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_TEXT; }

#endif		// mcENABLE_DATA


#ifdef mcENABLE_GUI
public:

	mcWRAPPER int gui_GetRowHeight(wxDC *dc, int i) const
		{ return hlp()->gui_GetRowHeight(dc, i); }

	mcWRAPPER int gui_GetRowWidth(wxDC *dc, int i) const
		{ return hlp()->gui_GetRowWidth(dc, i); }

	mcWRAPPER int gui_Draw(wxDC &dc, int x, int y, long flags, int w, wxColour *color) const
		{ return hlp()->gui_Draw(dc, x, y, flags, w, color); }

#endif		// mcENABLE_GUI
};

#endif		// TEXT_H

