///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Value.h
//! \brief  Defines the mcValue classes.
//!
//! See Value.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef VALUE_H
#define VALUE_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "Value.h"
#endif

// required includes
#include "mc/Setup.h"
#include <math.h>			// In any case we'll need the standard pow() since GMP
                            // does not have a mpf_pow(mpf_t rop, mpf_t base, mpf_t exp)

#ifdef mcUSE_GMP

	#include <gmp.h>		// The main include of the GNU Multiple Precision library

#else

	// DUMMY GMP-LIKE INTERFACE (only for debugging)
    // ---------------------------------------------
	typedef struct { double val; } __mpf_struct;
	#define __mpq_struct		__mpf_struct
	#define __mpz_struct		__mpf_struct
	#define mp_exp_t			int

	typedef __mpz_struct* mpz_t;
	typedef const __mpz_struct* mpz_src;
	typedef __mpq_struct* mpq_t;
	typedef const __mpq_struct* mpq_src;
	typedef __mpf_struct* mpf_t;
	typedef const __mpq_struct* mpf_src;


	// MPF functions

	static unsigned long mpf_get_ui(mpf_src op) { return op.val; }
	static void mpf_set_z(mpf_t op, mpz_src t) { op.val = t.val; }
	static void mpf_set_q(mpf_t op, mpq_src t) { op.val = t.val; }
	static void mpf_init(mpf_t op) { /*op = new __mpf_struct;*/ op.val = 0; }
	static void mpf_clear(mpf_t op) { /*delete op;*/ op.val = 0; }
	static void mpf_set(mpf_t op, mpf_src t) { op.val = t.val; }
	static void mpf_set_ui(mpf_t op, unsigned long newval) { op.val = newval; }
	static void mpf_set_str(mpf_t op, const char *str, int) { op.val = atoi(str); }
	static void mpf_abs(mpf_t op, mpf_src t) { op.val = abs(t.val); }
	static void mpf_pow_ui(mpf_t op, mpf_src base, unsigned long exp) { op.val = pow(base.val, exp); }
	static void mpf_floor(mpf_t op, mpf_src t) { op.val = floor(t.val); }
	static void mpf_ceil(mpf_t op, mpf_src t) { op.val = ceil(t.val); }
	static void mpf_trunc(mpf_t op, mpf_src t) { op.val = ceil(t.val); }
	static void mpf_add_ui(mpf_t op, mpf_src t, unsigned long add) { op.val = t.val + add; }
	static void mpf_sub_ui(mpf_t op, mpf_src t, unsigned long add) { op.val = t.val - add; }
	static void mpf_add(mpf_t op, mpf_src t, mpf_src add) { op.val = t.val + add.val; }
	static void mpf_sub(mpf_t op, mpf_src t, mpf_src add) { op.val = t.val - add.val; }
	static void mpf_mul(mpf_t op, mpf_src t, mpf_src add) { op.val = t.val * add.val; }
	static void mpf_div(mpf_t op, mpf_src t, mpf_src add) { op.val = t.val / add.val; }
	static void mpf_set_d(mpf_t op, double t) { op.val = t; }
	static double mpf_get_d(mpf_src op) { return op.val; }
	static int mpf_integer_p(mpf_src op) { return ceil(op.val) == floor(op.val); }

	static int mpf_cmp_d(mpf_src op, double op2) { if (op.val > op2) return 1;
											if (op.val == op2) return 0;
											return -1; }
	static int mpf_cmp_si(mpf_src op, long op2) { return mpf_cmp_d(op, (double)op2); }
	static int mpf_cmp(mpf_src op, mpf_src op2) { return mpf_cmp_d(op, op2.val); }

	static char *mpf_get_str(char *str, int *exp, int base, int digits, mpf_src op) { return wxT(""); }

	static void mp_set_memory_functions (
	static void *(*alloc_func_ptr) (size_t),
	static void *(*realloc_func_ptr) (static void *, size_t, size_t),
	static void (*free_func_ptr) (static void *, size_t)) {}



	// MPZ functions

	#define mpz_init		mpf_init
	#define mpz_clear		mpf_clear
	#define mpz_set			mpf_set
	#define mpz_set_ui		mpf_set_ui
	#define mpz_get_ui		mpf_get_ui
	#define mpz_set_str		mpf_set_str
	#define mpz_abs			mpf_abs
	#define mpz_pow_ui		mpf_pow_ui

	static void mpz_gcd(mpz_t res, mpz_src, mpz_src) { res.val = 0; }
	static void mpz_lcm(mpz_t res, mpz_src, mpz_src) { res.val = 0; }
	static void mpz_set_q(mpz_t res, mpq_src d) { res.val = d.val; }
	static void mpz_set_f(mpz_t res, mpf_src d) { res.val = d.val; }
	static int mpz_sizeinbase(mpf_src d, int) { return 10; }
	static char *mpz_get_str(char *str, int, mpz_src op) { return wxT(""); }



	// MPQ functions

	#define mpq_init		mpf_init
	#define mpq_clear		mpf_clear
	#define mpq_set			mpf_set
	#define mpq_get_ui		mpf_get_ui
	#define mpq_set_str		mpf_set_str
	#define mpq_abs			mpf_abs

	static void mpq_get_den(mpz_t op, mpq_src) { op.val = 1; }
	static void mpq_get_num(mpz_t op, mpq_src t) { op.val = t.val; }
	static void mpq_set_d(mpq_t op, double d) { op.val = d; }
	static void mpq_set_z(mpq_t op, mpz_src d) { op.val = d.val; }
	static void mpq_set_f(mpq_t op, mpf_src d) { op.val = d.val; }
	static void mpq_canonicalize(mpz_t) {}
	static void mpq_set_ui(mpz_t op, unsigned long n, unsigned long d) { op.val = n/d; }
	static double mpq_get_d(mpq_src op) { return op.val; }

#endif



//! The default precision for mcValue-derived classes.
#define mcVALUE_DEFAULTPRECISION							30

//! The default threshold for mcValue::GetSmartStr.
#define mcVALUE_SMARTSTR_THRESHOLD							10

#if wxUSE_UNICODE
	#define WX2GMP(x)		(wxString(x).mb_str(wxConvUTF8).data())
#else
	#define WX2GMP(x)		(wxString(x).mb_str(wxConvUTF8))
#endif

#define GMP2WX(x)			(wxString(x, wxConvUTF8))



// these will be defined later
class mcIntegerValue;
class mcRationalValue;
class mcRealValue;
class mcComplexValue;


//! Allocates n bytes and returns a pointer to the starting address of the 
//! allocated memory. Uses the NEW operator.
//! Used by MathCore to enable GMP to use new and delete C++ operators.
void *mcGMPAlloc(size_t n);

//! Reallocates the oldsz bytes pointed by p returning an address to newsz
//! allocated bytes. Uses NEW and DELETE [].
//! Used by MathCore to enable GMP to use new and delete C++ operators.
void *mcGMPRealloc(void *p, size_t oldsz, size_t newsz);

//! Frees the given pointer of the given size. Uses the DELETE [] operator.
//! Used by MathCore to enable GMP to use new and delete C++ operators.
void mcGMPFree(void *, size_t);




//! \name The GMP/MPFR wrapped structures.
//! The structures used by mcValue-derived classes to implement COW. @{

//! The bit mask used to divide flags from reference count in a
//! GMP struct wrapper.
#define mcVALUE_MASK					0xFFFF0000

#define mcVALUE_NAN_FLAG				0x000F	//!< The value is a NAN (Not a Number).
#define mcVALUE_POSINF_FLAG				0x00F0	//!< The value is +INF.
#define mcVALUE_NEGINF_FLAG				0x0F00	//!< The value is -INF.

//! Defines a structure named NAME derived from BASE which have an
//! additional field required to store both the reference count
//! (in the lower 16 bits) and the flags (in the upper 16 bits).
#define mcVALUE_DEFINE_GMP_STRUCTWRAPPER(name, base)			\
	struct name : base	{										\
		wxUint32 flags;											\
																\
		void SetRefCount(wxUint16 n)							\
			{ flags = (n & ~mcVALUE_MASK) | (GetFlags()<<16); }	\
		wxUint16 GetRefCount() const							\
			{ return (wxUint16)(flags & ~mcVALUE_MASK); }		\
		void SetFlags(wxUint16 n)								\
			{ flags = (((int)n) << 16) | GetRefCount(); }		\
		wxUint16 GetFlags() const								\
			{ return (wxUint16)(flags >> 16); }					\
	};

mcVALUE_DEFINE_GMP_STRUCTWRAPPER(mcMPF, __mpf_struct);
mcVALUE_DEFINE_GMP_STRUCTWRAPPER(mcMPQ, __mpq_struct);
mcVALUE_DEFINE_GMP_STRUCTWRAPPER(mcMPZ, __mpz_struct);

//@}



///////////////////////////////////////////////////////////////////////////
//! \name The mcValue macros
//! The macros used to define & implement those parts of mcValue-derived
//! classes that are redundant code. @{

//! Implements InternalInit() and InternalDestroy().
//! These two functions are used to create and init the structure wrapped
//! by this class.
#define mcVALUE_DEFINE_INTERNALS(x, fncprefix)						\
	void InternalInit(int flags = 0) {								\
		mp = new x; fncprefix##_init(mp);							\
		mp->SetRefCount((wxUint16)1);								\
		mp->SetFlags((wxInt16)flags); }								\
	void InternalDestroy()											\
		{ fncprefix##_clear(mp); delete mp; }

//! Defines and implements the Ref() and Unref() functions used for
//! reference counting.
#define mcVALUE_DEFINE_REFCOUNT(x, y)								\
	/*! The internal GMP structure wrapped by this class. */		\
	x *mp;															\
																	\
	void CopyMP(const x *n)	{										\
		mcASSERT(mp != NULL,										\
						wxT("This object is not initialized"));		\
		y##_set(mp, n); 	/* DO NOT USE MP() !! */				\
		mp->flags = n->flags;										\
	}																\
																	\
	/*! Increments the reference counter of the given struct. */	\
	void Ref(x *p) {												\
		mcASSERT(p->GetRefCount() > 0,								\
						wxT("This object cannot exist"));			\
		p->SetRefCount((wxUint16)(p->GetRefCount()+1));				\
	}																\
																	\
	/*! Decrements the reference counter of the given struct */		\
	/*! and eventually deletes it (if it reaches zero). */			\
	void Unref() {													\
		mcASSERT(mp->GetRefCount() > 0,								\
						wxT("This object cannot exist"));			\
																	\
		if (mp)												        \
			m_nMaxValueSharing = mcMAX(m_nMaxValueSharing,			\
											mp->GetRefCount());		\
																	\
		mp->SetRefCount((wxUint16)(mp->GetRefCount()-1));			\
		if (mp->GetRefCount() == 0)									\
			InternalDestroy();										\
	}
	
//! Defines and implements the non-const and the const getters of the
//! internal GMP structure. Used to implement Copy-On-Write (COW).
#define mcVALUE_DEFINE_INTERNAL_GETTERS(x)								\
	/*! Returns a private copy of the internal GMP structure. */		\
	x *MP() {															\
		mcASSERT(mp->GetRefCount() > 0,									\
						wxT("This object cannot exist"));				\
		if (mp->GetRefCount() == 1)										\
			return mp;													\
																		\
		/* make a private instance copying the shared object */			\
		x *old = mp;													\
		Unref();														\
		InternalInit();													\
		CopyMP(old);													\
		return mp;														\
	}																	\
																		\
	/*! Returns an (eventually shared) instance of */					\
	/*! the internal GMP structure. */									\
	const x *cMP() const {												\
		return mp;														\
	}

//! Defines the flags set/get functions.
#define mcVALUE_DEFINE_SETGET_FLAGS()									\
	int GetFlags() const			{ return (int)cMP()->GetFlags(); }	\
	void SetFlags(int n)			{ MP()->SetFlags((wxInt16)n); }		\
	bool hasSameFlags(int n) const	{ return GetFlags() == n; }

//! Defines some setters common to all mcValue-derived classes.
#define mcVALUE_DEFINE_SETTERS(x, fncprefix)							\
	void SetLong(long n) { fncprefix##_set_si(MP(), n); }				\
	void SetStr(const wxString &str, int base = 10)						\
		{ fncprefix##_set_str(MP(), WX2GMP(str), base); }

//! Defines some getters common to all mcValue-derived classes.
#define mcVALUE_DEFINE_GETTERS(x, fncprefix)							\
	long GetLong() const	{ return fncprefix##_get_ui(cMP()); }

//! Defines some miscellaneous function:
//! - ChangeSign() changes the sign of *this value: this function is 
//!   sometimes useful when the "-" unary operator is ambiguos.
#define mcVALUE_DEFINE_MISCELLANEOUS(rettype, fncprefix)				\
	rettype ChangeSign()												\
	{ rettype ret; fncprefix##_neg(ret.MP(), cMP()); return ret; }

//! Just a short version for
#define mcVALUE_DEFINE_GETSET(x, fncprefix)								\
	mcVALUE_DEFINE_SETGET_FLAGS()										\
	mcVALUE_DEFINE_GETTERS(x, fncprefix)								\
	mcVALUE_DEFINE_SETTERS(x, fncprefix)

#define mcVALUE_DEFINE_FLAGS_EQUAL_OPERATORS(y, fnc, z)													\
	bool operator==(y m) const		 {return (fnc(cMP(), z) == 0) && hasSameFlags(m.GetFlags());}		\
	bool operator!=(y m) const		 {return (fnc(cMP(), z) != 0) || !hasSameFlags(m.GetFlags());}

#define mcVALUE_DEFINE_NOFLAGS_EQUAL_OPERATORS(y, fnc, z)				\
	bool operator==(y m) const		 {return (fnc(cMP(), z) == 0);}		\
	bool operator!=(y m) const		 {return (fnc(cMP(), z) != 0);}	

//! Defines and implements the comparison operators.
#define mcVALUE_DEFINE_NOFLAGS_GTLT_OPERATORS(y, fnc, z)				\
	bool operator<=(y m) const		 {return (fnc(cMP(), z) <= 0);}		\
	bool operator>=(y m) const		 {return (fnc(cMP(), z) >= 0);}		\
	bool operator<(y m) const		 {return (fnc(cMP(), z) < 0);}		\
	bool operator>(y m) const		 {return (fnc(cMP(), z) > 0);}

#define mcVALUE_DEFINE_NOFLAGSCOMPARISON_OPERATORS(y, fnc, z)		\
	mcVALUE_DEFINE_NOFLAGS_GTLT_OPERATORS(y, fnc, z)				\
	mcVALUE_DEFINE_NOFLAGS_EQUAL_OPERATORS(y, fnc, z)

#define mcVALUE_DEFINE_FLAGSCOMPARISON_OPERATORS(y, fnc, z)			\
	mcVALUE_DEFINE_NOFLAGS_GTLT_OPERATORS(y, fnc, z)				\
	mcVALUE_DEFINE_FLAGS_EQUAL_OPERATORS(y, fnc, z)

//! Defines and implements the +,-,*,/ operators.
#define mcVALUE_DEFINE_ARITHMETIC_OPERATORS(rettype, x, y, fncprefix, z, w)	\
	friend rettype operator+(x a, y b)									\
	 {rettype ret; fncprefix##_add(ret.MP(), z, w); return ret;}		\
	friend rettype operator-(x a, y b)									\
	 {rettype ret; fncprefix##_sub(ret.MP(), z, w); return ret;}		\
	friend rettype operator*(x a, y b)									\
	 {rettype ret; fncprefix##_mul(ret.MP(), z, w); return ret;}		\
	friend rettype  operator/(x a, y b)									\
	 {rettype ret; fncprefix##_div(ret.MP(), z, w); return ret;}

//! Defines and implements unary operators.
#define mcVALUE_DEFINE_UNARY_OPERATORS(rettype, x, fncprefix)											\
	rettype &operator=(const rettype &m) 		{ Unref(); mp=m.mp; Ref(mp); return *this;}				\
	rettype &operator=(x &m) 					{ mp=&m; Ref(mp); return *this;}						\
	rettype &operator=(const wxString &str) 	{ SetStr(str); return *this;}							\
	rettype &operator=(int n) 					{ SetInt(n); return *this;}							\
	rettype operator++()						{ fncprefix##_add_ui(MP(), cMP(), 1); return *this;}	\
	rettype operator--()						{ fncprefix##_sub_ui(MP(), cMP(), 1); return *this;}	\
	rettype &operator+=(const rettype &m)		{*this = *this+m;return *this;}							\
	rettype &operator-=(const rettype &m)		{*this = *this-m;return *this;}							\
	rettype &operator*=(const rettype &m)		{*this = *this*m;return *this;}							\
	rettype &operator/=(const rettype &m)		{*this = *this/m;return *this;}

//! Defines all the operators for a mcValue-derived class.
#define mcVALUE_DEFINE_ALLOPERATORS(x, structname, y)									\
	mcVALUE_DEFINE_UNARY_OPERATORS(x, structname, y);									\
																						\
	mcVALUE_DEFINE_ARITHMETIC_OPERATORS(x, const x &, const x &, y, a.cMP(), b.cMP());	\
	mcVALUE_DEFINE_ARITHMETIC_OPERATORS(x, const x &, long, y, a.cMP(), x(b).cMP());	\
	mcVALUE_DEFINE_ARITHMETIC_OPERATORS(x, const x &, int, y, a.cMP(), x(b).cMP());		\
	mcVALUE_DEFINE_ARITHMETIC_OPERATORS(x, long, const x &, y, x(a).cMP(), b.cMP());	\
	mcVALUE_DEFINE_ARITHMETIC_OPERATORS(x, int, const x &, y, x(a).cMP(), b.cMP());		\
																						\
	mcVALUE_DEFINE_FLAGSCOMPARISON_OPERATORS(const x &, __g##y##_cmp, m.cMP());			\
	mcVALUE_DEFINE_NOFLAGSCOMPARISON_OPERATORS(long, __g##y##_cmp_si, m);				\
	mcVALUE_DEFINE_NOFLAGSCOMPARISON_OPERATORS(int, __g##y##_cmp_si, (long)m);			\
	mcVALUE_DEFINE_MISCELLANEOUS(x, y)

#define mcVALUE_DEFINE_DOUBLEOPERATORS(x, structname, y)								\
	mcVALUE_DEFINE_ARITHMETIC_OPERATORS(x, const x &, double, y, a.cMP(), x(b).cMP());	\
	mcVALUE_DEFINE_ARITHMETIC_OPERATORS(x, double, const x &, y, x(a).cMP(), b.cMP());	\
	mcVALUE_DEFINE_NOFLAGSCOMPARISON_OPERATORS(double, __g##y##_cmp_d, m)


//! Put this macro at the beginning of the class declaration.
#define mcVALUE_DEFINE_COW(x, y)					\
	protected:										\
		mcVALUE_DEFINE_INTERNALS(x, y);				\
		mcVALUE_DEFINE_REFCOUNT(x, y);				\
	public:											\
		mcVALUE_DEFINE_INTERNAL_GETTERS(x)

//! Defines some standard constructor + the virtual destructor.
#define mcVALUE_DEFINE_CONSTRUCTORS(x, y)								\
	x()							{ InternalInit(); }						\
	x(long n)					{ InternalInit(); SetLong(n); }			\
	x(int n)					{ InternalInit(); SetLong(n); }			\
	x(const wxString &str)		{ InternalInit(); SetStr(str); }		\
	x(y *m)						{ mp=m; Ref(mp); }						\
	x(const x &m)				{ mp=m.mp; Ref(mp); }					\
	virtual ~x()				{ Unref(); }

//@}





//! A variable holding an high-precision value.
//! This class links MathCore, wxWidgets and mpf_t libraries !!!
class mcValue
{
public:

	//! The maximum number of mcValue-derived classes which shared the same 
	//! GMP structure contemporary.
	static int m_nMaxValueSharing;

public:	
	mcValue() {}		// cannot build anything since everything is declared in derived classes.
	virtual ~mcValue() {}


	//! Initializes the static variables used by mcValue.
	//! This function must be called *before* using any other mcValue function.
	static void Init();

	//! Undoes what #Init() does: destroys static variables.
	//! After this function you cannot use mcValue anymore, unless you
	//! call #Init() another time...
	static void Cleanup();


public:		// standard conversions

	virtual void SetInt(int n)					{ SetLong(n); }
	virtual void SetLong(long n) = 0;
	virtual void SetStr(const wxString &str, int base = 10) = 0;
	
	virtual long GetLong() const = 0;
	virtual int GetInt() const			{ return (int)GetLong(); }

	operator int() const				{ return GetInt(); }
	operator long() const				{ return GetLong(); }
#ifndef __BORLANDC__ 
	operator unsigned long()			{ return (unsigned long)GetLong(); }
#endif

	//! Returns a string in exponential form, thus containing something like:
	//!
	//!   SnnnE+yyyy        example: -4318384739357974916E+6215
	//!
	//! or
	//!
	//!   SnnnE-yyyy        example: +8.208237913789131096523645193E-12873
	//!
	//! where:
	//! - "S" is the sign of the number (+ or -)
	//! - "n" is a digit (0-9)
	//! - wxT("E") is a simple "E" which means that the number must be multiplied by 10^yyyy
	//! - "y" is a digit of the exponent (0-9)
	virtual wxString GetStr() const = 0;

public:		// miscellaneous functions

	//! Returns TRUE if the given double is a "normal" number: not an infinite,
	//! not an 1.#INF and not a NaN...
	bool isValid() const			{ return !isNAN() & math_isFinite(); }

	//! Returns TRUE if this number is not mcValue::pNAN.
	bool isNAN() const				{ return (GetFlags() & mcVALUE_NAN_FLAG) != 0; }

	//! Returns TRUE if this number is not mcValue::pNegInf or mcValue::pPosInf.
	bool math_isFinite() const		{ return !isPosInf() & !isNegInf(); }

	bool isPosInf() const			{ return (GetFlags() & mcVALUE_POSINF_FLAG) != 0; }
	bool isNegInf() const			{ return (GetFlags() & mcVALUE_NEGINF_FLAG) != 0; }

	//! Returns TRUE if the given real number does not have decimal digits
	//! (that is, it can be converted to a mcIntegerValue without rounding).
	virtual bool isInteger() const = 0;

	//! Returns the flags of this number.
	//! See the mcVALUE_XXX_FLAG defines above.
	virtual int GetFlags() const = 0;

	//! Sets the flags of this number.
	//! See the mcVALUE_XXX_FLAG defines above.
	virtual void SetFlags(int n) = 0;

	//! math_Adds a flag to this number.
	//! See the mcVALUE_XXX_FLAG defines above.
	void math_AddFlag(int n)		{ SetFlags(n | GetFlags()); }
};

//! An integer value positive or negative.
class mcIntegerValue : public mcValue
{
	mcVALUE_DEFINE_COW(mcMPZ, mpz);

public:		// static

	static mcIntegerValue *pOne;
	static mcIntegerValue *pZero;
	static mcIntegerValue *pNAN;			//!< Not a number. See #isNAN().
	static mcIntegerValue *pPosInf;			//!< Positive infinite. See #math_isFinite().
	static mcIntegerValue *pNegInf;			//!< Negative infinite. See #math_isFinite().

public:
	mcVALUE_DEFINE_CONSTRUCTORS(mcIntegerValue, mcMPZ);
	mcIntegerValue(const mcRationalValue &m);
	mcIntegerValue(const mcRealValue &m);


public:

	mcVALUE_DEFINE_GETSET(mcIntegerValue, mpz);
	mcVALUE_DEFINE_ALLOPERATORS(mcIntegerValue, mcMPZ, mpz);

	//! Returns the absolute value of this number.
	mcIntegerValue abs() const
		{ mcIntegerValue ret; mpz_abs(ret.MP(), cMP()); return ret; }

	//! Raise this number to the given one.
	mcIntegerValue pow(unsigned long int exp) const
		{ mcIntegerValue ret; mpz_pow_ui(ret.MP(), cMP(), exp); return ret; }

	mcIntegerValue GetMax(const mcIntegerValue &n) const
		{ if (*this > n) return *this; else return n; }

	mcIntegerValue GetMin(const mcIntegerValue &n) const
		{ if (*this < n) return *this; else return n; }

	//! See mcValue#GetStr.
	wxString GetStr() const;

	//! mcIntegerValues are always integer... :-)
	bool isInteger() const								{ return TRUE; }

	//! Returns TRUE if this number and the given one are prime.
	bool isPrimeTo(const mcIntegerValue &n) const		{ return (GCD(n) == 1); }

	static mcIntegerValue GetComb(const mcIntegerValue &n1, const mcIntegerValue &n2)
	{ mcIntegerValue rop; mpz_bin_ui(rop.MP(), n1.cMP(), n2.GetLong()); return rop; }

	//! Returns the highest common factor between the two numbers supplied.
	//! If we have two integers, GCD is the largest integer 
	//! that divides evenly both numbers
	//! Euclid's GCD Algorithm is as follows - 
	//!
	//!	 GCD(0, 0) = 1
	//!	 GCD(x, 0) = x
	//!	 GCD(x, y) = GCD(y, x mod y)
	mcIntegerValue GCD(const mcIntegerValue &n) const	
	{ mcIntegerValue ret; mpz_gcd(ret.MP(), cMP(), n.cMP()); return ret; }
	
	//! Returns the least common multiple between the two numbers supplied.
	//! If we have two numbers, LCM is the smallest integer that both of 
	//! them divide evenly into.
	//! LCM Algorithm is as follows - 
	//!
	//!LCM(x, y)
	mcIntegerValue LCM(const mcIntegerValue &n) const
	{ mcIntegerValue ret; mpz_lcm(ret.MP(), cMP(), n.cMP()); return ret; }
};


//! A rational value stored internally by GMP in the form of two mcIntegerValues
//! representing the numerator and denominator. Thus, numbers like 2/3 = 0.6666...
//! can be stored without rounding and be approximated to any desired precision.
class mcRationalValue : public mcValue
{
	mcVALUE_DEFINE_COW(mcMPQ, mpq);

public:
	mcVALUE_DEFINE_CONSTRUCTORS(mcRationalValue, mcMPQ);
	mcRationalValue(const mcIntegerValue &m);
	mcRationalValue(const mcRealValue &m);
	mcRationalValue(const mcIntegerValue &num, const mcIntegerValue &den);


	mcIntegerValue GetDen() const			
		{ mcIntegerValue ret; mpq_get_den(ret.MP(), cMP()); return ret; }
	mcIntegerValue GetNum() const			
		{ mcIntegerValue ret; mpq_get_num(ret.MP(), cMP()); return ret; }

	//! Remove any factors that are common to the numerator and denominator 
	//! and make the denominator positive.
	void Canonicalize()
	{ mpq_canonicalize(MP()); }

public:

	mcVALUE_DEFINE_SETGET_FLAGS();
	mcVALUE_DEFINE_MISCELLANEOUS(mcRationalValue, mpq);
	//mcVALUE_DEFINE_ALLOPERATORS(mcRationalValue, mcMPQ, mpq)

	void SetLong(long n)		{ mpq_set_si(MP(), n, 1); }
	void SetDouble(double n)	{ mpq_set_d(MP(), n); }
	void SetStr(const wxString &str, int base = 10)
		{ mpq_set_str(MP(), WX2GMP(str), base); }

	double GetDouble() const		{ return mpq_get_d(cMP()); }
	long GetLong() const			{ return (long)GetDouble(); }

	//! See mcValue#GetStr.
	wxString GetStr() const;

	//! Returns TRUE if the denominator of this number is 1.
	bool isInteger() const					{ return GetDen() == 1; }

	//! Returns the absolute value of this number.
	mcRationalValue abs() const				
		{ mcRationalValue ret; mpq_abs(ret.MP(), cMP()); return ret; }

	//! Returns the fractional part of this number, removing the integer one.
	//! This function just subtracts the denominator from the numerator while
	//! the numerator is less than the denominator:
	//!
	//! 5/3 = 1.66666666666 ...     .      2/3 = 0.66666666666 ...
	//! 19/13 = 1,461538461538461   .      6/13 = 0,461538461538461
	mcRationalValue GetFractionalPart() const;

	//! Returns the number of fractional digits.
	int GetFractionalDigitCount(int digits = mcVALUE_DEFAULTPRECISION) const;

	//! Returns the integer part of this number, removing the fractional one.
	//! This function just subtracts to this number the value returned by
	//! #GetFractionalPart().
	mcIntegerValue GetIntegerPart() const;

	//! Transform a copy of this number multiplying it by
	//! 10^GetFractionalDigitCount(*this), thus transforming it into
	//! an integer number.
	mcIntegerValue MakeInteger(int digits = mcVALUE_DEFAULTPRECISION) const;
};


//! A real value like standard float and double types but with any desired
//! precision kept during calculations.
class mcRealValue : public mcValue
{
	mcVALUE_DEFINE_COW(mcMPF, mpf);

protected:

	//! Returns a raw string, not elaborated, as GMP returns.
	wxString GetBaseStr(int *p, int digits, int *n) const;

public:		// static

	static mcRealValue *pOne;
	static mcRealValue *pZero;
	static mcRealValue *pNAN;			//!< Not a number. See #isValid().
	static mcRealValue *pPosInf;			//!< Positive infinite. See #math_isFinite().
	static mcRealValue *pNegInf;			//!< Negative infinite. See #math_isFinite().

public:
	mcVALUE_DEFINE_CONSTRUCTORS(mcRealValue, mcMPF);
	mcRealValue(double f)					{ InternalInit(); SetDouble(f); }
	mcRealValue(const mcRationalValue &m)	{ InternalInit(m.GetFlags()); mpf_set_q(MP(), m.cMP()); }
	mcRealValue(const mcIntegerValue &m)	{ InternalInit(m.GetFlags()); mpf_set_z(MP(), m.cMP()); }


public:		// math.h-like functions
	
	mcRealValue pow(unsigned long n) const				
		{ mcRealValue ret; mpf_pow_ui(ret.MP(), cMP(), n); return ret; }
	mcRealValue pow(int n) const				
		{ mcRealValue ret; mpf_pow_ui(ret.MP(), cMP(), n); return ret; }
	mcRealValue pow(double n) const				
		{ mcRealValue ret; ret.SetDouble(::pow(GetDouble(), n)); return ret; }
	mcRealValue pow(const mcRealValue &n) const				
		{ return this->pow(n.GetDouble()); }
	mcRealValue root(const mcRealValue &n)
		{ mcRealValue exp=1.0 / n; return this->pow(exp); }

	mcRealValue abs() const				{ mcRealValue ret; mpf_abs(ret.MP(), cMP()); return ret; }
	mcRealValue floor() const			{ mcRealValue ret; mpf_floor(ret.MP(), cMP()); return ret; }
	mcRealValue ceil() const			{ mcRealValue ret; mpf_ceil(ret.MP(), cMP()); return ret; }
	mcRealValue trunc() const			{ mcRealValue ret; mpf_trunc(ret.MP(), cMP()); return ret; }

	//! Returns TRUE if all the approximation functions (#floor, #ceil, #trunc)
	//! return the same value.
	bool isInteger() const				{ return mpf_integer_p(cMP()) != 0; }

	//! Returns the reciprocal of this number.
	mcRealValue GetReciprocal() const	{ return 1.0 / (*this); }

public:		// operators

	mcVALUE_DEFINE_ALLOPERATORS(mcRealValue, mcMPF, mpf);
	mcVALUE_DEFINE_DOUBLEOPERATORS(mcRealValue, mcMPF, mpf);
	mcVALUE_DEFINE_GETSET(mcRealValue, mpf);
	mcVALUE_DEFINE_FLAGSCOMPARISON_OPERATORS(mcIntegerValue, mpf_cmp_si, m.GetLong());

	//! See mcValue#GetStr.
	wxString GetStr() const			{ return GetStr(mcVALUE_DEFAULTPRECISION); }
	wxString GetStr(int digits) const;

	//! Returns a string in extended form:
	//!
	//!   Snnnnnnnnnnnnnnnnnnnn        example: -4318384739357974916
	//! 
	//! This function uses GetStr() and then "unroll" the trailer exponent
	//! adding the required zeroes at the end/begin of the number (for positive/
	//! negative exponents) and the dot.
	//! \note This function could take a *lot* of memory for big big numbers
	//!       (like 123E+10000) and this can mean DoS (Denial Of Service):
	//!       use with care !!!
	virtual wxString GetExtStr(int digits = mcVALUE_DEFAULTPRECISION) const;

	//! Returns a string in exponential form, if the exponent is greater
	//! than mcVALUE_SMARTSTR_THRESHOLD; returns a string in extended form
	//! if the exponent is less than mcVALUE_SMARTSTR_THRESHOLD.
	virtual wxString GetSmartStr(int threshold = mcVALUE_SMARTSTR_THRESHOLD,
								int digits = mcVALUE_DEFAULTPRECISION) const;


	double GetDouble() const					{ return mpf_get_d(cMP()); }
	void SetDouble(double n)					{ mpf_set_d(MP(), n); }

	mcRealValue &operator=(double d)			{ SetDouble(d); return *this;}							\
	operator double() const						{ return GetDouble(); }

public:			// digit-manipulation functions

	// -------------------
	// Digit getters
	// -------------------

	int GetNumOfDigits(int digits = mcVALUE_DEFAULTPRECISION) const;
	mcRealValue GetFromDigitRange(int begin, int end = -1, int digits = mcVALUE_DEFAULTPRECISION) const;


	// --------------------
	// Digit manipulation
	// --------------------

	mcRealValue DeleteDigit(int pos, int digits = mcVALUE_DEFAULTPRECISION) const;
	mcRealValue InsertDigit(wxChar digit, int pos, int digits = mcVALUE_DEFAULTPRECISION) const;
	mcRealValue AppendDigit(wxChar digit, int digits = mcVALUE_DEFAULTPRECISION) const
	{ return InsertDigit(digit, -1, digits); }
};


//! A complex number.
class mcComplexValue : public mcValue
{


};





/*
class mcNaturalValue : public mcValue
{
};*/

/*
inline mcValue pow(const mcValue &x,const mcValue &y)		{return x.pow(y.GetLong());}
inline mcValue gcd(const mcValue &u,const mcValue &v)		{return u.GCD(v);}
inline mcValue lcm(const mcValue &u,const mcValue &v)		{return u.LCM(v);}
*/

#endif		// VALUE_H
